/**
 * Filename     : baseCompartmentChange.cc
 * Description  : A base class describing variable updates
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : October 2003
 * Revision     : $Id: baseCompartmentChange.cc 474 2010-08-30 15:23:20Z henrik $
 */
#include<vector>

#include "baseCompartmentChange.h"
#include "compartmentDivision.h"
#include "compartmentRemoval.h"
#include "../reactions/elongatedBacteria/CellDivision.h"
#include "../reactions/sphericalBacteria/CellDivision.h"

BaseCompartmentChange::~BaseCompartmentChange() {
  std::cerr << "BaseCompartmentChange::~BaseCompartmentChange()\n";
}

//!Factory creator, all creation should be mapped onto this one 
/*! Given the idValue a compartmentChange of the defined type is returned
(using new Class).*/
BaseCompartmentChange *
BaseCompartmentChange::
createCompartmentChange(std::vector<double> &paraValue,
												std::vector< std::vector<size_t> > &indValue, 
												const std::string &idValue ) {
	//Divisions
  //(compartmentDivision.h,compartmentDivision.cc)
  //////////////////////////////////////////////////////////////////////
  if( idValue=="sphere::divisionRandomDirection" ) 
    return new Sphere::DivisionRandomDirection(paraValue,indValue);
  else if( idValue=="sphere::divisionRandomAndSCDirection" ) 
    return new Sphere::DivisionRandomAndSCDirection(paraValue,indValue);
  else if( idValue=="sphere::divisionRandomDirectionOnSCSurface" ) 
    return new Sphere::DivisionRandomDirectionOnSCSurface(paraValue,indValue);
  else if( idValue=="sphere::divisionDirection" ) 
    return new Sphere::DivisionDirection(paraValue,indValue);
  else if( idValue=="sphere::divisionVelocity" ) 
    return new Sphere::DivisionVelocity(paraValue,indValue);
  else if( idValue=="divisionRandomDirectionSphere" ||
	   idValue=="divisionRandomAndSCDirectionSphere" ||
	   idValue=="divisionRandomDirectionSphereOnSCSurface" ||
	   idValue=="divisionDirectionSphere" ) {
    std::cerr << "baseCompartmentChange.cc All divisionXSphere names have been moved into " 
	      << "sphere::divisionX.";
    exit(0);
  }
	   
  else if( idValue=="divisionCellDirectionSphereBud" )
    return new DivisionCellDirectionSphereBud(paraValue,indValue);
  else if( idValue=="divisionCellDirectionEllipse" )
    return new DivisionCellDirectionEllipse(paraValue,indValue);
  else if( idValue=="divisionCellDirectionCigar" )
    return new DivisionCellDirectionCigar(paraValue,indValue);
  else if( idValue=="divisionCellDirectionCigarStochastic" )
    return new DivisionCellDirectionCigarStochastic(paraValue,indValue);
  else if( idValue=="elongatedBacteria::cellDivision2D" )
    return new elongatedBacteria::CellDivision<2>(paraValue,indValue);
  else if( idValue=="elongatedBacteria::cellDivision3D" )
    return new elongatedBacteria::CellDivision<3>(paraValue,indValue);
  else if( idValue=="sphericalBacteria::cellDivision2D" )
    return new sphericalBacteria::CellDivision<2>(paraValue,indValue);
  else if( idValue=="sphericalBacteria::cellDivision3D" )
    return new sphericalBacteria::CellDivision<3>(paraValue,indValue);
  else if( idValue=="divisionLens" )
    return new DivisionLens(paraValue,indValue);
  //Removals
  //(compartmentRemoval.h,compartmentRemoval.cc)
  //////////////////////////////////////////////////////////////////////
  else if( idValue=="removalThreshold" ) 
    return new RemovalThreshold(paraValue,indValue);
  else if( idValue=="removalRadialThreshold" ) 
    return new RemovalRadialThreshold(paraValue,indValue);
  else if( idValue=="removalThresholdBactPos" ) 
    return new RemovalThresholdBactPos(paraValue,indValue);
  //Default, if nothing found
  //////////////////////////////////////////////////////////////////////
  else {
    std::cerr << "\nBaseCompartmentChange::createCompartmentChange() "
	      << "WARNING: CompartmentChangetype " 
	      << idValue << " not known, no compartmentChange created.\n\7";
    exit(-1);
  }
 }

//!This creator reads from an open file and then calls for the main creator
BaseCompartmentChange * 
BaseCompartmentChange::createCompartmentChange(std::istream &IN ) {
  
  std::string idVal;
  size_t pNum,levelNum;
  IN >> idVal;
  IN >> pNum;
  IN >> levelNum;
  std::vector<size_t> varIndexNum( levelNum );
  for( size_t i=0 ; i<levelNum ; i++ )
    IN >> varIndexNum[i];

  std::vector<double> pVal( pNum );
  for( size_t i=0 ; i<pNum ; i++ )
    IN >> pVal[i];

  std::vector< std::vector<size_t> > varIndexVal( levelNum );
  for( size_t i=0 ; i<levelNum ; i++ )
    varIndexVal[i].resize( varIndexNum[i] );

  for( size_t i=0 ; i<levelNum ; i++ )
    for( size_t j=0 ; j<varIndexNum[i] ; j++ )
      IN >> varIndexVal[i][j];

  return createCompartmentChange(pVal,varIndexVal,idVal);
}

int BaseCompartmentChange::
flag(Compartment &compartment,std::vector< std::vector<double> > &y,
     std::vector< std::vector<double> > &dydt ) {
  std::cerr << "BaseCompartmentChange::flag() should not be used. "
	    << "Should always be mapped onto one of the real types.\n";
  exit(0);
}  

void BaseCompartmentChange::
update(Organism &O,Compartment &compartment,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > &dydt, double t ) {
  std::cerr << "BaseCompartmentChange::update() should not be used. "
	    << "Should always be mapped onto one of the real types.\n";
  exit(0);
}  
