//
// Filename     : costFunction.cc
// Description  : Classes defining cost functions
// Author(s)    : Pontus Melke (pontus@thep.lu.se)
// Created      : August 2006
// Revision     : $Id: costFunction.h 474 2010-08-30 15:23:20Z henrik $
//

#ifndef COSTFUNCTION_H
#define COSTFUNCTION_H

#include "baseCostFunction.h"

///
/// @brief Cost function using a mean-square-error function
///
/// MeanSquare is defined as::
///
///@f[ \frac{1}{T}\frac{1}{N}\frac{1}{M}\sum_{t_1}^{T}\sum_{i=1}^{N}\sum_{j=1}^{M}(x_{ij}(t) - \tilde{x}_{ij}(t))^2, @f]
///
///where @f$T@f$ is the number of time points stored in the vector
///costTemplateTime_,@f$N@f$ is the number of compartments, @f$M@f$ is the
///number of species used in the cost calculation (specified in costList_),
///@f$x_{ij}(t)@f$ is the value of species @f$j@f$ in compartment @f$i@f$
///obtained from simulation and @f$\tilde{x}_{ij}(t)@f$ is the experimental
///value of that same quantity.
///
class MeanSquare : public BaseCostFunction {
 public:
	double getCost(double simValue, double dataValue, double scaleFactor);
};

///
/// @brief Cost function using a normalized mean-square-error function
///
/// MeanSquareRelative is defined as::
///
///@f[ \frac{1}{T}\frac{1}{N}\frac{1}{M}\sum_{t_1}^{T}\sum_{i=1}^{N}\sum_{j=1}^{M}\left(\frac{x_{ij}(t) - \tilde{x}_{ij}(t)}{\tilde{x}_{ij}(t)}\right)^2, @f]
///
///where @f$T@f$ is the number of time points,@f$N@f$ is the number of
///compartments, @f$M@f$ is the number of species used in the cost calculation
///(specified in costList_), @f$x_{ij}(t)@f$ is the value of species @f$j@f$
///in compartment @f$i@f$ obtained from simulation and @f$\tilde{x}_{ij}(t)@f$
///is the experimental value of that same quantity.
///
class MeanSquareRelative : public BaseCostFunction {
 public:
	double getCost(double simValue, double dataValue, double scaleFactor);
};

///
/// @brief Cost function using a mean-square-error function normalized
/// for different spatial regions and concentrations
///
/// NormalizedMeanSquare is defined as::
///
///@f[ \frac{1}{T}\frac{1}{M}\sum_{t_1}^{T}\sum_{j=1}^{M}\sum_{g=1}^{N_g}\frac{1}{N}\sum_{i_g=1}^{N}\left(\frac{x_{ij}(t) - \tilde{x}_{ij}(t)}{\tilde{x}_{ij}(t)}\right)^2, @f]
///
/// where @f$T@f$ is the number of time points,@f$N@f$ is the number of
/// compartments of a specific type (e.g. a clv region with 1 should be
/// normalized equally as the non clv3 region of many more cells), @f$M@f$
/// is the number of species used in the cost calculation
/// (specified in costList_), @f$x_{ij}(t)@f$ is the value of species @f$j@f$
/// in compartment @f$i@f$ obtained from simulation and @f$\tilde{x}_{ij}(t)@f$
/// is the experimental value of that same quantity.
///
class NormalizedMeanSquare : public BaseCostFunction {
 public:
  double getCost(double simValue, double dataValue, double scaleFactor);
  double addCost(const std::vector< std::vector<double> > &y, const double t); 
};

#endif
