/**
 * Filename     : embedding.h
 * Description  : A class describing a embedding (compartment positions)
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : October 2003
 * Revision     : $Id: embedding.h 474 2010-08-30 15:23:20Z henrik $
 */
#ifndef EMBEDDING_H
#define EMBEDDING_H

#include<vector>
#include<string>
#include<iostream>
#include<fstream>

#include"../../compartment.h"
#include"../../reactions/simpleReaction.h"

//! A class for describing the updates of an embedding (compartment positions) 
/*! The Embedding class is describing positional properties of a
  compartment. An organism have typically one object of the type (if
  homolog compartments are assumed). The update rules comes from the
  simpleReactions included.
*/ 
class Embedding {
  
 private:
  
  std::string id_;
  std::vector<SimpleReaction*> reaction_;           
  int dimension_;
  inline void setDimension( int value );
  
 public:
  
  Embedding();
  Embedding( const Embedding &embeddingCopy );
  Embedding( char *inFile );
  Embedding( const std::string &inFile );
  Embedding( std::ifstream &IN );
  
  ~Embedding();
  
  // Get values
  inline std::string id() const;
  inline int numReaction() const;  
  inline int dimension() const;  
  
  inline SimpleReaction* reaction(int i) const;
  inline const std::vector<SimpleReaction*> reaction() const;
  
  // Set values
  inline void setId(const std::string &value);
  inline void setReaction(int i,SimpleReaction* value);
  inline void setReaction(const std::vector<SimpleReaction*> value);
  
  //Other functions
  void readEmbedding(std::ifstream &IN);
  int addReaction( std::ifstream &IN );
  void derivs(Compartment &compartment,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!Returns the id string
inline std::string Embedding::id() const {
  return id_;
}

//!Returns the number of reactions
inline int Embedding::numReaction() const {
  return reaction_.size();
}

//!Returns the number of dimensions (embedding variables)
inline int Embedding::dimension() const {
  return dimension_;
}

//!Returns a reaction (pointer)
inline SimpleReaction* Embedding::reaction(int i) const {
  return reaction_[i];
}

//!Returns the reaction pointers vector
inline const std::vector<SimpleReaction*> Embedding::reaction() const {
  return reaction_;
}

//!Sets the id string
inline void Embedding::setId(const std::string &value) {
  id_=value;
}

//!Sets the reaction(pointer) with index i
inline void Embedding::setReaction(int i,SimpleReaction* value) {
  reaction_[i]=value;
}

//!Sets all reaction(pointers) for a embedding
inline void Embedding::setReaction(const std::vector<SimpleReaction*> value) {
  reaction_ = value;
}

//!Sets the number of dimensions (private)
inline void Embedding::setDimension(int value) {
  dimension_=value;
}

#endif
