/**
 * Filename     : simulatorCellTemplate.cc
 * Description  : Organism simulation without updating neighborhood
 *              : Reading positions neighborhood and volumes from file
 *              : Normalizing volumes / neighborareas to get average volume=1
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : January 2005
 * Revision     : $Id: simulatorCellTemplate.cc 474 2010-08-30 15:23:20Z henrik $
 */
#include<vector>
#include<iostream>
#include"../../organism.h"
#include"../../solvers/rk5Adaptive.h"

int main(int argc,char *argv[]) {
  
  //Input handeling
  //////////////////////////////////////////////////////////////////////
  if( argc != 5 && argc != 6 ) {
    std::cerr << "usage: " << argv[0] << " modelFile initFile "
	      << "neighborhoodFile simulatorParaFile [spaceScalingFlag]\n";
    exit(-1);
  }
  std::string modelFile=argv[1];
  std::string initFile=argv[2];
  std::string neighFile=argv[3];
  std::string simPara=argv[4];
  int spaceScalingFlag=0;
  if( argc==6 )
    spaceScalingFlag = atoi(argv[5]); 
  
  //Create organism (model) and read neighborhood information
  //////////////////////////////////////////////////////////////////////
  Organism O(modelFile,initFile);
  std::cerr << O.numCompartment() << " cells added.\n";
  int numNeigh1,numNeigh2=0,numNeigh3=0;
  numNeigh1 = O.readNeighborsWithAreas(neighFile);
  std::cerr << numNeigh1 << " neighbors added.\n";
  
  //Find scaling factor to get average volume equal to one if applicable
  //////////////////////////////////////////////////////////////////////
  if( spaceScalingFlag ) {
    double averageVolume = 0.;
    int dim = O.dimension();
    double oneOverDim=1./dim;
    for( int i=0 ; i<O.numCompartment() ; i++ )
      averageVolume += O.compartment(i).variable(dim);
    averageVolume /= O.numCompartment();
    std::cerr << "Average volume before scaling: " << averageVolume << "\n";
    //Scale to get average volume equal to one
    double spaceScalingFactor = 1./averageVolume;
    spaceScalingFactor = pow(spaceScalingFactor,oneOverDim);
    O.scaleSpace(spaceScalingFactor);
    //Test average volume
    averageVolume=0.;
    for( int i=0 ; i<O.numCompartment() ; i++ )
      averageVolume += O.compartment(i).variable(dim);
    averageVolume /= O.numCompartment();
    std::cerr << "Average volume after scaling: " << averageVolume << "\n";
  }
  
  //Create simulator and simulate without updating the neighborhood
  //////////////////////////////////////////////////////////////////////
  Rk5Adaptive S(&O,simPara);
  S.simulateStaticNeighborhood();
}
