/**
 * Filename     : transportation.cc
 * Description  : Classes describing updates due to molecule transports
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : October 2003
 * Revision     : $Id: transport.cc 324 2007-12-07 15:44:17Z pontus $
 */
#include"transport.h"
#include"../../reactions/baseReaction.h"

Diffusion::Diffusion(std::vector<double> &paraValue, 
		     std::vector< std::vector<size_t> > 
		     &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=1 ) {
    std::cerr << "Diffusion::Diffusion() "
	      << "Uses only one parameter D.\n";
    exit(0);
  }
  if( indValue.size() != 1 || indValue[0].size() !=1 ) {
    std::cerr << "Diffusion::Diffusion() "
	      << "Variable index for size parameter needed.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("diffusion");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "D";
  setParameterId( tmp );
}

void Diffusion::derivs(Compartment &compartment,size_t species,
		       std::vector< std::vector<double> > &y,
		       std::vector< std::vector<double> > &dydt ) {
    
  size_t i=compartment.index();
  double Vi = y[i][variableIndex(0,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "Diffusion::derivs Given volume " << Vi 
	      << " unphysical.\n";
    exit(-1);
  }
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    //Only update if compartments index less than neighbors
    if( i<j ) {
      double Vj = y[j][variableIndex(0,0)];//Volume of the compartment neigh
      if( Vj<=0.0 ) {
	std::cerr << "Diffusion::derivs Given volume " << Vj
		  << " unphysical.\n";
	exit(-1);
      }
      //If no neighbor area defined assume area and distance 1
      double distance = 1.0,area=1.0;
      if( compartment.numNeighborArea()>n ) {
	area = compartment.neighborArea(n);
	distance=0.0;
	for( size_t d=compartment.topologyStart() ; 
	     d<compartment.numDimension() ; d++ )
	  distance += (y[j][d]-y[i][d])*(y[j][d]-y[i][d]);
	//Check that distance is larger than zero
	if( distance<=0 ) {
	  std::cerr << "Diffusion::derivs Calculated distance " 
		    << distance << " unphysical.\n";
	  exit(-1);
	}
	distance = sqrt(distance);
      }
      double diff = y[j][species]-y[i][species];
      dydt[i][species] += parameter(0)*diff*area/(Vi*distance);
      dydt[j][species] -= parameter(0)*diff*area/(Vj*distance);
    }
  }
}

DiffusionSimple::DiffusionSimple(std::vector<double> &paraValue, 
			       std::vector< std::vector<size_t> > 
			       &indValue ) 
{  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=1 ) {
    std::cerr << "DiffusionSimple::DiffusionSimple() "
	      << "Uses only one parameter D.\n";
    exit(0);
  }
  if( indValue.size() != 0 ) {
    std::cerr << "DiffusionSimple::DiffusionSimple() "
	      << "No variable index used.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("diffusionSimple");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "D";
  setParameterId( tmp );
}

void DiffusionSimple::derivs(Compartment &compartment,size_t species,
			     std::vector< std::vector<double> > &y,
			     std::vector< std::vector<double> > &dydt ) 
{    
  size_t i=compartment.index();
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    //Only update if compartments index less than neighbors
    if( i<j ) {
      double diff = y[j][species]-y[i][species];
      dydt[i][species] += parameter(0)*diff;
      dydt[j][species] -= parameter(0)*diff;
    }
  }
}

DiffusionSimpleSpatial::DiffusionSimpleSpatial(std::vector<double> &paraValue, 
																							 std::vector< std::vector<size_t> > 
																							 &indValue ) 
{  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=1 ) {
    std::cerr << "DiffusionSimpleSpatial::DiffusionSimpleSpatial() "
	      << "Uses only one parameter D.\n";
    exit(0);
  }
  if( indValue.size() != 0 ) {
    std::cerr << "DiffusionSimpleSpatial::DiffusionSimpleSpatial() "
	      << "No variable index used.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("diffusionSimpleSpatial");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "D";
  setParameterId( tmp );
}

void DiffusionSimpleSpatial::derivs(Compartment &compartment,size_t species,
																		std::vector< std::vector<double> > &y,
																		std::vector< std::vector<double> > &dydt ) {
  
  size_t i=compartment.index();
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    //Only update if compartments index less than neighbors
    if( i<j ) {
      //Area calculation
      double area;
      double d2=0.0;
      for( size_t dim=0 ; dim<compartment.numDimension() ; dim++ )
				d2 += (y[i][dim]-y[j][dim])*(y[i][dim]-y[j][dim]);
      double d = std::sqrt(d2);
      double ri=y[i][compartment.numDimension()];
      double rj=y[j][compartment.numDimension()];
      if( d>ri+rj ) area=0.0;
      else if( d<ri || d<rj ) {
				area = ri<rj ? ri : rj;
      }
      else
				area = std::sqrt( 0.25*(d2-2.*(ri*ri-rj*rj)*(ri*ri-rj*rj) +
																(ri-rj)*(ri-rj)/d2 ) );
      double diff = y[j][species]-y[i][species];
      dydt[i][species] += parameter(0)*area*diff;
      dydt[j][species] -= parameter(0)*area*diff;
    }
  }
}

DiffusionRestricted::DiffusionRestricted(std::vector<double> &paraValue, 
								 std::vector< std::vector<size_t> > 
								 &indValue )
{
	//Do some checks on the parameters and variable indeces
	//////////////////////////////////////////////////////////////////////
	if (paraValue.size() != 2) {
		std::cerr << "DiffusionRestricted::DiffusionRestricted() "
				<< "Uses two parameters D_out and D_in.\n";
		exit(EXIT_FAILURE);
	}
	if (indValue.size() != 2 || indValue[0].size() != 2 || 
	    indValue[1].size() != 1) {
		std::cerr << "DiffusionRestricted::DiffusionRestricted() "
				<< "Variable indices for two restrictMarkers, and size"
				<< " (second level) needed.\n";
		exit(EXIT_FAILURE);
	}
	if (indValue[0][0] == indValue[0][1] && paraValue[0] != paraValue[1]) {
		std::cerr << "DiffusionRestricted::DiffusionRestricted() "
				<< "Same type of compartments requires same D_out and D_in.\n";
		exit(EXIT_FAILURE);
	}
	//Set the variable values
	//////////////////////////////////////////////////////////////////////
	setId("diffusionRestricted");
	setParameter(paraValue);  
	setVariableIndex(indValue);
	
	//Set the parameter identities
	//////////////////////////////////////////////////////////////////////
	std::vector<std::string> tmp(numParameter());
	tmp.resize(numParameter());
	tmp[0] = "D_out";
	tmp[1] = "D_in";
	setParameterId(tmp);
}

void DiffusionRestricted::derivs(Compartment &compartment,size_t species,
						   std::vector< std::vector<double> > &y,
						   std::vector< std::vector<double> > &dydt )
{
	size_t i = compartment.index();
	//Check that this compartment is marked for this diffusion
	if (y[i][variableIndex(0,0)] < 0.5)
		return;

	static size_t dimension = compartment.numTopologyVariable() - 1; // Static?
	
	//Extract the volume of the compartment and assure it is physical
	double Vi = y[i][variableIndex(1,0)];
	assert( Vi>0.0 );
	for (size_t n = 0; n < compartment.numNeighbor(); n++) {
		size_t j = compartment.neighbor(n);
		//Only update if neighbor marked for this diffusion
		//If same type of compartments only update once (i>j)
		if (y[j][variableIndex(0,1)] > 0.5) {
			if (variableIndex(0,0) != variableIndex(0,1) || i > j) {
				
				//Get volume of neighbor and assure it is physical
				double Vj = y[j][variableIndex(1,0)];
				assert( Vj>0.0 );
				
				//If no neighbor area defined assume area and distance 1
				double distance = 1.0, area = 1.0;
				if (compartment.numNeighborArea() > n) {
					area = compartment.neighborArea(n);
					distance = 0.0;
					for (size_t d = compartment.topologyStart(); d < dimension; d++)
						distance += (y[j][d] - y[i][d]) * (y[j][d] - y[i][d]);
					assert( distance>0.0 );
					distance = std::sqrt(distance);
					assert( distance>0.0 );
				}
			
				//D_out
				double tmp = parameter(0) * y[i][species] * area / distance;
				dydt[i][species] -= tmp / Vi;
				dydt[j][species] += tmp / Vj;
				//D_in
				tmp = parameter(1) * y[j][species] * area / distance;
				dydt[i][species] += tmp / Vi;
				dydt[j][species] -= tmp / Vj;
			}  
		}
	}
}

PolarizationTransport::PolarizationTransport(std::vector<double> &paraValue, 
					     std::vector< std::vector<size_t> > 
					     &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=1 ) {
    std::cerr << "PolarizationTransport::PolarizationTransport() "
	      << "uses only one parameter D.\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationTransport::PolarizationTransport() "
	      << "Pin1 + polarization signal indeces needed in level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationTransport");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  setParameterId( tmp );
}

void PolarizationTransport::derivs(Compartment &compartment,size_t species,
				   std::vector< std::vector<double> > &y,
				   std::vector< std::vector<double> > &dydt ) {
    
  size_t i=compartment.index();
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationTransport::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
    sum += y[ compartment.neighbor(n) ][ variableIndex(0,1) ];
  //ad hoc way to include "wall area"
  sum /= compartment.numNeighbor();
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationTransport::derivs Given volume " << Vj
		<< " unphysical.\n";
      exit(-1);
    }
    
    double Pij;
    if( sum > 0.0 ) Pij = y[j][ variableIndex(0,1) ]/sum;
    else {
      Pij=1.;
      if( sum< 0.0 ) {
	std::cerr << "PolarizationTransport::derivs() WARNING Strange "
		  << "polarization signal.\n";
	std::cerr << i << " " << species << " " << variableIndex(0,0) << " " 
		  << variableIndex(0,1) << " " << sum << "\n";   
      }
    }
    double coeff = y[i][species]*y[i][variableIndex(0,0)]*Pij;
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;    
  }
}

PolarizationTransportFast::PolarizationTransportFast(std::vector<double> 
						     &paraValue, 
						     std::vector< 
						     std::vector<size_t> > 
						     &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=2 ) {
    std::cerr << "PolarizationTransportFast::PolarizationTransportFast() "
	      << "uses two parameters (D and alpha/beta).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationTransport::PolarizationTransport() "
	      << "Polarized molecule + polarization signal indeces needed in "
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationTransportFast");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b/a";
  setParameterId( tmp );
}

void PolarizationTransportFast::derivs(Compartment &compartment,size_t species,
				       std::vector< std::vector<double> > &y,
				       std::vector< std::vector<double> > 
				       &dydt ) {
    
  size_t i=compartment.index();
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationTransportFast::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
    sum += y[ compartment.neighbor(n) ][ variableIndex(0,1) ];
  //sum /= compartment.numNeighbor();//For adjusting for different num neigh
  sum += parameter(1);

  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationTransportFast::derivs Given volume " << Vj
		<< " unphysical.\n";
      exit(-1);
    }
    
    //double Pij = y[i][variableIndex(0,0)] * 
    //y[j][ variableIndex(0,1) ] /sum;
    double coeff =0.0;
    if( sum != 0.0 )
      coeff = y[i][species]*y[i][variableIndex(0,0)] *
	y[j][ variableIndex(0,1) ] /sum;
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

//!Constructor for the PolarizationTransportFastMM class
/*!  The Polarized transporter molecule P is divided into the cell
  cytosol (P_i) and its membranes (P_ij) following the dynamics
  
  dP_i/dt = -a P_i sum_k A_k + b sum_k P_ik 
  dP_ij/dt = a P_i A_j - b P_ij 
  
  where k is summed over neighboring cells, A is the polarizing signal
  and a,b are constants. The polarization is assumed to be fast and
  the fixed point values are assumed
  
  P_i = (b/a)*P / (sum_k A_k + (b/a))
  P_ij = P * A_j / (sum_k A_k + (b/a))
  
  where P is the total amount of polarized molecule in a cell.
  
  The polarized transport of a molecule X between two neighboring
  cells is then given by a Michaelis Menten formalism
  
  dX_i/dt = T * ( P_ij*X_i/(X_i+K) - P_ji*X_j/(X_j+K) ).
  
  The parameters given to this function are T and K'=b/a, and K.  The
  volumes for the cells are also included for correct concentration
  update.
*/
PolarizationTransportFastMM::PolarizationTransportFastMM(std::vector<double> 
							 &paraValue, 
							 std::vector< 
							 std::vector<size_t> > 
							 &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=3 ) {
    std::cerr << "PolarizationTransportFastMM::PolarizationTransportFastMM() "
	      << "uses three parameters (T, b/a and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationTransport::PolarizationTransport() "
	      << "Polarized molecule + polarization signal indeces needed in "
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationTransportFastMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b/a";
  tmp[2] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationTransportFastMM::derivs(Compartment &compartment,size_t species,
				       std::vector< std::vector<double> > &y,
				       std::vector< std::vector<double> > 
				       &dydt ) {
    
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationTransportFastMM::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
    sum += y[ compartment.neighbor(n) ][ variableIndex(0,1) ];
  sum /= compartment.numNeighbor();//To lower dep on num neighs
  sum += parameter(1);
  double invSum = 1.0/sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationTransportFastMM::derivs Given volume " << Vj
		<< " unphysical.\n";
      exit(-1);
    }
    
    double Pij = y[i][variableIndex(0,0)] * 
      y[j][ variableIndex(0,1) ] * invSum;
    double coeff = Pij*y[i][species]/( parameter(2)+y[i][species] );
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

//!Constructor for the SymmetricTransportFastMM class
/*!The active transport of a molecule X between two neighboring
  cells is given by a Michaelis Menten formalism
  
  dX_i/dt = T * ( P_ij*X_i/(X_i+K_M) - P_ji*X_j/(X_j+K_M) ).
  
  The parameters given to this function are T and K_M (maximal rate
  and MM constant). P is the efflux mediating molecule. The volumes
  for the cells are also included for correct concentration update.
*/
SymmetricTransportFastMM::
SymmetricTransportFastMM(std::vector<double> &paraValue, 
			 std::vector< std::vector<size_t> > &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=2 ) {
    std::cerr << "SymmetricTransportFastMM::SymmetricTransportFastMM() "
	      << "uses two parameters (T and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=1 
      || indValue[1].size() !=1  ) {
    std::cerr << "SymmetricTransport::SymmetricTransport() "
	      << "Polarized molecule index needed in "
	      << "level 1.\n" 
	      << "Variable index for size parameter needed "
	      << "at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("symmetricTransportFastMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void SymmetricTransportFastMM::derivs(Compartment &compartment,size_t species,
				       std::vector< std::vector<double> > &y,
				       std::vector< std::vector<double> > 
				       &dydt ) {
    
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "SymmetricTransportFastMM::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    if( i<j ) {
      double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
      if( Vj<=0.0 ) {
	std::cerr << "SymmetricTransportFastMM::derivs Given volume " << Vj
		  << " unphysical.\n";
	exit(-1);
      }
      double Pij = y[i][variableIndex(0,0)];
      double Pji = y[j][variableIndex(0,0)];
      // i -> j
      double coeff = Pij*y[i][species]/( parameter(2)+y[i][species] );
      //dydt[i][species] -= parameter(0)*coeff/Vi;
      //dydt[j][species] += parameter(0)*coeff/Vj;
      dydt[i][species] -= parameter(0)*coeff;
      dydt[j][species] += parameter(0)*coeff;
      // j -> i
      coeff = Pji*y[i][species]/( parameter(2)+y[i][species] );
      //dydt[i][species] += parameter(0)*coeff/Vi;
      //dydt[j][species] -= parameter(0)*coeff/Vj;
      dydt[i][species] += parameter(0)*coeff;
      dydt[j][species] -= parameter(0)*coeff;
    }
  }
}

//!Constructor for the PolarizationTransportInfluxFastMM class
/*!  The Polarized transporter molecule P is divided into the cell
  cytosol (P_i) and its membranes (P_ij) following the dynamics
  
  dP_i/dt = -a P_i sum_k A_k + b sum_k P_ik 
  dP_ij/dt = a P_i A_j - b P_ij 
  
  where k is summed over neighboring cells, A is the polarizing signal
  and a,b are constants. The polarization is assumed to be fast and
  the fixed point values are assumed
  
  P_i = (b/a)*P / (sum_k A_k + (b/a))
  P_ij = P * A_j / (sum_k A_k + (b/a))
  
  where P is the total amount of polarized molecule in a cell.
  
  The polarized transport of a molecule X between two neighboring
  cells is then given by a Michaelis Menten formalism
  
  dX_i/dt = T * ( P_ij*X_i/(X_i+K) * L_j/(L_i+L_j) 
  - P_ji*X_j/(X_j+K) * L_i/(L_i+L_j) ).
  
  The parameters given to this function are T and K'=b/a, and K. L is
  the influx carrier assumed to be symmetrically polarized. The
  volumes for the cells are also included for correct concentration
  update.
*/
PolarizationTransportInfluxFastMM::
PolarizationTransportInfluxFastMM(std::vector<double> &paraValue, 
				  std::vector< std::vector<size_t> > 
				  &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=3 ) {
    std::cerr << "PolarizationTransportInfluxFastMM::"
	      << "PolarizationTransportInfluxFastMM() "
	      << "uses three parameters (T, b/a and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=3 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationTransport::PolarizationTransport() "
	      << "Polarized efflux molecule, polarization signal, "
	      << "and influx carrier indeces needed in level 1.\n" 
	      << "Variable index for size parameter needed at "
	      << "second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationTransportInfluxFastMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b/a";
  tmp[2] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationTransportInfluxFastMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > &dydt ) {
    
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationTransportInfluxFastMM::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
    sum += y[ compartment.neighbor(n) ][ variableIndex(0,1) ];
  sum /= compartment.numNeighbor();//To lower dep on num neighs
  sum += parameter(1);
  double invSum = 1.0/sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationTransportInfluxFastMM::derivs "
		<< "Given volume " << Vj << " unphysical.\n";
      exit(-1);
    }
    
    double Pij = y[i][variableIndex(0,0)] * 
      y[j][ variableIndex(0,1) ] * invSum;
    double influxSum = y[i][variableIndex(0,2)]+y[j][variableIndex(0,2)];
    if( influxSum<=0.0 ) return;
    double coeff = Pij*y[i][species]*y[j][variableIndex(0,2)] /
      ( ( parameter(2)+y[i][species] )*influxSum );
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

//!Constructor for the PolarizationHillTransportInfluxFastMM class
/*!  The Polarized transporter molecule P is divided into the cell
  cytosol (P_i) and its membranes (P_ij) following the 'Hill' dynamics
  
  dP_i/dt = -a P_i sum_k f(A_k) + b sum_k P_ik 
  dP_ij/dt = a P_i f(A_j) - b P_ij , where 
  f(A_k) = A_k^n/(K^n + A_k^n)

  where k is summed over neighboring cells, A is the polarizing signal
  and a,b are rate constants and K, n are the Hill constant and
  coefficient. The polarization is assumed to be fast and the fixed
  point values are assumed
  
  P_i = (b/a)*P / (sum_k f(A_k) + (b/a))
  P_ij = P * f(A_j) / (sum_k f(A_k) + (b/a))
  
  where P is the total amount of polarized molecule in a cell.
  
  The polarized transport of a molecule X between two neighboring
  cells is then given by a Michaelis Menten formalism
  
  dX_i/dt = T * ( P_ij*X_i/(X_i+K_m) * L_j/(L_i+L_j) 
  - P_ji*X_j/(X_j+K_m) * L_i/(L_i+L_j) ).
  
  The parameters given to this function are T, K'=b/a, K_m, K, and
  n. L is the influx carrier assumed to be symmetrically
  polarized. The volumes for the cells are also included for correct
  concentration update.
*/
PolarizationHillTransportInfluxFastMM::
PolarizationHillTransportInfluxFastMM(std::vector<double> &paraValue, 
				  std::vector< std::vector<size_t> > 
				  &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=5 ) {
    std::cerr << "PolarizationHillTransportInfluxFastMM::"
	      << "PolarizationHillTransportInfluxFastMM() "
	      << "uses five parameters (T, b/a, K_M, K, n).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=3 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationHillTransport::PolarizationHillTransport() "
	      << "Polarized efflux molecule, polarization signal, "
	      << "and influx carrier indeces needed in level 1.\n" 
	      << "Variable index for size parameter needed at "
	      << "second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationHillTransportInfluxFastMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b/a";
  tmp[2] = "K_M";
  tmp[2] = "K_H";
  tmp[2] = "n_H";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationHillTransportInfluxFastMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > &dydt ) {
  
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationHillTransportInfluxFastMM::derivs "
	      << "Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  double KtoN = std::pow( parameter(3),parameter(4) );
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    double ytoN = std::pow( y[ compartment.neighbor(n) ]
			    [ variableIndex(0,1) ],parameter(4) );
    sum += ytoN/(KtoN+ytoN);
  }
  sum /= compartment.numNeighbor();//To lower dep on num neighs
  sum += parameter(1);
  double invSum = 1.0/sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationHillTransportInfluxFastMM::derivs "
		<< "Given volume " << Vj << " unphysical.\n";
      exit(-1);
    }
    
    double ytoN = std::pow(y[j][ variableIndex(0,1) ],parameter(4));
    double Pij = y[i][variableIndex(0,0)] * 
      ( ytoN/(KtoN+ytoN) ) * invSum;
    double influxSum = y[i][variableIndex(0,2)]+y[j][variableIndex(0,2)];
    if( influxSum<=0.0 ) return;
    double coeff = Pij*y[i][species]*y[j][variableIndex(0,2)] /
      ( ( parameter(2)+y[i][species] )*influxSum );
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

//!Constructor for the SymmetricTransportInfluxFastMM class
/*!The transport of a molecule X between two neighboring cells is
  given by a Michaelis Menten formalism
  
  dX_i/dt = - T*( P_i*L_j*X_i/(X_i+K_M) -
  P_j*L_i*X_j/(X_j+K_M))/(L_i+L_j).
  
  The parameters given to this function are T and K_M (max rate and MM
  constant). P and L are the efflux and influx carriers assumed to be
  symmetrically polarized. The volumes for the cells are also included
  for correct concentration update.
*/
SymmetricTransportInfluxFastMM::
SymmetricTransportInfluxFastMM(std::vector<double> &paraValue, 
				  std::vector< std::vector<size_t> > 
				  &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=2 ) {
    std::cerr << "SymmetricTransportInfluxFastMM::"
	      << "SymmetricTransportInfluxFastMM() "
	      << "uses two parameters (T and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "SymmetricTransport::SymmetricTransport() "
	      << "Efflux molecule "
	      << "and influx carrier indeces needed in level 1.\n" 
	      << "Variable index for size parameter needed at "
	      << "second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("symmetricTransportInfluxFastMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void SymmetricTransportInfluxFastMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > &dydt ) {
    
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "SymmetricTransportInfluxFastMM::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    if( i<j ) {
      double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
      if( Vj<=0.0 ) {
	std::cerr << "SymmetricTransportInfluxFastMM::derivs "
		  << "Given volume " << Vj << " unphysical.\n";
	exit(-1);
      }
      
      double Pij = y[i][variableIndex(0,0)];
      double Pji = y[j][variableIndex(0,0)];
      double Lij = y[i][variableIndex(0,1)];
      double Lji = y[j][variableIndex(0,1)];
    
      double influxSum = Lij+Lji;
      if( influxSum<=0.0 ) return;
      // i -> j
      double coeff = Pij*Lji*y[i][species] /
	( ( parameter(1)+y[i][species] )*influxSum );
      //dydt[i][species] -= parameter(0)*coeff/Vi;
      //dydt[j][species] += parameter(0)*coeff/Vj;
      dydt[i][species] -= parameter(0)*coeff;
      dydt[j][species] += parameter(0)*coeff;
      // j -> i
      coeff = Pji*Lij*y[j][species] /
	( ( parameter(1)+y[j][species] )*influxSum );
      //dydt[i][species] += parameter(0)*coeff/Vi;
      //dydt[j][species] -= parameter(0)*coeff/Vj;
      dydt[i][species] += parameter(0)*coeff;
      dydt[j][species] -= parameter(0)*coeff;
    }
  }
}

//!Constructor for the PolarizationTransportFastMMSpatial class
/*!  The Polarized transporter molecule P is divided into the cell
  cytosol (P_i) and its membranes (P_ij) following the dynamics
  
  dP_i/dt = -a P_i sum_k A_k + b sum_k P_ik 
  dP_ij/dt = a P_i A_j - b P_ij 
  
  where k is summed over neighboring cells, A is the polarizing signal
  and a,b are constants. The polarization is assumed to be fast and
  the fixed point values are assumed
  
  P_i = (b/a)*P / (sum_k A_k + (b/a))
  P_ij = P * A_j / (sum_k A_k + (b/a))
  
  where P is the total amount of polarized molecule in a cell.
  
  The polarized transport of a molecule X between two neighboring
  cells is then given by a Michaelis Menten formalism
  
  dX_i/dt = T * ( a_ij*P_ij*X_i/(X_i+K) - a_ji*P_ji*X_j/(X_j+K) ).
  
  The parameters given to this function are T and K'=b/a, and K.  a_ij
  is the crossing area. The volumes for the cells are also included for
  correct concentration update.
*/
PolarizationTransportFastMMSpatial::PolarizationTransportFastMMSpatial(std::vector<double> 
							 &paraValue, 
							 std::vector< 
							 std::vector<size_t> > 
							 &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=3 ) {
    std::cerr << "PolarizationTransportFastMMSpatial::PolarizationTransportFastMMSpatial() "
	      << "uses three parameters (T, b/a and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationTransportFastMMSpatial::PolarizationTransport() "
	      << "Polarized molecule + polarization signal indeces needed in "
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationTransportFastMMSpatial");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b/a";
  tmp[2] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationTransportFastMMSpatial::derivs(Compartment &compartment,size_t species,
				       std::vector< std::vector<double> > &y,
				       std::vector< std::vector<double> > 
				       &dydt ) {
    
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationTransportFastMMSpatial::derivs Given volume " 
	      << Vi << " unphysical.\n";
    exit(-1);
  }
  //Areas
  double areaSum=0.0;
  std::vector<double> area(compartment.numNeighbor());
  for( size_t k=0 ; k<compartment.numNeighbor() ; k++ ) {
    size_t n=compartment.neighbor(k);
    double d2=0.0;
    for( size_t dim=0 ; dim<compartment.numDimension() ; dim++ )
      d2 += (y[i][dim]-y[n][dim])*(y[i][dim]-y[n][dim]);
    double d = std::sqrt(d2);
    double ri=y[i][compartment.numDimension()];
    double rn=y[n][compartment.numDimension()];
    if( d>ri+rn ) area[k]=0.0;
    else if( d<ri || d<rn ) {
      area[k] = ri<rn ? ri : rn;
    }
    else
      area[k] = std::sqrt( 0.25*(d2-2.*(ri*ri-rn*rn)*(ri*ri-rn*rn) +
				 (ri-rn)*(ri-rn)/d2 ) );
    areaSum += area[k];
    if( areaSum<=0.0 ) areaSum=1.0;
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
    sum += y[ compartment.neighbor(n) ][ variableIndex(0,1) ]*area[n]/areaSum;
  //sum /= compartment.numNeighbor();//To lower dep on num neighs
  sum += parameter(1);
  double invSum = 1.0/sum;//To use mult instead of div

  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationTransportFastMMSpatial::derivs Given volume " << Vj
		<< " unphysical.\n";
      exit(-1);
    }
    
    double Pij = y[i][variableIndex(0,0)] * 
      y[j][ variableIndex(0,1) ] * invSum;
    double coeff = area[n]*Pij*y[i][species]/( parameter(2)+y[i][species] );
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

PolarizationHillTransportFast::
PolarizationHillTransportFast(std::vector<double> &paraValue, 
			      std::vector< std::vector<size_t> > &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=4 ) {
    std::cerr << "PolarizationHillTransportFast::"
	      << "PolarizationHillTransportFast() "
	      << "uses four parameters (T, b/a and K_H,n_H).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationHillTransportFast::"
	      << "PolarizationHillTransportFast() "
	      << "Polarized molecule + polarization signal indeces needed in "
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationHillTransportFast");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b/a";
  tmp[2] = "K_H";
  tmp[3] = "K_n";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationHillTransportFast::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > &dydt ) {
  
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationHillTransportFast::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    double Ak = y[ compartment.neighbor(n) ][ variableIndex(0,1) ]; 
    sum += std::pow(Ak,parameter(3))/
      (std::pow(parameter(2),parameter(3)) + std::pow(Ak,parameter(3)) );
  }
  sum /= compartment.numNeighbor();
  sum += parameter(1);
  double invSum = 1.0/sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationHillTransportFast::derivs Given volume " << Vj
		<< " unphysical.\n";
      exit(-1);
    }
    double Aj = y[j][ variableIndex(0,1) ];
    double Pij = y[i][variableIndex(0,0)] * invSum *
      std::pow(Aj,parameter(3))/
      (std::pow(parameter(2),parameter(3)) + std::pow(Aj,parameter(3)) );
    
    double coeff = Pij*y[i][species];
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

PolarizationHillTransportFastMM::
PolarizationHillTransportFastMM(std::vector<double> &paraValue, 
			      std::vector< std::vector<size_t> > &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=5 ) {
    std::cerr << "PolarizationHillTransportFastMM::"
	      << "PolarizationHillTransportFastMM() "
	      << "uses five parameters (T, b/a, K_H,n_H and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationHillTransportFastMM::"
	      << "PolarizationHillTransportFastMM() "
	      << "Polarized molecule + polarization signal indeces needed in "
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationHillTransportFastMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b/a";
  tmp[2] = "K_H";
  tmp[3] = "n_H";
  tmp[4] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationHillTransportFastMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > &dydt ) {
  
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationHillTransportFastMM::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    double Ak = y[ compartment.neighbor(n) ][ variableIndex(0,1) ]; 
    sum += std::pow(Ak,parameter(3))/
      (std::pow(parameter(2),parameter(3)) + std::pow(Ak,parameter(3)) );
  }
  sum /= compartment.numNeighbor();
  sum += parameter(1);
  double invSum = 1.0/sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationHillTransportFastMM::derivs Given volume " 
		<< Vj << " unphysical.\n";
      exit(-1);
    }
    double Aj = y[j][ variableIndex(0,1) ];
    double Pij = y[i][variableIndex(0,0)] * invSum *
      std::pow(Aj,parameter(3))/
      (std::pow(parameter(2),parameter(3)) + std::pow(Aj,parameter(3)) );
    
    double coeff = Pij*y[i][species]/(parameter(4)+y[i][species]);
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

//!Polarized transport where exponential feedback assumed 
PolarizationExpTransportFastMM::
PolarizationExpTransportFastMM(std::vector<double> &paraValue, 
			       std::vector< std::vector<size_t> > 
			       &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=3 ) {
    std::cerr << "PolarizationExpTransportFastMM::"
	      << "PolarizationExpTransportFastMM() "
	      << "uses five parameters (T, b, and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationExpTransportFastMM::"
	      << "PolarizationExpTransportFastMM() "
	      << "Polarized molecule + polarization signal indeces needed in "
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationExpTransportFastMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b";
  tmp[2] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationExpTransportFastMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > &dydt ) {
  
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationExpTransportFastMM::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    double Ak = y[ compartment.neighbor(n) ][ variableIndex(0,1) ]; 
    sum += std::pow(parameter(1),Ak);
  }
  //sum /= compartment.numNeighbor();
  double invSum = 1.0/sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationExpTransportFastMM::derivs Given volume " 
		<< Vj << " unphysical.\n";
      exit(-1);
    }
    double Aj = y[j][ variableIndex(0,1) ];
    double Pij = y[i][variableIndex(0,0)] * invSum *
      std::pow(parameter(1),Aj);
    
    double coeff = Pij*y[i][species]/(parameter(2)+y[i][species]);
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

//!Constructor for the PolarizationConstLinTransportFast class
PolarizationConstLinTransportFast::
PolarizationConstLinTransportFast(std::vector<double> &paraValue, 
			      std::vector< std::vector<size_t> > &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=3 ) {
    std::cerr << "PolarizationConstLinTransportFast::"
	      << "PolarizationConstLinTransportFast() "
	      << "uses three parameters (T, b/a and c).\n";
    exit(0);
  }
  if( indValue.size() != 1 || indValue[0].size() !=2 ) {
    std::cerr << "PolarizationConstLinTransportFast::"
	      << "PolarizationConstLinTransportFast() "
	      << "Polarized molecule + polarization signal indeces "
	      << "needed in level 1.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationConstLinTransportFast");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b/a";
  tmp[2] = "c";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationConstLinTransportFast::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > &dydt ) {
  
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  //double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  //if( Vi<=0.0 ) {
  //std::cerr << "PolarizationConstLinTransportFast::derivs Given volume " << Vi
  //      << " unphysical.\n";
  //exit(-1);
  //}
  //Polarization coefficient normalization constant
  double sum = parameter(2)*compartment.numNeighbor();
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    sum += y[ compartment.neighbor(n) ][ variableIndex(0,1) ]; 
  }
  sum /= compartment.numNeighbor();
  sum += parameter(1);
  if( sum<= 0.0 )
    return;
  double invSum = 1.0/sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    //double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    //if( Vj<=0.0 ) {
    //std::cerr << "PolarizationConstLinTransportFast::derivs Given volume " << Vj
    //	<< " unphysical.\n";
    //exit(-1);
    //}
    double PijA = y[i][variableIndex(0,0)] * invSum *
      ( parameter(2)+y[j][ variableIndex(0,1) ] ) * y[i][species];
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*PijA;
    dydt[j][species] += parameter(0)*PijA;
  }
}


//!Constructor for the PolarizationTransportFastB class
/*!  The Polarized transporter molecule P is divided into the cell
  cytosol (P_i) and its membranes (P_ij) following the dynamics
  
  dP_i/dt = -a P_i + b sum_k P_ik/(K+A_k) 
  dP_ij/dt = a P_i - b P_ij/(K+A_j) 
  
  where k is summed over neighboring cells, A is the polarizing signal
  and a,b,K are constants. The polarization is assumed to be fast and
  the fixed point values are assumed
  
  P_i = b/a * P / ( b/a + sum_k (K+A_k) )
  P_ij = (K+A_j) / ( b/a + sum_k (K+A_k) )
  
  where P is the total amount of polarized molecule in a cell.
  
  The polarized transport of a molecule X between two neighboring
  cells is then given by
  
  dX_i/dt = T * ( X_i*P_ij - X_j*P_ji ).
  
  The parameters given to this function are T, K and k=b/a. The
  volumes for the cells are also included for correct concentration
  update.
*/
PolarizationTransportFastB::PolarizationTransportFastB(std::vector<double> 
						       &paraValue, 
						       std::vector< 
						       std::vector<size_t> > 
						       &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=3 ) {
    std::cerr << "PolarizationTransportFastB::PolarizationTransportFastB() "
	      << "uses three parameters (T, K and b/a).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationTransport::PolarizationTransport() "
	      << "Polarized molecule + polarization signal indeces needed in "
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationTransportFastB");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "K";
  tmp[1] = "b/a";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationTransportFastB::derivs(Compartment &compartment,size_t species,
				       std::vector< std::vector<double> > &y,
				       std::vector< std::vector<double> > 
				       &dydt ) {
    
  size_t i=compartment.index();
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationTransportFastB::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=parameter(1) * compartment.numNeighbor();
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
    sum += y[ compartment.neighbor(n) ][ variableIndex(0,1) ];
  sum += parameter(2);
  double invSum = 1.0/sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationTransportFastB::derivs Given volume " << Vj
		<< " unphysical.\n";
      exit(-1);
    }
    
    double Pij = ( parameter(1) + y[j][ variableIndex(0,1) ]) * 
      y[i][variableIndex(0,0)] * invSum;
    double coeff = y[i][species]*Pij;
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

//!Constructor for the PolarizationTransportFastBMM class
/*!  The Polarized transporter molecule P is divided into the cell
  cytosol (P_i) and its membranes (P_ij) following the dynamics
  
  dP_i/dt = -a P_i + b sum_k P_ik/(K+A_k) 
  dP_ij/dt = a P_i - b P_ij/(K+A_j) 
  
  where k is summed over neighboring cells, A is the polarizing signal
  and a,b,K are constants. The polarization is assumed to be fast and
  the fixed point values are assumed
  
  P_i = b/a * P / ( b/a + sum_k (K+A_k) )
  P_ij = (K+A_j) / ( b/a + sum_k (K+A_k) )
  
  where P is the total amount of polarized molecule in a cell.
  
  The polarized transport of a molecule X between two neighboring
  cells is then given by
  
  dX_i/dt = T * ( P_ij*X_i/(X_i+K_M) - P_ji*X_j/(X_j+K_M) ).
  
  The parameters given to this function are T, K and k=b/a. The
  volumes for the cells are also included for correct concentration
  update.
*/
PolarizationTransportFastBMM::PolarizationTransportFastBMM(std::vector<double> 
							   &paraValue, 
							   std::vector< 
							   std::vector<size_t> > 
							   &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=4 ) {
    std::cerr << "PolarizationTransportFastBMM::PolarizationTransportFastBMM()"
	      << " uses four parameters (T, K, b/a and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationTransport::PolarizationTransport() "
	      << "Polarized molecule + polarization signal indeces needed in "
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationTransportFastBMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "K";
  tmp[2] = "b/a";
  tmp[3] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationTransportFastBMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > 
       &dydt ) {
  
  size_t i=compartment.index();
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationTransportFastBMM::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=parameter(1) * compartment.numNeighbor();
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
    sum += y[ compartment.neighbor(n) ][ variableIndex(0,1) ];
  sum += parameter(2);
  double invSum = 1.0/sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationTransportFastBMM::derivs Given volume " << Vj
		<< " unphysical.\n";
      exit(-1);
    }
    
    double Pij = ( parameter(1) + y[j][ variableIndex(0,1) ]) * 
      y[i][variableIndex(0,0)] * invSum;
    double coeff = Pij*y[i][species]/( y[i][species]+parameter(3) );
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

//!Constructor for the PolarizationTransportFastSwitchedMM class
/*!  The Polarized transporter molecule P is divided into the cell
  cytosol (P_i) and its membranes (P_ij) following the dynamics
  
  dP_i/dt = -a P_i sum_k A_k + b sum_k P_ik 
  dP_ij/dt = a P_i A_j - b P_ij 
  
  where k is summed over neighboring cells, A is the polarizing signal
  and a,b are constants. The polarization is assumed to be fast and
  the fixed point values are assumed
  
  P_i = (b/a)*P / (sum_k A_k + (b/a))
  P_ij = P * A_j / (sum_k A_k + (b/a))
  
  where P is the total amount of polarized molecule in a cell.
  
  The polarized transport of a molecule X between two neighboring
  cells is then given by a Michaelis Menten formalism
  
  dX_i/dt = T * ( P_ij*X_i/(X_i+K) - P_ji*X_j/(X_j+K) ).
  
  The parameters given to this function are T and K'=b/a, K_M and
  Th_switch.  The volumes for the cells are also included for correct
  concentration update.
*/
PolarizationTransportFastSwitchedMM::
PolarizationTransportFastSwitchedMM(std::vector<double> 
				    &paraValue, 
				    std::vector< 
				    std::vector<size_t> > 
				    &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=4 ) {
    std::cerr << "PolarizationTransportFastSwitchedMM::"
	      << "PolarizationTransportFastSwitchedMM() "
	      << "uses three parameters (T, b/a,K_M and Th_switch).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=3 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationTransport::PolarizationTransport() "
	      << "Polarized molecule + polarization signal + switch molecule "
	      << "indeces needed in"
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationTransportFastSwitchedMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b/a";
  tmp[2] = "K_M";
  tmp[3] = "Th_switch";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationTransportFastSwitchedMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > 
       &dydt ) {
    
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationTransportFastSwitchedMM::derivs Given volume " 
	      << Vi << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  double min = y[ compartment.neighbor(0) ][ variableIndex(0,1) ];
  double max=min;
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    sum += y[ compartment.neighbor(n) ][ variableIndex(0,1) ];
    if( y[ compartment.neighbor(n) ][ variableIndex(0,1) ]<min )
      min = y[ compartment.neighbor(n) ][ variableIndex(0,1) ];
    if( y[ compartment.neighbor(n) ][ variableIndex(0,1) ]>max )
      max = y[ compartment.neighbor(n) ][ variableIndex(0,1) ];
  }
  sum /= compartment.numNeighbor();
  sum += parameter(1);
  double invSum = 1.0;
  if( sum>0 )
    invSum /= sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationTransportFastSwitchedMM::derivs Given volume " 
		<< Vj << " unphysical.\n";
      exit(-1);
    }
    
    double Pij; 
    if( y[i][variableIndex(0,2)] > parameter(3) )
      Pij = y[i][variableIndex(0,0)] * y[j][ variableIndex(0,1) ] * invSum;
    else
      Pij = y[i][variableIndex(0,0)] * (max+min-y[j][ variableIndex(0,1) ]) 
	* invSum;
    double coeff = Pij*y[i][species]/( parameter(2)+y[i][species] );
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

//!Contructor for a polarized cell-wall transport
PolarizationLinFastTransportRestrictedMM::
PolarizationLinFastTransportRestrictedMM(std::vector<double> &paraValue, 
					 std::vector< std::vector<size_t> > 
					 &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=4 ) {
    std::cerr << "PolarizationLinFastTransportRestrictedMM::"
	      << "PolarizationLinFastTransportRestrictedMM() "
	      << "uses four parameters (T_eff, T_inf, b/a, and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=4 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationLinFastTransportRestrictedMM::"
	      << "PolarizationLinFastTransportRestrictedMM() "
	      << "Polarized molecule + polarization signal + "
	      << "cell marker + wall marker "
	      << "indeces needed in"
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationLinFastTransportRestrictedMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T_eff";
  tmp[1] = "T_inf";
  tmp[2] = "b/a";
  tmp[3] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationLinFastTransportRestrictedMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > 
       &dydt ) {
  
  size_t i=compartment.index();
  //Only if the comp has neighbors and is a cell
  if( compartment.numNeighbor() == 0 || 
      y[i][variableIndex(0,2)]<0.5 ) return;
  
  //Extract compartment volume and make sure it is physical
  double Vi = y[i][variableIndex(1,0)];
  assert( Vi>0.0 );
  //Polarization from CELL neighbors at level 1.
  assert( compartment.numNeighborAtLevel(1) == 
	  compartment.numNeighbor() );
  
  //Calulate normalization
  //////////////////////////////////////////////////////////////////////
  double sum=parameter(2);
  if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
    sum *=Vi;
    for( size_t n=0 ; n<compartment.numNeighborAtLevel(1) ; n++ )
      sum += y[ compartment.neighborAtLevel(1,n) ][ variableIndex(0,1) ]*
	compartment.neighborArea(n);
  }
  else
    for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
      sum += y[ compartment.neighborAtLevel(1,n) ][ variableIndex(0,1) ];
  
  double invSum = 1.0;
  if( sum>0.0 )
    invSum /= sum;//To use mult instead of div
  
  //Update all compartments
  //////////////////////////////////////////////////////////////////////
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);

    //Make sure neighbor marked as wall
    assert( y[j][variableIndex(0,3)]>0.5 );

    //Extract neighbor volume and make sure it is physical
    double Vj = y[j][variableIndex(1,0)];
    assert( Vj>0.0 );

    double Pij = y[i][variableIndex(0,0)] * Vi * 
      y[compartment.neighborAtLevel(1,n)][variableIndex(0,1)] * invSum;
    if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
      double coeff = compartment.neighborArea(n)*Pij*
	y[i][species]/( parameter(3)+y[i][species] );
      dydt[i][species] -= parameter(0)*coeff/Vi;
      dydt[j][species] += parameter(0)*coeff/Vj;
      coeff = compartment.neighborArea(n)*Pij*
	y[j][species]/( parameter(3)+y[j][species] );
      dydt[i][species] += parameter(1)*coeff/Vi;
      dydt[j][species] -= parameter(1)*coeff/Vj;
    }
    else {
      double coeff = Pij*y[i][species]/( parameter(3)+y[i][species] );
      dydt[i][species] -= parameter(0)*coeff/Vi;
      dydt[j][species] += parameter(0)*coeff/Vj;
      coeff = Pij*y[j][species]/( parameter(3)+y[j][species] );
      dydt[i][species] += parameter(1)*coeff/Vi;
      dydt[j][species] -= parameter(1)*coeff/Vj;
    }
  }
}

PolarizationConstHillFastTransportRestrictedMM::
PolarizationConstHillFastTransportRestrictedMM(std::vector<double> 
					       &paraValue, 
					       std::vector< 
					       std::vector<size_t> > 
					       &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=7 ) {
    std::cerr << "PolarizationConstHillFastTransportRestrictedMM::"
	      << "PolarizationConstHillFastTransportRestrictedMM() "
	      << "uses seven parameters (T_eff, T_inf, b/a, and c, "
	      << "K_H, n,and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=4 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationConstHillFastTransportRestrictedMM::"
	      << "PolarizationConstHillFastTransportRestrictedMM() "
	      << "Polarized molecule + polarization signal + "
	      << "cell marker + wall marker "
	      << "indeces needed in"
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Check parameters
  //////////////////////////////////////////////////////////////////////
  if( paraValue[0]<0.0 || paraValue[1]<0.0 || paraValue[2]<0.0 ||
      paraValue[3]<0.0 || paraValue[4]<0.0 || paraValue[5]<0.0 ||
      paraValue[6]<0.0 ) {
    std::cerr << "PolarizationConstHillFastTransportRestrictedMM::"
	      << "PolarizationConstHillFastTransportRestrictedMM()"
	      << " Parameters need to be positive.\n";
    exit(0);
  }
  if( paraValue[3]>1.0 ) {
    std::cerr << "PolarizationConstHillFastTransportRestrictedMM::"
	      << "PolarizationConstHillFastTransportRestrictedMM()"
	      << " Parameter c (Lin/Hill fraction need to be "
	      << "smaller than 1.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationConstHillFastTransportRestrictedMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T_eff";
  tmp[1] = "T_inf";
  tmp[2] = "b/a";
  tmp[3] = "c_p";
  tmp[4] = "K_H";
  tmp[5] = "n";
  tmp[6] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationConstHillFastTransportRestrictedMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > 
       &dydt ) {
  
  size_t i=compartment.index();
  //Only if the comp has neighbors and is a cell
  if( compartment.numNeighbor() == 0 || 
      y[i][variableIndex(0,2)]<0.5 ) return;
  
  static double KToNHill = std::pow(parameter(4),parameter(5));
  //Polarization function from CELL neighbors at level 1.
  assert( compartment.numNeighborAtLevel(1) == 
	  compartment.numNeighbor() );
  //Extract volume of the compartment and make sure it is physical
  double Vi = y[i][variableIndex(1,0)];
  assert( Vi>0.0 );
  
  //Calculate f(n) and normalization
  //////////////////////////////////////////////////////////////////////
  std::vector<double> f(compartment.numNeighbor() );
  for( size_t n=0 ; n<compartment.numNeighborAtLevel(1) ; n++ ) {
    double yToN = std::pow(y[ compartment.neighborAtLevel(1,n) ]
			   [ variableIndex(0,1) ] , parameter(5));
    f[n] = (1.0-parameter(3)) + parameter(3)*yToN/(KToNHill+yToN);
  }

  double sum=parameter(2);
  if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
    sum *=Vi;
    for( size_t n=0 ; n<compartment.numNeighborAtLevel(1) ; n++ )
      sum += f[n]*compartment.neighborArea(n);
  }
  else
    for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
      sum += f[n];
  
  double invSum = 1.0;
  if( sum>0.0 )
    invSum /= sum;//To use mult instead of div
  
  //Update all compartments
  //////////////////////////////////////////////////////////////////////
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    
    //Make sure neighbor marked as wall 
    assert( y[j][variableIndex(0,3)]>0.5 ); 
    
    //Extract volume of neighbor and make sure it is physical
    double Vj = y[j][variableIndex(1,0)];
    assert( Vj>0.0 );
    
    double Pij = y[i][variableIndex(0,0)] * Vi * f[n] * invSum;
    if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
      double coeff = compartment.neighborArea(n)*Pij*
	y[i][species]/( parameter(6)+y[i][species] );
      dydt[i][species] -= parameter(0)*coeff/Vi;
      dydt[j][species] += parameter(0)*coeff/Vj;
      coeff = compartment.neighborArea(n)*Pij*
	y[j][species]/( parameter(6)+y[j][species] );
      dydt[i][species] += parameter(1)*coeff/Vi;
      dydt[j][species] -= parameter(1)*coeff/Vj;
    }
    else {
      double coeff = Pij*y[i][species]/( parameter(6)+y[i][species] );
      dydt[i][species] -= parameter(0)*coeff/Vi;
      dydt[j][species] += parameter(0)*coeff/Vj;
      coeff = Pij*y[j][species]/( parameter(6)+y[j][species] );
      dydt[i][species] += parameter(1)*coeff/Vi;
      dydt[j][species] -= parameter(1)*coeff/Vj;
    }
  }
}

PolarizationConstFastTransportRestrictedMM::
PolarizationConstFastTransportRestrictedMM(std::vector<double> 
					   &paraValue, 
					   std::vector< 
					   std::vector<size_t> > 
					   &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=4 ) {
    std::cerr << "PolarizationConstFastTransportRestrictedMM::"
	      << "PolarizationConstFastTransportRestrictedMM() "
	      << "uses seven parameters (T_eff, T_inf, b/a, and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=3 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationConstFastTransportRestrictedMM::"
	      << "PolarizationConstFastTransportRestrictedMM() "
	      << "Polarized molecule + "
	      << "cell marker + wall marker "
	      << "indeces needed in"
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at "
	      << "second level.\n";
    exit(0);
  }
  //Check parameters
  //////////////////////////////////////////////////////////////////////
  if( paraValue[0]<0.0 || paraValue[1]<0.0 || paraValue[2]<0.0 ||
      paraValue[3]<0.0 ) {
    std::cerr << "PolarizationConstFastTransportRestrictedMM::"
	      << "PolarizationConstFastTransportRestrictedMM()"
	      << " Parameters need to be positive.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationConstFastTransportRestrictedMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T_eff";
  tmp[1] = "T_inf";
  tmp[2] = "b/a";
  tmp[3] = "K_M";
  setParameterId( tmp );
}

void PolarizationConstFastTransportRestrictedMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > 
       &dydt ) {
  
  size_t i=compartment.index();
  //Only if the comp has neighbors and is a cell
  if( compartment.numNeighbor() == 0 || 
      y[i][variableIndex(0,1)]<0.5 ) return;
  
  //Polarization function from CELL neighbors at level 1.
  assert( compartment.numNeighborAtLevel(1) == 
	  compartment.numNeighbor() );
  //Extract volume of the compartment and make sure it is physical
  double Vi = y[i][variableIndex(1,0)];
  assert( Vi>0.0 );
  
  //Calculate normalization
  //////////////////////////////////////////////////////////////////////
  double sum=parameter(2);
  if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
    sum *=Vi;
    for( size_t n=0 ; n<compartment.numNeighborAtLevel(1) ; n++ )
      sum += compartment.neighborArea(n);
  }
  else
    sum += compartment.numNeighbor();//assuming area=1
  
  double invSum = 1.0;
  if( sum>0.0 )
    invSum /= sum;//To use mult instead of div
  
  //Update all compartments
  //////////////////////////////////////////////////////////////////////
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    
    //Make sure neighbor marked as wall 
    assert( y[j][variableIndex(0,2)]>0.5 ); 
    
    //Extract volume of neighbor and make sure it is physical
    double Vj = y[j][variableIndex(1,0)];
    assert( Vj>0.0 );
    
    double Pij = y[i][variableIndex(0,0)] * Vi * invSum;
    if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
      double coeff = compartment.neighborArea(n)*Pij*
	y[i][species]/( parameter(3)+y[i][species] );
      dydt[i][species] -= parameter(0)*coeff/Vi;
      dydt[j][species] += parameter(0)*coeff/Vj;
      coeff = compartment.neighborArea(n)*Pij*
	y[j][species]/( parameter(3)+y[j][species] );
      dydt[i][species] += parameter(1)*coeff/Vi;
      dydt[j][species] -= parameter(1)*coeff/Vj;
    }
    else {
      double coeff = Pij*y[i][species]/( parameter(3)+y[i][species] );
      dydt[i][species] -= parameter(0)*coeff/Vi;
      dydt[j][species] += parameter(0)*coeff/Vj;
      coeff = Pij*y[j][species]/( parameter(3)+y[j][species] );
      dydt[i][species] += parameter(1)*coeff/Vi;
      dydt[j][species] -= parameter(1)*coeff/Vj;
    }
  }
}

PolarizationConstFastTransportFractionRestrictedMM::
PolarizationConstFastTransportFractionRestrictedMM(std::vector<double> 
					   &paraValue, 
					   std::vector< 
					   std::vector<size_t> > 
					   &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=6 ) {
    std::cerr << "PolarizationConstFastTransportFractionRestrictedMM::"
	      << "PolarizationConstFastTransportFractionRestrictedMM() "
	      << "uses six parameters (T_eff, T_inf, b/a, K_M, f_i, f_ij).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=3 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationConstFastTransportFractionRestrictedMM::"
	      << "PolarizationConstFastTransportFractionRestrictedMM() "
	      << "Polarized molecule + "
	      << "cell marker + wall marker "
	      << "indeces needed in"
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at "
	      << "second level.\n";
    exit(0);
  }
  //Check parameters
  //////////////////////////////////////////////////////////////////////
  if( paraValue[0]<0.0 || paraValue[1]<0.0 || paraValue[2]<0.0 ||
      paraValue[3]<0.0 || paraValue[4]<0.0 || paraValue[4]>1.0 ||
      paraValue[5]<0.0 || paraValue[5]>1.0 ) {
    std::cerr << "PolarizationConstFastTransportFractionRestrictedMM::"
	      << "PolarizationConstFastTransportFractionRestrictedMM()"
	      << " Parameters need to be positive (and fractions).\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationConstFastTransportFractionRestrictedMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T_eff";
  tmp[1] = "T_inf";
  tmp[2] = "b/a";
  tmp[3] = "K_M";
  tmp[4] = "f_i";
  tmp[5] = "f_ij";
  setParameterId( tmp );
}

void PolarizationConstFastTransportFractionRestrictedMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > 
       &dydt ) {
  
  size_t i=compartment.index();
  //Only if the comp has neighbors and is a cell
  if( compartment.numNeighbor() == 0 || 
      y[i][variableIndex(0,1)]<0.5 ) return;
  
  //Extract volume of the compartment and make sure it is physical
  double Vi = y[i][variableIndex(1,0)];
  assert( Vi>0.0 );
  
  //Calculate normalization
  //////////////////////////////////////////////////////////////////////
  double invSum=1.0;
	//if ( compartment.numNeighborAtLevel(1)) {
	//assert(compartment.numNeighborAtLevel(1)==compartment.numNeighbor());
		double sum=parameter(2);
		if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
			sum *=Vi;
			for( size_t n=0 ; n<compartment.numNeighborAtLevel(1) ; ++n )
				sum += compartment.neighborArea(n);
		}
		else
			sum += compartment.numNeighbor();//assuming area=1
		
		//double invSum = 1.0;
		if( sum>0.0 )
			invSum /= sum;//To use mult instead of div
		//}
		
  //Update all compartments
  //////////////////////////////////////////////////////////////////////
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    
    //Make sure neighbor marked as wall 
    //assert( y[j][variableIndex(0,2)]>0.5 ); 
		if ( y[j][variableIndex(0,2)]>0.5 ) { 
			//Extract volume of neighbor and make sure it is physical
			double Vj = y[j][variableIndex(1,0)];
			assert( Vj>0.0 );
			
			double Pij = y[i][variableIndex(0,0)] * Vi  * invSum;
			if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
				double coeff = compartment.neighborArea(n)*Pij*
					parameter(4)*y[i][species]/( parameter(3)+parameter(4)*y[i][species] );
				dydt[i][species] -= parameter(0)*coeff/Vi;
				dydt[j][species] += parameter(0)*coeff/Vj;
				coeff = compartment.neighborArea(n)*Pij*
					parameter(5)*y[j][species]/( parameter(3)+parameter(5)*y[j][species] );
				dydt[i][species] += parameter(1)*coeff/Vi;
				dydt[j][species] -= parameter(1)*coeff/Vj;
			}
			else {
				double coeff = Pij*parameter(4)*y[i][species]/
					( parameter(3)+parameter(4)*y[i][species] );
				dydt[i][species] -= parameter(0)*coeff/Vi;
				dydt[j][species] += parameter(0)*coeff/Vj;
				coeff = Pij*parameter(5)*y[j][species]/
					( parameter(3)+parameter(5)*y[j][species] );
				dydt[i][species] += parameter(1)*coeff/Vi;
				dydt[j][species] -= parameter(1)*coeff/Vj;
			}
		}
	}
}

PolarizationConstHillFastTransportFractionRestrictedMM::
PolarizationConstHillFastTransportFractionRestrictedMM(std::vector<double>
											&paraValue, 
											std::vector< std::vector<size_t> > &indValue)
{  
	//Do some checks on the parameters and variable indeces
	//////////////////////////////////////////////////////////////////////
	if (paraValue.size() != 9) {
		std::cerr << "PolarizationConstHillFastTransportRestrictedMM::"
				<< "PolarizationConstHillFastTransportRestrictedMM() "
				<< "uses seven parameters (T_eff, T_inf, b/a, and c, "
				<< "K_H, n,and K_M, f_i and f_ij).\n";
		exit(EXIT_FAILURE);
	}
	if (indValue.size() != 2 || indValue[0].size() != 4 
	    || indValue[1].size() != 1) {
		std::cerr << "PolarizationConstHillFastTransportRestrictedMM::"
				<< "PolarizationConstHillFastTransportRestrictedMM() "
				<< "Polarized molecule + polarization signal + "
				<< "cell marker + wall marker "
				<< "indeces needed in"
				<< "level 1.\n" 
				<< "Variable index for size parameter needed at second level.\n";
		exit(EXIT_FAILURE);
	}
	//Check parameters
	//////////////////////////////////////////////////////////////////////
	if (paraValue[0] < 0.0 || paraValue[1] < 0.0 || paraValue[2] < 0.0 ||
	    paraValue[3] < 0.0 || paraValue[4] < 0.0 || paraValue[5] < 0.0 ||
	    paraValue[6] < 0.0 ) {
		std::cerr << "PolarizationConstHillFastTransportRestrictedMM::"
				<< "PolarizationConstHillFastTransportRestrictedMM()"
				<< " Parameters need to be positive.\n";
		exit(EXIT_FAILURE);
	}
	if (paraValue[3] > 1.0) {
		std::cerr << "PolarizationConstHillFastTransportRestrictedMM::"
				<< "PolarizationConstHillFastTransportRestrictedMM()"
				<< " Parameter c (Lin/Hill fraction need to be "
				<< "smaller than 1.\n";
		exit(EXIT_FAILURE);
	}
	//Set the variable values
	//////////////////////////////////////////////////////////////////////
	setId("polarizationConstHillFastTransportRestrictedMM");
	setParameter(paraValue);  
	setVariableIndex(indValue);
	
	//Set the parameter identities
	//////////////////////////////////////////////////////////////////////
	std::vector<std::string> tmp(numParameter());
	tmp.resize(numParameter());
	tmp[0] = "T_eff";
	tmp[1] = "T_inf";
	tmp[2] = "b/a";
	tmp[3] = "c_p";
	tmp[4] = "K_H";
	tmp[5] = "n";
	tmp[6] = "K_M";
	tmp[7] = "f_i";
	tmp[8] = "f_ij";
	setParameterId(tmp);
}

void PolarizationConstHillFastTransportFractionRestrictedMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > 
       &dydt ) {
  
  size_t i=compartment.index();
  //Only if the comp has neighbors and is a cell
  if( compartment.numNeighbor() == 0 || 
      y[i][variableIndex(0,2)]<0.5 ) return;
  
  static double KToNHill = std::pow(parameter(4),parameter(5));
  //Polarization function from CELL neighbors at level 1.
  assert( compartment.numNeighborAtLevel(1) == 
					compartment.numNeighbor() );
  //Extract volume of the compartment and make sure it is physical
  double Vi = y[i][variableIndex(1,0)];
  assert( Vi>0.0 );
  
  //Calculate f(n) and normalization
  //////////////////////////////////////////////////////////////////////
  std::vector<double> f(compartment.numNeighbor() );
  for( size_t n=0 ; n<compartment.numNeighborAtLevel(1) ; n++ ) {
    double yToN = std::pow(y[ compartment.neighborAtLevel(1,n) ]
													 [ variableIndex(0,1) ] , parameter(5));
    f[n] = (1.0-parameter(3)) + parameter(3)*yToN/(KToNHill+yToN);
  }
	
  double sum=parameter(2);
  if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
    sum *=Vi;
    for( size_t n=0 ; n<compartment.numNeighborAtLevel(1) ; n++ )
      sum += f[n]*compartment.neighborArea(n);
  }
  else
    for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
      sum += f[n];
  
  double invSum = 1.0;
  if( sum>0.0 )
    invSum /= sum;//To use mult instead of div
  
  //Update all compartments
  //////////////////////////////////////////////////////////////////////
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    
    //Make sure neighbor marked as wall 
    assert( y[j][variableIndex(0,3)]>0.5 ); 
    
    //Extract volume of neighbor and make sure it is physical
    double Vj = y[j][variableIndex(1,0)];
    assert( Vj>0.0 );
    
    double Pij = y[i][variableIndex(0,0)] * Vi * f[n] * invSum;
    if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
      double coeff = compartment.neighborArea(n)*Pij*
				parameter(7)*y[i][species] /
				( parameter(6)+parameter(7)*y[i][species] );
      dydt[i][species] -= parameter(0)*coeff/Vi;
      dydt[j][species] += parameter(0)*coeff/Vj;
      coeff = compartment.neighborArea(n)*Pij*
				parameter(8)*y[j][species] /
				( parameter(6)+parameter(8)*y[j][species] );
      dydt[i][species] += parameter(1)*coeff/Vi;
      dydt[j][species] -= parameter(1)*coeff/Vj;
    }
    else {
      double coeff = Pij*y[i][species]/( parameter(6)+y[i][species] );
      dydt[i][species] -= parameter(0)*coeff/Vi;
      dydt[j][species] += parameter(0)*coeff/Vj;
      coeff = Pij*y[j][species]/( parameter(6)+y[j][species] );
      dydt[i][species] += parameter(1)*coeff/Vi;
      dydt[j][species] -= parameter(1)*coeff/Vj;
    }
  }
}

DirectionalTransport::
DirectionalTransport(std::vector<double> &paraValue, 
										 std::vector< std::vector<size_t> > &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=1 ) {
    std::cerr << "DirectionalTransport::DirectionalTransport() "
							<< "uses only one parameter T_strength.\n";
    exit(0);
  }
  if( (indValue.size() != 1 && indValue.size() != 2 ) || 
			indValue[0].size() != 1 || 
			(indValue.size()==2 && indValue[1].size() !=1)  ) {
    std::cerr << "DirectionalTransport::DirectionalTransport() "
							<< "directional signal index (x,y,z?) needed in level 1.\n" 
							<< "Variable index for size parameter could be at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("directionalTransport");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T_strength";
  setParameterId( tmp );
}

void DirectionalTransport::derivs(Compartment &compartment,size_t species,
																	std::vector< std::vector<double> > &y,
																	std::vector< std::vector<double> > &dydt ) {
  
	size_t dimension = compartment.numTopologyVariable()-1;
  size_t i=compartment.index();
	double Vi=0,Vj=0;
	if( numVariableIndexLevel()>1 ) {
		Vi = y[i][variableIndex(1,0)];//Volume of the compartment
		if( Vi<=0.0 ) {
			std::cerr << "DirectionalTransport::derivs Given volume " << Vi
								<< " unphysical.\n";
			exit(-1);
		}
  }
  //Check all neighbors
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    //Only transport from this cell if this condition is satisfied
    //(Otherwise transport in opposite direction) 
    if( parameter(0)*(y[j][variableIndex(0,0)]-y[i][variableIndex(0,0)]) 
				> 0.0 ) {
      if( numVariableIndexLevel()>1 ) {
				Vj = y[j][variableIndex(1,0)];//Volume of the compartment
				if( Vj<=0.0 ) {
					std::cerr << "DirectionalTransport::derivs Given volume " << Vj
										<< " unphysical.\n";
					exit(-1);
				}
      }
      //Directional coefficient normalization constant ( |xi-xj|/|ri-rj| )
      //Assuming positional variables in the beginning of a y-row
      double distance=0.0;
      for( size_t d=0 ; d<dimension ; d++ )
				distance += (y[i][d]-y[j][d])*(y[i][d]-y[j][d]);
      distance = sqrt( distance );
      double norm;
      if( distance>0.0 )
				norm = fabs(y[j][variableIndex(0,0)]-y[i][variableIndex(0,0)])
					/distance;
      else {
				std::cerr << "DirectionalTransport::derivs() "
									<< "distance between cells <= 0 ("
									<< distance << ")." << std::endl;
				exit(-1);
      }
      
      double coeff = y[i][species]*norm;
			if( numVariableIndexLevel()>1 ) {
				dydt[i][species] -= fabs( parameter(0) ) * coeff / Vi; 
				dydt[j][species] += fabs( parameter(0) ) * coeff / Vj;
			}
			else {
				dydt[i][species] -= fabs( parameter(0) ) * coeff;
				dydt[j][species] += fabs( parameter(0) ) * coeff;
			}
    }  
  }
}

DirectionalTransportMM::DirectionalTransportMM(std::vector<double> &paraValue, 
					       std::vector< std::vector<size_t> > 
					       &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=2 ) {
    std::cerr << "DirectionalTransportMM::DirectionalTransportMM() "
	      << "uses two parameters T_strength and K_M.\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=1 
      || indValue[1].size() !=1  ) {
    std::cerr << "DirectionalTransportMM::DirectionalTransportMM() "
	      << "directional signal index (x,y,z?) needed in level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("directionalTransportMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T_strength";
  tmp[1] = "K_M";
  setParameterId( tmp );
}

void DirectionalTransportMM::derivs(Compartment &compartment,size_t species,
				  std::vector< std::vector<double> > &y,
				  std::vector< std::vector<double> > &dydt ) {
  
  size_t i=compartment.index();
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "DirectionalTransportMM::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Check all neighbors
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    //Only transport from this cell if this condition is satisfied
    //(Otherwise transport in opposite direction) 
    if( parameter(0)*(y[j][variableIndex(0,0)]-y[i][variableIndex(0,0)]) 
	> 0.0 ) {
      
      double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
      if( Vj<=0.0 ) {
	std::cerr << "DirectionalTransportMM::derivs Given volume " << Vj
		  << " unphysical.\n";
	exit(-1);
      }
      
      //Directional coefficient normalization constant ( |xi-xj|/|ri-rj| )
      //Assuming positional variables in the beginning of a y-row
      double distance=0.0;
      for( size_t d=0 ; d<compartment.numDimension() ; d++ )
	distance += (y[i][d]-y[j][d])*(y[i][d]-y[j][d]);
      distance = sqrt( distance );
      double norm;
      if( distance>0.0 )
	norm = fabs(y[j][variableIndex(0,0)]-y[i][variableIndex(0,0)])
	  /distance;
      else {
	std::cerr << "DirectionalTransportMM::derivs() "
		  << "distance between cells <= 0.\n";
	exit(-1);
      }
      
      double coeff = norm * y[i][species]/(y[i][species]+parameter(1));
      dydt[i][species] -= fabs( parameter(0) ) * coeff;
      dydt[j][species] += fabs( parameter(0) ) * coeff;
      //dydt[i][species] -= fabs( parameter(0) ) * coeff / Vi; 
      //dydt[j][species] += fabs( parameter(0) ) * coeff / Vj;
    }  
  }
}

DirectionalTransportSwitchedMM::
DirectionalTransportSwitchedMM(std::vector<double> &paraValue, 
			       std::vector< std::vector<size_t> > 
			       &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=3 ) {
    std::cerr << "DirectionalTransportSwitchedMM::"
	      << "DirectionalTransportSwitchedMM() "
	      << "uses three parameters T_strength and K_M Th_switch.\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "DirectionalTransportSwitchedMM::"
	      << "DirectionalTransportSwitchedMM() "
	      << "directional signal index (x,y,z) and switch molecule needed"
	      << " in level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("directionalTransportSwitchedMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T_strength";
  tmp[1] = "K_M";
  tmp[2] = "Th_sw";
  setParameterId( tmp );
}

void DirectionalTransportSwitchedMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > &dydt ) {
  
  size_t i=compartment.index();
  //Check for the switched behavior
  //(Caveat:is only checked for this cell and not its neighbor...)
  double T = parameter(0);
  if( y[i][variableIndex(0,1)] > parameter(2) )
    T = -T;
  
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "DirectionalTransportSwitchedMM::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Check all neighbors
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    //Only transport from this cell if this condition is satisfied
    //(Otherwise transport in opposite direction) 
    if( T*(y[j][variableIndex(0,0)]-y[i][variableIndex(0,0)]) 
	> 0.0 ) {
      
      double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
      if( Vj<=0.0 ) {
	std::cerr << "DirectionalTransportSwitchedMM::derivs Given volume " 
		  << Vj << " unphysical.\n";
	exit(-1);
      }
      
      //Directional coefficient normalization constant ( |xi-xj|/|ri-rj| )
      //Assuming positional variables in the beginning of a y-row
      double distance=0.0;
      for( size_t d=0 ; d<compartment.numDimension() ; d++ )
	distance += (y[i][d]-y[j][d])*(y[i][d]-y[j][d]);
      distance = sqrt( distance );
      double norm;
      if( distance>0.0 )
	norm = fabs(y[j][variableIndex(0,0)]-y[i][variableIndex(0,0)])
	  /distance;
      else {
	std::cerr << "DirectionalTransportSwitchedMM::derivs() "
		  << "distance between cells <= 0.\n";
	exit(-1);
      }
      
      double coeff = norm * y[i][species]/(y[i][species]+parameter(1));
      dydt[i][species] -= std::fabs(T)*coeff;
      dydt[j][species] += std::fabs(T)*coeff;
      //dydt[i][species] -= fabs( parameter(0) ) * coeff / Vi; 
      //dydt[j][species] += fabs( parameter(0) ) * coeff / Vj;
    }  
  }
}

MitchisonTransport::MitchisonTransport(std::vector<double> &paraValue, 
				       std::vector< std::vector<size_t> > 
				       &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=1 ) {
    std::cerr << "MitchisonTransport::MitchisonTransport() "
	      << "uses only one non-compartment specific parameter.\n"
	      << "This is a strength P (and is equal to 1 in M's paper.\n";
    exit(0);
  }
  if( indValue.size() != 0 ) {
    std::cerr << "MitchisonTransport::MitchisonTransport() "
	      << "No extra indeces needed.\n"; 
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("mitchisonTransport");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp[0]="P";
  setParameterId( tmp );
}

void MitchisonTransport::derivs(Compartment &compartment,size_t species,
				std::vector< std::vector<double> > &y,
				std::vector< std::vector<double> > &dydt ) {
  
  double volume,PI=3.1415;
  size_t i=compartment.index();
  
  if( compartment.numNeighbor() != compartment.numParameter() ) {
    std::cerr << "MitchisonTransport::derivs() "
	      << "One parameter for each neighbor must be defined.\n";
    exit(-1);
  }
  double r = y[i][compartment.numDimension()];
  if( compartment.numDimension()==2 )
    volume = PI*r*r;
  else if( compartment.numDimension()==3 )
    volume = 4.0*PI*r*r*r/3.0;
  else {
    std::cerr << "MitchisonTransport::derivs() "
	      << "wrong dimension for update.\n";
    exit(-1);
  }
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    
    double Pij=compartment.parameter(n);
    
    double volumeNeighbor;
    //Caveat: Assumes that also neighbors have same dimension
    double rN = y[j][compartment.numDimension()];
    if( compartment.numDimension()==2 )
      volumeNeighbor = PI*rN*rN;
    else if( compartment.numDimension()==3 )
      volumeNeighbor = 4.0*PI*rN*rN*rN/3.0;
    else {
      std::cerr << "MitchisonTransport::derivs() "
		<< "wrong dimension for update.\n";
      exit(-1);
    }
    
    //derivative with strength parameter 1 to be close to concentration diff...
    double volumeMean=0.5*(volume+volumeNeighbor);
    //For no volume dependance
    //volume=volumeNeighbor=volumeMean=1.0;
    if( volume>0. ) 
      dydt[i][species] -= parameter(0)* y[i][species]*Pij * volumeMean/volume;
    else {
      std::cerr << "MitchisonTransport::derivs "
		<< "Division by zero volume.\n" 
		<< volume << " " << volumeNeighbor << "\n";
      std::cerr << i << " " << species << " " << variableIndex(1,0) << "\n"; 
      exit(-1);
    }  
    if( volumeNeighbor>0. ) 
      dydt[j][species] += parameter(0)* y[i][species]*Pij * 
	volumeMean/volumeNeighbor;
    else {
      std::cerr << "MitchisonTransport::derivs "
		<< "Division by zero neighbor volume.\n"
		<< volume << " " << volumeNeighbor << "\n";
      exit(-1);
    }  
  }
}

ActiveTransportRestricted::ActiveTransportRestricted(std::vector<double>
						     &paraValue, 
						     std::vector< 
						     std::vector<size_t> > 
						     &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=2 ) {
    std::cerr << "ActiveTransportRestricted::ActiveTransportRestricted() "
	      << "Uses two parameters T_out T_in.\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=3
      || indValue[1].size() !=1 ) {
    std::cerr << "ActiveTransportRestricted::ActiveTransportRestricted() "
	      << "Variable index for helper(in neighbor), compartment"
	      << " and neighbor markers, and size (2nd level) needed\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("activeTransportRestricted");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T_out";
  tmp[1] = "T_in";
  setParameterId( tmp );
}

void ActiveTransportRestricted::derivs(Compartment &compartment,size_t species,
				       std::vector< std::vector<double> > &y,
				       std::vector< std::vector<double> > 
				       &dydt ) {
  
  //double volume,PI=3.1415;
  size_t i=compartment.index();
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "ActiveTransportRestricted::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }

  //Do this update from compartmentMarked cells (not neighborMarked)
  if( y[i][variableIndex(0,1)] < 0.5 ) {
    return;
  }
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    if( y[j][variableIndex(0,2)] > 0.5 ) {//Only when neighbor is marked
      double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
      if( Vj<=0.0 ) {
	std::cerr << "ActiveTransportRestricted::derivs Given volume " << Vj
		  << " unphysical.\n";
	exit(-1);
      }
      double compConc = y[i][species];
      double neighConc = y[j][species];
      size_t helperCol=variableIndex(0,0);
      //If no neighbor area defined assume area and distance 1
      double distance = 1.0,area=1.0;
      if( compartment.numNeighborArea()>n ) {
	area = compartment.neighborArea(n);
	distance=0.0;
	for( size_t d=compartment.topologyStart() ; d<compartment.numDimension() 
	       ; d++ )
	  distance += (y[j][d]-y[i][d])*(y[j][d]-y[i][d]);
	if( distance<=0 ) {
	  std::cerr << "ActiveTransportRestricted::derivs Calculated "
		    << "distance " << distance << " unphysical.\n";
	  exit(-1);
	}
	distance = sqrt(distance);
      }
      //T_out
      double tmp = parameter(0)*compConc*y[j][helperCol]*area/distance;
      dydt[i][species] -= tmp/Vi;
      dydt[j][species] += tmp/Vj;
      //T_in
      tmp = parameter(1)*neighConc*y[j][helperCol]*area/distance;
      dydt[i][species] += tmp/Vi;
      dydt[j][species] -= tmp/Vj;
    }
  }
}

ActiveTransportMMRestricted::ActiveTransportMMRestricted(std::vector<double>
							 &paraValue, 
							 std::vector< 
							 std::vector<size_t> > 
							 &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=4 ) {
    std::cerr << "ActiveTransportMMRestricted::ActiveTransportMMRestricted() "
	      << "Uses four parameters Tmax_out k_out Tmax_in "
	      << "k_in.\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=3
      || indValue[1].size() !=1 ) {
    std::cerr << "ActiveTransportMMRestricted::ActiveTransportMMRestricted() "
	      << "Variable index for helper, compartment and "
	      << "neighbor markers, and size (2nd level) needed.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("activeTransportMMRestricted");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "Tmax_out";
  tmp[1] = "k_out";
  tmp[2] = "Tmax_in";
  tmp[3] = "k_in";
  setParameterId( tmp );
}

void ActiveTransportMMRestricted::derivs(Compartment &compartment,size_t species,
					 std::vector< std::vector<double> > &y,
					 std::vector< std::vector<double> > 
					 &dydt ) {
  
  size_t i=compartment.index();
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "ActiveTransportMMRestricted::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Do this update from marked compartments only
  if( y[i][variableIndex(0,1)] < 0.5 ) {
    return;
  }
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    if( y[j][variableIndex(0,2)] > 0.5 ) {//Only when neighbor is marked
      
      double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
      if( Vj<=0.0 ) {
	std::cerr << "ActiveTransportMMRestricted::derivs Given volume " << Vj
		  << " unphysical.\n";
	exit(-1);
      }
      double cellConc = y[i][species];
      double wallConc = y[j][species];
      
      //Tmax_out,k_out
      double tmp = parameter(0)*cellConc*y[j][variableIndex(0,0)]/
	(parameter(1)+y[j][variableIndex(0,0)]);
      dydt[i][species] -= tmp/Vi;
      dydt[j][species] += tmp/Vj;
      //Tmax_in,k_in
      tmp = parameter(2)*wallConc*y[j][variableIndex(0,0)]/
	(parameter(3)+y[i][variableIndex(0,0)]);
      dydt[i][species] += tmp/Vi;
      dydt[j][species] -= tmp/Vj;
    }
  }
}

//!Constructor for the ActiveTransportB class
/*! Uses T_out*A_ij*X_i for out and T_in*A_ij*X_ij for transport
  between and compartment and neighbor. (out means comp to neigh, in
  is neigh to comp). X is the transported molecule, A is a helper
  molecule promoting in/efflux. The difference from
  ActiveTransportRestricted() is that the flux depends on
  concentration in a compartment defined in compartment.parameter.*/
ActiveTransportBRestricted::ActiveTransportBRestricted(std::vector<double>
						       &paraValue, 
						       std::vector< 
						       std::vector<size_t> > 
						       &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=2 ) {
    std::cerr << "ActiveTransportBRestricted::ActiveTransportBRestricted() "
	      << "Uses two parameters T_out T_in.\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=3
      || indValue[1].size() !=1 ) {
    std::cerr << "ActiveTransportBRestricted::ActiveTransportBRestricted() "
	      << "Variable index for helper, compMarker"
	      << " and neighMarker, and size (2nd level) needed.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("activeTransportBRestricted");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T_out";
  tmp[1] = "T_in";
  setParameterId( tmp );
}

//! Derivative contribution for active transport
void ActiveTransportBRestricted::derivs(Compartment &compartment,size_t species,
					std::vector< std::vector<double> > &y,
					std::vector< std::vector<double> > 
					&dydt ) {
    
  //double volume,PI=3.1415;
  size_t i=compartment.index();
  
  //Get compartment volume and assure it is physical
  double Vi = y[i][variableIndex(1,0)];
  assert( Vi>0.0 );
  
  //Do this update from marked compartments
  if( y[i][variableIndex(0,1)] < 0.5 ) {
    return;
  }
  //Check that the compartment have parameters corresponding 
  //to the neighboring CELLS 
  //Caveat: this is now handled by multiple layers of neighs 
  assert( compartment.numNeighbor() == compartment.numParameter() );
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    if( y[j][variableIndex(0,2)] > 0.5 ) {//Only when neighbor is marked

      size_t jHelper = static_cast<size_t>( compartment.parameter(n) );
      //Get neighbor volume and make sure it is physical 

      double Vj = y[j][variableIndex(1,0)];
      assert( Vj>0.0 );

      double compConc = y[i][species];
      double neighConc = y[j][species];
      size_t helperCol=variableIndex(0,0);
      double tmp;
      //Only update if jHelper is not the background
      if( jHelper != static_cast<size_t>(-1) ) {
	//If no neighbor area defined assume area and distance equal to 1
	double distance = 1.0,area=1.0;
	if( compartment.numNeighborArea()>n ) {
	  area = compartment.neighborArea(n);
	  distance=0.0;
	  for( size_t d=compartment.topologyStart() ; d<compartment.numDimension() 
		 ; d++ )
	    distance += (y[j][d]-y[i][d])*(y[j][d]-y[i][d]);
	  assert( distance>0.0 );;
	  distance = sqrt(distance);
	}
	//T_out
	tmp = parameter(0)*compConc*y[jHelper][helperCol]*area/distance;
	dydt[i][species] -= tmp/Vi;
	dydt[j][species] += tmp/Vj;
	//T_in
	tmp = parameter(1)*neighConc*y[jHelper][helperCol]*area/distance;
	dydt[i][species] += tmp/Vi;
	dydt[j][species] -= tmp/Vj;
      }
    }
  }
}

//! Transport assuming a membrane between compartments 
MembraneTransport::MembraneTransport(std::vector<double> &paraValue, 
				     std::vector< std::vector<size_t> > 
				     &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=1 ) {
    std::cerr << "MembraneTransport::MembraneTransport() "
	      << "Uses only one parameter P (m/s).\n";
    exit(0);
  }
  if( indValue.size() != 1 || indValue[0].size() !=1 ) {
    std::cerr << "MembraneTransport::MembraneTransport() "
	      << "Variable index for size parameter needed.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("membraneTransport");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "P";
  setParameterId( tmp );
}

//! Derivative contribution for a degradation proportional to concentration
void MembraneTransport::derivs(Compartment &compartment,size_t species,
			       std::vector< std::vector<double> > &y,
			       std::vector< std::vector<double> > &dydt ) {
  
  size_t i=compartment.index();
  double Vi = y[i][variableIndex(0,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "MembraneTransport::derivs Given volume " << Vi 
	      << " unphysical.\n";
    exit(-1);
  }
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    //Only update if compartments index less than neighbors
    if( i<j ) {
      double Vj = y[j][variableIndex(0,0)];//Volume of the compartment neigh
      if( Vj<=0.0 ) {
	std::cerr << "MembraneTransport::derivs Given volume " << Vj
		  << " unphysical.\n";
	exit(-1);
      }
      //If no neighbor area defined assume area 1
      double area=1.0;
      if( compartment.numNeighborArea()>n )
	area = compartment.neighborArea(n);

      double diff = y[j][species]-y[i][species];
      dydt[i][species] += parameter(0)*diff*area/Vi;
      dydt[j][species] -= parameter(0)*diff*area/Vj;
    }
  }
}

//!Constructor for the MembraneTransportRestricted class
MembraneTransportRestricted::MembraneTransportRestricted(std::vector<double> 
							 &paraValue, 
							 std::vector< 
							 std::vector<size_t> > 
							 &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=2 ) {
    std::cerr << "MembraneTransportRestricted::MembraneTransportRestricted() "
	      << "Uses two parameters P_out and P_in (m/s).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 || 
      indValue[1].size() !=1 ) {
    std::cerr << "MembraneTransportRestricted::MembraneTransportRestricted() "
	      << "Variable indices for two restrictMarkers, and size"
	      << " (second level) needed.\n";
    exit(0);
  }
  if( indValue[0][0] == indValue[0][1] && paraValue[0] != paraValue[1] ) {
    std::cerr << "MembraneTransportRestricted::MembraneTransportRestricted() "
	      << "Same type of compartments requires same P_out and P_in.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("membraneTransportRestricted");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "P_out";
  tmp[1] = "P_in";
  setParameterId( tmp );
}

//! Derivative contribution for a restricted membrane transport
void MembraneTransportRestricted::derivs(Compartment &compartment,size_t species,
					 std::vector< std::vector<double> > &y,
					 std::vector< std::vector<double> > 
					 &dydt ) {
  
  size_t i=compartment.index();
  //Check that this compartment has neighbors and is marked for diffusion
  if( y[i][variableIndex(0,0)] < 0.5 || compartment.numNeighbor()<1 )
    return;
  
  //Get volume and assure it is physical
  double Vi = y[i][variableIndex(1,0)];
  assert( Vi>0.0 );
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    //Only update if neighbor marked for this diffusion
    //If same type of compartments only update once (i>j)
    if( y[j][variableIndex(0,1)] > 0.5 )
      if( variableIndex(0,0) != variableIndex(0,1) || i>j ) {
	
	//Get neighbor volume and assure it is physical
	double Vj = y[j][variableIndex(1,0)];
	assert( Vj>0.0 );
	//If no neighbor area defined assume area 1
	double area=1.0;
	if( compartment.numNeighborArea()>n )
	  area = compartment.neighborArea(n);
	
	//D_out
	double tmp = parameter(0)*y[i][species]*area;
	dydt[i][species] -= tmp/Vi;
	dydt[j][species] += tmp/Vj;
	//D_in
	tmp = parameter(1)*y[j][species]*area;
	dydt[i][species] += tmp/Vi;
	dydt[j][species] -= tmp/Vj;
      }  
  }
}

//!Constructor for ActiveMembraneTransportRestricted
ActiveMembraneTransportRestricted::
ActiveMembraneTransportRestricted(std::vector<double> &paraValue, 
				  std::vector< std::vector<size_t> > &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=2 ) {
    std::cerr << "ActiveMembraneTransportRestricted::"
	      << "ActiveMembraneTransportRestricted() "
	      << "Uses two parameters T_out T_in length/(time conc).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=3
      || indValue[1].size() !=1 ) {
    std::cerr << "ActiveMembraneTransportRestricted::"
	      << "ActiveMembraneTransportRestricted() "
	      << "Variable index for helper(in neighbor), compartment"
	      << " and neighbor markers, and size (2nd level) needed\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("activeMembraneTransportRestricted");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T_out";
  tmp[1] = "T_in";
  setParameterId( tmp );
}

//! Derivative contribution for active membrane transport
void ActiveMembraneTransportRestricted::derivs(Compartment &compartment,
					       size_t species,
					       std::vector< 
					       std::vector<double> > &y,
					       std::vector< 
					       std::vector<double> > &dydt ) {
  
  size_t i=compartment.index();
  //Do this update from compartmentMarked cells with neighbors
  if( y[i][variableIndex(0,1)] < 0.5 || compartment.numNeighbor()<1 ) {
    return;
  }
  
  //Get compartment volume and assure it is physical
  double Vi = y[i][variableIndex(1,0)];
  assert( Vi>0.0 );
  
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    if( y[j][variableIndex(0,2)] > 0.5 ) {//Only when neighbor is marked

      //Get neighbor volume and assure it is physical
      double Vj = y[j][variableIndex(1,0)];
      assert( Vj>0.0 );

      double compConc = y[i][species];
      double neighConc = y[j][species];
      size_t helperCol=variableIndex(0,0);
      //If no neighbor area defined assume area 1 
      double area=1.0;
      if( compartment.numNeighborArea()>n )
	area = compartment.neighborArea(n);
      
      //T_out
      double tmp = parameter(0)*compConc*y[j][helperCol]*area;
      dydt[i][species] -= tmp/Vi;
      dydt[j][species] += tmp/Vj;
      //T_in
      tmp = parameter(1)*neighConc*y[j][helperCol]*area;
      dydt[i][species] += tmp/Vi;
      dydt[j][species] -= tmp/Vj;
    }
  }
}

//!Constructor for ActiveMembraneTransportRestrictedMM
ActiveMembraneTransportRestrictedMM::
ActiveMembraneTransportRestrictedMM(std::vector<double> &paraValue, 
				    std::vector< std::vector<size_t> > 
				    &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=3 ) {
    std::cerr << "ActiveMembraneTransportRestrictedMM::"
	      << "ActiveMembraneTransportRestrictedMM() "
	      << "Uses three parameters T_out T_in length/(time conc) "
	      << " and K_M.\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=3
      || indValue[1].size() !=1 ) {
    std::cerr << "ActiveMembraneTransportRestrictedMM::"
	      << "ActiveMembraneTransportRestrictedMM() "
	      << "Variable index for helper(in neighbor), compartment"
	      << " and neighbor markers, and size (2nd level) needed\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("activeMembraneTransportRestrictedMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T_out";
  tmp[1] = "T_in";
  tmp[2] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for active membrane transport
void ActiveMembraneTransportRestrictedMM::derivs(Compartment &compartment,
						 size_t species,
						 std::vector< 
						 std::vector<double> > &y,
						 std::vector< 
						 std::vector<double> > 
						 &dydt ) {
  
  size_t i=compartment.index();
  //Do this update from compartmentMarked cells with neighbors
  if( y[i][variableIndex(0,1)] < 0.5 || compartment.numNeighbor()<1 ) {
    return;
  }
  
  //Get compartment volume and assure it is physical
  double Vi = y[i][variableIndex(1,0)];
  assert( Vi>0.0 );
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    if( y[j][variableIndex(0,2)] > 0.5 ) {//Only when neighbor is marked
      //Get neighbor volume and assure it is physical
      double Vj = y[j][variableIndex(1,0)];
      assert( Vj>0.0 );
      
      double compConcFac = y[i][species]/(parameter(2)+y[i][species]);
      double neighConcFac = y[j][species]/(parameter(2)+y[j][species]);
      size_t helperCol=variableIndex(0,0);
      //If no neighbor area defined assume area 1 
      double area=1.0;
      if( compartment.numNeighborArea()>n )
	area = compartment.neighborArea(n);
      
      //T_out
      double tmp = parameter(0)*compConcFac*y[j][helperCol]*area;
      dydt[i][species] -= tmp/Vi;
      dydt[j][species] += tmp/Vj;
      //T_in
      tmp = parameter(1)*neighConcFac*y[j][helperCol]*area;
      dydt[i][species] += tmp/Vi;
      dydt[j][species] -= tmp/Vj;
    }
  }
}

//!Constructor for the ActiveMembraneTransportBRestricted class
ActiveMembraneTransportBRestricted::
ActiveMembraneTransportBRestricted(std::vector<double> &paraValue, 
				   std::vector< std::vector<size_t> > 
				   &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=2 ) {
    std::cerr << "ActiveMembraneTransportBRestricted::"
	      << "ActiveMembraneTransportBRestricted() "
	      << "Uses two parameters T_out T_in (length/(time conc).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=3
      || indValue[1].size() !=1 ) {
    std::cerr << "ActiveMembraneTransportBRestricted::"
	      << "ActiveMembraneTransportBRestricted() "
	      << "Variable index for helper, compMarker"
	      << " and neighMarker, and size (2nd level) needed.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("activeMembraneTransportBRestricted");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T_out";
  tmp[1] = "T_in";
  setParameterId( tmp );
}

//! Derivative contribution for active Membrane transport (b variant of help)
void ActiveMembraneTransportBRestricted::derivs(Compartment &compartment,
						size_t species,
						std::vector< 
						std::vector<double> > &y,
						std::vector< 
						std::vector<double> > &dydt ) {
  
  size_t i=compartment.index();
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "ActiveMembraneTransportBRestricted::derivs Given volume " 
	      << Vi << " unphysical.\n";
    exit(-1);
  }
  //Do this update from marked compartments
  if( y[i][variableIndex(0,1)] < 0.5 ) {
    return;
  }
  //Check that the compartment have parameters corresponding to the neighbor
  //cells
  if( compartment.numNeighbor() != compartment.numParameter() ) {
    std::cerr << "ActiveMembraneTransportBRestricted::derivs "
	      << "Requires neighboring CELLS in the parameter fields"
	      << " of the compartment.\n";
    exit(0);
  }	
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    if( y[j][variableIndex(0,2)] > 0.5 ) {//Only when neighbor is marked
      size_t jHelper = static_cast<size_t>( compartment.parameter(n) );
      double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
      if( Vj<=0.0 ) {
	std::cerr << "ActiveMembraneTransportBRestricted::derivs Given volume "
		  << Vj << " unphysical.\n";
	exit(-1);
      }
      double compConc = y[i][species];
      double neighConc = y[j][species];
      size_t helperCol=variableIndex(0,0);
      double tmp;
      //Only if jHelper not background
      if( jHelper != static_cast<size_t>(-1) ) {
	//If no neighbor area defined assume area 1
	double area=1.0;
	if( compartment.numNeighborArea()>n )
	  area = compartment.neighborArea(n);

	//T_out
	tmp = parameter(0)*compConc*y[jHelper][helperCol]*area;
	dydt[i][species] -= tmp/Vi;
	dydt[j][species] += tmp/Vj;
	//T_in
	tmp = parameter(1)*neighConc*y[jHelper][helperCol]*area;
	dydt[i][species] += tmp/Vi;
	dydt[j][species] -= tmp/Vj;
      }
    }
  }
}

//!Constructor for the ActiveMembraneTransportBRestricted class
ActiveMembraneTransportBHillRestricted::
ActiveMembraneTransportBHillRestricted(std::vector<double> &paraValue, 
				   std::vector< std::vector<size_t> > 
				   &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=4 ) {
    std::cerr << "ActiveMembraneTransportBHillRestricted::"
	      << "ActiveMembraneTransportBHillRestricted() "
	      << "Uses two parameters T_out T_in (length/(time conc),"
	      << "n_H K_H.\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=3
      || indValue[1].size() !=1 ) {
    std::cerr << "ActiveMembraneTransportBHillRestricted::"
	      << "ActiveMembraneTransportBHillRestricted() "
	      << "Variable index for helper, compMarker"
	      << " and neighMarker, and size (2nd level) needed.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("activeMembraneTransportBHillRestricted");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T_out";
  tmp[1] = "T_in";
  tmp[2] = "n_H";
  tmp[3] = "K_H";
  setParameterId( tmp );
}

//! Derivative contribution for active Membrane transport (b variant of help)
void ActiveMembraneTransportBHillRestricted::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > &dydt ) {
  
  size_t i=compartment.index();
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "ActiveMembraneTransportBHillRestricted::derivs() "
	      << "Given volume " << Vi << " unphysical.\n";
    exit(-1);
  }
  //Do this update from marked compartments
  if( y[i][variableIndex(0,1)] < 0.5 ) {
    return;
  }
  //Check that the compartment have parameters corresponding to the neighbor
  //cells
  if( compartment.numNeighbor() != compartment.numParameter() ) {
    std::cerr << "ActiveMembraneTransportBHillRestricted::derivs "
	      << "Requires neighboring CELLS in the parameter fields"
	      << " of the compartment.\n";
    exit(0);
  }	
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    if( y[j][variableIndex(0,2)] > 0.5 ) {//Only when neighbor is marked
      size_t jHelper = size_t( compartment.parameter(n) );
      double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
      if( Vj<=0.0 ) {
	std::cerr << "ActiveMembraneTransportBHillRestricted::derivs() "
		  << "Given volume " << Vj << " unphysical.\n";
	exit(-1);
      }
      double compConc = y[i][species];
      double neighConc = y[j][species];
      size_t helperCol=variableIndex(0,0);
      double tmp,factor;
      //Only if jHelper not background
      if( jHelper != static_cast<size_t>(-1) ) {
	//If no neighbor area defined assume area 1
	double area=1.0;
	if( compartment.numNeighborArea()>n )
	  area = compartment.neighborArea(n);
	
	//T_out
	factor = std::pow(y[jHelper][helperCol],parameter(2)) /
	  ( std::pow( parameter(3),parameter(2) ) +
	    std::pow(y[jHelper][helperCol],parameter(2) ) );
	tmp = parameter(0)*compConc*area*factor;
	dydt[i][species] -= tmp/Vi;
	dydt[j][species] += tmp/Vj;
	//T_in
	tmp = parameter(1)*neighConc*area*factor;
	dydt[i][species] += tmp/Vi;
	dydt[j][species] -= tmp/Vj;
      }
    }
  }
}

//! Constructor for the PolarizationTransportFastGeneralFGRestricted class
/*!
  The Polarized transporter molecule \f$P\f$ is divided into the cell
  cytosol (\f$P_i\f$) and its membranes (\f$P_{ij}\f$) following the dynamics

  \f[
  \frac{dP_i}{dt} = - P_i sum_k f(A_{i,k}) + sum_k g(A_{i,k})P_{i,k} \\
  \f]
  \f[
  \frac{dP_{i,j}}{dt} = P_i f(A_{i,j}) - P_{i,j} g(A_{i,j})
  \f]

  where \f$k\f$ is summed over neighboring cells, \f$A_i\f$ is the
  polarizing signal in cell \f$i\f$ and \f$A_{i,j}\f$ is the
  polarizing signal in the wall between cell \f$i\f$ and \f$j\f$. The
  polarization is assumed to be fast and the fixed point values are
  assumed

  \f[
  P^\ast_{i,j} = \frac
  {\frac{f(A_{i,j})}{g(A_{i,j})}P_{\textnormal{tot}}}
  {1+sum_k\frac{f(A_{i,k})}{g(A_{i,k})}}
  \f]

  where \f$P_{\textnormal{tot}}\f$ is the total amount of polarized molecule in a cell.

  The polarized transport of a molecule \f$A\f$ between two neighboring compartments (of different types) are then given by
  
  \f[
  \frac{dA_{i}}{dt} = - T A_i sum_k P^\ast_{i,k}
  \f]
  \f[
  \frac{dA_{i,j}}{dt} = T(P^\ast_{i,j} A_i + P^\ast_{j,i} A_j)
  \f]

*/
PolarizationTransportFastGeneralFGRestricted::PolarizationTransportFastGeneralFGRestricted(
  std::vector<double> &paraValue,
  std::vector< std::vector<size_t> > &indValue) 
{
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if(paraValue.size() != 5) {
    std::cerr << "PolarizationTransportFastGeneralFGRestricted::"
	      << "PolarizationTransportFastGeneralFGRestricted() "
	      << "uses five parameters (T, k_f, n_f, k_g, n_g).\n";
    exit(0);
  }
  if(indValue.size() != 2 || indValue[0].size() != 2 
     || indValue[1].size() != 2) {
    std::cerr << "PolarizationTransportFastGeneralFGRestricted::"
	      << "PolarizationTransportFastGeneralFGRestricted()\n "
	      << " - Level 1: Polarized molecule + polarization signal.\n"
	      << " - Level 2: Markers for restricted transport.\n";
    exit(0);
  }

  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationTransportFastGeneralFGRestricted");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp(numParameter());
  tmp.resize(numParameter());
  tmp[0] = "T";
  tmp[1] = "k_f";
  tmp[2] = "n_f";
  tmp[3] = "k_g";
  tmp[4] = "n_n";
  setParameterId(tmp);
}

//! Derivative contribution for an restricted (general) active transport in polarized direction
/*!  
  Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.
*/
void PolarizationTransportFastGeneralFGRestricted::derivs(
  Compartment &compartment,size_t species,
  std::vector< std::vector<double> > &y,
  std::vector< std::vector<double> > &dydt )
{
  size_t i = compartment.index();

  //Check that this compartment is marked for this transport
  if (y[i][variableIndex(1,0)] < 0.5)
    return;

  //Polarization coefficient normalization constant
  double sum = 0.0;
  double tmp;
  for(size_t n = 0; n < compartment.numNeighbor(); n++) {
    tmp = f(y[compartment.neighbor(n)][variableIndex(0,1)]);
    tmp /= g(y[compartment.neighbor(n)][variableIndex(0,1)]);
    sum += tmp;
  }
  sum += 1;

  for(size_t n = 0; n < compartment.numNeighbor(); n++) {
    size_t j = compartment.neighbor(n);
 
    //Only update if neighbor marked for this transport
    //If same type of compartments only update once (i>j)

    double Pij, coeff;
    if (y[j][variableIndex(1,1)] > 0.5) {
      if (variableIndex(1,0) != variableIndex(1,1) || i > j) {
	Pij = f(y[j][variableIndex(0,1)]);
	Pij /= g(y[j][variableIndex(0,1)]);
	Pij *= y[i][variableIndex(0,0)];
	Pij /= sum;
  	coeff = y[i][species] * Pij;

	dydt[i][species] -= parameter(0)*coeff;
	dydt[j][species] += parameter(0)*coeff;
      }
    }
  }
}

//! Function to be used in PolarizationTransportFastGeneralFGRestricted
double PolarizationTransportFastGeneralFGRestricted::f(double arg)
{
  return parameter(1) * pow(arg, parameter(2));
}

//! Function to be used in PolarizationTransportFastGeneralFGRestricted
double PolarizationTransportFastGeneralFGRestricted::g(double arg)
{
  return parameter(3) * pow(arg, parameter(4));
}

//!Constructor for the DiffusionSimpleRestricted class
DiffusionSimpleRestricted::DiffusionSimpleRestricted(
  std::vector<double> &paraValue, 
  std::vector< std::vector<size_t> > &indValue) 
{
 
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if (paraValue.size() != 2) {
    std::cerr << "DiffusionSimpleRestricted::DiffusionSimpleRestricted() "
	      << "Uses two parameters D_out and D_in.\n";
    exit(0);
  }
  if (indValue[0].size() != 2) {
    std::cerr << "DiffusionRestricted::DiffusionRestricted() "
	      << "Variable indices for two restrictMarkers.\n";
    exit(0);
  }
  if (indValue[0][0] == indValue[0][1] && paraValue[0] != paraValue[1]) {
    std::cerr << "DiffusionRestricted::DiffusionRestricted() "
	      << "Same type of compartments requires same D_out and D_in.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("diffusionSimpleRestricted");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize(numParameter());
  tmp[0] = "D_out";
  tmp[1] = "D_in";
  setParameterId(tmp);
}

//! Derivative contribution for a (simple) restricted diffusion (btw allowed compartm)
/*! This is the same as diffusion but is restricted to only apply
  between restricted compartments (e.g. cells and walls, or cells and
  cells) marked in variableIndex (0,0) and (0,1). */ 
void DiffusionSimpleRestricted::derivs(Compartment &compartment,size_t species,
				 std::vector< std::vector<double> > &y,
				 std::vector< std::vector<double> > &dydt )
{  
  size_t i = compartment.index();

  //Check that this compartment is marked for this diffusion
  if (y[i][variableIndex(0,0)] < 0.5)
    return;

  for (size_t n = 0; n < compartment.numNeighbor(); n++) {
    size_t j = compartment.neighbor(n);

    //Only update if neighbor marked for this diffusion
    //If same type of compartments only update once (i>j)

    if (y[j][variableIndex(0,1)] > 0.5) {
      if (variableIndex(0,0) != variableIndex(0,1) || i > j) {
	//D_out
	double tmp = parameter(0) * y[i][species];
	dydt[i][species] -= tmp;
	dydt[j][species] += tmp;
	//D_in
	tmp = parameter(1) * y[j][species];
	dydt[i][species] += tmp;
	dydt[j][species] -= tmp;
      }  
    }
  }
}

/*
	Warning!! g() = 0 is valid in an analysis done with pen and paper, but the
	way the simulator adds factors together produces nan!.
*/

GeneralPolarizationTransportFast::GeneralPolarizationTransportFast(
   std::vector<double> &paraValue, 
	 std::vector< std::vector<size_t> > &indValue)
{
	//Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if (paraValue.size() != 5) {
    std::cerr << "GeneralPolarizationTransportFast::" 
							<< "GeneralPolarizationTransportFast() "
							<< "uses five parameters (T, k_f, n_f, k_g and n_g).\n";
    exit(EXIT_FAILURE);
  }
	
  if (indValue.size() != 2 || indValue[0].size() != 2 
      || indValue[1].size() != 1) {
    std::cerr << "GeneralPolarizationTransport::"
							<< "GeneralPolarizationTransport() "
							<< "Polarized molecule + polarization signal indeces needed in "
							<< "level 1.\n" 
							<< "Variable index for size parameter needed at second level.\n";
    exit(EXIT_FAILURE);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("generalPolarizationTransportFast");
  setParameter(paraValue);  
  setVariableIndex(indValue);

  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize(numParameter());
	tmp[0] = "T";
  tmp[1] = "k_f";
  tmp[2] = "n_f";
	tmp[3] = "k_g";
	tmp[4] = "n_g";	
  setParameterId( tmp );
}

void GeneralPolarizationTransportFast::derivs(
	 Compartment &compartment, 
	 size_t species,
	 std::vector< std::vector<double> > &y,
	 std::vector< std::vector<double> > &dydt)
{
	double k_f = parameter(1);
	double n_f = parameter(2);
	double k_g = parameter(3);
	double n_g = parameter(4);
	double a_j;

  size_t i = compartment.index();

  //Polarization coefficient normalization constant
  double sum = 0.0;
  for (size_t n = 0; n < compartment.numNeighbor(); n++) {
		double a_j = y[compartment.neighbor(n)][variableIndex(0,1)];
    sum += f(a_j, k_f, n_f) / g(a_j, k_g, n_g);
	}
  sum += 1;

  for (size_t n = 0; n < compartment.numNeighbor(); n++) {
    size_t j = compartment.neighbor(n);
    a_j = y[j][variableIndex(0,1)];
 
    double coeff = 0.0;
		coeff = y[i][species] * y[i][variableIndex(0,0)];
		coeff *= f(a_j, k_f, n_f) / g(a_j, k_g, n_g);

    if (sum != 0.0)
			coeff /= sum;
		else {
			std::cerr << "GeneralPolarizationTransportFast::derivs " 
								<< "sum == 0.0.\n";
			exit(EXIT_FAILURE);
		}
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

double GeneralPolarizationTransportFast::f(double a, double k, double n)
{
	return k * std::pow(a, n);
}

double GeneralPolarizationTransportFast::g(double a, double k, double n)
{
	return k;
}


