//
// Filename     : transport.h
// Description  : Classes describing transports of molecules
// Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
// Created      : October 2003
// Revision     : $Id: transport.h 203 2007-03-20 10:50:07Z henrik $
//
#ifndef TRANSPORT_H
#define TRANSPORT_H

#include<cmath>

#include"baseReaction.h"

///
/// @brief The class Diffusion is a diffusion update including topological properties
///
/// Generates a diffusion update where the topological properties such as
/// volumes are taken into account according to:
///
/// @f[ \frac{dy_{ij}}{dt} = \frac{A_n}{d_n} D (y_{nj}-y_{ij})/V_i @f]
///
/// and 
///
/// @f[ \frac{dy_{nj}}{dt} = \frac{A_n}{d_n} D (y_{nj}-y_{ij})/V_n. @f]
///
/// where \e j is the molecular index. The update only applies if the neighbor
/// index \e n is larger than \e i. The update needs the size index as a
/// variable index (V_i,V_n). If applicable the A_n and d_n is the cross
/// section area and distance between the compartments.
///
class Diffusion : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which sets the parameters and variable
	/// indices that defines the reaction.
	///
	/// @param paraValue vector with parameters
	///
	/// @param indValue vector of vectors with variable indices
	///
	/// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
	///
  Diffusion(std::vector<double> &paraValue, 
	    std::vector< std::vector<size_t> > &indValue );
  
	///
	/// @brief Derivative function for this reaction class
	///
	/// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
	///
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};


///
/// @brief The class DiffusionSimple represents the simplest diffusion update
/// NOT including topological properties
///
/// Generates a diffusion update where NO topological properties such as
/// volumes are taken into account according to:
///
/// @f[ \frac{dy_{ij}}{dt} = D (y_{nj}-y_{ij}) @f]
///
/// and 
///
/// @f[ \frac{dy_{nj}}{dt} = D (y_{nj}-y_{ij}) @f]
///
/// where \e j is the molecular index. The update only applies if the neighbor
/// index \e n is larger than \e i.
///
class DiffusionSimple : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which sets the parameters and variable
	/// indices that defines the reaction.
	///
	/// @param paraValue vector with parameters
	///
	/// @param indValue vector of vectors with variable indices
	///
	/// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
	///
  DiffusionSimple(std::vector<double> &paraValue, 
		  std::vector< std::vector<size_t> > &indValue );
  
	///
	/// @brief Derivative function for this reaction class
	///
	/// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
	///
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

// The class DiffusionSimpleSpatial is the simplest diffusion update
/* Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated. The
  parameter is D in dc/dt = D*(c_n-c) (and dc_n/dt = -D*(c_n-c)). The
  update only applys if the neighbor index is larger.
*/
class DiffusionSimpleSpatial : public BaseReaction {
  
 public:
  
  DiffusionSimpleSpatial(std::vector<double> &paraValue, 
		  std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief The class DiffusionRestricted is diffusion between specific compartments
///
/// A restricted diffusion (btw allowed compartm). This is the same as
/// diffusion but is restricted to only apply between restricted compartments
/// (e.g. cells and walls, or cells and cells) marked in variableIndex (0,0)
/// and (0,1).
///
class DiffusionRestricted : public BaseReaction {
  
 public:
  
  DiffusionRestricted(std::vector<double> &paraValue, 
		      std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief The class polarizationTransport moves molecules across cell walls activly
///
/// The parameter is the diffusion constant \e D in 
///
/// @f[ dc/dt = -D*V_m(cc)/V @f]
///
/// and 
///
/// @f[ dc_n/dt = D*V_m(cc)/V_n. @f] 
///
/// V_m is the average volume. cc is actually T*P*c where T is the efflux
/// carrier concentration P is the polarization coefficient (based on
/// neighbors concentration of "pol species", and c is the concentration of
/// the transported species. The update will be mirrored by a similar from j
/// to i. The update needs the efflux carrier and polarization signal as
/// indeces in the first level and the size index as a variable index in the
/// second level.
///
class PolarizationTransport : public BaseReaction {
  
 public:
  
  PolarizationTransport(std::vector<double> &paraValue, 
			std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

/// 
/// @brief The class polarizationTransportFast moves molecules activly
///
/// The Polarized transporter molecule P is divided into the cell cytosol
/// (P_i) and its membranes (P_ij) following the dynamics
///
/// @f[ dP_i/dt = -a P_i \sum_k A_k + b \sum_k P_{ik} @f]
/// @f[ dP_{ij}/dt = a P_i A_j - b P_{ij} @f]
///
/// where \e k is summed over neighboring cells, \e A is the polarizing signal
/// and \e a, \e b are constants. The polarization is assumed to be fast and
/// the fixed point values are assumed
///  
/// @f[ P_i = (b/a)*P / (\sum_k A_k + b/a) @f]
/// @f[ P_ij = P * A_j / (\sum_k A_k + (b/a)) @f]
///  
/// where \e P is the total amount of polarized molecule in a cell.
///  
/// The polarized transport of a molecule X between two neighboring cells is
/// then given by
///  
/// @f[ dX_i/dt = T ( X_i P_{ij} - X_j P_ji ). @f]
///  
/// The parameters given to this function are T and K=b/a.  The volumes for
/// the cells are also included for correct concentration update.
///
class PolarizationTransportFast : public BaseReaction {
  
 public:
  
  PolarizationTransportFast(std::vector<double> &paraValue, 
			    std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief The reaction moves molecules actively. 
///
/// The Polarized transporter molecule @f$P@f$ is divided into the cell
/// cytosol (@f$P_i@f$) and its membranes (@f$P_{ij}@f$) following the dynamics
/// @f[
/// \frac{dP_i}{dt} = - \sum_j f(A_j)\cdot P_i + \sum_j g(A_j)\cdot P_{ij} 
/// @f]
/// @f[
/// \frac{dP_{ij}}{dt} = f(A_j)\cdot P_i - f(A_j)\cdot P_{ij}
/// @f]
///
/// where @f$j@f$ is summed over neighboring cells, @f$A@f$ is the
/// polarizing signal and @f$f(A)@f$ and @f$g(A)@f$ are arbitrary
/// functions. The polarization is assumed to be fast and the fixed
/// point values are assumed to be
/// @f[ 
/// P_{ij}^\ast =
/// \frac{f(A_j)}{g(A_j)}\frac{P}{1+\sum_k
/// \frac{f(A_k)}{g(A_k)}}
/// @f]
/// where @f$k@f$ is summed over all neighboring cells and @f$P@f$ is
/// the total amount of polarized molecule in a cell. The polarized
/// transport of a molecule X between two neighboring cells is then
/// given by
/// @f[ \frac{dX_i}{dt} = - T (X_i\cdot P_{ij} -
/// X_j\cdot P_{ji}).
/// @f]
/// 
/// The parameters given to this function are @f$T@f$, @f$k_f@f$,
/// @f$n_f@f$, @f$k_g@f$ and @f$n_g@f$. First level variables are
/// @f$P@f$ and @f$X@f$.
/// 
class GeneralPolarizationTransportFast : public BaseReaction 
{
 public:
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which sets the parameters and variable
	/// indices that defines the reaction.
	///
	/// @param paraValue vector with parameters
	///
	/// @param indValue vector of vectors with variable indices
	///
	/// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  GeneralPolarizationTransportFast(std::vector<double> &paraValue, 
			    std::vector< std::vector<size_t> > &indValue );

  ///
	/// @brief Derivative function for this reaction class
	///
	/// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
	///
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
	///
	/// @brief Hard coded general function f(A).
	///
	double f(double a, double k, double n);
	///
	/// @brief Hard coded general function g(A).
	///
	double g(double a, double k, double n);
};

//!Moves molecules activly with a Michaelis Menten type of equation
class PolarizationTransportFastMM : public BaseReaction {
  
 public:
  
  PolarizationTransportFastMM(std::vector<double> &paraValue, 
			      std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!Moves molecules activly symmetric with a Michaelis Menten equation
class SymmetricTransportFastMM : public BaseReaction {
  
 public:
  
  SymmetricTransportFastMM(std::vector<double> &paraValue, 
			   std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!Moves molecules activly with a Michaelis Menten type of equation
class PolarizationTransportInfluxFastMM : public BaseReaction {
  
 public:
  
  PolarizationTransportInfluxFastMM(std::vector<double> &paraValue, 
				    std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!Moves molecules activly with a Michaelis Menten type of equation
class PolarizationHillTransportInfluxFastMM : public BaseReaction {
  
 public:
  
  PolarizationHillTransportInfluxFastMM(std::vector<double> &paraValue, 
					std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!Moves molecules activly symmetric with a Michaelis Menten equation
class SymmetricTransportInfluxFastMM : public BaseReaction {
  
 public:
  
  SymmetricTransportInfluxFastMM(std::vector<double> &paraValue, 
				    std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!Moves molecules activly with a Michaelis Menten type of equation
class PolarizationTransportFastMMSpatial : public BaseReaction {
  
 public:
  
  PolarizationTransportFastMMSpatial(std::vector<double> &paraValue, 
				     std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!The class polarizationHillTransportFast moves molecules activly
class PolarizationHillTransportFast : public BaseReaction {
  
 public:
  
  PolarizationHillTransportFast(std::vector<double> &paraValue, 
				std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!The class polarizationHillTransportFastMM moves molecules activly with MM
class PolarizationHillTransportFastMM : public BaseReaction {
  
 public:
  
  PolarizationHillTransportFastMM(std::vector<double> &paraValue, 
				std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!Moves molecules activly with MM, and exponential polarization feedback
class PolarizationExpTransportFastMM : public BaseReaction {
  
 public:
  
  PolarizationExpTransportFastMM(std::vector<double> &paraValue, 
				 std::vector< std::vector<size_t> > 
				 &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief This class moves molecules activly with a const+lin polarization feedback
///
/// The Polarized transporter molecule P is divided into the cell cytosol
/// (@f$P_i@f$) and its membranes (@f$P_ij@f$) following the dynamics
///
/// @f[ \frac{dP_i}{dt} = -a P_i \sum_k (c+A_k) + b \sum_k P_{ik} @f]
/// @f[ \frac{dP_{ij}}{dt} = a P_i (c+A_j) - b P_{ij} @f]
///
/// where k is summed over neighboring cells, A is the polarizing signal and
/// a,b,c are constants. The polarization is assumed to be fast and the fixed
/// point values are assumed
///
/// @f[ P_i = (b/a)*P / (\sum_k A_k + (b/a)) @f]
/// @f[ P_{ij} = \frac{P (c+A_j)}{(\sum_k (c+A_k) + (b/a))} @f]
///
/// where P is the total amount of polarized molecule in a cell.
///
/// The polarized transport of a molecule X between two neighboring cells is
/// then given by
///
/// @f[ \frac{dX_i}{dt} = T ( P_{ij}*X_i - P_{ji}*X_j ). @f]
///
/// The parameters given to this function are T and K'=b/a, and c.  a_ij is
/// the crossing area. The volumes for the cells are also included for correct
/// concentration update.
///
class PolarizationConstLinTransportFast : public BaseReaction {
  
 public:
  
  PolarizationConstLinTransportFast(std::vector<double> &paraValue, 
				    std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!The class polarizationTransportFastB moves molecules activly
class PolarizationTransportFastB : public BaseReaction {
  
 public:
  
  PolarizationTransportFastB(std::vector<double> &paraValue, 
			     std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!The class polarizationTransportFastBMM moves molecules activly
class PolarizationTransportFastBMM : public BaseReaction {
  
 public:
  
  PolarizationTransportFastBMM(std::vector<double> &paraValue, 
			       std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!Moves molecules activly with a built in switch behavior
class PolarizationTransportFastSwitchedMM : public BaseReaction {
  
 public:
  
  PolarizationTransportFastSwitchedMM(std::vector<double> &paraValue, 
				      std::vector< std::vector<size_t> > 
				      &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief Moves molecules activly with a MM type of equation and polarized
///
/// The Polarized transporter molecule P is divided into the cell cytosol
/// (@f$P_i@f$) and its membranes (@f$P_ij@f$) following the dynamics
///
/// @f[ \frac{dP_i}{dt} = -a P_i \sum_k (A_k) + b \sum_k P_{ik} @f]
/// @f[ \frac{dP_{ij}}{dt} = a P_i (A_j) - b P_{ij} @f]
///
/// where k is summed over neighboring cells (which are assumed to be in
/// neighbor level 1), A is the polarizing signal and a,b are constants. The
/// polarization is assumed to be fast and the fixed point values are assumed
///
/// @f[ P_i = \frac{(b/a)P}{(\sum_k a_k*A_k + V_i*(b/a))} @f]
/// @f[ P_{ij} = \frac{P (A_j)}{(\sum_k a_k*A_k + V_i*(b/a))} @f]
///
/// where P is the total amount of polarized molecule in a cell (P_i*V_i), and
/// a_k is the membrane area (if those are given, otherwise a_k=1).
///
/// The polarized transport of a molecule X between a cell (i) and neighboring
/// walls (ij) is then given by
///
/// @f[ dX_i/dt = 1/V_i sum_j a_ij ( T_inf P_ij*X_ij/(K+X_ij) 
/// - T_eff P_ij*X_i/(K+X_i) ). @f]
/// @f[ dX_ij/dt = 1/V_ij sum_j a_ij ( -T_inf P_ij*X_ij/(K+X_ij) 
/// + T_eff P_ij*X_i/(K+X_i) ). @f]
///
/// The parameters given to this function are T_eff, T_inf and K'=b/a. a_ij is
/// the crossing area. The volumes for the cells are also included for correct
/// concentration update. The derivative is updated from cellular
/// compartments.
///
class PolarizationLinFastTransportRestrictedMM : public BaseReaction {
  
 public:
  
  PolarizationLinFastTransportRestrictedMM(std::vector<double> 
					   &paraValue, 
					   std::vector< std::vector<size_t> > 
					   &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief Moves molecules activly with a MM type of equation and polarized
///
/// The Polarized transporter molecule P is divided into the cell cytosol
/// (P_i) and its membranes (P_ij) following the dynamics
///  
/// @f[ dP_i/dt = -a P_i \sum_k f(A_k) + b \sum_k P_{ik} @f]
/// @f[ dP_{ij}/dt = a P_i f(A_j) - b P_{ij} @f] 
///
/// where k is summed over neighboring cells (which are assumed to be in
/// neighboe level 1), f(A_j)=((1-c)+cA_j^n/(Kh^n+a_j^n)) is the polarizing
/// signal and a,b are constants. The polarization is assumed to be fast and
/// the fixed point values are assumed
///  
/// @f[ P_i = (b/a)*P / (\sum_k a_k*f(A_k) + V_i*(b/a)) @f]
/// @f[ P_{ij} = P * f(A_j) / (\sum_k a_k*f(A_k) + V_i*(b/a)) @f]
///
/// where P is the total amount of polarized molecule in a cell (P_i*V_i), and
/// a_k is the membrane area (if those are given, otherwise a_k=1).
///  
/// The polarized transport of a molecule X between a cell (i) and neighboring
/// walls (ij) is then given by
///  
/// @f[ dX_i/dt = 1/V_i sum_j a_ij ( T_inf P_ij*X_ij/(K+X_ij) 
/// - T_eff P_ij*X_i/(K+X_i) ). @f]
/// @f[ dX_ij/dt = 1/V_ij sum_j a_ij ( -T_inf P_ij*X_ij/(K+X_ij) 
/// + T_eff P_ij*X_i/(K+X_i) ). @f]
///
/// The parameters given to this function are T_eff, T_inf and K'=b/a, c, Kh,
/// n, and K.  a_ij is the crossing area. The volumes for the cells are also
/// included for correct concentration update. The derivative is updated from
/// cellular compartments.
///
class PolarizationConstHillFastTransportRestrictedMM : 
public BaseReaction {
  
 public:
  
  PolarizationConstHillFastTransportRestrictedMM(std::vector<double> 
						 &paraValue, 
						 std::vector< 
						 std::vector<size_t> > 
						 &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief Moves molecules activly with a MM type of equation and unpolarized
///
/// The transporter molecule P is divided into the cell cytosol (P_i) and its
/// membranes (P_ij) following the dynamics
///
/// @f[ \frac{dP_i}{dt} = -a P_i \sum_k 1 + b \sum_k P_{ik} @f]
/// @f[ \frac{dP_{ij}}{dt} = a P_i - b P_{ij} @f]
///
/// where k is summed over neighboring cells (which are assumed to be in
/// neighbor level 1), a,b are constants. The membranization is assumed to be
/// fast and the fixed point values are assumed
///
/// @f[ P_i = \frac{(b/a)*P}{(\sum_k a_k*1 + V_i*(b/a))} @f]
/// @f[ P_ij = \frac{P * 1}{(\sum_k a_k*1 + V_i*(b/a))} @f]
///
/// where P is the total amount of transport molecule in a cell (P_i*V_i), and
/// a_k is the membrane area (if those are given, otherwise a_k=1).
///
/// The polarized transport of a molecule X between a cell (i) and neighboring
/// walls (ij) is then given by
///
/// @f[ dX_i/dt = 1/V_i sum_j a_{ij} ( T_inf P_{ij}*X_{ij}/(K+X_{ij}) 
/// - T_{eff} P_{ij}*X_i/(K+X_i) ) . @f]
/// @f[dX_{ij}/dt = 1/V_{ij} sum_j a_{ij} ( -T_inf P_{ij}*X_ij/(K+X_{ij}) 
/// + T_{eff} P_{ij}*X_i/(K+X_i) ). @f]
///
/// The parameters given to this function are T_eff, T_inf and K'=b/a, and K.
/// a_ij is the crossing area. The volumes for the cells are also included for
/// correct concentration update. The derivative is updated from cellular
/// compartments.
///
class PolarizationConstFastTransportRestrictedMM : public BaseReaction {
  
 public:
  
  PolarizationConstFastTransportRestrictedMM(std::vector<double> 
																						 &paraValue, 
																						 std::vector< 
																						 std::vector<size_t> > 
																						 &indValue );
  
  void derivs(Compartment &compartment,size_t species,
							std::vector< std::vector<double> > &y,
							std::vector< std::vector<double> > &dydt );
};

///
/// @brief Moves molecules activly with a MM type of equation and unpolarized
///
/// The transporter molecule P is divided into the cell cytosol (P_i) and its
/// membranes (P_ij) following the dynamics
///
/// @f[ \frac{dP_i}{dt} = -a P_i \sum_k 1 + b \sum_k P_{ik} @f]
/// @f[ \frac{dP_{ij}}{dt} = a P_i - b P_{ij} @f] 
///
/// where k is summed over neighboring cells (which are assumed to be in
/// neighboe level 1), a,b are constants. The membranization is assumed to be
/// fast and the fixed point values are assumed
///
/// @f[ P_i = \frac{(b/a)*P}{(\sum_k a_k*1 + V_i*(b/a))} @f]
/// @f[ P_{ij} = \frac{P * 1}{(\sum_k a_k*1 + V_i*(b/a))} @f]
///
/// where P is the total amount of transport molecule in a cell (P_i*V_i), and
/// a_k is the membrane area (if those are given, otherwise a_k=1).
///
/// The polarized transport of a molecule X between a cell (i) and neighboring
/// walls (ij) is then given by
///
/// @f[ dX_i/dt = 1/V_i sum_j a_ij ( T_inf P_ij*f_ij*X_ij/(K+f_ij*X_ij) 
/// - T_eff P_ij*f_i*X_i/(K+f_i*X_i) ). @f]
/// @f[ dX_ij/dt = 1/V_ij sum_j a_ij ( -T_inf P_ij*f_ij*X_ij/(K+f_ij*X_ij) 
/// + T_eff P_ij*f_i*X_i/(K+f_i*X_i) ). @f]
///
/// The parameters given to this function are T_eff, T_inf and K'=b/a, , K,
/// f_i, and f_ij.  a_ij is the crossing area. The volumes for the cells are
/// also included for correct concentration update. The derivative is updated
/// from cellular compartments.
///
class PolarizationConstFastTransportFractionRestrictedMM : 
public BaseReaction {
  
 public:
  
  PolarizationConstFastTransportFractionRestrictedMM(std::vector<double> 
						     &paraValue, 
						     std::vector< 
						     std::vector<size_t> > 
						     &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief Moves molecules activly with a MM type of equation and polarized
///
/// The transporter molecule P is divided into the cell cytosol (P_i) and its
/// membranes (P_ij) following the dynamics
///
/// @f[ dP_i/dt = -a P_i \sum_k f(A_k) + b \sum_k P_{ik} @f]
/// @f[ dP_{ij}/dt = a P_i f(A_j) - b P_{ij} @f] 
///
/// where k is summed over neighboring cells (which are assumed to be in
/// neighboe level 1), f(A_j)=((1-c)+cA_j^n/(Kh^n+a_j^n)) is the polarizing
/// signal and a,b are constants. The polarization is assumed to be fast and
/// the fixed point values are assumed
///  
/// @f[ P_i = (b/a)*P / (\sum_k a_k*f(A_k) + V_i*(b/a)) @f]
/// @f[ P_{ij} = P * f(A_j) / (\sum_k a_k*f(A_k) + V_i*(b/a)) @f]
///
/// where P is the total amount of polarized molecule in a cell (P_i*V_i), and
/// a_k is the membrane area (if those are given, otherwise a_k=1).
///  
/// The polarized transport of a molecule X between a cell (i) and neighboring
/// walls (ij) is then given by
///
/// @f[ dX_i/dt = 1/V_i sum_j a_ij ( T_inf P_ij*f_ij*X_ij/(K+f_ij*X_ij) 
/// - T_eff P_ij*f_i*X_i/(K+f_i*X_i) ). @f]
/// @f[ dX_ij/dt = 1/V_ij sum_j a_ij ( -T_inf P_ij*f_ij*X_ij/(K+f_ij*X_ij) 
/// + T_eff P_ij*f_i*X_i/(K+f_i*X_i) ). @f]
///
/// The parameters given to this function are T_eff, T_inf and K'=b/a, , K,
/// f_i, and f_ij.  a_ij is the crossing area. The volumes for the cells are
/// also included for correct concentration update. The derivative is updated
/// from cellular compartments.
///
class PolarizationConstHillFastTransportFractionRestrictedMM : 
	public BaseReaction 
{
public:
  
  PolarizationConstHillFastTransportFractionRestrictedMM(std::vector<double> 
																												 &paraValue, 
																												 std::vector< 
																												 std::vector<size_t> > 
																												 &indValue );
  
  void derivs(Compartment &compartment,size_t species,
							std::vector< std::vector<double> > &y,
							std::vector< std::vector<double> > &dydt );
};

///
/// @brief The class directionalTransport moves molecules in a specific direction
///
/// A strength constant, and a direction variable index is needed. If the
/// constant is less than zero the transport is in the negative direction. The
/// transport is proportional with the molecular content, and normalized as
/// direction length divided by total length between the two cells.
///
/// The parameter is T_strength in 
///
/// @f[ dc/dt = -|T_strength|*c*|x-x_n|/|r-r_n|*V_m/V*\Theta @f]
///
/// and
///
/// @f[ dc_n/dt = -dc/dt @f] 
///
/// The absolute value of the parameter concerns the reversed direction if the
/// parameter is negative. V_m is the average volume. x is the direction
/// given, r is the cell position, and c is the concentration of the
/// transported species. The Theta result in an application only if transport
/// out from this cell. The update will be mirrored by a similar from j to i
/// (if that is the direction instead). The update needs the directional
/// variable as index in the first level and the size index as a variable
/// index in the second level.
///
class DirectionalTransport : public BaseReaction {
  
 public:
  
  DirectionalTransport(std::vector<double> &paraValue, 
		       std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief The class directionalTransport moves molecules in a specific direction
///
/// A strength constant, and a direction variable index is needed. If the
/// constant is less than zero the transport is in the negative direction. The
/// transport is proportional with the molecular content, and normalized as
/// direction length divided by total length between the two cells. The
/// transport equation is in the Michaelis Menten formalism (which is the only
/// thing that differs from the DirectionalTransport class).
///
/// The parameter is T_strength in 
///
/// @f[ dc/dt = -|T_strength|*c/(c+K_M)*|x-x_n|/|r-r_n|/V * \Theta @f]
/// 
/// and 
/// 
/// @f[ dc_n/dt = -dc/dt @f]. 
/// 
/// The absolute value of the parameter concerns the reversed direction if the
/// parameter is negative. V_m is the average volume. x is the direction
/// given, r is the cell position, and c is the concentration of the
/// transported species. The Theta result in an application only if transport
/// out from this cell. The update will be mirrored by a similar from j to i
/// (if that is the direction instead). The update needs the directional
/// variable as index in the first level and the size index as a variable
/// index in the second level.
///
class DirectionalTransportMM : public BaseReaction {
  
 public:
  
  DirectionalTransportMM(std::vector<double> &paraValue, 
			 std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief The class directionalTransportSwitchedMM moves molecules in a
/// specific direction
///
/// A strength constant, and a direction variable index is needed. If the
/// constant is less than zero the transport is in the negative direction. The
/// transport is proportional with the molecular content, and normalized as
/// direction length divided by total length between the two cells. The
/// transport equation is in the Michaelis Menten formalism. A molecule switch
/// is implemented to chnge the direction in some cells. (which is the only
/// thing that differs from the DirectionalTransportMM class).
///
/// The parameter is T_strength in 
/// 
/// @f[ dc/dt = -|T_strength|*c/(c+K_M)*|x-x_n|/|r-r_n|/V * \Theta @f]
///
/// and
/// 
/// @f[ dc_n/dt = -dc/dt. @f]
/// 
/// The absolute value of the parameter concerns the reversed direction if the
/// parameter is negative. The switching is checking for a molecule
/// concentration and reverses the direction if the concentration is above a
/// threshold. V_m is the average volume. x is the direction given, r is the
/// cell position, and c is the concentration of the transported species. The
/// Theta result in an addition only if transport out from this cell. The
/// update will be mirrored by a similar from j to i (if that is the direction
/// instead). The update needs the directional variable as index in the first
/// level and the size index as a variable index in the second level.
///
class DirectionalTransportSwitchedMM : public BaseReaction {
  
 public:
  
  DirectionalTransportSwitchedMM(std::vector<double> &paraValue, 
				 std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief The class MitchisonTransport moves molecules across cell walls activly
///
/// This update uses Mitchinsons flux model, where intracellular diffusion has
/// been removed. It requires parameters for each neighbor which need to be
/// updated seperatly {p_ij}.
///
class MitchisonTransport : public BaseReaction {
  
 public:
  
  MitchisonTransport(std::vector<double> &paraValue, 
		     std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief The class ActiveTransportRestricted moves molecules between cells and walls activly
///
/// Uses T_out*A_ij*X_i for out and T_in*A_ij*X_ij for transport between and
/// compartment and neighbor. (out means comp to neigh, in neigh to comp). X
/// is the transported molecule, A is a helper molecule (in neighbor) acting
/// as in/efflux carrier.
///
class ActiveTransportRestricted : public BaseReaction {
  
 public:
  
  ActiveTransportRestricted(std::vector<double> &paraValue, 
			    std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief The class ActiveTransportMMRestricted moves molecules between cells
/// and walls activly.
///
/// Uses 
/// 
/// @f[ Tmax_out*A_{ij}/(k_out+A_ij)*X_i @f]
///
/// for out and
///
/// @f[ Tmax_in*A_{ij}/(k_in+A_{ij})*X_{ij} @f]
///
/// for transport between a comp and neighbor. (out means comp to neighbor, in
/// is neighbor to compartment). X is the transported molecule, A is a helper
/// acting as in/efflux carrier.
///
class ActiveTransportMMRestricted : public BaseReaction {
  
 public:
  
    ActiveTransportMMRestricted(std::vector<double> &paraValue, 
				std::vector< std::vector<size_t> > &indValue );
    
    void derivs(Compartment &compartment,size_t species,
		std::vector< std::vector<double> > &y,
		std::vector< std::vector<double> > &dydt );
};

//The class CellWallTransportB moves molecules between cells and walls activly
/* Same as ActiveTransportRestricted but uses specific neighbours,
  e.g. the neighboring cell (not membrane) for "catalyzing" the
  transport.*/
class ActiveTransportBRestricted : public BaseReaction {
  
 public:
  
    ActiveTransportBRestricted(std::vector<double> &paraValue, 
			       std::vector< std::vector<size_t> > &indValue );
    
    void derivs(Compartment &compartment,size_t species,
		std::vector< std::vector<double> > &y,
		std::vector< std::vector<double> > &dydt );
};

//!The class membraneTransport is the simplest membrane transport update
class MembraneTransport : public BaseReaction {
  
 public:
  
  MembraneTransport(std::vector<double> &paraValue, 
		    std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!The class MembraneTransportRestricted is between specific compartments
class MembraneTransportRestricted : public BaseReaction {
  
 public:
  
  MembraneTransportRestricted(std::vector<double> &paraValue, 
			      std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!The class ActiveMembraneTransportRestricted is active membrane transport
class ActiveMembraneTransportRestricted : public BaseReaction {
  
 public:
  
  ActiveMembraneTransportRestricted(std::vector<double> &paraValue, 
				    std::vector< std::vector<size_t> > 
				    &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!The class ActiveMembraneTransportRestrictedMM is active membrane transport
class ActiveMembraneTransportRestrictedMM : public BaseReaction {
  
 public:
  
  ActiveMembraneTransportRestrictedMM(std::vector<double> &paraValue, 
				    std::vector< std::vector<size_t> > 
				    &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!The class CellWallTransportB moves molecules between cells and walls activly
class ActiveMembraneTransportBRestricted : public BaseReaction {
  
 public:
  
  ActiveMembraneTransportBRestricted(std::vector<double> &paraValue, 
				     std::vector< std::vector<size_t> > 
				     &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!The class CellWallTransportB moves molecules between cells and walls activly
class ActiveMembraneTransportBHillRestricted : public BaseReaction {
  
 public:
  
  ActiveMembraneTransportBHillRestricted(std::vector<double> &paraValue, 
					 std::vector< std::vector<size_t> > 
					 &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};


//! The class polarizationTransportFastGeneralFGRestricted
class PolarizationTransportFastGeneralFGRestricted : public BaseReaction
{
 public:
  PolarizationTransportFastGeneralFGRestricted(
    std::vector<double> &paraValue, 
    std::vector< std::vector<size_t> > &indValue);
  
  void  derivs(Compartment &compartment, size_t species,
	       std::vector< std::vector<double> > &y,
	       std::vector< std::vector<double> > &dydt);
  double f(double arg);
  double g(double arg);			       
};

//!The class DiffusionSimpleRestricted is diffusion between specific compartments
class DiffusionSimpleRestricted : public BaseReaction {
  
 public:
  
  DiffusionSimpleRestricted(std::vector<double> &paraValue, 
		      std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

#endif
