#include "../optimizers/baseOptimizer.h"
#include "../cost/baseEstimator.h"
#include "../solvers/baseSolver.h"
#include "../common/myConfig.h"
#include "../common/mySignal.h"
#include "../common/myRandom.h"

int main(int argc,char *argv[]) {
  
  myConfig::registerOption("verbose", 1);
  myConfig::registerOption("debug_output", 1);
  
  if((argc !=5) && (argc != 2) && (argc != 4)) {
    std::cerr << "possible usages: 1: " << argv[0] << " modelFile " 
	      << "simulatorParaFile analyzerFile optimizerFile\n";
    std::cerr << "                 2: " << argv[0] 
	      << " (model + solver + analyzer + optimizer)File "
	      << "[i.e. everything consecutively in the same file}\n";
    std::cerr << "  (old usage )   3: " << argv[0] << " modelFile " 
	      << "simulatorParaFile (analyzer+optimizer)File\n";
    exit(-1);
  }
  
  if(argc == 5){
    char* modelFile=argv[1];
    std::string simPara=argv[2];
    std::string analyzerFile = argv[3];
    std::string optimizerFile = argv[4];
    
    myRandom::Randomize();
    myTimes::getTime();
    
    //
    // Define the organism (model), simulator and the optimizer
    //	
    Organism O(modelFile);
    std::cerr << "Organism created from model " << modelFile << "." << std::endl;
    
    BaseSolver *S = BaseSolver::getSolver(&O, simPara);
    std::cerr << "Solver created from file " << simPara << "." << std::endl;
    
    O.initiateParameter();
    
    BaseOptimizationProblem *E = new BaseEstimator(S, analyzerFile);
    
    BaseOptimizer *Opt = BaseOptimizer::createOptimizer(E, optimizerFile);
    //Add optimizer to signal handler.
    mySignal::addOptimizer(Opt);
    
    //Start optimizing  
    Opt->optimize();
    
    //Delete simulator and optimizer
    delete S;
    delete E;
    delete Opt;
  }
  
  /*if(argc == 4){
    std::cerr << "!Obs! : The syntax of the optimizer part has changed, you should now put :\n     'IdOptimizer    TypeOfScaling    NbArgumentsForScaling    ArgumentsForScaling    NbArgumentsForOptimizer ...'\n     instead of only 'IdOptimizer NumArgsForOptimizer ...' as before.\n     Put 'IdOpt 0 0 nbArgsForOpt ... to stay in non scaling mode as before.\n";   
    
    char* modelFile=argv[1];
    std::string simPara=argv[2];
    std::string estoptFile = argv[3];
    
    myRandom::Randomize();
    myTimes::getTime();
    
    //
    // Define the organism (model), simulator and the optimizer
    //	
    Organism O(modelFile);
    std::cerr << "Organism created from model " << modelFile << "." << std::endl;
    
    BaseSolver *S = BaseSolver::getSolver(&O, simPara);
    std::cerr << "Solver created from file " << simPara << "." << std::endl;
    
    O.initiateParameter();
    
    std::istream *IN = myFiles::openFile(estoptFile);
    if (!IN) {
    std::cerr << "./optimizer - main : "
    << "Cannot open (analyzer-optimizer)file " << estoptFile << std::endl;
    exit(-1);
    }
    
    BaseEstimator *E = new BaseEstimator(S, (std::ifstream &) *IN);
    
    BaseOptimizer *Opt = BaseOptimizer::createOptimizer(E, (std::ifstream&) *IN);
    //Add optimizer to signal handler.
    mySignal::addOptimizer(Opt);
    
    //Start optimizing  
    Opt->optimize();
    
    //Delete simulator and optimizer
    delete S;
    delete E;
    delete Opt;	  
    }*/
  
  if(argc == 2){
    char* totalFile=argv[1];
    
    std::istream *IN = myFiles::openFile(totalFile);
    if (!IN) {
      std::cerr << "./optimizer - main : "
		<< "Cannot open (analyzer-optimizer)file " << totalFile << std::endl;
      exit(-1);
    } 
    
    myRandom::Randomize();
    myTimes::getTime();
    
    Organism O((std::ifstream&) *IN);
    std::cerr << "Organism created from a first part of file " << totalFile << "." << std::endl;
    
    BaseSolver *S = BaseSolver::getSolver(&O, (std::ifstream&) *IN);
    std::cerr << "Solver created from a second part of file " << totalFile << "." << std::endl;
    
    O.initiateParameter();
    
    BaseEstimator *E = new BaseEstimator(S, (std::ifstream&) *IN);
    std::cerr << "Estimator created from a third part of file " << totalFile << "." << std::endl;
    
    
    BaseOptimizer *Opt = BaseOptimizer::createOptimizer(E, (std::ifstream&) *IN);
    std::cerr << "Estimator created from a fourth and last part of file " 
	      << totalFile << "." << std::endl;
    delete IN;
    //Add optimizer to signal handler.
    mySignal::addOptimizer(Opt);
    
    //Start optimizing  
    Opt->optimize();
    
    //Delete simulator and optimizer
    delete S;
    delete E;
    delete Opt;
  }
  
  return 0;
}
