//
// Filename     : adhocReaction.cc
// Description  : Classes describing some adhoc reactions
// Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
// Created      : October 2008
// Revision     : $Id:$
//

#include "baseReaction.h"
#include "adhocReaction.h"

SumCompartmentMass::SumCompartmentMass(std::vector<double> &paraValue, 
																			 std::vector< std::vector<size_t> > &indValue ) 
{
  //
  // Do some checks on the parameters and variable indeces
  //
  if( paraValue.size()!=1 ) {
    std::cerr << "SumCompartmentMass::SumCompartmentMass() one parameter used "
	      << "(converion factor)." << std::endl;
    exit(0);
  }
  int fail=0;
  size_t i=1;
  while( i<indValue.size() ) {
    if (indValue[i].size()!=2) {
      fail=1;
    }
    ++i;
  }
  if( indValue.size()<2 || indValue[0].size() != 3 || fail) {
    std::cerr << "SumCompartmentMass::SumCompartmentMass() "
	      << "First level of indices is (Volume concentration massSummed)"
	      << std::endl
	      << "Then for each additional level compartmentStart and compartmentEnd"
	      << " should be given." << std::endl;
    exit(0);
  }
  //
  // Set the variable values
  //
  setId("sumCompartmentMass");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  //
  // Set the parameter identities
  //
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "conversionFactor";
  setParameterId( tmp );
}

void SumCompartmentMass::
derivs(Compartment &compartment,size_t species,DataMatrix &y,DataMatrix &dydt) 
{
	//Do nothing (everything is done in the update).
}

void SumCompartmentMass::update(double h, double t,DataMatrix &y) 
{
  for (size_t k=1; k<numVariableIndexLevel(); ++k) {
    double sum=0.0;
    for (size_t c=variableIndex(k,0); c<=variableIndex(k,1); ++c)
      sum += y[c][variableIndex(0,0)]*y[c][variableIndex(0,1)];
    double val = parameter(0)*sum;
    for (size_t c=variableIndex(k,0); c<=variableIndex(k,1); ++c)
      y[c][variableIndex(0,2)] = val;
  }
}

DiffusionDivided::DiffusionDivided(std::vector<double> &paraValue, 
				   std::vector< std::vector<size_t> > &indValue ) 
{
  //
  // Do some checks on the parameters and variable indeces
  //
  if( paraValue.size()!=2 ) {
    std::cerr << "DiffusionDivided::DiffusionDivided() two parameters used "
	      << "diffusion rate and leakage neighbors" << std::endl;
    exit(0);
  }
  if( indValue.size() ) {
    std::cerr << "DiffusionDivided::DiffusionDivided() "
	      << "No indices used in this reaction." << std::endl;
    exit(0);
  }
  //
  // Set the variable values
  //
  setId("diffusionDivided");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  //
  // Set the parameter identities
  //
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "D";
  tmp[1] = "leakageNumber";
  setParameterId( tmp );
}

void DiffusionDivided::
derivs(Compartment &compartment,size_t species,DataMatrix &y,DataMatrix &dydt) 
{
  size_t i=compartment.index();
  size_t N = compartment.numNeighbor();
  double rate = parameter(0)/(N+parameter(1));
  double diff = rate*y[i][species];
  // cell-cell diffusion part
  for( size_t n=0 ; n<N ; n++ ) {
    size_t j=compartment.neighbor(n);
    dydt[i][species] -= diff;
    dydt[j][species] += diff;
  }
  // cell-background diffusion part
  dydt[i][species] -= parameter(1)*diff;
}

Add::Add(std::vector<double> &paraValue, 
	 std::vector< std::vector<size_t> > &indValue ) 
{
  //
  // Do some checks on the parameters and variable indeces
  //
  if( paraValue.size()!=1 ) {
    std::cerr << "Add::Add() one parameters used "
	      << "Constant to add." << std::endl;
    exit(0);
  }
  if( indValue.size()!=2 || indValue[0].size()!=1 ) {
    std::cerr << "Add::Add() "
	      << "Two levels of variable indices are used, "
	      << "one where the addition is stored (single index)"
	      << ", and one for variables to add is used." << std::endl;
    exit(0);
  }
  //
  // Set the variable values
  //
  setId("add");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  //
  // Set the parameter identities
  //
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "const";
  setParameterId( tmp );
}

void Add::
derivs(Compartment &compartment,size_t species,DataMatrix &y,DataMatrix &dydt) 
{
  // Nothing to be done for the derivative function.
}

void Add::
update(double h, double t,DataMatrix &y) 
{
  size_t N = y.size();
  for (size_t i=0; i<N; i++ ) {
    double sum = parameter(0);
    for (size_t k=0; k<numVariableIndex(1); ++k)
      sum += y[i][variableIndex(1,k)];
    y[i][variableIndex(0,0)] = sum;
  }
}


SwitchCompartmentThreshold::SwitchCompartmentThreshold(std::vector<double> &paraValue, 
																											 std::vector< std::vector<size_t> > &indValue ) 
{
  //
  // Do some checks on the parameters and variable indeces
  //
  if( paraValue.size()!=2 ) {
    std::cerr << "SwitchCompartmentThreshold::SwitchCompartmentThreshold() two parameters used "
	      << "Threshold and marker for >threshold (1) or <threshold (-1)." << std::endl;
    exit(0);
  }
  if( indValue.size()!=2 || indValue[0].size()!=1 || indValue[1].size()!=2) {
    std::cerr << "SwitchCompartmentThreshold::SwitchCompartmentThreshold() "
							<< "Two levels of variable indices are used, "
							<< "one for the threshold variable (single index)"
							<< ", and one for variables to change (first set to zero second to one is used." 
							<< std::endl;
    exit(0);
  }
  //
  // Set the variable values
  //
  setId("add");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  //
  // Set the parameter identities
  //
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "const";
  setParameterId( tmp );
}

void SwitchCompartmentThreshold::
derivs(Compartment &compartment,size_t species,DataMatrix &y,DataMatrix &dydt) 
{
  // Nothing to be done for the derivative function.
}

void SwitchCompartmentThreshold::
update(double h, double t,DataMatrix &y) 
{
	for (size_t i=0; i<y.size(); ++i)
		if ( ( parameter(1)>=0 && y[i][variableIndex(0,0)]>parameter(0) ) || 
				 ( parameter(1)<0 && y[i][variableIndex(0,0)]<parameter(0) ) ) {
			y[i][variableIndex(1,0)]=0;
			y[i][variableIndex(1,1)]=1;
		}
}



