//
// Filename     : adhocReaction.h
// Description  : Classes describing some adhoc updates
// Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
// Created      : October 2008
// Revision     : $Id:$
//
#ifndef ADHOCREACTION_H
#define ADHOCREATION_H

#include "baseReaction.h"

///
/// @brief SumCompartmentMass sums up the mass in a specified set of compartments
///
/// 
/// This method sums up the mass in a specified set of compartments, to be
/// able to compare with coarse grained data. It saves the summed mass in all
/// compartments included in the set. It has no derivative contribution, but
/// does the summing in an update.
///
/// @f[ y_{ij} = p_{0} \sum_k V_{k} y_{kl} @f]
///
/// where the compartment i is included among the set of k $f[V_{k}$f] is the
/// volume and $f[y_{kl}$f] is the concentration (at index l) to be
/// summed. $f[p_{0}$f] is a conversion factor usually set to get correct
/// units.
///
///
class SumCompartmentMass : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which checks and sets the parameters and
  /// variable indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  /// In a model file the reaction is defined as:
  ///
  /// @verbatim
  /// SumCompartmentMass 1 K+1 N_k1 [N_k2] ...
  /// p_0
  /// V_index conc_index save_index
  /// compartmentStart_index compartmentEnd_index
  /// [compartmentStart_index compartmentEnd_index]
  /// ...
  /// @endverbatim
  ///
  SumCompartmentMass(std::vector<double> &paraValue, 
		     std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t specie,DataMatrix &y,DataMatrix &dydt);	
  
  ///
  /// @brief Update function for this reaction class
  ///
  /// @see BaseReaction::update(double h, double t, ...)
  ///
  void update(double h, double t, DataMatrix &y);
};

///
/// @brief Includes diffusion divided among neighbors + ino background
///
/// This method implements diffusion out of a compartment into its
/// neighbors and to the background. The diffusion is divided among
/// neighbors and background to estimate the crossing area. 
/// A caveat is that this approximated area does not have to 
/// be the same from one cell to another as from the other to 
/// the first.
///
/// @f[ \frac{dy_{ij}}{dt} = - \frac{p_{0}}{N+p_{1}} \sum_n y_{ij}  @f]
///
/// where the diffusion is for all compartments i for molecule j.
/// N is the number of neighbors and the sum is over the neighbors.
/// Each neighbor recieves the same amount that is diffused from this cell
/// while there is an additional leakage to the background set by p1.
///
class DiffusionDivided : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which checks and sets the parameters and
  /// variable indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  /// In a model file the reaction is defined as:
  ///
  /// @verbatim
  /// diffusionDivided 2 0
  /// p_0 p_1
  /// @endverbatim
  ///
  DiffusionDivided(std::vector<double> &paraValue, 
		   std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t specie,DataMatrix &y,DataMatrix &dydt);	
};

///
/// @brief Adds the variables in the list given plus a constant
///
/// Adds a list of variables and saves it in the specie value.
/// This is done in the update function and nothing is done in the 
/// derivs function.
/// In a model file the reaction is defined as:
///
/// @verbatim
/// add 1 1 N
/// const
/// X_1 ... X_N
/// @endverbatim
///
class Add : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which checks and sets the parameters and
  /// variable indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  Add(std::vector<double> &paraValue, 
      std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief This class does not use derivatives for updates.
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t specie,DataMatrix &y,DataMatrix &dydt);

  ///
  /// @brief Update function for this reaction class
  ///
  /// @see BaseReaction::update(double h, double t, ...)
  ///
  void update(double h, double t, DataMatrix &y);
};

///
/// @brief Turns off one and turns on compartment markers at thereshold
///
/// Adds a list of variables and saves it in the specie value.
/// This is done in the update function and nothing is done in the 
/// derivs function.
/// In a model file the reaction is defined as:
///
/// @verbatim
/// SwitchCompartmentThreshold 1 1 N
/// threshold negativeFlag
/// removeIndex addIndex
/// @endverbatim
///
class SwitchCompartmentThreshold : public BaseReaction {
  
 public:
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which checks and sets the parameters and
  /// variable indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
		SwitchCompartmentThreshold(std::vector<double> &paraValue, 
															 std::vector< std::vector<size_t> > &indValue );
		
  ///
  /// @brief This class does not use derivatives for updates.
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t specie,DataMatrix &y,DataMatrix &dydt);

  ///
  /// @brief Update function for this reaction class
  ///
  /// @see BaseReaction::update(double h, double t, ...)
  ///
		void update(double h, double t, DataMatrix &y);
};

#endif
