//
// Filename     : creation.cc
// Description  : Classes describing creation updates
// Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
//              : Pontus Melke
// Created      : December 2003
// Revision     : $Id: creation.cc 493 2011-06-17 11:02:50Z henrik $
//

#include<cmath>

#include"baseReaction.h"
#include"creation.h"

CreationLinear::CreationLinear(std::vector<double> &paraValue, 
			       std::vector< std::vector<size_t> > 
			       &indValue ) 
{  
	// Do some checks on the parameters and variable indeces
	if( paraValue.size()!=1 ) {
		std::cerr << "CreationLinear::CreationLinear() "
			  << "Uses only one parameter k_c\n";
		exit(0);
	}
	if( indValue.size() != 0 && indValue.size() != 1 ) {
		std::cerr << "CreationLinear::CreationLinear() "
			  << "At most one level of variables are allowed.\n";
		exit(0);
	}
	
	// Set the variable values
	setId("creationLinear");
	setParameter(paraValue);  
	setVariableIndex(indValue);
  
	// Set the parameter identities
	std::vector<std::string> tmp( numParameter() );
	tmp.resize( numParameter() );
	tmp[0] = "k_c";
	setParameterId( tmp );
}

void CreationLinear::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
	double rate = parameter(0);
	if( numVariableIndexLevel() )
	  for( size_t k=0; k<numVariableIndex(0); ++k )
	    rate *= y[compartment.index()][ variableIndex(0,k) ];
	dydt[compartment.index()][varIndex] += rate;
}

CreationZero::CreationZero(std::vector<double> &paraValue, 
			   std::vector< std::vector<size_t> > 
			   &indValue ) 
{  
	// Do some checks on the parameters and variable indeces
	if( paraValue.size()!=1 ) {
		std::cerr << "CreationZero::CreationZero() "
			  << "Uses only zero parameter k_c\n";
		exit(0);
	}
	if( indValue.size() ) {
		std::cerr << "CreationZero::CreationZero() "
			  << "No variable index is used.\n";
		exit(0);
	}

	// Set the variable values
	setId("creationZero");
	setParameter(paraValue);  
	setVariableIndex(indValue);
  
	// Set the parameter identities
	std::vector<std::string> tmp( numParameter() );
	tmp.resize( numParameter() );
	tmp[0] = "k_c";
	setParameterId( tmp );
}

void CreationZero::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
	dydt[compartment.index()][varIndex] += parameter(0);
}

size_t CreationZero::
Jacobian(Compartment &compartment,size_t varIndex,DataMatrix &y,JacobianMatrix &A) 
{  
	return 0;
}

double CreationZero::
propensity(Compartment &compartment,size_t varIndex,DataMatrix &y)
{
  return parameter(0);
}

void CreationZero::
discreteUpdate(Compartment &compartment,size_t varIndex,DataMatrix &y)
{
	y[compartment.index()][varIndex] += 1.0;
}

CreationOne::CreationOne(std::vector<double> &paraValue, 
												 std::vector< std::vector<size_t> > &indValue ) 
{  
	// Do some checks on the parameters and variable indices
	if( paraValue.size()!=1 ) {
		std::cerr << "CreationOne::CreationOne() "
			  << "Uses only one parameter k_c\n";
		exit(0);
	}
	if( indValue.size() != 1 || indValue[0].size() != 1  ) {
		std::cerr << "CreationOne::CreationOne() "
							<< "One variable index needed.\n";
		exit(0);
	}
	
	// Set the variable values
	setId("creationOne");
	setParameter(paraValue);  
	setVariableIndex(indValue);
  
	// Set the parameter identities
	std::vector<std::string> tmp( numParameter() );
	tmp.resize( numParameter() );
	tmp[0] = "k_c";
	setParameterId( tmp );
}

void CreationOne::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{
	dydt[compartment.index()][varIndex] += parameter(0)*
		y[compartment.index()][ variableIndex(0,0) ];
}

size_t CreationOne::
Jacobian(Compartment &compartment,size_t varIndex,DataMatrix &y,JacobianMatrix &A) 
{
	size_t i = compartment.index()*y[0].size() + varIndex; 
	size_t j = compartment.index()*y[0].size() + variableIndex(0,0); 
	A(i,j) += parameter(0);
	return 0;
}

double CreationOne::
propensity(Compartment &compartment,size_t varIndex,DataMatrix &y)
{
  return parameter(0)*y[compartment.index()][ variableIndex(0,0) ];
}

void CreationOne::
discreteUpdate(Compartment &compartment,size_t varIndex,DataMatrix &y)
{
	y[compartment.index()][varIndex] += 1.0;
}

CreationTwo::CreationTwo(std::vector<double> &paraValue, 
			 std::vector< std::vector<size_t> > 
			 &indValue ) 
{  
	// Do some checks on the parameters and variable indeces
	if( paraValue.size()!=1 ) {
		std::cerr << "CreationTwo::CreationTwo() "
			  << "Uses only one parameter k_c\n";
		exit(0);
	}
	if( indValue.size() != 1 || indValue[0].size() != 2  ) {
		std::cerr << "CreationTwo::CreationTwo() "
			  << "Two variable index needed.\n";
		exit(0);
	}

	// Set the variable values
	setId("creationTwo");
	setParameter(paraValue);  
	setVariableIndex(indValue);
  
	// Set the parameter identities
	std::vector<std::string> tmp( numParameter() );
	tmp.resize( numParameter() );
	tmp[0] = "k_c";
	setParameterId( tmp );
}

void CreationTwo::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
	dydt[compartment.index()][varIndex] += parameter(0)*
		y[compartment.index()][ variableIndex(0,0) ]*
		y[compartment.index()][ variableIndex(0,1) ];
}

size_t CreationTwo::
Jacobian(Compartment &compartment,size_t varIndex,DataMatrix &y,JacobianMatrix &A) 
{  
	if (variableIndex(0,0)==variableIndex(0,1)) {
		size_t i = compartment.index()*y[0].size() + varIndex; 
		size_t j = compartment.index()*y[0].size() + variableIndex(0,0); 
		A(i,j) += 2.0*parameter(0)*y[compartment.index()][ variableIndex(0,0) ];
	}
	else {
		size_t i = compartment.index()*y[0].size() + varIndex; 
		size_t j = compartment.index()*y[0].size() + variableIndex(0,0); 
		A(i,j) += parameter(0)*y[compartment.index()][ variableIndex(0,1) ];
		j = compartment.index()*y[0].size() + variableIndex(0,1); 
		A(i,j) += parameter(0)*y[compartment.index()][ variableIndex(0,0) ];
	}
	return 1;
}

CreationZeroDegradationOne::CreationZeroDegradationOne(std::vector<double>
                                                       &paraValue,
                                                       std::vector<
                                                       std::vector<size_t> >
                                                       &indValue ) 
{  
	// Do some checks on the parameters and variable indeces
	if( paraValue.size()!=2 ) {
		std::cerr << "CreationZeroDegradationOne::CreationZeroDegradationOne() "
			  << "Uses two parameters K,k_d\n";
		exit(0);
	}
	if( indValue.size() ) {
		std::cerr << "CreationZeroDegradationOne::CreationZeroDegradationOne() "
			  << "No variable index is used.\n";
		exit(0);
	}

	// Set the variable values
	setId("creationZeroDegradationOne");
	setParameter(paraValue);
	setVariableIndex(indValue);
  
	// Set the parameter identities
	std::vector<std::string> tmp( numParameter() );
	tmp.resize( numParameter() );
	tmp[0] = "K";
	tmp[1] = "k_d";
	setParameterId( tmp );
}

void CreationZeroDegradationOne::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
	dydt[compartment.index()][varIndex] += parameter(0)*
		(1.0-parameter(1)*y[compartment.index()][varIndex]);
}

size_t CreationZeroDegradationOne::
Jacobian(Compartment &compartment,size_t varIndex,DataMatrix &y,JacobianMatrix &A) 
{  
	size_t i = compartment.index()*y[0].size() + varIndex; 
	A(i,i) -= parameter(0)*parameter(1);
	return 0;
}

CreationZeroDegradationTwo::CreationZeroDegradationTwo(std::vector<double>
                                                       &paraValue,
                                                       std::vector<
                                                       std::vector<size_t> >
                                                       &indValue ) 
{	
	// Do some checks on the parameters and variable indeces
	if( paraValue.size()!=2 ) {
		std::cerr << "CreationZeroDegradationTwo::CreationZeroDegradationTwo() "
			  << "Uses two parameters K,k_d\n";
		exit(0);
	}
	if( indValue.size()!=1 || indValue[0].size() != 1 ) {
		std::cerr << "CreationZeroDegradationTwo::CreationZeroDegradationTwo() "
			  << "one variable index is used.\n";
		exit(0);
	}

	// Set the variable values
	setId("creationZeroDegradationTwo");
	setParameter(paraValue);
	setVariableIndex(indValue);
	
	// Set the parameter identities
	std::vector<std::string> tmp( numParameter() );
	tmp.resize( numParameter() );
	tmp[0] = "K";
	tmp[1] = "k_d";
	setParameterId( tmp );
}

void CreationZeroDegradationTwo::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
	dydt[compartment.index()][varIndex] += parameter(0)*
		(1.0-parameter(1)*y[compartment.index()][varIndex]*
		 y[compartment.index()][variableIndex(0,0)]);
}

CreationOneDegradationOne::CreationOneDegradationOne(std::vector<double> 
						     &paraValue, 
						     std::vector< 
						     std::vector<size_t> > 
						     &indValue ) 
{  
	// Do some checks on the parameters and variable indeces
	if( paraValue.size()!=2 ) {
		std::cerr << "CreationOneDegradationOne::CreationOneDegradationOne() "
			  << "Uses two parameters K,k_d\n";
		exit(0);
	}
	if( indValue.size()!=1 || indValue[0].size() != 1  ) {
		std::cerr << "CreationOneDegradationOne::CreationOneDegradationOne() "
			  << "one variable indices are used.\n";
		exit(0);
	}
	
	// Set the variable values
	setId("creationOneDegradationOne");
	setParameter(paraValue);  
	setVariableIndex(indValue);
  
	// Set the parameter identities
	std::vector<std::string> tmp( numParameter() );
	tmp.resize( numParameter() );
	tmp[0] = "K";
	tmp[1] = "k_d";
	setParameterId( tmp );
}

void CreationOneDegradationOne::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
	dydt[compartment.index()][varIndex] += parameter(0)*
		(y[compartment.index()][variableIndex(0,0)]-parameter(1)*y[compartment.index()][varIndex]);
}

CreationOneDegradationTwo::CreationOneDegradationTwo(std::vector<double>
						     &paraValue,
						     std::vector<
						     std::vector<size_t> >
						     &indValue ) 
{  
	// Do some checks on the parameters and variable indeces
	if( paraValue.size()!=2 ) {
		std::cerr << "CreationOneDegradationTwo::CreationOneDegradationTwo() "
			  << "Uses two parameters K,k_d\n";
		exit(0);
	}
	if( indValue.size()!=1 || indValue[0].size() != 2  ) {
		std::cerr << "CreationOneDegradationTwo::CreationOneDegradationTwo() "
			  << "two variable indices are used.\n";
		exit(0);
	}

	// Set the variable values
	setId("creationOneDegradationTwo");
	setParameter(paraValue);
	setVariableIndex(indValue);
  
	// Set the parameter identities
	std::vector<std::string> tmp( numParameter() );
	tmp.resize( numParameter() );
	tmp[0] = "K";
	tmp[1] = "k_d";
	setParameterId( tmp );
}

void CreationOneDegradationTwo::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{
	dydt[compartment.index()][varIndex] += parameter(0)*
		( y[compartment.index()][variableIndex(0,0)] -
		  parameter(1)*y[compartment.index()][varIndex]*
		  y[compartment.index()][variableIndex(0,1)] );
}

CreationSpatialThreshold::CreationSpatialThreshold(std::vector<double> &paraValue, 
						   std::vector< std::vector<size_t> > 
						   &indValue ) 
{  
	// Do some checks on the parameters and variable indeces
	if( paraValue.size()!=3 ) {
		std::cerr << "CreationSpatialThreshold::CreationSpatialThreshold() "
			  << "Uses three parameters k_c X_th X_sign\n";
		exit(0);
	}
	if( indValue.size() != 1 || indValue[0].size() != 1 ) {
		std::cerr << "CreationSpatialThreshold::CreationSpatialThreshold()"
			  << "Variable index indicating threshold variable needed.\n";
		exit(0);
	}
	//Sign should be -/+1
	if( paraValue[2] != -1 && paraValue[2] != 1 ) {
		std::cerr << "CreationSpatialThreshold::CreationSpatialThreshold() "
			  << "X_sign should be +/-1\n";
		exit(0);
	}
	
	// Set the variable values
	setId("creationSpatialThreshold");
	setParameter(paraValue);  
	setVariableIndex(indValue);
  
	// Set the parameter identities
	std::vector<std::string> tmp( numParameter() );
	tmp.resize( numParameter() );
	tmp[0] = "k_c";
	tmp[1] = "X_th";
	tmp[2] = "X_sign";  
	setParameterId( tmp );
}

void CreationSpatialThreshold::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
	if( y[compartment.index()][variableIndex(0,0)]*parameter(2)>parameter(1)*parameter(2) )
		dydt[compartment.index()][varIndex] += parameter(0);
}

CreationSpatialSphere::CreationSpatialSphere(std::vector<double> &paraValue, 
					     std::vector< std::vector<size_t> > 
					     &indValue ) 
{  
	// Do some checks on the parameters and variable indeces
	if( paraValue.size()!=3 ) {
		std::cerr << "CreationSpatialSphere::CreationSpatialSphere() "
			  << "Uses three parameters k_c R and R_sign\n";
		exit(0);
	}
	if( indValue.size() != 0 ) {
		std::cerr << "CreationSpatialSphere::CreationSpatialSphere() "
			  << "No variable index needed.\n";
		exit(0);
	}
	// Sign should be -/+1
	if( paraValue[2] != -1 && paraValue[2] != 1 ) {
		std::cerr << "CreationSpatialSphere::CreationSpatialSphere() "
			  << "R_sign should be +/-1\n";
		exit(0);
	}
	
	// Set the variable values
	setId("creationSpatialSphere");
	setParameter(paraValue);  
	setVariableIndex(indValue);
  
	// Set the parameter identities
	std::vector<std::string> tmp( numParameter() );
	tmp.resize( numParameter() );
	tmp[0] = "k_c";
	tmp[1] = "R";
	tmp[2] = "R_sign";  
	setParameterId( tmp );
}

void CreationSpatialSphere::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
	double r=0.0;
	size_t dimension = compartment.numTopologyVariable()-1;
	if( dimension<2 ) {
		r += y[compartment.index()][ 0 ]*y[compartment.index()][ 0 ];
	}
	for(size_t i=0 ; i<2 ; i++ )
		r += y[compartment.index()][ i ]*y[compartment.index()][ i ];
	r = std::sqrt(r);
  
	if( r*parameter(2)>parameter(1)*parameter(2) )
		dydt[compartment.index()][varIndex] += parameter(0);
}

CreationSpatialSphereCylinder::
CreationSpatialSphereCylinder(std::vector<double> &paraValue, 
			      std::vector< std::vector<size_t> > 
			      &indValue ) 
{  
	// Do some checks on the parameters and variable indices
	if( paraValue.size()!=3 ) {
		std::cerr << "CreationSpatialSphereCylinder::"
			  << "CreationSpatialSphereCylinder() "
			  << "Uses three parameters k_c R and R_sign\n";
		exit(0);
	}
	if( indValue.size() != 0 ) {
		std::cerr << "CreationSpatialSphereCylinder::"
			  << "CreationSpatialSphereCylinder() "
			  << "No variable index needed.\n";
		exit(0);
	}
	//k and R should be nonnegative
	if( paraValue[0] < 0.0 || paraValue[1] < 0.0 ) {
		std::cerr << "CreationSpatialSphereCylinder::"
			  << "CreationSpatialSphereCylinder() "
			  << "k_c and R should be nonnegative\n";
		exit(0);
	}
	//Sign should be -/+1
	if( paraValue[2] != -1 && paraValue[2] != 1 ) {
		std::cerr << "CreationSpatialSphereCylinder::CreationSpatialSphereCylinder() "
			  << "R_sign should be +/-1\n";
		exit(0);
	}

	// Set the variable values
	setId("creationSpatialSphereCylinder");
	setParameter(paraValue);  
	setVariableIndex(indValue);
  
	// Set the parameter identities
	std::vector<std::string> tmp( numParameter() );
	tmp.resize( numParameter() );
	tmp[0] = "k_c";
	tmp[1] = "R";
	tmp[2] = "R_sign";  
	setParameterId( tmp );
}

void CreationSpatialSphereCylinder::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
	size_t dimension = compartment.numTopologyVariable()-1;
	size_t i=compartment.index();
	double r=0.0;
	for(size_t d=0 ; d<dimension-1 ; d++ )
		r += y[i][d]*y[i][d];
	if( y[i][dimension-1]>0.0 )
		r += y[i][dimension-1]*y[i][dimension-1];
	r = std::sqrt(r);
  
	if( r*parameter(2)>parameter(1)*parameter(2) )
		dydt[i][varIndex] += parameter(0);
}

CreationSpatialMax::CreationSpatialMax(std::vector<double> &paraValue, 
				       std::vector< std::vector<size_t> > 
				       &indValue ) 
{  
	// Do some checks on the parameters and variable indeces
	if( paraValue.size()!=2 ) {
		std::cerr << "CreationSpatialMax::CreationSpatialMax() "
			  << "Uses two parameters k_c X_sign\n";
		exit(0);
	}
	if( indValue.size() != 1 || indValue[0].size() != 1 ) {
		std::cerr << "CreationSpatialMax::CreationSpatialMax()"
			  << "Variable index indicating max/min variable needed.\n";
		exit(0);
	}
	//Sign should be -/+1
	if( paraValue[1] != -1 && paraValue[1] != 1 ) {
		std::cerr << "CreationSpatialMax::CreationSpatialMax() "
			  << "X_sign should be +/-1\n";
		exit(0);
	}
	
	// Set the variable values
	setId("creationSpatialMax");
	setParameter(paraValue);  
	setVariableIndex(indValue);
  
	// Set the parameter identities
	std::vector<std::string> tmp( numParameter() );
	tmp.resize( numParameter() );
	tmp[0] = "k_c";
	tmp[1] = "X_sign";
	setParameterId( tmp );
}

void CreationSpatialMax::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{
	if( compartment.index()>0 )
		return;
	
	double minmax=y[0][variableIndex(0,0)];
	size_t minmaxI=0;
	
	if( parameter(1)==1 ) {
		for( size_t i=0 ; i<y.size() ; ++i ) {
			if( y[i][variableIndex(0,0)]>minmax ) {
				minmax=y[i][variableIndex(0,0)];
				minmaxI=i;					
			}
		}
	}
	else if( parameter(1) ==-1 ) {
		for( size_t i=0 ; i<y.size() ; ++i ) {
			if( y[i][variableIndex(0,0)]<minmax ) {
				minmax=y[i][variableIndex(0,0)];
				minmaxI=i;								
			}
		}
	}
	else {
		std::cerr << "CreationSpatialMax::derivs() Wrong value for p_1"
			  << std::endl;
		exit(-1);
	}
	dydt[minmaxI][varIndex] += parameter(0);
}

CreationExponent::CreationExponent(std::vector<double> &paraValue, 
				   std::vector< std::vector<size_t> > 
				   &indValue ) 
{  
	// Do some checks on the parameters and variable indeces
	if( paraValue.size()!=2 ) {
		std::cerr << "CreationExponent::CreationExponent() "
			  << "Uses two parameters k_c and k_e\n";
		exit(0);
	}
	if( indValue.size() != 1 || indValue[0].size() != 2) {
		std::cerr << "CreationExponent::CreationExponent() "
			  << "Uses two variable indices.\n";
		exit(0);
	}
	
	// Set the variable values
	setId("creationExponent");
	setParameter(paraValue);  
	setVariableIndex(indValue);
  
	// Set the parameter identities
	std::vector<std::string> tmp( numParameter() );
	tmp.resize( numParameter() );
	tmp[0] = "k_c";
	tmp[0] = "k_e";
	setParameterId( tmp );
}

void CreationExponent::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
	dydt[compartment.index()][varIndex] += parameter(0) * std::pow(
		y[compartment.index()][ variableIndex(0,0) ],
		parameter(1)*y[compartment.index()][ variableIndex(0,1) ] );
}

Constant::Constant(std::vector<double> &paraValue, 
		   std::vector< std::vector<size_t> > 
		   &indValue ) 
{  
	// Do some checks on the parameters and variable indeces
	if( paraValue.size()!=1 ) {
		std::cerr << "Constant::Constant() "
			  << "Use one parameter, value\n";
		exit(0);
	}
	if( indValue.size() != 1 || indValue[0].size() != 1  ) {
		std::cerr << "Constant::Constant() "
			  << "One variable index needed.\n";
		exit(0);
	}
  
	
	// Set the variable values
	setId("constant");
	setParameter(paraValue);  
	setVariableIndex(indValue);
  
	// Set the parameter identities
	std::vector<std::string> tmp( numParameter() );
	tmp.resize( numParameter() );
	tmp[0] = "value";
	setParameterId( tmp );
}

void Constant::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt ) 
{ 
	if (y[compartment.index()][ variableIndex(0,0) ] )
		y[compartment.index()][ varIndex ]=parameter(0);
}

CreationNeighbor::CreationNeighbor(std::vector<double> &paraValue, 
				   std::vector< std::vector<size_t> > 
				   &indValue ) 
{  
	// Do some checks on the parameters and variable indeces
	if( paraValue.size()!=1 ) {
		std::cerr << "CreationNeighbor::CreationNeighbor() "
			  << "Uses only one parameter k_c\n";
		exit(0);
	}
	if( indValue.size() != 2 ) {
		std::cerr << "CreationNeighbor::CreationNeighbor()"
			  << "Two levels of variables are used, the first with membrane variables, "
			  << "the second with neighbor variable.\n";
		exit(0);
	}
	
	// Set the variable values
	setId("creationNeighbor");
	setParameter(paraValue);  
	setVariableIndex(indValue);
  
	// Set the parameter identities
	std::vector<std::string> tmp( numParameter() );
	tmp.resize( numParameter() );
	tmp[0] = "k_c";
	setParameterId( tmp );
}

void CreationNeighbor::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
	if (numVariableIndex(0)) {
		if (numVariableIndex(0)>compartment.numNeighbor()) {
			std::cerr << "CreationNeighbor:::derivs() "
				  << "Wrong number of neighbors!" << std::endl;
			std::cerr << "cell " << compartment.index() << " numVariableIndex " << numVariableIndex(0)
				  << " numNeighbor " << compartment.numNeighbor() << std::endl; 
			exit(-1);
		}
		for( size_t k=0; k<numVariableIndex(0); ++k ) {
			//std::cerr << "cell " << compartment.index() << " variable " << variableIndex(0,k) 
			//				<< " updated from neighbor " << compartment.neighbor(k) << " var " 
			//				<< variableIndex(1,0) << std::endl;
			dydt[compartment.index()][variableIndex(0,k)] += parameter(0)*y[compartment.neighbor(k)][variableIndex(1,0)]*y[compartment.neighbor(k)][variableIndex(1,0)];
		}
	}
}


