#ifndef DEGRADATION_H
#define DEGRADATION_H
//
// Filename     : degradation.h
// Description  : Classes describing degradation of molecules
// Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
// Created      : October 2003
// Revision     : $Id: degradation.h 486 2011-04-07 19:42:37Z henrik $
//

#include "baseReaction.h"

///
/// @brief DegradationLinear applies a degradation rate proportional to a
/// number of user defined variables plus itself
///
/// The variable update is given by 
///
/// @f[ \frac{dc}{dt} = - k_d c \prod_i C_i @f]
///
/// where @f$ k_d @f$ is the degradation rate and @f$ c @f$ is the
/// concentration of the degraded molecule and the product is over the user
/// supplied variables (indices) @f$ C_i @f$.
///
/// In a model file the reaction is defined as
///
/// @verbatim
/// degradationLinear 1 1 N_i
/// k_c
/// index1 ... indexN_i
/// @endverbatim
///
/// Alternatively if no index supplied (degradation proportional to itself)
/// the first line can be replaced by 'degradationLinear 1 0'.
///
class DegradationLinear : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which sets the parameters and variable
	/// indices that defines the reaction.
	///
	/// @param paraValue vector with parameters
	///
	/// @param indValue vector of vectors with variable indices
	///
	/// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
	///
  DegradationLinear(std::vector<double> &paraValue, 
										std::vector< std::vector<size_t> > &indValue );
  
	///
	/// @brief Derivative function for this reaction class
	///
	/// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
	///
  void derivs(Compartment &compartment,size_t species,DataMatrix &y,DataMatrix &dydt);
};

///
/// @brief Implements a degradation proportional to its own concentration
///
/// The update to the variable is
///
/// @f[ \frac{dc}{dt} -= k_d c @f]
///
/// where @f$ k_d @f$ is the degradation rate.
///
/// In a model file the reaction is defined as
///
/// @verbatim
/// degradationOne 1 0
/// k_d
/// @endverbatim
///
class DegradationOne : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which sets the parameters and variable
	/// indices that defines the reaction.
	///
	/// @param paraValue vector with parameters
	///
	/// @param indValue vector of vectors with variable indices
	///
	/// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
	///
  DegradationOne(std::vector<double> &paraValue, 
								 std::vector< std::vector<size_t> > &indValue );
  
	///
	/// @brief Derivative function for this reaction class
	///
	/// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
	///
  void derivs(Compartment &compartment,size_t species,DataMatrix &y,DataMatrix &dydt);
	///
	/// @brief Jacobian function for this reaction class
	///
	/// @see BaseReaction::Jacobian(Compartment &compartment,size_t species,...)
	///
  size_t Jacobian(Compartment &compartment,size_t species,DataMatrix &y,JacobianMatrix &A);

  ///
  /// @brief Propensity calculation (probability to happen in [t,t+dt]) for this reaction class
  ///
  /// @see BaseReaction::propensity(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  double propensity(Compartment &compartment,size_t species,DataMatrix &y);

  ///
  /// @brief Discrete update for this reaction used in stochastic simulations
  ///
  /// @see BaseReaction::discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  void discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y);
};

///
/// @brief Implements a degradation proportional to its own concentration and
/// an additional variable.
///
/// The update to the variable is
///
/// @f[ \frac{dc}{dt} -= k_d c C @f]
///
/// where @f$ k_d @f$ is the degradation rate, and C is the user supplied
/// variable.
///
/// In a model file the reaction is defined as
///
/// @verbatim
/// degradationTwo 1 1 1
/// k_d
/// index1
/// @endverbatim
///
class DegradationTwo : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which sets the parameters and variable
	/// indices that defines the reaction.
	///
	/// @param paraValue vector with parameters
	///
	/// @param indValue vector of vectors with variable indices
	///
	/// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
	///
  DegradationTwo(std::vector<double> &paraValue, 
								 std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t species,DataMatrix &y,DataMatrix &dydt);
  ///
  /// @brief Jacobian function for this reaction class
  ///
  /// @see BaseReaction::Jacobian(Compartment &compartment,size_t species,...)
  ///
  size_t Jacobian(Compartment &compartment,size_t species,DataMatrix &y,JacobianMatrix &A);
};

///
/// @brief Implements a degradation proportional to its own concentration and
/// two additional variables.
///
/// The update to the variable is
///
/// @f[ \frac{dc}{dt} -= k_d c C_1 C_2 @f]
///
/// where @f$ k_d @f$ is the degradation rate, and @f$ C_1,C_2 @f$ are the
/// user supplied variable.
///
/// In a model file the reaction is defined as
///
/// @verbatim
/// degradationThree 1 1 2
/// k_d
/// index1 index2
/// @endverbatim
///
class DegradationThree : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which sets the parameters and variable
	/// indices that defines the reaction.
	///
	/// @param paraValue vector with parameters
	///
	/// @param indValue vector of vectors with variable indices
	///
	/// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
	///
  DegradationThree(std::vector<double> &paraValue, 
									 std::vector< std::vector<size_t> > &indValue );
  
	///
	/// @brief Derivative function for this reaction class
	///
	/// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
	///
  void derivs(Compartment &compartment,size_t species,DataMatrix &y,DataMatrix &dydt);
};

///
/// @brief Implements a degradation proportional to its own concentration and
/// a Hill function of another.
///
/// The update to the variable is
///
/// @f[ \frac{dc}{dt} -= k_d c C_1^n/(C_1^n+K^n) @f]
///
/// where @f$ k_d @f$ is the degradation rate, and @f$ C_1 @f$ is the
/// user supplied variable.
///
/// In a model file the reaction is defined as
///
/// @verbatim
/// degradatioHill 3 1 1
/// k_d K n
/// index1
/// @endverbatim
///
class DegradationHill : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which sets the parameters and variable
	/// indices that defines the reaction.
	///
	/// @param paraValue vector with parameters
	///
	/// @param indValue vector of vectors with variable indices
	///
	/// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
	///
  DegradationHill(std::vector<double> &paraValue, 
									std::vector< std::vector<size_t> > &indValue );
  
	///
	/// @brief Derivative function for this reaction class
	///
	/// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
	///
  void derivs(Compartment &compartment,size_t species,DataMatrix &y,DataMatrix &dydt);
};

///
/// @brief Implements a degradation proportional to its own concentration and
/// a (decreasing) Hill function of another.
///
/// The update to the variable is
///
/// @f[ \frac{dc}{dt} -= k_d c K^n/(C_1^n+K^n) @f]
///
/// where @f$ k_d @f$ is the degradation rate, and @f$ C_1 @f$ is the
/// user supplied variable.
///
/// In a model file the reaction is defined as
///
/// @verbatim
/// degradatioHillR 3 1 1
/// k_d K n
/// index1
/// @endverbatim
///
class DegradationHillR : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which sets the parameters and variable
	/// indices that defines the reaction.
	///
	/// @param paraValue vector with parameters
	///
	/// @param indValue vector of vectors with variable indices
	///
	/// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
	///
  DegradationHillR(std::vector<double> &paraValue, 
									std::vector< std::vector<size_t> > &indValue );
  
	///
	/// @brief Derivative function for this reaction class
	///
	/// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
	///
  void derivs(Compartment &compartment,size_t species,DataMatrix &y,DataMatrix &dydt);
};

///
/// @brief Molecular degradation for cells above/below a threshold position 
///
/// This class uses a threshold in a given dimension to determine if a
/// constant (rate @f$ k_d @f$) molecular degradation should be
/// applied. Threshold value is given as parameter, and a second parameter is
/// the sign which is determining if degradation is applied above (sign=1) or
/// below (sign=-1) the threshold value. The thrshold variable index is also
/// given.
///
/// In a model file the reaction is defined as
///
/// @verbatim
/// degradationSpatialThreshold 3 1 1
/// k_d var_th sign
/// index1
/// @endverbatim
///
class DegradationSpatialThreshold : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which sets the parameters and variable
	/// indices that defines the reaction.
	///
	/// @param paraValue vector with parameters
	///
	/// @param indValue vector of vectors with variable indices
	///
	/// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
	///
  DegradationSpatialThreshold(std::vector<double> &paraValue, 
													 std::vector< std::vector<size_t> > &indValue );
  
	///
	/// @brief Derivative function for this reaction class
	///
	/// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
	///
  void derivs(Compartment &compartment,size_t species,DataMatrix &y,DataMatrix &dydt);
};

#endif


