//
// Filename     : grn.h
// Description  : Classes describing gene regulatory network updates
// Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
// Created      : October 2003
// Revision     : $Id: grn.h 490 2011-04-19 07:25:30Z henrik $
//
#ifndef GRN_H
#define GRN_H

#include <cmath>

#include "baseReaction.h"

///
/// @brief The class Grn use a neural network inspired mathematics for gene
/// regulation
///
/// This class uses a neural network inspired update with a sigmoidal function
/// for gene regulation as defined in Mjolsness et al (1991). The update is
/// done according to:
///
/// @f[ \frac{dy_{ij}}{dt} = \frac{1}{p_1}g\left( p_0 + \sum_k p_k y_{ik} \right) @f]
///
/// where g() is a sigmoidal function, the @f$p_k@f$ are the parameters given
/// from position 2 and forward, and the k indices for @f$y_{ik}@f$ are given
/// at the first level of variableIndex. Hence the number of parameters must
/// be two more than the number of indices given at the first level.
///
/// @see Grn::sigmoid(double x) for implementation of sigmoid function.
///
class Grn : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which checks and sets the parameters and
  /// variable indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  Grn(std::vector<double> &paraValue, 
      std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t specie,DataMatrix &y,DataMatrix &dydt);

  ///
  /// @brief Sigmoidal function used in the derivs function
  ///
  /// Sigmoidal function defined by
  ///
  /// @f[ g(x) = \frac{1}{2}\left( 1 + \frac{x}{\sqrt{1+x^2}}\right)@f]
  ///
  inline double sigmoid( double value );  
};

inline double Grn::sigmoid(double x) 
{ 
  return 0.5*(1 + x/sqrt(1+x*x) );
}

///
/// @brief The class Gsrn use a neural network inspired mathematics with
/// neighbor signals for gene regulation
///
/// This class uses a neural network inspired update with a sigmoidal function
/// for gene regulation as defined in Mjolsness et al (1991). This class adds
/// a direct contribution from neighboring cells, compared to the Grn
/// class. The update is done according to:
///
/// @f[ \frac{dy_{ij}}{dt} = \frac{1}{p_1}g\left( p_0 + \sum_k p_k y_{ik} +
/// \sum_{k'} p_{k'} y_{jk'} \right) @f]
///
/// where g() is a sigmoidal function. The @f$p_k@f$ are the parameters given
/// from position 2 and K steps forward and defines the intracellular
/// contributions. The @f$p_{k'}@f$ are given from 2+K and forward and defines
/// the intercellular (from neighbors) contribution. The k indices for
/// @f$y_{ik}@f$ are given at the first level of variableIndex
/// (intracellular), and the k' indices are given in the second level
/// (intercellular). Hence the number of parameters must be two more than the
/// number of indices given at the first and second levels.
///
/// @see Gsrn::sigmoid(double x) for implementation of sigmoid function.
///
class Gsrn : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which sets the parameters and variable
  /// indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  Gsrn(std::vector<double> &paraValue, 
       std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t specie,DataMatrix &y,DataMatrix &dydt);
  
  ///
  /// @brief Sigmoidal function used in the derivs function
  ///
  /// Sigmoidal function defined by
  ///
  /// @f[ g(x) = \frac{1}{2}\left( 1 + \frac{x}{\sqrt{1+x^2}}\right)@f]
  ///
  inline double sigmoid( double value );  
};

inline double Gsrn::sigmoid(double x) 
{ 
  return 0.5*(1 + x/sqrt(1+x*x) );
}

///
/// @brief The class Gsrn use a neural network inspired mathematics with a
/// ligand/receptor neighbor signals for gene regulation
///
/// This class uses a neural network inspired update with a sigmoidal function
/// for gene regulation as defined in Mjolsness et al (1991). This class adds
/// a ligand/receptor contribution from neighboring/internal cells, compared
/// to the Gsrn class. The update is done according to:
///
/// @f[ \frac{dy_{ij}}{dt} = \frac{1}{p_1}g\left( p_0 + \sum_k p_k y_{ik} +
/// \sum_{k'} p_{k'} y_{jk'} + \sum_{k''} p_{k''} y_{jk''}y_{ik'''} \right) @f]
///
/// where g() is a sigmoidal function. The @f$p_k@f$ are the parameters given
/// from position 2 and K steps forward and defines the intracellular
/// contributions. The @f$p_{k'}@f$ are given from 2+K and forward and defines
/// the intercellular (from neighbors) contribution. The k indices for
/// @f$y_{ik}@f$ are given at the first level of variableIndex
/// (intracellular), the k' indices are given in the second level
/// (intercellular), and the k'/k''' are given in the third level ( and comes
/// in ligand/receptor pairs). Hence the number of parameters must be two more
/// than the number of indices given at the first and second levels plus half
/// the number given in the third layer.
///
/// @see Gsrn2::sigmoid(double x) for implementation of sigmoid function.
///
class Gsrn2 : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which sets the parameters and variable
  /// indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  Gsrn2(std::vector<double> &paraValue, 
	std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t specie,DataMatrix &y,DataMatrix &dydt);
  
  ///
  /// @brief Sigmoidal function used in the derivs function
  ///
  /// Sigmoidal function defined by
  ///
  /// @f[ g(x) = \frac{1}{2}\left( 1 + \frac{x}{\sqrt{1+x^2}}\right)@f]
  ///
  inline double sigmoid( double value );  
};

inline double Gsrn2::sigmoid(double x) 
{ 
  return 0.5*(1 + x/sqrt(1+x*x) );
}

///
/// @brief This class describes a simple Michaelis-Menten production
///
/// This reaction is a Michaelis-Menten formalism for production given by
///
/// @f[ \frac{dy_{ij}}{dt} = p_0 \frac{y_{ik}}{p_1+y_{ik}}@f]
///
/// where p_0 is the maximal rate (@f$V_{max}@f$), and @f$p_1@f$ is the half
/// maximal rate constant (@f$K_{half}@f$). The k index is given in the first
/// level of varIndex.
///
class MichaelisMenten : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which sets the parameters and variable
  /// indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  MichaelisMenten(std::vector<double> &paraValue, 
		  std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t specie,DataMatrix &y,DataMatrix &dydt);

  ///
  /// @brief Propensity calculation (probability to happen in [t,t+dt]) for this reaction class
  ///
  /// @see BaseReaction::propensity(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  double propensity(Compartment &compartment,size_t species,DataMatrix &y);

  ///
  /// @brief Discrete update for this reaction used in stochastic simulations
  ///
  /// @see BaseReaction::discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  void discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y);
};

///
/// @brief This class describes a simple Michaelis-Menten production with a
/// restricting variable
///
/// This reaction is a Michaelis-Menten formalism for production with a
/// restricting variable given by
///
/// @f[ \frac{dy_{ij}}{dt} = p_0 \frac{y_{ik} y_{ik'}}{p_1+y_{ik}}@f]
///
/// where p_0 is the maximal rate (@f$V_{max}@f$), and @f$p_1@f$ is the half
/// maximal rate constant (@f$K_{half}@f$). The k index is given in the first
/// level of varIndex, and the k' in the second level.
///
class MichaelisMentenRestricted : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which sets the parameters and variable
  /// indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  MichaelisMentenRestricted(std::vector<double> &paraValue, 
			    std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t specie,DataMatrix &y,DataMatrix &dydt);
};

///
/// @brief This class describes a Hill-type production with activators and
/// repressors
///
/// This reaction is a Michaelis-Menten formalism for production with a
/// restricting variable given by
///
/// @f[ \frac{dy_{ij}}{dt} = p_0 \frac{y_{ik}^{p_2}}{p_1^{p_2}+y_{ik}^{p_2}}...\frac{p_{1'}^{p_{2'}}}{p_{1'}^{p_{2'}}+y_{ik'}^{p_{2'}}}@f]
///
/// where p_0 is the maximal rate (@f$V_{max}@f$), and @f$p_1@f$ is the Hill
/// constant (@f$K_{half}@f$), and @f$p_2@f$ is the Hill coefficient (n). The
/// k index is given in the first level of varIndex and corresponds to the
/// activators, and the k' in the second level which corresponds to the
/// repressors. In both layers each index corresponds to a pair of parameters
/// (K,n) that are preceded by the @f$V_{max}@f$ parameter.
///
class Hill : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which sets the parameters and variable
  /// indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  Hill(std::vector<double> &paraValue, 
       std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t specie,DataMatrix &y,DataMatrix &dydt);
  
  ///
  /// @brief Jacobian function for this reaction class
  ///
  /// @see BaseReaction::Jacobian(Compartment &compartment,size_t species,...)
  ///
  size_t Jacobian(Compartment &compartment,size_t species,DataMatrix &y,JacobianMatrix &A);  

  ///
  /// @brief Propensity calculation (probability to happen in [t,t+dt]) for this reaction class
  ///
  /// @see BaseReaction::propensity(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  double propensity(Compartment &compartment,size_t species,DataMatrix &y);

  ///
  /// @brief Discrete update for this reaction used in stochastic simulations
  ///
  /// @see BaseReaction::discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  void discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y);
};

///
/// @brief This class describes a simple Hill-type production (repression)
///
/// This reaction uses a single input and describes a Hill repression
/// production by
///
/// @f[ \frac{dy_{ij}}{dt} = p_0
/// \frac{y_{ik}^{p_2}}{p_1^{p_2}+y_{ik}^{p_2}}@f]
///
/// where p_0 is the maximal rate (@f$V_{max}@f$), and @f$p_1@f$ is the Hill
/// constant (@f$K_{half}@f$), and @f$p_2@f$ is the Hill coefficient. The
/// single k index is given in the first level of varIndex.
///
class HillRepressor : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which sets the parameters and variable
  /// indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  HillRepressor(std::vector<double> &paraValue, 
		std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t specie,DataMatrix &y,DataMatrix &dydt);  

  ///
  /// @brief Jacobian function for this reaction class
  ///
  /// @see BaseReaction::Jacobian(Compartment &compartment,size_t species,...)
  ///
  size_t Jacobian(Compartment &compartment,size_t species,DataMatrix &y,JacobianMatrix &A);  
};

///
/// @brief This class describes a SheaAckers-type production with activators and
/// repressors
///
/// This reaction is a Shea-Ackers formalism for transcription given by
///
/// @f[ \frac{dy_{ij}}{dt} = \frac{\Sum_k a_k y_{ik}}{1 + \Sum_l b_l y_{il}}@f]
///
/// The k index is given in the first level of varIndex and corresponds to the
/// activators, and the l in the second level which corresponds to the
/// repressors. For each variable there is one parameter.
///
class SheaAckers : public BaseReaction {
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which sets the parameters and variable
  /// indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  SheaAckers(std::vector<double> &paraValue, 
	     std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t specie,DataMatrix &y,DataMatrix &dydt);
  
  ///
  /// @brief Jacobian function for this reaction class
  ///
  /// @see BaseReaction::Jacobian(Compartment &compartment,size_t species,...)
  ///
  size_t Jacobian(Compartment &compartment,size_t species,DataMatrix &y,JacobianMatrix &A);  
};

///
/// @brief This class describes a SheaAckers-type production with activators and
/// repressors
///
/// This reaction is a Shea-Ackers formalism for transcription given by
///
/// 
///
/// The k index is given in the first level of varIndex and corresponds to the
/// activators, and the l in the second level which corresponds to the
/// repressors. For each variable there is one parameter.
///
class SheaAckersDimer : public BaseReaction {
  
	
	
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which sets the parameters and variable
  /// indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  SheaAckersDimer(std::vector<double> &paraValue, 
	     std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t specie,DataMatrix &y,DataMatrix &dydt);
  
  ///
  /// @brief Jacobian function for this reaction class
  ///
  /// @see BaseReaction::Jacobian(Compartment &compartment,size_t species,...)
  ///
  size_t Jacobian(Compartment &compartment,size_t species,DataMatrix &y,JacobianMatrix &A);  
};


#endif
