//
// Filename     : growth.h
// Description  : Classes describing growth updates
// Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
// Created      : October 2003
// Revision     : $Id: growth.h 499 2011-09-15 12:56:59Z henrik $
//
#ifndef GROWTH_H
#define GROWTH_H

#include<cmath>

#include"baseReaction.h"
///
/// @brief The class constantGrowth describes a constant compartment growth
///
/// A constant growth is applied to the last topology variable, e.g. if a 
/// compartment is described by a sphere x y z r, the update will be
///
/// @f[ \frac{dr}{dt} = p_0
///
/// where the rate is set by the only used parameter (no variable indices needs 
/// to be supplied). In the model file the reaction is given by
///
/// @verbatim
/// growthConstant 1 0
/// p_0
/// @endverbatim
///
class GrowthConstant : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which checks and sets the parameters and
	/// variable indices that defines the neighborhood rule.
	///
	/// @param paraValue Vector with parameters used.
	///
	/// @param indValue Vector of vectors with variable indices used by the
	/// reaction.
	///
	/// @see BaseReaction::createReaction(std::vector<double> &, std::vector<
	/// std::vector<size_t> > &, const std::string &)
	///
  GrowthConstant(std::vector<double> &paraValue, 
		 std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};
///
// @brief A compartment growth proportional to size
///
/// An exponential growth is applied to the last topology variable, e.g. if a 
/// compartment is described by a sphere x y z r, the update will be
///
/// @f[ \frac{dr}{dt} = p_0 r
///
/// where the rate is set by the only used parameter (no variable indices needs 
/// to be supplied). In the model file the reaction is given by
///
/// @verbatim
/// growthExponential 1 0
/// p_0
/// @endverbatim
///
class GrowthExponential : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which checks and sets the parameters and
	/// variable indices that defines the neighborhood rule.
	///
	/// @param paraValue Vector with parameters used.
	///
	/// @param indValue Vector of vectors with variable indices used by the
	/// reaction.
	///
	/// @see BaseReaction::createReaction(std::vector<double> &, std::vector<
	/// std::vector<size_t> > &, const std::string &)
	///
  GrowthExponential(std::vector<double> &paraValue, 
		    std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief Assumes a compartment growth prop to volume and a molecular content*/
///
///
// @brief A compartment growth proportional to size
///
/// An exponential growth is applied to the last topology variable, e.g. if a 
/// compartment is described by a sphere x y z r, the update will be
///
/// @f[ \frac{dr}{dt} = p_0 r x
///
/// where the rate is set by the only used parameter and multiplied
/// with the molecular value x. In the model file the reaction is given by
///
/// @verbatim
/// growthExponentialOne 1 1 1
/// p_0
/// xIndex
/// @endverbatim
///
class GrowthExponentialOne : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which checks and sets the parameters and
	/// variable indices that defines the neighborhood rule.
	///
	/// @param paraValue Vector with parameters used.
	///
	/// @param indValue Vector of vectors with variable indices used by the
	/// reaction.
	///
	/// @see BaseReaction::createReaction(std::vector<double> &, std::vector<
	/// std::vector<size_t> > &, const std::string &)
	///
  GrowthExponentialOne(std::vector<double> &paraValue, 
		       std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!The class GrowthExponentialRestricted 
/*! Assumes a compartment growth prop to volume, but only for a cells
  having a varible value above a threshold.*/
class GrowthExponentialRestricted : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which checks and sets the parameters and
	/// variable indices that defines the neighborhood rule.
	///
	/// @param paraValue Vector with parameters used.
	///
	/// @param indValue Vector of vectors with variable indices used by the
	/// reaction.
	///
	/// @see BaseReaction::createReaction(std::vector<double> &, std::vector<
	/// std::vector<size_t> > &, const std::string &)
	///
  GrowthExponentialRestricted(std::vector<double> &paraValue, 
			      std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!The class GrowthExponentialTrunc 
/*! Assumes a compartment growth prop to volume, but decreased when
  approaching a maximal size (where growth becomes zero).*/
class GrowthExponentialTrunc : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which checks and sets the parameters and
	/// variable indices that defines the neighborhood rule.
	///
	/// @param paraValue Vector with parameters used.
	///
	/// @param indValue Vector of vectors with variable indices used by the
	/// reaction.
	///
	/// @see BaseReaction::createReaction(std::vector<double> &, std::vector<
	/// std::vector<size_t> > &, const std::string &)
	///
  GrowthExponentialTrunc(std::vector<double> &paraValue, 
			 std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!The class GrowthExponentialRestrictedTrunc 
/*! Assumes a compartment growth prop to volume, but only for a cells
  having a varible value above a threshold. Growth is also decreased
  when reacing a maximal size (where growth is zero).*/
class GrowthExponentialRestrictedTrunc : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which checks and sets the parameters and
	/// variable indices that defines the neighborhood rule.
	///
	/// @param paraValue Vector with parameters used.
	///
	/// @param indValue Vector of vectors with variable indices used by the
	/// reaction.
	///
	/// @see BaseReaction::createReaction(std::vector<double> &, std::vector<
	/// std::vector<size_t> > &, const std::string &)
	///
  GrowthExponentialRestrictedTrunc(std::vector<double> &paraValue, 
			      std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!Dilution corrects for concentration changes due to compartment growth
class Dilution : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which checks and sets the parameters and
	/// variable indices that defines the neighborhood rule.
	///
	/// @param paraValue Vector with parameters used.
	///
	/// @param indValue Vector of vectors with variable indices used by the
	/// reaction.
	///
	/// @see BaseReaction::createReaction(std::vector<double> &, std::vector<
	/// std::vector<size_t> > &, const std::string &)
	///
  Dilution(std::vector<double> &paraValue, 
	   std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!The class EllipticGrowthExponentialTrunc 
/*! EllipticGrowthExponentialTrunc assumes a constant compartment
  growth prop to focal distance D, truncated at DMax.*/
class EllipticGrowthExponentialTrunc : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which checks and sets the parameters and
	/// variable indices that defines the neighborhood rule.
	///
	/// @param paraValue Vector with parameters used.
	///
	/// @param indValue Vector of vectors with variable indices used by the
	/// reaction.
	///
	/// @see BaseReaction::createReaction(std::vector<double> &, std::vector<
	/// std::vector<size_t> > &, const std::string &)
	///
  EllipticGrowthExponentialTrunc(std::vector<double> &paraValue, 
		 std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

/// @brief CigarGrowthExponentialTrunc assumes a constant compartment
/// growth prop to focal distance D, truncated at DMax.*/
///
/// Describing an exponential
/// truncated constant growth using the values in y and add the results
/// to dydt. Compartment is the cell and varIndex is the index of the
/// focal length variable D to be updated.  The parameters are k_growth,
/// DMax, k_spring in dD/dt=k_growth*D(1-D/DMax). Finally, we imagine a
/// spring with spring constant k_spring attached between the two focal
/// points, and update the cell coordinates accordingly. NOTE!
/// Coordinates are not updated using this class right now.
///
class CigarGrowthExponentialTrunc : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which checks and sets the parameters and
	/// variable indices that defines the neighborhood rule.
	///
	/// @param paraValue Vector with parameters used.
	///
	/// @param indValue Vector of vectors with variable indices used by the
	/// reaction.
	///
	/// @see BaseReaction::createReaction(std::vector<double> &, std::vector<
	/// std::vector<size_t> > &, const std::string &)
	///
  CigarGrowthExponentialTrunc(std::vector<double> &paraValue, 
			      std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!The class is used to describe growth of a sphere-bud compartment 
class SphereBudGrowthTruncNeigh : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which checks and sets the parameters and
	/// variable indices that defines the neighborhood rule.
	///
	/// @param paraValue Vector with parameters used.
	///
	/// @param indValue Vector of vectors with variable indices used by the
	/// reaction.
	///
	/// @see BaseReaction::createReaction(std::vector<double> &, std::vector<
	/// std::vector<size_t> > &, const std::string &)
	///
  SphereBudGrowthTruncNeigh(std::vector<double> &paraValue, 
			    std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief Calculates a new radius from adding r+0.5dr of internal
/// neighbor and 0.5dr from itself
class LensRadiusGrowth : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which checks and sets the parameters and
	/// variable indices that defines the neighborhood rule.
	///
	/// @param paraValue Vector with parameters used.
	///
	/// @param indValue Vector of vectors with variable indices used by the
	/// reaction.
	///
	/// @see BaseReaction::createReaction(std::vector<double> &, std::vector<
	/// std::vector<size_t> > &, const std::string &)
	///
  LensRadiusGrowth(std::vector<double> &paraValue, 
									 std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,
							std::vector< std::vector<double> > &y,
							std::vector< std::vector<double> > &dydt );
};

class LensLayerGrowth : public BaseReaction {
  
 public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which checks and sets the parameters and
	/// variable indices that defines the neighborhood rule.
	///
	/// @param paraValue Vector with parameters used.
	///
	/// @param indValue Vector of vectors with variable indices used by the
	/// reaction.
	///
	/// @see BaseReaction::createReaction(std::vector<double> &, std::vector<
	/// std::vector<size_t> > &, const std::string &)
	///
  LensLayerGrowth(std::vector<double> &paraValue, 
									std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,
							std::vector< std::vector<double> > &y,
							std::vector< std::vector<double> > &dydt );
};

class LensDilution : public BaseReaction {
  
public:
  
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which checks and sets the parameters and
	/// variable indices that defines the neighborhood rule.
	///
	/// @param paraValue Vector with parameters used.
	///
	/// @param indValue Vector of vectors with variable indices used by the
	/// reaction.
	///
	/// @see BaseReaction::createReaction(std::vector<double> &, std::vector<
	/// std::vector<size_t> > &, const std::string &)
	///
  LensDilution(std::vector<double> &paraValue, 
							 std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,
							std::vector< std::vector<double> > &y,
							std::vector< std::vector<double> > &dydt );
};

#endif
