/**
 * Filename     : massAction.cc
 * Description  : Classes describing mass action reactions
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : November 2003
 * Revision     : $Id: massAction.cc 489 2011-04-13 11:21:21Z henrik $
 */
#include"massAction.h"
#include"baseReaction.h"

MassAction::MassAction(std::vector<double> &paraValue, 
		       std::vector< std::vector<size_t> > &indValue ) 
{  
  //
  // Do some checks on the parameters and variable indeces
  //
  if( paraValue.size()!=1 ) {
    std::cerr << "MassAction::MassAction() "
	      << "Uses only one parameter k_f\n";
    exit(0);
  }
  if( indValue.size() !=2 ) {
    std::cerr << "MassAction::MassAction() "
	      << "Two levels of variable indeces are used.\n"
	      << "One for reactants and one for products\n";
    exit(0);
  }
  if( indValue[0].size()<1 ) {
    std::cerr << "MassAction::MassAction() "
	      << "If no reactants given, there will be no reaction...\n";
    exit(0);
  }
  //
  // Set the variable values
  //
  setId("massAction");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  //
  // Set the parameter identities
  //
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "k_f";
  setParameterId( tmp );
}

void MassAction::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
  double rate = parameter(0);
  if( numVariableIndex(0) )
    for( size_t i=0 ; i< numVariableIndex(0) ; i++ )
      rate *= y[compartment.index()][variableIndex(0,i)];
  else //No reaction defined...
    return;
  
  if (rate<=0.0) //No update
    return;
  
  for( size_t i=0 ; i< numVariableIndex(0) ; ++i )
    dydt[compartment.index()][variableIndex(0,i)] -= rate;
  
  for( size_t i=0 ; i< numVariableIndex(1) ; ++i )
    dydt[compartment.index()][variableIndex(1,i)] += rate;
}

size_t MassAction::
Jacobian(Compartment &compartment,size_t varIndex,DataMatrix &y,JacobianMatrix &A) 
{  
  double rate = parameter(0);
  if( numVariableIndex(0) )
    for (size_t k=0; k< numVariableIndex(0); ++k)
      rate *= y[compartment.index()][variableIndex(0,k)];
  else //No reaction defined...
    return 0;
  if (rate<=0.0) //No update
    return 1;
  //
  // Update all A
  //
  for (size_t k=0; k<numVariableIndex(0); ++k ) {
    double element = rate / y[compartment.index()][variableIndex(0,k)];
    size_t j = compartment.index()*y[0].size() + variableIndex(0,k); 
    // Reactants
    for (size_t kk=0; kk<numVariableIndex(0); ++kk ) {
      size_t i = compartment.index()*y[0].size() + variableIndex(0,kk); 
      A(i,j) -= element;
    }
    // products
    for (size_t kk=0; kk<numVariableIndex(1); ++kk ) {
      size_t i = compartment.index()*y[0].size() + variableIndex(1,kk); 
      A(i,j) += element;
    }
  }
  return 1;
}

double MassAction::
propensity(Compartment &compartment,size_t varIndex,DataMatrix &y)
{
  double rate = parameter(0);
  if( numVariableIndex(0) )
    for (size_t k=0; k<numVariableIndex(0); ++k)
      rate *= y[compartment.index()][variableIndex(0,k)];
  else //No reaction defined...
    return 0.0;
  if (rate<=0.0) //No update
    return 0.0;
  return rate;
}

void MassAction::
discreteUpdate(Compartment &compartment,size_t varIndex,DataMatrix &y)
{
  if( numVariableIndex(0) )
    for (size_t k=0; k<numVariableIndex(0); ++k)
      y[compartment.index()][variableIndex(0,k)] -= 1.0;
  if( numVariableIndex(1) )
    for (size_t k=0; k<numVariableIndex(1); ++k)
      y[compartment.index()][variableIndex(1,k)] += 1.0;
}

MassAction1to1::MassAction1to1(std::vector<double> &paraValue, 
															 std::vector< std::vector<size_t> > &indValue ) 
{  
  //
  // Do some checks on the parameters and variable indeces
  //
  if( paraValue.size()!=1 ) {
    std::cerr << "MassAction1to1::MassAction1to1() "
	      << "Uses only one parameter k_f\n";
    exit(0);
  }
  if( indValue.size() !=2 || indValue[0].size() !=1 
			|| indValue[1].size() !=1) {
    std::cerr << "MassAction1to1::MassAction1to1() "
							<< "Two levels of variable indeces are used.\n"
							<< "One for a single reactant and one for a single product.\n";
    exit(0);
  }
  //
  // Set the variable values
  //
  setId("massAction1to1");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  //
  // Set the parameter identities
  //
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "k_f";
  setParameterId( tmp );
}

void MassAction1to1::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
  double rate = parameter(0)*y[compartment.index()][variableIndex(0,0)];
  if (rate<=0.0) //No update
    return;
	dydt[compartment.index()][variableIndex(0,0)] -= rate;
	dydt[compartment.index()][variableIndex(1,0)] += rate;
}

size_t MassAction1to1::
Jacobian(Compartment &compartment,size_t varIndex,DataMatrix &y,JacobianMatrix &A) 
{  
  double rate = parameter(0);
	
  if (rate<=0.0) //No update
    return 1;
  //
  // Update all A
  //
	size_t j = compartment.index()*y[0].size() + variableIndex(0,0); 
	// Reactant
	A(j,j) -= rate;
	// product
	size_t i = compartment.index()*y[0].size() + variableIndex(1,0); 
	A(i,j) += rate;
  return 1;
}

double MassAction1to1::
propensity(Compartment &compartment,size_t varIndex,DataMatrix &y)
{
  double rate = parameter(0)*y[compartment.index()][variableIndex(0,0)];
  if (rate<=0.0) //No update
    return 0.0;
  return rate;
}

void MassAction1to1::
discreteUpdate(Compartment &compartment,size_t varIndex,DataMatrix &y)
{
	y[compartment.index()][variableIndex(0,0)] -= 1.0;
	y[compartment.index()][variableIndex(1,0)] += 1.0;
}

MassAction1to2::
MassAction1to2(std::vector<double> &paraValue, 
	       std::vector< std::vector<size_t> > &indValue ) 
{  
  //
  // Do some checks on the parameters and variable indeces
  //
  if( paraValue.size()!=1 ) {
    std::cerr << "MassAction1to2::MassAction1to2() "
	      << "Uses only one parameter k_f\n";
    exit(0);
  }
  if( indValue.size() !=2 || indValue[0].size() !=1 
      || indValue[1].size() !=2) {
    std::cerr << "MassAction1to2::MassAction1to2() "
	      << "Two levels of variable indeces are used.\n"
	      << "One for a single reactant and one for two products.\n";
    exit(0);
  }
  //
  // Set the variable values
  //
  setId("massAction1to2");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  //
  // Set the parameter identities
  //
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "k_f";
  setParameterId( tmp );
}

void MassAction1to2::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
  double rate = parameter(0)*y[compartment.index()][variableIndex(0,0)];
  if (rate<=0.0) //No update
    return;
  dydt[compartment.index()][variableIndex(0,0)] -= rate;
  dydt[compartment.index()][variableIndex(1,0)] += rate;
  dydt[compartment.index()][variableIndex(1,1)] += rate;
}

size_t MassAction1to2::
Jacobian(Compartment &compartment,size_t varIndex,DataMatrix &y,JacobianMatrix &A) 
{  
  double rate = parameter(0);	
  if (rate<=0.0) //No update
    return 1;
  //
  // Update all A
  //
  size_t j = compartment.index()*y[0].size() + variableIndex(0,0); 
  // Reactant
  A(j,j) -= rate;
  // products
  size_t i = compartment.index()*y[0].size() + variableIndex(1,0); 
  A(i,j) += rate;
  i = compartment.index()*y[0].size() + variableIndex(1,1); 
  A(i,j) += rate;
  return 1;
}

double MassAction1to2::
propensity(Compartment &compartment,size_t varIndex,DataMatrix &y)
{
  double rate = parameter(0)*y[compartment.index()][variableIndex(0,0)];
  if (rate<=0.0) //No update
    return 0.0;
  return rate;
}

void MassAction1to2::
discreteUpdate(Compartment &compartment,size_t varIndex,DataMatrix &y)
{
  y[compartment.index()][variableIndex(0,0)] -= 1.0;
  y[compartment.index()][variableIndex(1,0)] += 1.0;
  y[compartment.index()][variableIndex(1,1)] += 1.0;
}

MassAction1to3::
MassAction1to3(std::vector<double> &paraValue, 
	       std::vector< std::vector<size_t> > &indValue ) 
{  
  //
  // Do some checks on the parameters and variable indeces
  //
  if( paraValue.size()!=1 ) {
    std::cerr << "MassAction1to3::MassAction1to3() "
	      << "Uses only one parameter k_f\n";
    exit(0);
  }
  if( indValue.size() !=2 || indValue[0].size() !=1 
			|| indValue[1].size() !=3) {
    std::cerr << "MassAction1to3::MassAction1to3() "
							<< "Two levels of variable indeces are used.\n"
							<< "One for a single reactant and one for three products.\n";
    exit(0);
  }
  //
  // Set the variable values
  //
  setId("massAction1to3");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  //
  // Set the parameter identities
  //
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "k_f";
  setParameterId( tmp );
}

void MassAction1to3::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
  double rate = parameter(0)*y[compartment.index()][variableIndex(0,0)];
  if (rate<=0.0) //No update
    return;
	dydt[compartment.index()][variableIndex(0,0)] -= rate;
	dydt[compartment.index()][variableIndex(1,0)] += rate;
	dydt[compartment.index()][variableIndex(1,1)] += rate;
	dydt[compartment.index()][variableIndex(1,2)] += rate;
}

size_t MassAction1to3::
Jacobian(Compartment &compartment,size_t varIndex,DataMatrix &y,JacobianMatrix &A) 
{  
  double rate = parameter(0);	
  if (rate<=0.0) //No update
    return 1;
  //
  // Update all A
  //
	size_t j = compartment.index()*y[0].size() + variableIndex(0,0); 
	// Reactant
	A(j,j) -= rate;
	// products
	size_t i = compartment.index()*y[0].size() + variableIndex(1,0); 
	A(i,j) += rate;
	i = compartment.index()*y[0].size() + variableIndex(1,1); 
	A(i,j) += rate;
	i = compartment.index()*y[0].size() + variableIndex(1,2); 
	A(i,j) += rate;
  return 1;
}

double MassAction1to3::
propensity(Compartment &compartment,size_t varIndex,DataMatrix &y)
{
  double rate = parameter(0)*y[compartment.index()][variableIndex(0,0)];
  if (rate<=0.0) //No update
    return 0.0;
  return rate;
}

void MassAction1to3::
discreteUpdate(Compartment &compartment,size_t varIndex,DataMatrix &y)
{
	y[compartment.index()][variableIndex(0,0)] -= 1.0;
	y[compartment.index()][variableIndex(1,0)] += 1.0;
	y[compartment.index()][variableIndex(1,1)] += 1.0;
	y[compartment.index()][variableIndex(1,2)] += 1.0;
}

MassAction1to4::
MassAction1to4(std::vector<double> &paraValue, 
							 std::vector< std::vector<size_t> > &indValue ) 
{  
  //
  // Do some checks on the parameters and variable indeces
  //
  if( paraValue.size()!=1 ) {
    std::cerr << "MassAction1to4::MassAction1to4() "
	      << "Uses only one parameter k_f\n";
    exit(0);
  }
  if( indValue.size() !=2 || indValue[0].size() !=1 
			|| indValue[1].size() !=4) {
    std::cerr << "MassAction1to4::MassAction1to4() "
							<< "Two levels of variable indeces are used.\n"
							<< "One for a single reactant and one for four products.\n";
    exit(0);
  }
  //
  // Set the variable values
  //
  setId("massAction1to4");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  //
  // Set the parameter identities
  //
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "k_f";
  setParameterId( tmp );
}

void MassAction1to4::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
  double rate = parameter(0)*y[compartment.index()][variableIndex(0,0)];
  if (rate<=0.0) //No update
    return;
	dydt[compartment.index()][variableIndex(0,0)] -= rate;
	dydt[compartment.index()][variableIndex(1,0)] += rate;
	dydt[compartment.index()][variableIndex(1,1)] += rate;
	dydt[compartment.index()][variableIndex(1,2)] += rate;
	dydt[compartment.index()][variableIndex(1,3)] += rate;
}

size_t MassAction1to4::
Jacobian(Compartment &compartment,size_t varIndex,DataMatrix &y,JacobianMatrix &A) 
{  
  double rate = parameter(0);	
  if (rate<=0.0) //No update
    return 1;
  //
  // Update all A
  //
	size_t j = compartment.index()*y[0].size() + variableIndex(0,0); 
	// Reactant
	A(j,j) -= rate;
	// products
	size_t i = compartment.index()*y[0].size() + variableIndex(1,0); 
	A(i,j) += rate;
	i = compartment.index()*y[0].size() + variableIndex(1,1); 
	A(i,j) += rate;
	i = compartment.index()*y[0].size() + variableIndex(1,2); 
	A(i,j) += rate;
	i = compartment.index()*y[0].size() + variableIndex(1,3); 
	A(i,j) += rate;
  return 1;
}

double MassAction1to4::
propensity(Compartment &compartment,size_t varIndex,DataMatrix &y)
{
  double rate = parameter(0)*y[compartment.index()][variableIndex(0,0)];
  if (rate<=0.0) //No update
    return 0.0;
  return rate;
}

void MassAction1to4::
discreteUpdate(Compartment &compartment,size_t varIndex,DataMatrix &y)
{
	y[compartment.index()][variableIndex(0,0)] -= 1.0;
	y[compartment.index()][variableIndex(1,0)] += 1.0;
	y[compartment.index()][variableIndex(1,1)] += 1.0;
	y[compartment.index()][variableIndex(1,2)] += 1.0;
	y[compartment.index()][variableIndex(1,3)] += 1.0;
}

MassAction2to1::
MassAction2to1(std::vector<double> &paraValue, 
	       std::vector< std::vector<size_t> > &indValue ) 
{  
  //
  // Do some checks on the parameters and variable indeces
  //
  if( paraValue.size()!=1 ) {
    std::cerr << "MassAction2to1::MassAction2to1() "
	      << "Uses only one parameter k_f\n";
    exit(0);
  }
  if( indValue.size() !=2 || indValue[0].size() !=2 
      || indValue[1].size() !=1) {
    std::cerr << "MassAction2to1::MassAction2to1() "
	      << "Two levels of variable indeces are used.\n"
	      << "One for two reactants and one for a single product.\n";
    exit(0);
  }
  //
  // Set the variable values
  //
  setId("massAction2to1");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  //
  // Set the parameter identities
  //
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "k_f";
  setParameterId( tmp );
}

void MassAction2to1::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
  double rate = parameter(0)*y[compartment.index()][variableIndex(0,0)]
    *y[compartment.index()][variableIndex(0,1)];
  if (rate<=0.0) //No update
    return;
  dydt[compartment.index()][variableIndex(0,0)] -= rate;
  dydt[compartment.index()][variableIndex(0,1)] -= rate;
  dydt[compartment.index()][variableIndex(1,0)] += rate;
}

size_t MassAction2to1::
Jacobian(Compartment &compartment,size_t varIndex,DataMatrix &y,JacobianMatrix &A) 
{  
  double rate = parameter(0);
  
  if (rate<=0.0) //No update
    return 1;
  //
  // Update all A
  //
  // Reactants
  size_t j = compartment.index()*y[0].size() + variableIndex(0,0); 
  size_t jj = compartment.index()*y[0].size() + variableIndex(0,1); 
  // product
  size_t i = compartment.index()*y[0].size() + variableIndex(1,0); 
  double element = rate*y[compartment.index()][variableIndex(0,1)];
  A(j,j) -= element;
  A(jj,j) -= element;
  A(i,j) += element;
  element = rate*y[compartment.index()][variableIndex(0,0)];
  A(jj,jj) -= element;
  A(j,jj) -= element;
  A(i,jj) += element;
  return 1;
}

double MassAction2to1::
propensity(Compartment &compartment,size_t varIndex,DataMatrix &y)
{
  double rate = parameter(0)*y[compartment.index()][variableIndex(0,0)];
  if (variableIndex(0,0)!=variableIndex(0,1))
    rate *= y[compartment.index()][variableIndex(0,1)];
  else
    rate *= 0.5*(y[compartment.index()][variableIndex(0,1)]-1.0);
  if (rate<=0.0) //No update
    return 0.0;
  return rate;
}

void MassAction2to1::
discreteUpdate(Compartment &compartment,size_t varIndex,DataMatrix &y)
{
  y[compartment.index()][variableIndex(0,0)] -= 1.0;
  y[compartment.index()][variableIndex(0,1)] -= 1.0;
  y[compartment.index()][variableIndex(1,0)] += 1.0;
}

MassAction3to1::
MassAction3to1(std::vector<double> &paraValue, 
	       std::vector< std::vector<size_t> > &indValue ) 
{  
  //
  // Do some checks on the parameters and variable indeces
  //
  if( paraValue.size()!=1 ) {
    std::cerr << "MassAction3to1::MassAction3to1() "
	      << "Uses only one parameter k_f\n";
    exit(0);
  }
  if( indValue.size() !=2 || indValue[0].size() !=3 
			|| indValue[1].size() !=1) {
    std::cerr << "MassAction3to1::MassAction3to1() "
							<< "Two levels of variable indeces are used.\n"
							<< "One for three reactants and one for a single product.\n";
    exit(0);
  }
  //
  // Set the variable values
  //
  setId("massAction3to1");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  //
  // Set the parameter identities
  //
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "k_f";
  setParameterId( tmp );
}

void MassAction3to1::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
  double rate = parameter(0)*y[compartment.index()][variableIndex(0,0)]
		*y[compartment.index()][variableIndex(0,1)]
		*y[compartment.index()][variableIndex(0,2)];
  if (rate<=0.0) //No update
    return;
	dydt[compartment.index()][variableIndex(0,0)] -= rate;
	dydt[compartment.index()][variableIndex(0,1)] -= rate;
	dydt[compartment.index()][variableIndex(0,2)] -= rate;
	dydt[compartment.index()][variableIndex(1,0)] += rate;
}

size_t MassAction3to1::
Jacobian(Compartment &compartment,size_t varIndex,DataMatrix &y,JacobianMatrix &A) 
{  
  double rate = parameter(0);
	
  if (rate<=0.0) //No update
    return 1;
  //
  // Update all A
  //
	// Reactants
	size_t j = compartment.index()*y[0].size() + variableIndex(0,0); 
	size_t jj = compartment.index()*y[0].size() + variableIndex(0,1); 
	size_t jjj = compartment.index()*y[0].size() + variableIndex(0,2); 
	// product
	size_t i = compartment.index()*y[0].size() + variableIndex(1,0); 
	double element = rate*y[compartment.index()][variableIndex(0,1)]*
		y[compartment.index()][variableIndex(0,2)];
	A(j,j) -= element;
	A(jj,j) -= element;
	A(jjj,j) -= element;
	A(i,j) += element;
	element = rate*y[compartment.index()][variableIndex(0,0)]*
		y[compartment.index()][variableIndex(0,2)];
	A(j,jj) -= element;
	A(jj,jj) -= element;
	A(jjj,jj) -= element;
	A(i,jj) += element;
	element = rate*y[compartment.index()][variableIndex(0,0)]*
		y[compartment.index()][variableIndex(0,1)];
	A(j,jjj) -= element;
	A(jj,jjj) -= element;
	A(jjj,jjj) -= element;
	A(i,jjj) += element;
  return 1;
}

double MassAction3to1::
propensity(Compartment &compartment,size_t varIndex,DataMatrix &y)
{
  double rate = parameter(0)*y[compartment.index()][variableIndex(0,0)];
	if (variableIndex(0,0)!=variableIndex(0,1)) {// 0 != 1
		rate *= y[compartment.index()][variableIndex(0,1)];
		if (variableIndex(0,0)!=variableIndex(0,2) &&
				variableIndex(0,1)!=variableIndex(0,2) ) {// 0 != 1 != 2
			rate *= y[compartment.index()][variableIndex(0,2)];
		}
		else {// 0 != 1 && (2=0 || 2=1)
			rate *= 0.5*(y[compartment.index()][variableIndex(0,2)]-1.0);
		}
	}
	else {// 0 = 1
		rate *= 0.5*(y[compartment.index()][variableIndex(0,1)]-1.0);
		if (variableIndex(0,0)!=variableIndex(0,2)) { // 0 = 1 && 2 != 0 (&& 2 != 1)
			rate *= y[compartment.index()][variableIndex(0,2)];
		}
		else {//0=1=2
			rate *= (y[compartment.index()][variableIndex(0,2)]-2.0)/3.0;
		}
	}
  if (rate<=0.0) //No update
    return 0.0;
  return rate;
}

void MassAction3to1::
discreteUpdate(Compartment &compartment,size_t varIndex,DataMatrix &y)
{
	y[compartment.index()][variableIndex(0,0)] -= 1.0;
	y[compartment.index()][variableIndex(0,1)] -= 1.0;
	y[compartment.index()][variableIndex(0,2)] -= 1.0;
	y[compartment.index()][variableIndex(1,0)] += 1.0;
}

MassActionEnzymatic::MassActionEnzymatic(std::vector<double> &paraValue, 
					 std::vector< std::vector<size_t> > 
					 &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=1 ) {
    std::cerr << "MassActionEnzymatic::MassActionEnzymatic() "
	      << "Uses only one parameter k_f\n";
    exit(0);
  }
  if( indValue.size() !=3 ) {
    std::cerr << "MassActionEnzymatic::MassActionEnzymatic() "
	      << "Three levels of variable indices are used.\n"
	      << "One for reactants, one for products and one for enzymes\n";
    exit(0);
  }
  if( indValue[0].size()<1 ) {
    std::cerr << "MassActionEnzymatic::MassActionEnzymatic() "
	      << "If no reactants given, there will be no reaction...\n";
    exit(0);
  }
  if( indValue[2].size()<1 ) {
    std::cerr << "MassActionEnzymatic::MassActionEnzymatic() "
	      << "If no enzymes given, there will be no reaction...\n";
    exit(0);
  }
  
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("massActionEnzymatic");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "k_f";
  setParameterId( tmp );
}

void MassActionEnzymatic::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
  double rate = parameter(0);
  if( numVariableIndex(0) )
    for( size_t i=0 ; i< numVariableIndex(0) ; i++ )
      rate *= y[compartment.index()][variableIndex(0,i)];
  else //No reaction defined...
    return;
  
  if( numVariableIndex(2) )
    for( size_t i=0 ; i< numVariableIndex(2) ; i++ )
      rate *= y[compartment.index()][variableIndex(2,i)];
  else //No reaction defined...
    return;
  
  if (rate<=0.0) //No update
    return;

  for( size_t i=0 ; i< numVariableIndex(0) ; i++ )
    dydt[compartment.index()][variableIndex(0,i)] -= rate;
  
  for( size_t i=0 ; i< numVariableIndex(1) ; i++ )
    dydt[compartment.index()][variableIndex(1,i)] += rate;
}

size_t MassActionEnzymatic::
Jacobian(Compartment &compartment,size_t varIndex,DataMatrix &y,JacobianMatrix &A) 
{  
  double rate = parameter(0);
  if( numVariableIndex(0) )
    for (size_t k=0; k<numVariableIndex(0); ++k)
      rate *= y[compartment.index()][variableIndex(0,k)];
  else //No reaction defined...
    return 0;
  if( numVariableIndex(2) )
    for (size_t k=0 ; k<numVariableIndex(2); ++k)
      rate *= y[compartment.index()][variableIndex(2,k)];
  else //No reaction defined...
    return 0;
  if (rate<=0.0) //No update
    return 1;
  //
  // Update all A
  //
  // Dependence on reactants
  for (size_t k=0; k<numVariableIndex(0); ++k ) {
    double element = rate / y[compartment.index()][variableIndex(0,k)];
    size_t j = compartment.index()*y[0].size() + variableIndex(0,k); 
    // Reactants
    for (size_t kk=0; kk<numVariableIndex(0); ++kk ) {
      size_t i = compartment.index()*y[0].size() + variableIndex(0,kk); 
      A(i,j) -= element;
    }
    // products
    for (size_t kk=0; kk<numVariableIndex(1); ++kk ) {
      size_t i = compartment.index()*y[0].size() + variableIndex(1,kk); 
      A(i,j) += element;
    }
  }
  // Dependence on enzymes
  for (size_t k=0; k<numVariableIndex(2); ++k ) {
    double element = rate / y[compartment.index()][variableIndex(2,k)];
    size_t j = compartment.index()*y[0].size() + variableIndex(2,k); 
    // Reactants
    for (size_t kk=0; kk<numVariableIndex(0); ++kk ) {
      size_t i = compartment.index()*y[0].size() + variableIndex(0,kk); 
      A(i,j) -= element;
    }
    // products
    for (size_t kk=0; kk<numVariableIndex(1); ++kk ) {
      size_t i = compartment.index()*y[0].size() + variableIndex(1,kk); 
      A(i,j) += element;
    }
  }
  return 1;
}

MassActionMM::MassActionMM(std::vector<double> &paraValue, 
			   std::vector< std::vector<size_t> > 
			   &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=2 ) {
    std::cerr << "MassActionMM::MassActionMM() "
	      << "Uses two parameters V_max and K\n";
    exit(0);
  }
  if( indValue.size() !=2 ) {
    std::cerr << "MassActionMM::MassActionMM() "
	      << "Two levels of variable indeces are used.\n"
	      << "One for reactant and one for product\n";
    exit(0);
  }
  if( indValue[0].size() !=1 ) {
    std::cerr << "MassActionMM::MassActionMM() "
	      << "This reaction needs one reactant...\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("massActionMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "V_max";
  tmp[1] = "K";
  setParameterId( tmp );
}

void MassActionMM::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{  
  double rate = parameter(0)*y[compartment.index()][variableIndex(0,0)]
    / ( parameter(1) + y[compartment.index()][variableIndex(0,0)] );
  
  dydt[compartment.index()][variableIndex(0,0)] -= rate;  
  dydt[compartment.index()][variableIndex(1,0)] += rate;
}

MassActionEnzymaticMM::MassActionEnzymaticMM(std::vector<double> &paraValue,
																						 std::vector< std::vector<size_t> >
																						 &indValue ) {
	
//Do some checks on the parameters and variable indeces
//////////////////////////////////////////////////////////////////////
	if( paraValue.size()!=2 ) {
		std::cerr << "MassActionEnzymaticMM::MassActionMM() "
							<< "Uses two parameters V_max and K\n";
		exit(0);
	}
	if( indValue.size() !=3 ) {
		std::cerr << "MassActionEnzymaticMM::MassActionEnzymaticMM() "
							<< "Three levels of variable indeces are used.\n"
							<< "One for reactant, one for product and one for the enzyme\n";
		exit(0);
	}
	if( indValue[0].size() !=1 ) {
		std::cerr << "MassActionEnzymaticMM::MassActionEnzymaticMM() "
							<< "This reaction needs one reactant...\n";
		exit(0);
}
//Set the variable values
//////////////////////////////////////////////////////////////////////
	setId("massActionEnzymaticMM");
	setParameter(paraValue);
	setVariableIndex(indValue);
	
//Set the parameter identities
//////////////////////////////////////////////////////////////////////
	std::vector<std::string> tmp( numParameter() );
	tmp.resize( numParameter() );
	tmp[0] = "V_max";
	tmp[1] = "K";
	setParameterId( tmp );
}

void MassActionEnzymaticMM::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{
  double rate = parameter(0)*y[compartment.index()][variableIndex(0,0)]
    / ( parameter(1) + y[compartment.index()][variableIndex(0,0)] );
  
  rate *= y[compartment.index()][variableIndex(2,0)];
  
  dydt[compartment.index()][variableIndex(0,0)] -= rate;
  dydt[compartment.index()][variableIndex(1,0)] += rate;
}

MassActionMMFraction::MassActionMMFraction(std::vector<double> &paraValue,
																					 std::vector< std::vector<size_t> >
																					 &indValue ) {
    
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
	if( paraValue.size()!=3 ) {
		std::cerr << "MassActionMMFraction::MassActionMMFraction() "
							<< "Uses three parameters V_max, K and fraction\n";
		exit(0);
	}
	if( indValue.size() !=2 ) {
		std::cerr << "MassActionMMFraction::MassActionMMFraction() "
							<< "Two levels of variable indeces are used.\n"
							<< "One for reactant and one for product\n";
		exit(0);
	}
	if( indValue[0].size() !=1 ) {
		std::cerr << "MassActionMMFraction::MassActionMMFraction() "
							<< "This reaction needs one reactant...\n";
		exit(0);
	}
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
	setId("massActionMMFraction");
	setParameter(paraValue);
	setVariableIndex(indValue);
	
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
	std::vector<std::string> tmp( numParameter() );
	tmp.resize( numParameter() );
	tmp[0] = "V_max";
	tmp[1] = "K";
	tmp[2] = "f";
	setParameterId( tmp );
}

void MassActionMMFraction::
derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt) 
{
  double rate = parameter(0)*parameter(2)*y[compartment.index()][variableIndex(0,0)]
    / ( parameter(1) + parameter(2)*y[compartment.index()][variableIndex(0,0)] );
  
  dydt[compartment.index()][variableIndex(0,0)] -= rate;
  dydt[compartment.index()][variableIndex(1,0)] += rate;
}
