/**
 * Filename     : massAction.h
 * Description  : Classes describing mass action reactions
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : November 2003
 * Revision     : $Id: massAction.h 489 2011-04-13 11:21:21Z henrik $
 */
#ifndef MASSACTION_H
#define MASSACTION_H

#include "baseReaction.h"

///
/// @brief A one way mass action reaction
///
/// The reactant indeces are in level zero and products in level one in
/// variableIndex. One parameter (rate constant) is needed.
///
/// Reactants [R] and products [P] stored in variableIndexLevel 0/1 are
/// updated. The parameter is k_f in 
///
/// @f[ \frac{d[P]}{dt} = k_f * \prod [R] @f]
/// @f[ \frac{d[R]}{dt} = - k_f \prod [R] @f]
///
/// In a model file the reaction is defined as:
///
/// @verbatim
/// massAction 1 2 N_R N_P
/// k_f
/// R_1 ... R_{N_R}
/// R_1 ... R_{N_R}
/// @endverbatim
///
/// @note varIndex is not used.
///
class MassAction : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which sets the parameters and variable
  /// indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  MassAction(std::vector<double> &paraValue, 
	     std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt );
  ///
  /// @brief Jacobian function for this reaction class
  ///
  /// @see BaseReaction::Jacobian(Compartment &compartment,size_t species,...)
  ///
  size_t Jacobian(Compartment &compartment,size_t species,DataMatrix &y,JacobianMatrix &A);

  ///
  /// @brief Propensity calculation (probability to happen in [t,t+dt]) for this reaction class
  ///
  /// @see BaseReaction::propensity(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  /// @note This function is under construction and currently only works for the case
  /// when the reactants are of different kinds.
  ///
  double propensity(Compartment &compartment,size_t species,DataMatrix &y);

  ///
  /// @brief Discrete update for this reaction used in stochastic simulations
  ///
  /// @see BaseReaction::discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  void discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y);
};

///
/// @brief A one way mass action reaction assuming a single reactant and a single product
///
/// The reactant index are in level zero and the product in level one in
/// variableIndex. One parameter (rate constant) is needed.
///
/// Reactant [R] and product [P] stored in variableIndexLevel 0/1 are
/// updated. The parameter is k_f in 
///
/// @f[ \frac{d[P]}{dt} = k_f R @f]
/// @f[ \frac{d[R]}{dt} = - k_f R @f]
///
/// In a model file the reaction is defined as:
///
/// @verbatim
/// massAction1to1 1 2 1 1
/// k_f
/// R
/// P
/// @endverbatim
///
/// @note varIndex is not used.
///
class MassAction1to1 : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which sets the parameters and variable
  /// indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  MassAction1to1(std::vector<double> &paraValue, 
		 std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt );
  ///
  /// @brief Jacobian function for this reaction class
  ///
  /// @see BaseReaction::Jacobian(Compartment &compartment,size_t species,...)
  ///
  size_t Jacobian(Compartment &compartment,size_t species,DataMatrix &y,JacobianMatrix &A);

  ///
  /// @brief Propensity calculation (probability to happen in [t,t+dt]) for this reaction class
  ///
  /// @see BaseReaction::propensity(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  double propensity(Compartment &compartment,size_t species,DataMatrix &y);

  ///
  /// @brief Discrete update for this reaction used in stochastic simulations
  ///
  /// @see BaseReaction::discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  void discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y);
};

///
/// @brief A one way mass action reaction assuming a single reactant and a two products
///
/// The reactant index are in level zero and the two products in level one in
/// variableIndex. One parameter (rate constant) is needed.
///
/// Reactant R and products @f$P_1@f$, @f$P_2@f$ stored in variableIndexLevel 0/1 are
/// updated. The parameter is k_f in 
///
/// @f[ \frac{d[P_i]}{dt} = k_f R @f]
/// @f[ \frac{d[R]}{dt} = - k_f R @f]
///
/// In a model file the reaction is defined as:
///
/// @verbatim
/// massAction1to2 1 2 1 2
/// k_f
/// R
/// P_1 P_2
/// @endverbatim
///
/// @note varIndex is not used.
///
class MassAction1to2 : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which sets the parameters and variable
  /// indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  MassAction1to2(std::vector<double> &paraValue, 
		 std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt );
  ///
  /// @brief Jacobian function for this reaction class
  ///
  /// @see BaseReaction::Jacobian(Compartment &compartment,size_t species,...)
  ///
  size_t Jacobian(Compartment &compartment,size_t species,DataMatrix &y,JacobianMatrix &A);

  ///
  /// @brief Propensity calculation (probability to happen in [t,t+dt]) for this reaction class
  ///
  /// @see BaseReaction::propensity(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  double propensity(Compartment &compartment,size_t species,DataMatrix &y);

  ///
  /// @brief Discrete update for this reaction used in stochastic simulations
  ///
  /// @see BaseReaction::discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  void discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y);
};

///
/// @brief A one way mass action reaction assuming a single reactant and three products
///
/// The reactant index are in level zero and the three products in level one in
/// variableIndex. One parameter (rate constant) is needed.
///
/// Reactant R and products @f$P_i@f$ stored in variableIndexLevel 0/1 are
/// updated. The parameter is k_f in 
///
/// @f[ \frac{d[P_i]}{dt} = k_f R @f]
/// @f[ \frac{d[R]}{dt} = - k_f R @f]
///
/// In a model file the reaction is defined as:
///
/// @verbatim
/// massAction1to3 1 2 1 3
/// k_f
/// R
/// P_1 P_2 P_3
/// @endverbatim
///
/// @note varIndex is not used.
///
class MassAction1to3 : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which sets the parameters and variable
  /// indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  MassAction1to3(std::vector<double> &paraValue, 
		 std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt );
  ///
  /// @brief Jacobian function for this reaction class
  ///
  /// @see BaseReaction::Jacobian(Compartment &compartment,size_t species,...)
  ///
  size_t Jacobian(Compartment &compartment,size_t species,DataMatrix &y,JacobianMatrix &A);

  ///
  /// @brief Propensity calculation (probability to happen in [t,t+dt]) for this reaction class
  ///
  /// @see BaseReaction::propensity(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  double propensity(Compartment &compartment,size_t species,DataMatrix &y);

  ///
  /// @brief Discrete update for this reaction used in stochastic simulations
  ///
  /// @see BaseReaction::discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  void discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y);
};

///
/// @brief A one way mass action reaction assuming a single reactant and four products
///
/// The reactant index are in level zero and the four products in level one in
/// variableIndex. One parameter (rate constant) is needed.
///
/// Reactant R and products @f$P_i@f$ stored in variableIndexLevel 0/1 are
/// updated. The parameter is k_f in 
///
/// @f[ \frac{d[P_i]}{dt} = k_f R @f]
/// @f[ \frac{d[R]}{dt} = - k_f R @f]
///
/// In a model file the reaction is defined as:
///
/// @verbatim
/// massAction1to4 1 2 1 4
/// k_f
/// R
/// P_1 P_2 P_3 P_4
/// @endverbatim
///
/// @note varIndex is not used.
///
class MassAction1to4 : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which sets the parameters and variable
  /// indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  MassAction1to4(std::vector<double> &paraValue, 
		 std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt );
  ///
  /// @brief Jacobian function for this reaction class
  ///
  /// @see BaseReaction::Jacobian(Compartment &compartment,size_t species,...)
  ///
  size_t Jacobian(Compartment &compartment,size_t species,DataMatrix &y,JacobianMatrix &A);

  ///
  /// @brief Propensity calculation (probability to happen in [t,t+dt]) for this reaction class
  ///
  /// @see BaseReaction::propensity(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  double propensity(Compartment &compartment,size_t species,DataMatrix &y);

  ///
  /// @brief Discrete update for this reaction used in stochastic simulations
  ///
  /// @see BaseReaction::discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  void discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y);
};

///
/// @brief A one way mass action reaction assuming two reactants and a single product
///
/// The reactant index are in level zero and the product in level one in
/// variableIndex. One parameter (rate constant) is needed.
///
/// Reactants @f$R_1,R_2@f$ and product P stored in variableIndexLevel 0/1 are
/// updated. The parameter is k_f in 
///
/// @f[ \frac{d[P]}{dt} = k_f R_1 R_2 @f]
/// @f[ \frac{d[R_i]}{dt} = - k_f R_1 R_2 @f]
///
/// In a model file the reaction is defined as:
///
/// @verbatim
/// massAction2to1 1 2 2 1
/// k_f
/// R_1 R_2
/// P
/// @endverbatim
///
/// @note varIndex is not used.
///
class MassAction2to1 : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which sets the parameters and variable
  /// indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  MassAction2to1(std::vector<double> &paraValue, 
		 std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt );
  ///
  /// @brief Jacobian function for this reaction class
  ///
  /// @see BaseReaction::Jacobian(Compartment &compartment,size_t species,...)
  ///
  size_t Jacobian(Compartment &compartment,size_t species,DataMatrix &y,JacobianMatrix &A);

  ///
  /// @brief Propensity calculation (probability to happen in [t,t+dt]) for this reaction class
  ///
  /// @see BaseReaction::propensity(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  double propensity(Compartment &compartment,size_t species,DataMatrix &y);

  ///
  /// @brief Discrete update for this reaction used in stochastic simulations
  ///
  /// @see BaseReaction::discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  void discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y);
};

///
/// @brief A one way mass action reaction assuming three reactants and a single product
///
/// The reactant indices are in level zero and the product in level one in
/// variableIndex. One parameter (rate constant) is needed.
///
/// Reactants @f$R_i@f$ and product P stored in variableIndexLevel 0/1 are
/// updated. The parameter is k_f in 
///
/// @f[ \frac{d[P]}{dt} = k_f R_1 R_2 R_3 @f]
/// @f[ \frac{d[R_i]}{dt} = - k_f R_1 R_2 R_3 @f]
///
/// In a model file the reaction is defined as:
///
/// @verbatim
/// massAction2to1 1 2 3 1
/// k_f
/// R_1 R_2 R_3
/// P
/// @endverbatim
///
/// @note varIndex is not used.
///
class MassAction3to1 : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which sets the parameters and variable
  /// indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  MassAction3to1(std::vector<double> &paraValue, 
		 std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt );
  ///
  /// @brief Jacobian function for this reaction class
  ///
  /// @see BaseReaction::Jacobian(Compartment &compartment,size_t species,...)
  ///
  size_t Jacobian(Compartment &compartment,size_t species,DataMatrix &y,JacobianMatrix &A);

  ///
  /// @brief Propensity calculation (probability to happen in [t,t+dt]) for this reaction class
  ///
  /// @see BaseReaction::propensity(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  double propensity(Compartment &compartment,size_t species,DataMatrix &y);

  ///
  /// @brief Discrete update for this reaction used in stochastic simulations
  ///
  /// @see BaseReaction::discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y)
  ///
  void discreteUpdate(Compartment &compartment,size_t species,DataMatrix &y);
};

///
/// @brief A one way mass action reaction with enzyme contribution
///
/// The reactant indeces are in level zero, products in level one, and enzymes
/// in level two in variableIndex. One parameter (rate constant) is needed.
///
/// Reactants [R], products [P] and enzymes [E] are stored in
/// variableIndexLevel 0/1/2. [R] and [P] are updated. The parameter is k_f in
/// 
/// @f[ \frac{d[P]}{dt} = k_f \prod [E] \prod [R] @f]
/// @f[ \frac{d[R]}{dt} = - k_f \prod [E] \prod [R] @f]
/// @f[ \frac{d[E]}{dt} = 0 @f]
///
/// @note varIndex is not used.
/// 
class MassActionEnzymatic : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which sets the parameters and variable
  /// indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  MassActionEnzymatic(std::vector<double> &paraValue, 
		      std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt );
  ///
  /// @brief Jacobian function for this reaction class
  ///
  /// @see BaseReaction::Jacobian(Compartment &compartment,size_t species,...)
  ///
  size_t Jacobian(Compartment &compartment,size_t species,DataMatrix &y,JacobianMatrix &A);
};

///
/// @brief A one way mass action enzymatic reaction using Michaelis Menten
///
/// The reactant index is in level zero and the product in level one in
/// variableIndex.  V,K parameters are needed.
///
/// Reactants [R] and products [P] stored in variableIndexLevel 0/1 are
/// updated. The update is 
///
/// @f[ \frac{d[P]}{dt} = \frac{V_{max} [R]}{(K+[R])} @f]
/// @f[ \frac{d[R]}{dt} = - \frac{d[P]}{dt} @f]
///
/// @note varIndex is not used.
///
class MassActionMM : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which sets the parameters and variable
  /// indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  MassActionMM(std::vector<double> &paraValue, 
	       std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt );
};

///
/// @brief A one way mass action enzymatic reaction using Michaelis Menten
///
/// The reactant index is in level zero and the product in level one in
/// variableIndex.  V,K parameters are needed.
///
/// Reactants [R] and products [P] stored in variableIndexLevel 0/1 are
/// updated, and [E] is in variableIndexLevel 2 . The update is 
///
/// @f[ \frac{d[P]}{dt} = \frac{V_{max} [E][R]}{(K+[R])} @f]
/// @f[ \frac{d[R]}{dt} = - \frac{d[P]}{dt} @f]
/// @f[ \frac{d[E]}{dt} = 0 @f]
///
/// @note varIndex is not used.
///
class MassActionEnzymaticMM : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which sets the parameters and variable
  /// indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  MassActionEnzymaticMM(std::vector<double> &paraValue, 
			std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt );
};

///
/// @brief A one way mass action enzymatic reaction using Michaelis Menten and
/// a fraction of the molecular concentration
///
/// The reactant index is in level zero and the product in level one in
/// variableIndex.  V,K,f parameters are needed.
///
/// Reactant [R] and product [P] stored in variableIndexLevel 0/1 are updated
///
/// @f[ \frac{d[P]}{dt} = \frac{V_{max} f [R]}{(K + f[R])} @f]
/// @f[ \frac{d[R]}{dt} = - \frac{d[P]}{dt} @f]
///
/// @note varIndex is not used.
///
class MassActionMMFraction : public BaseReaction {
  
 public:
  
  ///
  /// @brief Main constructor
  ///
  /// This is the main constructor which sets the parameters and variable
  /// indices that defines the reaction.
  ///
  /// @param paraValue vector with parameters
  ///
  /// @param indValue vector of vectors with variable indices
  ///
  /// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  MassActionMMFraction(std::vector<double> &paraValue, 
		       std::vector< std::vector<size_t> > &indValue );
  
  ///
  /// @brief Derivative function for this reaction class
  ///
  /// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
  ///
  void derivs(Compartment &compartment,size_t varIndex,DataMatrix &y,DataMatrix &dydt );
};

#endif

