#include "auxinTransport.h"

namespace SimplifiedAuxinTransportJTB
{
	AuxinTransport::AuxinTransport(std::vector<double> &paraValue, 
		std::vector< std::vector<size_t> > &indValue)
	{
		if (paraValue.size() != 8)
		{
			std::cerr << "SimplifiedAuxinTransport::AuxinTransport::AuxinTransport "
			<< "Uses eight parameters: \n"
			<< "D, "             // 0
			<< "k1, "            // 1
			<< "k2, "            // 2
			<< "K_H, "           // 3
			<< "n, "             // 4
			<< "K_M, "           // 5
			<< "c_a, "           // 6
			<< "and d_a.\n";     // 7
			exit(EXIT_FAILURE);
		}
		
		if (indValue.size() != 1 || indValue[0].size() != 2)
		{
			std::cerr << "SimplifiedAuxinTransport::AuxinTransport::AuxinTransport "
			<< "Two variable indices are needed: \n"
			<< "Auxin and "      // 0
			<< "PIN1.\n";        // 1
			exit(EXIT_FAILURE);
		}
		
		setId("auxinTransport");
		setParameter(paraValue);  
		setVariableIndex(indValue);
		
		std::vector<std::string> tmp(numParameter());
		tmp.resize(numParameter());
		
		tmp[0] = "D";
		tmp[1] = "k1";
		tmp[2] = "k2";
		tmp[3] = "K_H";
		tmp[4] = "n";
		tmp[5] = "K_M";
		tmp[6] = "c_a";
		tmp[7] = "d_a";
		
		setParameterId(tmp);
	}
	
	void AuxinTransport::derivs(Compartment &compartment, size_t species,
		std::vector< std::vector<double> > &y,
		std::vector< std::vector<double> > &dydt)
	{
		size_t i = compartment.index();
		
		double sum = 1.0;
		
		for (size_t n = 0; n < compartment.numNeighbor(); ++n)
		{
			size_t k = compartment.neighbor(n);
			sum += PIN1Feedback(y[k][variableIndex(0, 0)]);
		}

		const double D = parameter(0);

		for (size_t n = 0; n < compartment.numNeighbor(); ++n)
		{
			size_t j = compartment.neighbor(n);
			
			double passive = D * y[i][variableIndex(0, 0)];
			
			double Pij = (PIN1Feedback(y[j][variableIndex(0, 0)]) * y[i][variableIndex(0, 1)]) / sum;		
			
			double active = AuxinActiveTransport(y[i][variableIndex(0, 0)]) * Pij; 
			
			dydt[i][species] -= (passive + active);
			dydt[j][species] += (passive + active);
		}

		const double C_a = parameter(6);
		const double D_a = parameter(7);
		
		dydt[i][species] += 2.0 * C_a;
		dydt[i][species] -= 2.0 * D_a * y[i][variableIndex(0, 0)];
	}
	
	double AuxinTransport::PIN1Feedback(double auxin)
	{
		const double k1 = parameter(1);
		const double k2 = parameter(2);
		const double K_H = parameter(3);
		const double n = parameter(4);

		return k1 + (k2 * std::pow(auxin, n) / (std::pow(K_H, n) + std::pow(auxin, n)));
	}
	
	double AuxinTransport::AuxinActiveTransport(double auxin)
	{
		const double K_M = parameter(5);
		return auxin / (K_M + auxin);
	}	
}















