/**
 * Filename     : polarizationTransport.cc
 * Description  : Classes describing updates due to polarized molecule transports
 * Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
 * Created      : December 2007
 * Revision     : $Id: transport.cc 324 2007-12-07 15:44:17Z pontus $
 */
#include"polarizationTransport.h"
#include"../baseReaction.h"

PolarizationTransport::PolarizationTransport(std::vector<double> &paraValue, 
					     std::vector< std::vector<size_t> > 
					     &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=1 ) {
    std::cerr << "PolarizationTransport::PolarizationTransport() "
	      << "uses only one parameter D.\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationTransport::PolarizationTransport() "
	      << "Pin1 + polarization signal indeces needed in level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationTransport");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  setParameterId( tmp );
}

void PolarizationTransport::derivs(Compartment &compartment,size_t species,
				   std::vector< std::vector<double> > &y,
				   std::vector< std::vector<double> > &dydt ) {
    
  size_t i=compartment.index();
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationTransport::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
    sum += y[ compartment.neighbor(n) ][ variableIndex(0,1) ];
  //ad hoc way to include "wall area"
  sum /= compartment.numNeighbor();
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationTransport::derivs Given volume " << Vj
		<< " unphysical.\n";
      exit(-1);
    }
    
    double Pij;
    if( sum > 0.0 ) Pij = y[j][ variableIndex(0,1) ]/sum;
    else {
      Pij=1.;
      if( sum< 0.0 ) {
	std::cerr << "PolarizationTransport::derivs() WARNING Strange "
		  << "polarization signal.\n";
	std::cerr << i << " " << species << " " << variableIndex(0,0) << " " 
		  << variableIndex(0,1) << " " << sum << "\n";   
      }
    }
    double coeff = y[i][species]*y[i][variableIndex(0,0)]*Pij;
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;    
  }
}

PolarizationTransportFast::PolarizationTransportFast(std::vector<double> 
						     &paraValue, 
						     std::vector< 
						     std::vector<size_t> > 
						     &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=2 ) {
    std::cerr << "PolarizationTransportFast::PolarizationTransportFast() "
	      << "uses two parameters (D and alpha/beta).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationTransport::PolarizationTransport() "
	      << "Polarized molecule + polarization signal indeces needed in "
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationTransportFast");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b/a";
  setParameterId( tmp );
}

void PolarizationTransportFast::derivs(Compartment &compartment,size_t species,
				       std::vector< std::vector<double> > &y,
				       std::vector< std::vector<double> > 
				       &dydt ) {
    
  size_t i=compartment.index();
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationTransportFast::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
    sum += y[ compartment.neighbor(n) ][ variableIndex(0,1) ];
  //sum /= compartment.numNeighbor();//For adjusting for different num neigh
  sum += parameter(1);

  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationTransportFast::derivs Given volume " << Vj
		<< " unphysical.\n";
      exit(-1);
    }
    
    //double Pij = y[i][variableIndex(0,0)] * 
    //y[j][ variableIndex(0,1) ] /sum;
    double coeff =0.0;
    if( sum != 0.0 )
      coeff = y[i][species]*y[i][variableIndex(0,0)] *
	y[j][ variableIndex(0,1) ] /sum;
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

//!Constructor for the PolarizationTransportFastMM class
/*!  The Polarized transporter molecule P is divided into the cell
  cytosol (P_i) and its membranes (P_ij) following the dynamics
  
  dP_i/dt = -a P_i sum_k A_k + b sum_k P_ik 
  dP_ij/dt = a P_i A_j - b P_ij 
  
  where k is summed over neighboring cells, A is the polarizing signal
  and a,b are constants. The polarization is assumed to be fast and
  the fixed point values are assumed
  
  P_i = (b/a)*P / (sum_k A_k + (b/a))
  P_ij = P * A_j / (sum_k A_k + (b/a))
  
  where P is the total amount of polarized molecule in a cell.
  
  The polarized transport of a molecule X between two neighboring
  cells is then given by a Michaelis Menten formalism
  
  dX_i/dt = T * ( P_ij*X_i/(X_i+K) - P_ji*X_j/(X_j+K) ).
  
  The parameters given to this function are T and K'=b/a, and K.  The
  volumes for the cells are also included for correct concentration
  update.
*/
PolarizationTransportFastMM::PolarizationTransportFastMM(std::vector<double> 
							 &paraValue, 
							 std::vector< 
							 std::vector<size_t> > 
							 &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=3 ) {
    std::cerr << "PolarizationTransportFastMM::PolarizationTransportFastMM() "
	      << "uses three parameters (T, b/a and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationTransport::PolarizationTransport() "
	      << "Polarized molecule + polarization signal indeces needed in "
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationTransportFastMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b/a";
  tmp[2] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationTransportFastMM::derivs(Compartment &compartment,size_t species,
				       std::vector< std::vector<double> > &y,
				       std::vector< std::vector<double> > 
				       &dydt ) {
    
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationTransportFastMM::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
    sum += y[ compartment.neighbor(n) ][ variableIndex(0,1) ];
  sum /= compartment.numNeighbor();//To lower dep on num neighs
  sum += parameter(1);
  double invSum = 1.0/sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationTransportFastMM::derivs Given volume " << Vj
		<< " unphysical.\n";
      exit(-1);
    }
    
    double Pij = y[i][variableIndex(0,0)] * 
      y[j][ variableIndex(0,1) ] * invSum;
    double coeff = Pij*y[i][species]/( parameter(2)+y[i][species] );
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

//!Constructor for the SymmetricTransportFastMM class
/*!The active transport of a molecule X between two neighboring
  cells is given by a Michaelis Menten formalism
  
  dX_i/dt = T * ( P_ij*X_i/(X_i+K_M) - P_ji*X_j/(X_j+K_M) ).
  
  The parameters given to this function are T and K_M (maximal rate
  and MM constant). P is the efflux mediating molecule. The volumes
  for the cells are also included for correct concentration update.
*/
SymmetricTransportFastMM::
SymmetricTransportFastMM(std::vector<double> &paraValue, 
			 std::vector< std::vector<size_t> > &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=2 ) {
    std::cerr << "SymmetricTransportFastMM::SymmetricTransportFastMM() "
	      << "uses two parameters (T and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=1 
      || indValue[1].size() !=1  ) {
    std::cerr << "SymmetricTransport::SymmetricTransport() "
	      << "Polarized molecule index needed in "
	      << "level 1.\n" 
	      << "Variable index for size parameter needed "
	      << "at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("symmetricTransportFastMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void SymmetricTransportFastMM::derivs(Compartment &compartment,size_t species,
				       std::vector< std::vector<double> > &y,
				       std::vector< std::vector<double> > 
				       &dydt ) {
    
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "SymmetricTransportFastMM::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    if( i<j ) {
      double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
      if( Vj<=0.0 ) {
	std::cerr << "SymmetricTransportFastMM::derivs Given volume " << Vj
		  << " unphysical.\n";
	exit(-1);
      }
      double Pij = y[i][variableIndex(0,0)];
      double Pji = y[j][variableIndex(0,0)];
      // i -> j
      double coeff = Pij*y[i][species]/( parameter(2)+y[i][species] );
      //dydt[i][species] -= parameter(0)*coeff/Vi;
      //dydt[j][species] += parameter(0)*coeff/Vj;
      dydt[i][species] -= parameter(0)*coeff;
      dydt[j][species] += parameter(0)*coeff;
      // j -> i
      coeff = Pji*y[i][species]/( parameter(2)+y[i][species] );
      //dydt[i][species] += parameter(0)*coeff/Vi;
      //dydt[j][species] -= parameter(0)*coeff/Vj;
      dydt[i][species] += parameter(0)*coeff;
      dydt[j][species] -= parameter(0)*coeff;
    }
  }
}

//!Constructor for the PolarizationTransportInfluxFastMM class
/*!  The Polarized transporter molecule P is divided into the cell
  cytosol (P_i) and its membranes (P_ij) following the dynamics
  
  dP_i/dt = -a P_i sum_k A_k + b sum_k P_ik 
  dP_ij/dt = a P_i A_j - b P_ij 
  
  where k is summed over neighboring cells, A is the polarizing signal
  and a,b are constants. The polarization is assumed to be fast and
  the fixed point values are assumed
  
  P_i = (b/a)*P / (sum_k A_k + (b/a))
  P_ij = P * A_j / (sum_k A_k + (b/a))
  
  where P is the total amount of polarized molecule in a cell.
  
  The polarized transport of a molecule X between two neighboring
  cells is then given by a Michaelis Menten formalism
  
  dX_i/dt = T * ( P_ij*X_i/(X_i+K) * L_j/(L_i+L_j) 
  - P_ji*X_j/(X_j+K) * L_i/(L_i+L_j) ).
  
  The parameters given to this function are T and K'=b/a, and K. L is
  the influx carrier assumed to be symmetrically polarized. The
  volumes for the cells are also included for correct concentration
  update.
*/
PolarizationTransportInfluxFastMM::
PolarizationTransportInfluxFastMM(std::vector<double> &paraValue, 
				  std::vector< std::vector<size_t> > 
				  &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=3 ) {
    std::cerr << "PolarizationTransportInfluxFastMM::"
	      << "PolarizationTransportInfluxFastMM() "
	      << "uses three parameters (T, b/a and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=3 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationTransport::PolarizationTransport() "
	      << "Polarized efflux molecule, polarization signal, "
	      << "and influx carrier indeces needed in level 1.\n" 
	      << "Variable index for size parameter needed at "
	      << "second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationTransportInfluxFastMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b/a";
  tmp[2] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationTransportInfluxFastMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > &dydt ) {
    
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationTransportInfluxFastMM::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
    sum += y[ compartment.neighbor(n) ][ variableIndex(0,1) ];
  sum /= compartment.numNeighbor();//To lower dep on num neighs
  sum += parameter(1);
  double invSum = 1.0/sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationTransportInfluxFastMM::derivs "
		<< "Given volume " << Vj << " unphysical.\n";
      exit(-1);
    }
    
    double Pij = y[i][variableIndex(0,0)] * 
      y[j][ variableIndex(0,1) ] * invSum;
    double influxSum = y[i][variableIndex(0,2)]+y[j][variableIndex(0,2)];
    if( influxSum<=0.0 ) return;
    double coeff = Pij*y[i][species]*y[j][variableIndex(0,2)] /
      ( ( parameter(2)+y[i][species] )*influxSum );
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

//!Constructor for the PolarizationHillTransportInfluxFastMM class
/*!  The Polarized transporter molecule P is divided into the cell
  cytosol (P_i) and its membranes (P_ij) following the 'Hill' dynamics
  
  dP_i/dt = -a P_i sum_k f(A_k) + b sum_k P_ik 
  dP_ij/dt = a P_i f(A_j) - b P_ij , where 
  f(A_k) = A_k^n/(K^n + A_k^n)

  where k is summed over neighboring cells, A is the polarizing signal
  and a,b are rate constants and K, n are the Hill constant and
  coefficient. The polarization is assumed to be fast and the fixed
  point values are assumed
  
  P_i = (b/a)*P / (sum_k f(A_k) + (b/a))
  P_ij = P * f(A_j) / (sum_k f(A_k) + (b/a))
  
  where P is the total amount of polarized molecule in a cell.
  
  The polarized transport of a molecule X between two neighboring
  cells is then given by a Michaelis Menten formalism
  
  dX_i/dt = T * ( P_ij*X_i/(X_i+K_m) * L_j/(L_i+L_j) 
  - P_ji*X_j/(X_j+K_m) * L_i/(L_i+L_j) ).
  
  The parameters given to this function are T, K'=b/a, K_m, K, and
  n. L is the influx carrier assumed to be symmetrically
  polarized. The volumes for the cells are also included for correct
  concentration update.
*/
PolarizationHillTransportInfluxFastMM::
PolarizationHillTransportInfluxFastMM(std::vector<double> &paraValue, 
				  std::vector< std::vector<size_t> > 
				  &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=5 ) {
    std::cerr << "PolarizationHillTransportInfluxFastMM::"
	      << "PolarizationHillTransportInfluxFastMM() "
	      << "uses five parameters (T, b/a, K_M, K, n).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=3 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationHillTransport::PolarizationHillTransport() "
	      << "Polarized efflux molecule, polarization signal, "
	      << "and influx carrier indeces needed in level 1.\n" 
	      << "Variable index for size parameter needed at "
	      << "second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationHillTransportInfluxFastMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b/a";
  tmp[2] = "K_M";
  tmp[2] = "K_H";
  tmp[2] = "n_H";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationHillTransportInfluxFastMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > &dydt ) {
  
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationHillTransportInfluxFastMM::derivs "
	      << "Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  double KtoN = std::pow( parameter(3),parameter(4) );
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    double ytoN = std::pow( y[ compartment.neighbor(n) ]
			    [ variableIndex(0,1) ],parameter(4) );
    sum += ytoN/(KtoN+ytoN);
  }
  sum /= compartment.numNeighbor();//To lower dep on num neighs
  sum += parameter(1);
  double invSum = 1.0/sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationHillTransportInfluxFastMM::derivs "
		<< "Given volume " << Vj << " unphysical.\n";
      exit(-1);
    }
    
    double ytoN = std::pow(y[j][ variableIndex(0,1) ],parameter(4));
    double Pij = y[i][variableIndex(0,0)] * 
      ( ytoN/(KtoN+ytoN) ) * invSum;
    double influxSum = y[i][variableIndex(0,2)]+y[j][variableIndex(0,2)];
    if( influxSum<=0.0 ) return;
    double coeff = Pij*y[i][species]*y[j][variableIndex(0,2)] /
      ( ( parameter(2)+y[i][species] )*influxSum );
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

//!Constructor for the SymmetricTransportInfluxFastMM class
/*!The transport of a molecule X between two neighboring cells is
  given by a Michaelis Menten formalism
  
  dX_i/dt = - T*( P_i*L_j*X_i/(X_i+K_M) -
  P_j*L_i*X_j/(X_j+K_M))/(L_i+L_j).
  
  The parameters given to this function are T and K_M (max rate and MM
  constant). P and L are the efflux and influx carriers assumed to be
  symmetrically polarized. The volumes for the cells are also included
  for correct concentration update.
*/
SymmetricTransportInfluxFastMM::
SymmetricTransportInfluxFastMM(std::vector<double> &paraValue, 
				  std::vector< std::vector<size_t> > 
				  &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=2 ) {
    std::cerr << "SymmetricTransportInfluxFastMM::"
	      << "SymmetricTransportInfluxFastMM() "
	      << "uses two parameters (T and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "SymmetricTransport::SymmetricTransport() "
	      << "Efflux molecule "
	      << "and influx carrier indeces needed in level 1.\n" 
	      << "Variable index for size parameter needed at "
	      << "second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("symmetricTransportInfluxFastMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void SymmetricTransportInfluxFastMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > &dydt ) {
    
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "SymmetricTransportInfluxFastMM::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    if( i<j ) {
      double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
      if( Vj<=0.0 ) {
	std::cerr << "SymmetricTransportInfluxFastMM::derivs "
		  << "Given volume " << Vj << " unphysical.\n";
	exit(-1);
      }
      
      double Pij = y[i][variableIndex(0,0)];
      double Pji = y[j][variableIndex(0,0)];
      double Lij = y[i][variableIndex(0,1)];
      double Lji = y[j][variableIndex(0,1)];
    
      double influxSum = Lij+Lji;
      if( influxSum<=0.0 ) return;
      // i -> j
      double coeff = Pij*Lji*y[i][species] /
	( ( parameter(1)+y[i][species] )*influxSum );
      //dydt[i][species] -= parameter(0)*coeff/Vi;
      //dydt[j][species] += parameter(0)*coeff/Vj;
      dydt[i][species] -= parameter(0)*coeff;
      dydt[j][species] += parameter(0)*coeff;
      // j -> i
      coeff = Pji*Lij*y[j][species] /
	( ( parameter(1)+y[j][species] )*influxSum );
      //dydt[i][species] += parameter(0)*coeff/Vi;
      //dydt[j][species] -= parameter(0)*coeff/Vj;
      dydt[i][species] += parameter(0)*coeff;
      dydt[j][species] -= parameter(0)*coeff;
    }
  }
}

//!Constructor for the PolarizationTransportFastMMSpatial class
/*!  The Polarized transporter molecule P is divided into the cell
  cytosol (P_i) and its membranes (P_ij) following the dynamics
  
  dP_i/dt = -a P_i sum_k A_k + b sum_k P_ik 
  dP_ij/dt = a P_i A_j - b P_ij 
  
  where k is summed over neighboring cells, A is the polarizing signal
  and a,b are constants. The polarization is assumed to be fast and
  the fixed point values are assumed
  
  P_i = (b/a)*P / (sum_k A_k + (b/a))
  P_ij = P * A_j / (sum_k A_k + (b/a))
  
  where P is the total amount of polarized molecule in a cell.
  
  The polarized transport of a molecule X between two neighboring
  cells is then given by a Michaelis Menten formalism
  
  dX_i/dt = T * ( a_ij*P_ij*X_i/(X_i+K) - a_ji*P_ji*X_j/(X_j+K) ).
  
  The parameters given to this function are T and K'=b/a, and K.  a_ij
  is the crossing area. The volumes for the cells are also included for
  correct concentration update.
*/
PolarizationTransportFastMMSpatial::PolarizationTransportFastMMSpatial(std::vector<double> 
							 &paraValue, 
							 std::vector< 
							 std::vector<size_t> > 
							 &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=3 ) {
    std::cerr << "PolarizationTransportFastMMSpatial::PolarizationTransportFastMMSpatial() "
	      << "uses three parameters (T, b/a and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationTransportFastMMSpatial::PolarizationTransport() "
	      << "Polarized molecule + polarization signal indeces needed in "
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationTransportFastMMSpatial");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b/a";
  tmp[2] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationTransportFastMMSpatial::derivs(Compartment &compartment,size_t species,
				       std::vector< std::vector<double> > &y,
				       std::vector< std::vector<double> > 
				       &dydt ) {
    
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationTransportFastMMSpatial::derivs Given volume " 
	      << Vi << " unphysical.\n";
    exit(-1);
  }
  //Areas
  double areaSum=0.0;
  std::vector<double> area(compartment.numNeighbor());
  for( size_t k=0 ; k<compartment.numNeighbor() ; k++ ) {
    size_t n=compartment.neighbor(k);
    double d2=0.0;
    for( size_t dim=0 ; dim<compartment.numDimension() ; dim++ )
      d2 += (y[i][dim]-y[n][dim])*(y[i][dim]-y[n][dim]);
    double d = std::sqrt(d2);
    double ri=y[i][compartment.numDimension()];
    double rn=y[n][compartment.numDimension()];
    if( d>ri+rn ) area[k]=0.0;
    else if( d<ri || d<rn ) {
      area[k] = ri<rn ? ri : rn;
    }
    else
      area[k] = std::sqrt( 0.25*(d2-2.*(ri*ri-rn*rn)*(ri*ri-rn*rn) +
				 (ri-rn)*(ri-rn)/d2 ) );
    areaSum += area[k];
    if( areaSum<=0.0 ) areaSum=1.0;
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
    sum += y[ compartment.neighbor(n) ][ variableIndex(0,1) ]*area[n]/areaSum;
  //sum /= compartment.numNeighbor();//To lower dep on num neighs
  sum += parameter(1);
  double invSum = 1.0/sum;//To use mult instead of div

  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationTransportFastMMSpatial::derivs Given volume " << Vj
		<< " unphysical.\n";
      exit(-1);
    }
    
    double Pij = y[i][variableIndex(0,0)] * 
      y[j][ variableIndex(0,1) ] * invSum;
    double coeff = area[n]*Pij*y[i][species]/( parameter(2)+y[i][species] );
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

PolarizationHillTransportFast::
PolarizationHillTransportFast(std::vector<double> &paraValue, 
			      std::vector< std::vector<size_t> > &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=4 ) {
    std::cerr << "PolarizationHillTransportFast::"
	      << "PolarizationHillTransportFast() "
	      << "uses four parameters (T, b/a and K_H,n_H).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationHillTransportFast::"
	      << "PolarizationHillTransportFast() "
	      << "Polarized molecule + polarization signal indeces needed in "
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationHillTransportFast");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b/a";
  tmp[2] = "K_H";
  tmp[3] = "K_n";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationHillTransportFast::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > &dydt ) {
  
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationHillTransportFast::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    double Ak = y[ compartment.neighbor(n) ][ variableIndex(0,1) ]; 
    sum += std::pow(Ak,parameter(3))/
      (std::pow(parameter(2),parameter(3)) + std::pow(Ak,parameter(3)) );
  }
  sum /= compartment.numNeighbor();
  sum += parameter(1);
  double invSum = 1.0/sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationHillTransportFast::derivs Given volume " << Vj
		<< " unphysical.\n";
      exit(-1);
    }
    double Aj = y[j][ variableIndex(0,1) ];
    double Pij = y[i][variableIndex(0,0)] * invSum *
      std::pow(Aj,parameter(3))/
      (std::pow(parameter(2),parameter(3)) + std::pow(Aj,parameter(3)) );
    
    double coeff = Pij*y[i][species];
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

PolarizationHillTransportFastMM::
PolarizationHillTransportFastMM(std::vector<double> &paraValue, 
			      std::vector< std::vector<size_t> > &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=5 ) {
    std::cerr << "PolarizationHillTransportFastMM::"
	      << "PolarizationHillTransportFastMM() "
	      << "uses five parameters (T, b/a, K_H,n_H and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationHillTransportFastMM::"
	      << "PolarizationHillTransportFastMM() "
	      << "Polarized molecule + polarization signal indeces needed in "
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationHillTransportFastMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b/a";
  tmp[2] = "K_H";
  tmp[3] = "n_H";
  tmp[4] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationHillTransportFastMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > &dydt ) {
  
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationHillTransportFastMM::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    double Ak = y[ compartment.neighbor(n) ][ variableIndex(0,1) ]; 
    sum += std::pow(Ak,parameter(3))/
      (std::pow(parameter(2),parameter(3)) + std::pow(Ak,parameter(3)) );
  }
  sum /= compartment.numNeighbor();
  sum += parameter(1);
  double invSum = 1.0/sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationHillTransportFastMM::derivs Given volume " 
		<< Vj << " unphysical.\n";
      exit(-1);
    }
    double Aj = y[j][ variableIndex(0,1) ];
    double Pij = y[i][variableIndex(0,0)] * invSum *
      std::pow(Aj,parameter(3))/
      (std::pow(parameter(2),parameter(3)) + std::pow(Aj,parameter(3)) );
    
    double coeff = Pij*y[i][species]/(parameter(4)+y[i][species]);
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

//!Polarized transport where exponential feedback assumed 
PolarizationExpTransportFastMM::
PolarizationExpTransportFastMM(std::vector<double> &paraValue, 
			       std::vector< std::vector<size_t> > 
			       &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=3 ) {
    std::cerr << "PolarizationExpTransportFastMM::"
	      << "PolarizationExpTransportFastMM() "
	      << "uses five parameters (T, b, and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationExpTransportFastMM::"
	      << "PolarizationExpTransportFastMM() "
	      << "Polarized molecule + polarization signal indeces needed in "
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationExpTransportFastMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b";
  tmp[2] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationExpTransportFastMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > &dydt ) {
  
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationExpTransportFastMM::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    double Ak = y[ compartment.neighbor(n) ][ variableIndex(0,1) ]; 
    sum += std::pow(parameter(1),Ak);
  }
  //sum /= compartment.numNeighbor();
  double invSum = 1.0/sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationExpTransportFastMM::derivs Given volume " 
		<< Vj << " unphysical.\n";
      exit(-1);
    }
    double Aj = y[j][ variableIndex(0,1) ];
    double Pij = y[i][variableIndex(0,0)] * invSum *
      std::pow(parameter(1),Aj);
    
    double coeff = Pij*y[i][species]/(parameter(2)+y[i][species]);
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

//!Constructor for the PolarizationConstLinTransportFast class
PolarizationConstLinTransportFast::
PolarizationConstLinTransportFast(std::vector<double> &paraValue, 
			      std::vector< std::vector<size_t> > &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=3 ) {
    std::cerr << "PolarizationConstLinTransportFast::"
	      << "PolarizationConstLinTransportFast() "
	      << "uses three parameters (T, b/a and c).\n";
    exit(0);
  }
  if( indValue.size() != 1 || indValue[0].size() !=2 ) {
    std::cerr << "PolarizationConstLinTransportFast::"
	      << "PolarizationConstLinTransportFast() "
	      << "Polarized molecule + polarization signal indeces "
	      << "needed in level 1.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationConstLinTransportFast");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b/a";
  tmp[2] = "c";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationConstLinTransportFast::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > &dydt ) {
  
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  //double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  //if( Vi<=0.0 ) {
  //std::cerr << "PolarizationConstLinTransportFast::derivs Given volume " << Vi
  //      << " unphysical.\n";
  //exit(-1);
  //}
  //Polarization coefficient normalization constant
  double sum = parameter(2)*compartment.numNeighbor();
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    sum += y[ compartment.neighbor(n) ][ variableIndex(0,1) ]; 
  }
  sum /= compartment.numNeighbor();
  sum += parameter(1);
  if( sum<= 0.0 )
    return;
  double invSum = 1.0/sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    //double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    //if( Vj<=0.0 ) {
    //std::cerr << "PolarizationConstLinTransportFast::derivs Given volume " << Vj
    //	<< " unphysical.\n";
    //exit(-1);
    //}
    double PijA = y[i][variableIndex(0,0)] * invSum *
      ( parameter(2)+y[j][ variableIndex(0,1) ] ) * y[i][species];
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*PijA;
    dydt[j][species] += parameter(0)*PijA;
  }
}


//!Constructor for the PolarizationTransportFastB class
/*!  The Polarized transporter molecule P is divided into the cell
  cytosol (P_i) and its membranes (P_ij) following the dynamics
  
  dP_i/dt = -a P_i + b sum_k P_ik/(K+A_k) 
  dP_ij/dt = a P_i - b P_ij/(K+A_j) 
  
  where k is summed over neighboring cells, A is the polarizing signal
  and a,b,K are constants. The polarization is assumed to be fast and
  the fixed point values are assumed
  
  P_i = b/a * P / ( b/a + sum_k (K+A_k) )
  P_ij = (K+A_j) / ( b/a + sum_k (K+A_k) )
  
  where P is the total amount of polarized molecule in a cell.
  
  The polarized transport of a molecule X between two neighboring
  cells is then given by
  
  dX_i/dt = T * ( X_i*P_ij - X_j*P_ji ).
  
  The parameters given to this function are T, K and k=b/a. The
  volumes for the cells are also included for correct concentration
  update.
*/
PolarizationTransportFastB::PolarizationTransportFastB(std::vector<double> 
						       &paraValue, 
						       std::vector< 
						       std::vector<size_t> > 
						       &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=3 ) {
    std::cerr << "PolarizationTransportFastB::PolarizationTransportFastB() "
	      << "uses three parameters (T, K and b/a).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationTransport::PolarizationTransport() "
	      << "Polarized molecule + polarization signal indeces needed in "
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationTransportFastB");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "K";
  tmp[1] = "b/a";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationTransportFastB::derivs(Compartment &compartment,size_t species,
				       std::vector< std::vector<double> > &y,
				       std::vector< std::vector<double> > 
				       &dydt ) {
    
  size_t i=compartment.index();
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationTransportFastB::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=parameter(1) * compartment.numNeighbor();
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
    sum += y[ compartment.neighbor(n) ][ variableIndex(0,1) ];
  sum += parameter(2);
  double invSum = 1.0/sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationTransportFastB::derivs Given volume " << Vj
		<< " unphysical.\n";
      exit(-1);
    }
    
    double Pij = ( parameter(1) + y[j][ variableIndex(0,1) ]) * 
      y[i][variableIndex(0,0)] * invSum;
    double coeff = y[i][species]*Pij;
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

//!Constructor for the PolarizationTransportFastBMM class
/*!  The Polarized transporter molecule P is divided into the cell
  cytosol (P_i) and its membranes (P_ij) following the dynamics
  
  dP_i/dt = -a P_i + b sum_k P_ik/(K+A_k) 
  dP_ij/dt = a P_i - b P_ij/(K+A_j) 
  
  where k is summed over neighboring cells, A is the polarizing signal
  and a,b,K are constants. The polarization is assumed to be fast and
  the fixed point values are assumed
  
  P_i = b/a * P / ( b/a + sum_k (K+A_k) )
  P_ij = (K+A_j) / ( b/a + sum_k (K+A_k) )
  
  where P is the total amount of polarized molecule in a cell.
  
  The polarized transport of a molecule X between two neighboring
  cells is then given by
  
  dX_i/dt = T * ( P_ij*X_i/(X_i+K_M) - P_ji*X_j/(X_j+K_M) ).
  
  The parameters given to this function are T, K and k=b/a. The
  volumes for the cells are also included for correct concentration
  update.
*/
PolarizationTransportFastBMM::PolarizationTransportFastBMM(std::vector<double> 
							   &paraValue, 
							   std::vector< 
							   std::vector<size_t> > 
							   &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=4 ) {
    std::cerr << "PolarizationTransportFastBMM::PolarizationTransportFastBMM()"
	      << " uses four parameters (T, K, b/a and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=2 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationTransport::PolarizationTransport() "
	      << "Polarized molecule + polarization signal indeces needed in "
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationTransportFastBMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "K";
  tmp[2] = "b/a";
  tmp[3] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationTransportFastBMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > 
       &dydt ) {
  
  size_t i=compartment.index();
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationTransportFastBMM::derivs Given volume " << Vi
	      << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=parameter(1) * compartment.numNeighbor();
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
    sum += y[ compartment.neighbor(n) ][ variableIndex(0,1) ];
  sum += parameter(2);
  double invSum = 1.0/sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationTransportFastBMM::derivs Given volume " << Vj
		<< " unphysical.\n";
      exit(-1);
    }
    
    double Pij = ( parameter(1) + y[j][ variableIndex(0,1) ]) * 
      y[i][variableIndex(0,0)] * invSum;
    double coeff = Pij*y[i][species]/( y[i][species]+parameter(3) );
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

//!Constructor for the PolarizationTransportFastSwitchedMM class
/*!  The Polarized transporter molecule P is divided into the cell
  cytosol (P_i) and its membranes (P_ij) following the dynamics
  
  dP_i/dt = -a P_i sum_k A_k + b sum_k P_ik 
  dP_ij/dt = a P_i A_j - b P_ij 
  
  where k is summed over neighboring cells, A is the polarizing signal
  and a,b are constants. The polarization is assumed to be fast and
  the fixed point values are assumed
  
  P_i = (b/a)*P / (sum_k A_k + (b/a))
  P_ij = P * A_j / (sum_k A_k + (b/a))
  
  where P is the total amount of polarized molecule in a cell.
  
  The polarized transport of a molecule X between two neighboring
  cells is then given by a Michaelis Menten formalism
  
  dX_i/dt = T * ( P_ij*X_i/(X_i+K) - P_ji*X_j/(X_j+K) ).
  
  The parameters given to this function are T and K'=b/a, K_M and
  Th_switch.  The volumes for the cells are also included for correct
  concentration update.
*/
PolarizationTransportFastSwitchedMM::
PolarizationTransportFastSwitchedMM(std::vector<double> 
				    &paraValue, 
				    std::vector< 
				    std::vector<size_t> > 
				    &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=4 ) {
    std::cerr << "PolarizationTransportFastSwitchedMM::"
	      << "PolarizationTransportFastSwitchedMM() "
	      << "uses three parameters (T, b/a,K_M and Th_switch).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=3 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationTransport::PolarizationTransport() "
	      << "Polarized molecule + polarization signal + switch molecule "
	      << "indeces needed in"
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationTransportFastSwitchedMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T";
  tmp[1] = "b/a";
  tmp[2] = "K_M";
  tmp[3] = "Th_switch";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationTransportFastSwitchedMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > 
       &dydt ) {
    
  size_t i=compartment.index();
  if( compartment.numNeighbor() == 0 ) return;
  double Vi = y[i][variableIndex(1,0)];//Volume of the compartment
  if( Vi<=0.0 ) {
    std::cerr << "PolarizationTransportFastSwitchedMM::derivs Given volume " 
	      << Vi << " unphysical.\n";
    exit(-1);
  }
  //Polarization coefficient normalization constant
  double sum=0.0;
  double min = y[ compartment.neighbor(0) ][ variableIndex(0,1) ];
  double max=min;
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    sum += y[ compartment.neighbor(n) ][ variableIndex(0,1) ];
    if( y[ compartment.neighbor(n) ][ variableIndex(0,1) ]<min )
      min = y[ compartment.neighbor(n) ][ variableIndex(0,1) ];
    if( y[ compartment.neighbor(n) ][ variableIndex(0,1) ]>max )
      max = y[ compartment.neighbor(n) ][ variableIndex(0,1) ];
  }
  sum /= compartment.numNeighbor();
  sum += parameter(1);
  double invSum = 1.0;
  if( sum>0 )
    invSum /= sum;//To use mult instead of div
  
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    double Vj = y[j][variableIndex(1,0)];//Volume of the compartment
    if( Vj<=0.0 ) {
      std::cerr << "PolarizationTransportFastSwitchedMM::derivs Given volume " 
		<< Vj << " unphysical.\n";
      exit(-1);
    }
    
    double Pij; 
    if( y[i][variableIndex(0,2)] > parameter(3) )
      Pij = y[i][variableIndex(0,0)] * y[j][ variableIndex(0,1) ] * invSum;
    else
      Pij = y[i][variableIndex(0,0)] * (max+min-y[j][ variableIndex(0,1) ]) 
	* invSum;
    double coeff = Pij*y[i][species]/( parameter(2)+y[i][species] );
    //dydt[i][species] -= parameter(0)*coeff/Vi;
    //dydt[j][species] += parameter(0)*coeff/Vj;
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

//!Contructor for a polarized cell-wall transport
PolarizationLinFastTransportRestrictedMM::
PolarizationLinFastTransportRestrictedMM(std::vector<double> &paraValue, 
					 std::vector< std::vector<size_t> > 
					 &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=4 ) {
    std::cerr << "PolarizationLinFastTransportRestrictedMM::"
	      << "PolarizationLinFastTransportRestrictedMM() "
	      << "uses four parameters (T_eff, T_inf, b/a, and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=4 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationLinFastTransportRestrictedMM::"
	      << "PolarizationLinFastTransportRestrictedMM() "
	      << "Polarized molecule + polarization signal + "
	      << "cell marker + wall marker "
	      << "indeces needed in"
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationLinFastTransportRestrictedMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T_eff";
  tmp[1] = "T_inf";
  tmp[2] = "b/a";
  tmp[3] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationLinFastTransportRestrictedMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > 
       &dydt ) {
  
  size_t i=compartment.index();
  //Only if the comp has neighbors and is a cell
  if( compartment.numNeighbor() == 0 || 
      y[i][variableIndex(0,2)]<0.5 ) return;
  
  //Extract compartment volume and make sure it is physical
  double Vi = y[i][variableIndex(1,0)];
  assert( Vi>0.0 );
  //Polarization from CELL neighbors at level 1.
  assert( compartment.numNeighborAtLevel(1) == 
	  compartment.numNeighbor() );
  
  //Calulate normalization
  //////////////////////////////////////////////////////////////////////
  double sum=parameter(2);
  if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
    sum *=Vi;
    for( size_t n=0 ; n<compartment.numNeighborAtLevel(1) ; n++ )
      sum += y[ compartment.neighborAtLevel(1,n) ][ variableIndex(0,1) ]*
	compartment.neighborArea(n);
  }
  else
    for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
      sum += y[ compartment.neighborAtLevel(1,n) ][ variableIndex(0,1) ];
  
  double invSum = 1.0;
  if( sum>0.0 )
    invSum /= sum;//To use mult instead of div
  
  //Update all compartments
  //////////////////////////////////////////////////////////////////////
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);

    //Make sure neighbor marked as wall
    assert( y[j][variableIndex(0,3)]>0.5 );

    //Extract neighbor volume and make sure it is physical
    double Vj = y[j][variableIndex(1,0)];
    assert( Vj>0.0 );

    double Pij = y[i][variableIndex(0,0)] * Vi * 
      y[compartment.neighborAtLevel(1,n)][variableIndex(0,1)] * invSum;
    if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
      double coeff = compartment.neighborArea(n)*Pij*
	y[i][species]/( parameter(3)+y[i][species] );
      dydt[i][species] -= parameter(0)*coeff/Vi;
      dydt[j][species] += parameter(0)*coeff/Vj;
      coeff = compartment.neighborArea(n)*Pij*
	y[j][species]/( parameter(3)+y[j][species] );
      dydt[i][species] += parameter(1)*coeff/Vi;
      dydt[j][species] -= parameter(1)*coeff/Vj;
    }
    else {
      double coeff = Pij*y[i][species]/( parameter(3)+y[i][species] );
      dydt[i][species] -= parameter(0)*coeff/Vi;
      dydt[j][species] += parameter(0)*coeff/Vj;
      coeff = Pij*y[j][species]/( parameter(3)+y[j][species] );
      dydt[i][species] += parameter(1)*coeff/Vi;
      dydt[j][species] -= parameter(1)*coeff/Vj;
    }
  }
}

PolarizationConstHillFastTransportRestrictedMM::
PolarizationConstHillFastTransportRestrictedMM(std::vector<double> 
					       &paraValue, 
					       std::vector< 
					       std::vector<size_t> > 
					       &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=7 ) {
    std::cerr << "PolarizationConstHillFastTransportRestrictedMM::"
	      << "PolarizationConstHillFastTransportRestrictedMM() "
	      << "uses seven parameters (T_eff, T_inf, b/a, and c, "
	      << "K_H, n,and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=4 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationConstHillFastTransportRestrictedMM::"
	      << "PolarizationConstHillFastTransportRestrictedMM() "
	      << "Polarized molecule + polarization signal + "
	      << "cell marker + wall marker "
	      << "indeces needed in"
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at second level.\n";
    exit(0);
  }
  //Check parameters
  //////////////////////////////////////////////////////////////////////
  if( paraValue[0]<0.0 || paraValue[1]<0.0 || paraValue[2]<0.0 ||
      paraValue[3]<0.0 || paraValue[4]<0.0 || paraValue[5]<0.0 ||
      paraValue[6]<0.0 ) {
    std::cerr << "PolarizationConstHillFastTransportRestrictedMM::"
	      << "PolarizationConstHillFastTransportRestrictedMM()"
	      << " Parameters need to be positive.\n";
    exit(0);
  }
  if( paraValue[3]>1.0 ) {
    std::cerr << "PolarizationConstHillFastTransportRestrictedMM::"
	      << "PolarizationConstHillFastTransportRestrictedMM()"
	      << " Parameter c (Lin/Hill fraction need to be "
	      << "smaller than 1.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationConstHillFastTransportRestrictedMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T_eff";
  tmp[1] = "T_inf";
  tmp[2] = "b/a";
  tmp[3] = "c_p";
  tmp[4] = "K_H";
  tmp[5] = "n";
  tmp[6] = "K_M";
  setParameterId( tmp );
}

//! Derivative contribution for an active transport in polarized direction
/*! Deriving the time derivative contribution from a degradation using
  the values in y and add the results to dydt. Compartment is the cell
  and species is the index of the variable to be updated.*/
void PolarizationConstHillFastTransportRestrictedMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > 
       &dydt ) {
  
  size_t i=compartment.index();
  //Only if the comp has neighbors and is a cell
  if( compartment.numNeighbor() == 0 || 
      y[i][variableIndex(0,2)]<0.5 ) return;
  
  static double KToNHill = std::pow(parameter(4),parameter(5));
  //Polarization function from CELL neighbors at level 1.
  assert( compartment.numNeighborAtLevel(1) == 
	  compartment.numNeighbor() );
  //Extract volume of the compartment and make sure it is physical
  double Vi = y[i][variableIndex(1,0)];
  assert( Vi>0.0 );
  
  //Calculate f(n) and normalization
  //////////////////////////////////////////////////////////////////////
  std::vector<double> f(compartment.numNeighbor() );
  for( size_t n=0 ; n<compartment.numNeighborAtLevel(1) ; n++ ) {
    double yToN = std::pow(y[ compartment.neighborAtLevel(1,n) ]
			   [ variableIndex(0,1) ] , parameter(5));
    f[n] = (1.0-parameter(3)) + parameter(3)*yToN/(KToNHill+yToN);
  }

  double sum=parameter(2);
  if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
    sum *=Vi;
    for( size_t n=0 ; n<compartment.numNeighborAtLevel(1) ; n++ )
      sum += f[n]*compartment.neighborArea(n);
  }
  else
    for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
      sum += f[n];
  
  double invSum = 1.0;
  if( sum>0.0 )
    invSum /= sum;//To use mult instead of div
  
  //Update all compartments
  //////////////////////////////////////////////////////////////////////
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    
    //Make sure neighbor marked as wall 
    assert( y[j][variableIndex(0,3)]>0.5 ); 
    
    //Extract volume of neighbor and make sure it is physical
    double Vj = y[j][variableIndex(1,0)];
    assert( Vj>0.0 );
    
    double Pij = y[i][variableIndex(0,0)] * Vi * f[n] * invSum;
    if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
      double coeff = compartment.neighborArea(n)*Pij*
	y[i][species]/( parameter(6)+y[i][species] );
      dydt[i][species] -= parameter(0)*coeff/Vi;
      dydt[j][species] += parameter(0)*coeff/Vj;
      coeff = compartment.neighborArea(n)*Pij*
	y[j][species]/( parameter(6)+y[j][species] );
      dydt[i][species] += parameter(1)*coeff/Vi;
      dydt[j][species] -= parameter(1)*coeff/Vj;
    }
    else {
      double coeff = Pij*y[i][species]/( parameter(6)+y[i][species] );
      dydt[i][species] -= parameter(0)*coeff/Vi;
      dydt[j][species] += parameter(0)*coeff/Vj;
      coeff = Pij*y[j][species]/( parameter(6)+y[j][species] );
      dydt[i][species] += parameter(1)*coeff/Vi;
      dydt[j][species] -= parameter(1)*coeff/Vj;
    }
  }
}

PolarizationConstFastTransportRestrictedMM::
PolarizationConstFastTransportRestrictedMM(std::vector<double> 
					   &paraValue, 
					   std::vector< 
					   std::vector<size_t> > 
					   &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=4 ) {
    std::cerr << "PolarizationConstFastTransportRestrictedMM::"
	      << "PolarizationConstFastTransportRestrictedMM() "
	      << "uses seven parameters (T_eff, T_inf, b/a, and K_M).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=3 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationConstFastTransportRestrictedMM::"
	      << "PolarizationConstFastTransportRestrictedMM() "
	      << "Polarized molecule + "
	      << "cell marker + wall marker "
	      << "indeces needed in"
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at "
	      << "second level.\n";
    exit(0);
  }
  //Check parameters
  //////////////////////////////////////////////////////////////////////
  if( paraValue[0]<0.0 || paraValue[1]<0.0 || paraValue[2]<0.0 ||
      paraValue[3]<0.0 ) {
    std::cerr << "PolarizationConstFastTransportRestrictedMM::"
	      << "PolarizationConstFastTransportRestrictedMM()"
	      << " Parameters need to be positive.\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationConstFastTransportRestrictedMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T_eff";
  tmp[1] = "T_inf";
  tmp[2] = "b/a";
  tmp[3] = "K_M";
  setParameterId( tmp );
}

void PolarizationConstFastTransportRestrictedMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > 
       &dydt ) {
  
  size_t i=compartment.index();
  //Only if the comp has neighbors and is a cell
  if( compartment.numNeighbor() == 0 || 
      y[i][variableIndex(0,1)]<0.5 ) return;
  
  //Polarization function from CELL neighbors at level 1.
  assert( compartment.numNeighborAtLevel(1) == 
	  compartment.numNeighbor() );
  //Extract volume of the compartment and make sure it is physical
  double Vi = y[i][variableIndex(1,0)];
  assert( Vi>0.0 );
  
  //Calculate normalization
  //////////////////////////////////////////////////////////////////////
  double sum=parameter(2);
  if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
    sum *=Vi;
    for( size_t n=0 ; n<compartment.numNeighborAtLevel(1) ; n++ )
      sum += compartment.neighborArea(n);
  }
  else
    sum += compartment.numNeighbor();//assuming area=1
  
  double invSum = 1.0;
  if( sum>0.0 )
    invSum /= sum;//To use mult instead of div
  
  //Update all compartments
  //////////////////////////////////////////////////////////////////////
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    
    //Make sure neighbor marked as wall 
    assert( y[j][variableIndex(0,2)]>0.5 ); 
    
    //Extract volume of neighbor and make sure it is physical
    double Vj = y[j][variableIndex(1,0)];
    assert( Vj>0.0 );
    
    double Pij = y[i][variableIndex(0,0)] * Vi * invSum;
    if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
      double coeff = compartment.neighborArea(n)*Pij*
	y[i][species]/( parameter(3)+y[i][species] );
      dydt[i][species] -= parameter(0)*coeff/Vi;
      dydt[j][species] += parameter(0)*coeff/Vj;
      coeff = compartment.neighborArea(n)*Pij*
	y[j][species]/( parameter(3)+y[j][species] );
      dydt[i][species] += parameter(1)*coeff/Vi;
      dydt[j][species] -= parameter(1)*coeff/Vj;
    }
    else {
      double coeff = Pij*y[i][species]/( parameter(3)+y[i][species] );
      dydt[i][species] -= parameter(0)*coeff/Vi;
      dydt[j][species] += parameter(0)*coeff/Vj;
      coeff = Pij*y[j][species]/( parameter(3)+y[j][species] );
      dydt[i][species] += parameter(1)*coeff/Vi;
      dydt[j][species] -= parameter(1)*coeff/Vj;
    }
  }
}

PolarizationConstFastTransportFractionRestrictedMM::
PolarizationConstFastTransportFractionRestrictedMM(std::vector<double> 
					   &paraValue, 
					   std::vector< 
					   std::vector<size_t> > 
					   &indValue ) {
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if( paraValue.size()!=6 ) {
    std::cerr << "PolarizationConstFastTransportFractionRestrictedMM::"
	      << "PolarizationConstFastTransportFractionRestrictedMM() "
	      << "uses six parameters (T_eff, T_inf, b/a, K_M, f_i, f_ij).\n";
    exit(0);
  }
  if( indValue.size() != 2 || indValue[0].size() !=3 
      || indValue[1].size() !=1  ) {
    std::cerr << "PolarizationConstFastTransportFractionRestrictedMM::"
	      << "PolarizationConstFastTransportFractionRestrictedMM() "
	      << "Polarized molecule + "
	      << "cell marker + wall marker "
	      << "indeces needed in"
	      << "level 1.\n" 
	      << "Variable index for size parameter needed at "
	      << "second level.\n";
    exit(0);
  }
  //Check parameters
  //////////////////////////////////////////////////////////////////////
  if( paraValue[0]<0.0 || paraValue[1]<0.0 || paraValue[2]<0.0 ||
      paraValue[3]<0.0 || paraValue[4]<0.0 || paraValue[4]>1.0 ||
      paraValue[5]<0.0 || paraValue[5]>1.0 ) {
    std::cerr << "PolarizationConstFastTransportFractionRestrictedMM::"
	      << "PolarizationConstFastTransportFractionRestrictedMM()"
	      << " Parameters need to be positive (and fractions).\n";
    exit(0);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationConstFastTransportFractionRestrictedMM");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize( numParameter() );
  tmp[0] = "T_eff";
  tmp[1] = "T_inf";
  tmp[2] = "b/a";
  tmp[3] = "K_M";
  tmp[4] = "f_i";
  tmp[5] = "f_ij";
  setParameterId( tmp );
}

void PolarizationConstFastTransportFractionRestrictedMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > 
       &dydt ) {
  
  size_t i=compartment.index();
  //Only if the comp has neighbors and is a cell
  if( compartment.numNeighbor() == 0 || 
      y[i][variableIndex(0,1)]<0.5 ) return;
  
  //Extract volume of the compartment and make sure it is physical
  double Vi = y[i][variableIndex(1,0)];
  assert( Vi>0.0 );
  
  //Calculate normalization
  //////////////////////////////////////////////////////////////////////
  double invSum=1.0;
	//if ( compartment.numNeighborAtLevel(1)) {
	//assert(compartment.numNeighborAtLevel(1)==compartment.numNeighbor());
		double sum=parameter(2);
		if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
			sum *=Vi;
			for( size_t n=0 ; n<compartment.numNeighborAtLevel(1) ; ++n )
				sum += compartment.neighborArea(n);
		}
		else
			sum += compartment.numNeighbor();//assuming area=1
		
		//double invSum = 1.0;
		if( sum>0.0 )
			invSum /= sum;//To use mult instead of div
		//}
		
  //Update all compartments
  //////////////////////////////////////////////////////////////////////
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    
    //Make sure neighbor marked as wall 
    //assert( y[j][variableIndex(0,2)]>0.5 ); 
		if ( y[j][variableIndex(0,2)]>0.5 ) { 
			//Extract volume of neighbor and make sure it is physical
			double Vj = y[j][variableIndex(1,0)];
			assert( Vj>0.0 );
			
			double Pij = y[i][variableIndex(0,0)] * Vi  * invSum;
			if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
				double coeff = compartment.neighborArea(n)*Pij*
					parameter(4)*y[i][species]/( parameter(3)+parameter(4)*y[i][species] );
				dydt[i][species] -= parameter(0)*coeff/Vi;
				dydt[j][species] += parameter(0)*coeff/Vj;
				coeff = compartment.neighborArea(n)*Pij*
					parameter(5)*y[j][species]/( parameter(3)+parameter(5)*y[j][species] );
				dydt[i][species] += parameter(1)*coeff/Vi;
				dydt[j][species] -= parameter(1)*coeff/Vj;
			}
			else {
				double coeff = Pij*parameter(4)*y[i][species]/
					( parameter(3)+parameter(4)*y[i][species] );
				dydt[i][species] -= parameter(0)*coeff/Vi;
				dydt[j][species] += parameter(0)*coeff/Vj;
				coeff = Pij*parameter(5)*y[j][species]/
					( parameter(3)+parameter(5)*y[j][species] );
				dydt[i][species] += parameter(1)*coeff/Vi;
				dydt[j][species] -= parameter(1)*coeff/Vj;
			}
		}
	}
}

PolarizationConstHillFastTransportFractionRestrictedMM::
PolarizationConstHillFastTransportFractionRestrictedMM(std::vector<double>
											&paraValue, 
											std::vector< std::vector<size_t> > &indValue)
{  
	//Do some checks on the parameters and variable indeces
	//////////////////////////////////////////////////////////////////////
	if (paraValue.size() != 9) {
		std::cerr << "PolarizationConstHillFastTransportRestrictedMM::"
				<< "PolarizationConstHillFastTransportRestrictedMM() "
				<< "uses seven parameters (T_eff, T_inf, b/a, and c, "
				<< "K_H, n,and K_M, f_i and f_ij).\n";
		exit(EXIT_FAILURE);
	}
	if (indValue.size() != 2 || indValue[0].size() != 4 
	    || indValue[1].size() != 1) {
		std::cerr << "PolarizationConstHillFastTransportRestrictedMM::"
				<< "PolarizationConstHillFastTransportRestrictedMM() "
				<< "Polarized molecule + polarization signal + "
				<< "cell marker + wall marker "
				<< "indeces needed in"
				<< "level 1.\n" 
				<< "Variable index for size parameter needed at second level.\n";
		exit(EXIT_FAILURE);
	}
	//Check parameters
	//////////////////////////////////////////////////////////////////////
	if (paraValue[0] < 0.0 || paraValue[1] < 0.0 || paraValue[2] < 0.0 ||
	    paraValue[3] < 0.0 || paraValue[4] < 0.0 || paraValue[5] < 0.0 ||
	    paraValue[6] < 0.0 ) {
		std::cerr << "PolarizationConstHillFastTransportRestrictedMM::"
				<< "PolarizationConstHillFastTransportRestrictedMM()"
				<< " Parameters need to be positive.\n";
		exit(EXIT_FAILURE);
	}
	if (paraValue[3] > 1.0) {
		std::cerr << "PolarizationConstHillFastTransportRestrictedMM::"
				<< "PolarizationConstHillFastTransportRestrictedMM()"
				<< " Parameter c (Lin/Hill fraction need to be "
				<< "smaller than 1.\n";
		exit(EXIT_FAILURE);
	}
	//Set the variable values
	//////////////////////////////////////////////////////////////////////
	setId("polarizationConstHillFastTransportRestrictedMM");
	setParameter(paraValue);  
	setVariableIndex(indValue);
	
	//Set the parameter identities
	//////////////////////////////////////////////////////////////////////
	std::vector<std::string> tmp(numParameter());
	tmp.resize(numParameter());
	tmp[0] = "T_eff";
	tmp[1] = "T_inf";
	tmp[2] = "b/a";
	tmp[3] = "c_p";
	tmp[4] = "K_H";
	tmp[5] = "n";
	tmp[6] = "K_M";
	tmp[7] = "f_i";
	tmp[8] = "f_ij";
	setParameterId(tmp);
}

void PolarizationConstHillFastTransportFractionRestrictedMM::
derivs(Compartment &compartment,size_t species,
       std::vector< std::vector<double> > &y,
       std::vector< std::vector<double> > 
       &dydt ) {
  
  size_t i=compartment.index();
  //Only if the comp has neighbors and is a cell
  if( compartment.numNeighbor() == 0 || 
      y[i][variableIndex(0,2)]<0.5 ) return;
  
  static double KToNHill = std::pow(parameter(4),parameter(5));
  //Polarization function from CELL neighbors at level 1.
  assert( compartment.numNeighborAtLevel(1) == 
					compartment.numNeighbor() );
  //Extract volume of the compartment and make sure it is physical
  double Vi = y[i][variableIndex(1,0)];
  assert( Vi>0.0 );
  
  //Calculate f(n) and normalization
  //////////////////////////////////////////////////////////////////////
  std::vector<double> f(compartment.numNeighbor() );
  for( size_t n=0 ; n<compartment.numNeighborAtLevel(1) ; n++ ) {
    double yToN = std::pow(y[ compartment.neighborAtLevel(1,n) ]
													 [ variableIndex(0,1) ] , parameter(5));
    f[n] = (1.0-parameter(3)) + parameter(3)*yToN/(KToNHill+yToN);
  }
	
  double sum=parameter(2);
  if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
    sum *=Vi;
    for( size_t n=0 ; n<compartment.numNeighborAtLevel(1) ; n++ )
      sum += f[n]*compartment.neighborArea(n);
  }
  else
    for( size_t n=0 ; n<compartment.numNeighbor() ; n++ )
      sum += f[n];
  
  double invSum = 1.0;
  if( sum>0.0 )
    invSum /= sum;//To use mult instead of div
  
  //Update all compartments
  //////////////////////////////////////////////////////////////////////
  for( size_t n=0 ; n<compartment.numNeighbor() ; n++ ) {
    size_t j=compartment.neighbor(n);
    
    //Make sure neighbor marked as wall 
    assert( y[j][variableIndex(0,3)]>0.5 ); 
    
    //Extract volume of neighbor and make sure it is physical
    double Vj = y[j][variableIndex(1,0)];
    assert( Vj>0.0 );
    
    double Pij = y[i][variableIndex(0,0)] * Vi * f[n] * invSum;
    if( compartment.numNeighbor()==compartment.numNeighborArea() ) {
      double coeff = compartment.neighborArea(n)*Pij*
				parameter(7)*y[i][species] /
				( parameter(6)+parameter(7)*y[i][species] );
      dydt[i][species] -= parameter(0)*coeff/Vi;
      dydt[j][species] += parameter(0)*coeff/Vj;
      coeff = compartment.neighborArea(n)*Pij*
				parameter(8)*y[j][species] /
				( parameter(6)+parameter(8)*y[j][species] );
      dydt[i][species] += parameter(1)*coeff/Vi;
      dydt[j][species] -= parameter(1)*coeff/Vj;
    }
    else {
      double coeff = Pij*y[i][species]/( parameter(6)+y[i][species] );
      dydt[i][species] -= parameter(0)*coeff/Vi;
      dydt[j][species] += parameter(0)*coeff/Vj;
      coeff = Pij*y[j][species]/( parameter(6)+y[j][species] );
      dydt[i][species] += parameter(1)*coeff/Vi;
      dydt[j][species] -= parameter(1)*coeff/Vj;
    }
  }
}

PolarizationTransportFastGeneralFGRestricted::PolarizationTransportFastGeneralFGRestricted(
  std::vector<double> &paraValue,
  std::vector< std::vector<size_t> > &indValue) 
{
  
  //Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if(paraValue.size() != 5) {
    std::cerr << "PolarizationTransportFastGeneralFGRestricted::"
	      << "PolarizationTransportFastGeneralFGRestricted() "
	      << "uses five parameters (T, k_f, n_f, k_g, n_g).\n";
    exit(0);
  }
  if(indValue.size() != 2 || indValue[0].size() != 2 
     || indValue[1].size() != 2) {
    std::cerr << "PolarizationTransportFastGeneralFGRestricted::"
	      << "PolarizationTransportFastGeneralFGRestricted()\n "
	      << " - Level 1: Polarized molecule + polarization signal.\n"
	      << " - Level 2: Markers for restricted transport.\n";
    exit(0);
  }

  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("polarizationTransportFastGeneralFGRestricted");
  setParameter(paraValue);  
  setVariableIndex(indValue);
  
  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp(numParameter());
  tmp.resize(numParameter());
  tmp[0] = "T";
  tmp[1] = "k_f";
  tmp[2] = "n_f";
  tmp[3] = "k_g";
  tmp[4] = "n_n";
  setParameterId(tmp);
}

void PolarizationTransportFastGeneralFGRestricted::derivs(
  Compartment &compartment,size_t species,
  std::vector< std::vector<double> > &y,
  std::vector< std::vector<double> > &dydt )
{
  size_t i = compartment.index();

  //Check that this compartment is marked for this transport
  if (y[i][variableIndex(1,0)] < 0.5)
    return;

  //Polarization coefficient normalization constant
  double sum = 0.0;
  double tmp;
  for(size_t n = 0; n < compartment.numNeighbor(); n++) {
    tmp = f(y[compartment.neighbor(n)][variableIndex(0,1)]);
    tmp /= g(y[compartment.neighbor(n)][variableIndex(0,1)]);
    sum += tmp;
  }
  sum += 1;

  for(size_t n = 0; n < compartment.numNeighbor(); n++) {
    size_t j = compartment.neighbor(n);
 
    //Only update if neighbor marked for this transport
    //If same type of compartments only update once (i>j)

    double Pij, coeff;
    if (y[j][variableIndex(1,1)] > 0.5) {
      if (variableIndex(1,0) != variableIndex(1,1) || i > j) {
	Pij = f(y[j][variableIndex(0,1)]);
	Pij /= g(y[j][variableIndex(0,1)]);
	Pij *= y[i][variableIndex(0,0)];
	Pij /= sum;
  	coeff = y[i][species] * Pij;

	dydt[i][species] -= parameter(0)*coeff;
	dydt[j][species] += parameter(0)*coeff;
      }
    }
  }
}

double PolarizationTransportFastGeneralFGRestricted::f(double arg)
{
  return parameter(1) * pow(arg, parameter(2));
}

double PolarizationTransportFastGeneralFGRestricted::g(double arg)
{
  return parameter(3) * pow(arg, parameter(4));
}

/*
	Warning!! g() = 0 is valid in an analysis done with pen and paper, but the
	way the simulator adds factors together produces nan!.
*/
GeneralPolarizationTransportFast::GeneralPolarizationTransportFast(
   std::vector<double> &paraValue, 
	 std::vector< std::vector<size_t> > &indValue)
{
	//Do some checks on the parameters and variable indeces
  //////////////////////////////////////////////////////////////////////
  if (paraValue.size() != 5) {
    std::cerr << "GeneralPolarizationTransportFast::" 
							<< "GeneralPolarizationTransportFast() "
							<< "uses five parameters (T, k_f, n_f, k_g and n_g).\n";
    exit(EXIT_FAILURE);
  }
	
  if (indValue.size() != 2 || indValue[0].size() != 2 
      || indValue[1].size() != 1) {
    std::cerr << "GeneralPolarizationTransport::"
							<< "GeneralPolarizationTransport() "
							<< "Polarized molecule + polarization signal indeces needed in "
							<< "level 1.\n" 
							<< "Variable index for size parameter needed at second level.\n";
    exit(EXIT_FAILURE);
  }
  //Set the variable values
  //////////////////////////////////////////////////////////////////////
  setId("generalPolarizationTransportFast");
  setParameter(paraValue);  
  setVariableIndex(indValue);

  //Set the parameter identities
  //////////////////////////////////////////////////////////////////////
  std::vector<std::string> tmp( numParameter() );
  tmp.resize(numParameter());
	tmp[0] = "T";
  tmp[1] = "k_f";
  tmp[2] = "n_f";
	tmp[3] = "k_g";
	tmp[4] = "n_g";	
  setParameterId( tmp );
}

void GeneralPolarizationTransportFast::derivs(
	 Compartment &compartment, 
	 size_t species,
	 std::vector< std::vector<double> > &y,
	 std::vector< std::vector<double> > &dydt)
{
	double k_f = parameter(1);
	double n_f = parameter(2);
	double k_g = parameter(3);
	double n_g = parameter(4);
	double a_j;

  size_t i = compartment.index();

  //Polarization coefficient normalization constant
  double sum = 0.0;
  for (size_t n = 0; n < compartment.numNeighbor(); n++) {
		double a_j = y[compartment.neighbor(n)][variableIndex(0,1)];
    sum += f(a_j, k_f, n_f) / g(a_j, k_g, n_g);
	}
  sum += 1;

  for (size_t n = 0; n < compartment.numNeighbor(); n++) {
    size_t j = compartment.neighbor(n);
    a_j = y[j][variableIndex(0,1)];
 
    double coeff = 0.0;
		coeff = y[i][species] * y[i][variableIndex(0,0)];
		coeff *= f(a_j, k_f, n_f) / g(a_j, k_g, n_g);

    if (sum != 0.0)
			coeff /= sum;
		else {
			std::cerr << "GeneralPolarizationTransportFast::derivs " 
								<< "sum == 0.0.\n";
			exit(EXIT_FAILURE);
		}
    dydt[i][species] -= parameter(0)*coeff;
    dydt[j][species] += parameter(0)*coeff;
  }
}

double GeneralPolarizationTransportFast::f(double a, double k, double n)
{
	return k * std::pow(a, n);
}

double GeneralPolarizationTransportFast::g(double a, double k, double n)
{
	return k;
}


