//
// Filename     : polarizationTransport.h
// Description  : Classes describing polarized transports of molecules
// Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
// Created      : December 2007
// Revision     : $Id: transport.h 203 2007-03-20 10:50:07Z henrik $
//
#ifndef POLARIZATIONTRANSPORT_H
#define POLARIZATIONTRANSPORT_H

#include<cmath>

#include"../baseReaction.h"

///
/// @brief The class polarizationTransport moves molecules across cell walls activly
///
/// The parameter is the diffusion constant \e D in 
///
/// @f[ dc/dt = -D*V_m(cc)/V @f]
///
/// and 
///
/// @f[ dc_n/dt = D*V_m(cc)/V_n. @f] 
///
/// V_m is the average volume. cc is actually T*P*c where T is the efflux
/// carrier concentration P is the polarization coefficient (based on
/// neighbors concentration of "pol species", and c is the concentration of
/// the transported species. The update will be mirrored by a similar from j
/// to i. The update needs the efflux carrier and polarization signal as
/// indeces in the first level and the size index as a variable index in the
/// second level.
///
class PolarizationTransport : public BaseReaction {
  
 public:
  
  PolarizationTransport(std::vector<double> &paraValue, 
			std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

/// 
/// @brief The class polarizationTransportFast moves molecules activly
///
/// The Polarized transporter molecule P is divided into the cell cytosol
/// (P_i) and its membranes (P_ij) following the dynamics
///
/// @f[ dP_i/dt = -a P_i \sum_k A_k + b \sum_k P_{ik} @f]
/// @f[ dP_{ij}/dt = a P_i A_j - b P_{ij} @f]
///
/// where \e k is summed over neighboring cells, \e A is the polarizing signal
/// and \e a, \e b are constants. The polarization is assumed to be fast and
/// the fixed point values are assumed
///  
/// @f[ P_i = (b/a)*P / (\sum_k A_k + b/a) @f]
/// @f[ P_ij = P * A_j / (\sum_k A_k + (b/a)) @f]
///  
/// where \e P is the total amount of polarized molecule in a cell.
///  
/// The polarized transport of a molecule X between two neighboring cells is
/// then given by
///  
/// @f[ dX_i/dt = T ( X_i P_{ij} - X_j P_ji ). @f]
///  
/// The parameters given to this function are T and K=b/a.  The volumes for
/// the cells are also included for correct concentration update.
///
class PolarizationTransportFast : public BaseReaction {
  
 public:
  
  PolarizationTransportFast(std::vector<double> &paraValue, 
			    std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief The reaction moves molecules actively. 
///
/// The Polarized transporter molecule @f$P@f$ is divided into the cell
/// cytosol (@f$P_i@f$) and its membranes (@f$P_{ij}@f$) following the dynamics
/// @f[
/// \frac{dP_i}{dt} = - \sum_j f(A_j)\cdot P_i + \sum_j g(A_j)\cdot P_{ij} 
/// @f]
/// @f[
/// \frac{dP_{ij}}{dt} = f(A_j)\cdot P_i - f(A_j)\cdot P_{ij}
/// @f]
///
/// where @f$j@f$ is summed over neighboring cells, @f$A@f$ is the
/// polarizing signal and @f$f(A)@f$ and @f$g(A)@f$ are arbitrary
/// functions. The polarization is assumed to be fast and the fixed
/// point values are assumed to be
/// @f[ 
/// P_{ij}^\ast =
/// \frac{f(A_j)}{g(A_j)}\frac{P}{1+\sum_k
/// \frac{f(A_k)}{g(A_k)}}
/// @f]
/// where @f$k@f$ is summed over all neighboring cells and @f$P@f$ is
/// the total amount of polarized molecule in a cell. The polarized
/// transport of a molecule X between two neighboring cells is then
/// given by
/// @f[ \frac{dX_i}{dt} = - T (X_i\cdot P_{ij} -
/// X_j\cdot P_{ji}).
/// @f]
/// 
/// The parameters given to this function are @f$T@f$, @f$k_f@f$,
/// @f$n_f@f$, @f$k_g@f$ and @f$n_g@f$. First level variables are
/// @f$P@f$ and @f$X@f$.
/// 
class GeneralPolarizationTransportFast : public BaseReaction 
{
 public:
	///
	/// @brief Main constructor
	///
	/// This is the main constructor which sets the parameters and variable
	/// indices that defines the reaction.
	///
	/// @param paraValue vector with parameters
	///
	/// @param indValue vector of vectors with variable indices
	///
	/// @see BaseReaction::createReaction(std::vector<double> &paraValue,...)
  ///
  GeneralPolarizationTransportFast(std::vector<double> &paraValue, 
			    std::vector< std::vector<size_t> > &indValue );

  ///
	/// @brief Derivative function for this reaction class
	///
	/// @see BaseReaction::derivs(Compartment &compartment,size_t species,...)
	///
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
	///
	/// @brief Hard coded general function f(A).
	///
	double f(double a, double k, double n);
	///
	/// @brief Hard coded general function g(A).
	///
	double g(double a, double k, double n);
};

//!Moves molecules activly with a Michaelis Menten type of equation
class PolarizationTransportFastMM : public BaseReaction {
  
 public:
  
  PolarizationTransportFastMM(std::vector<double> &paraValue, 
			      std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!Moves molecules activly symmetric with a Michaelis Menten equation
class SymmetricTransportFastMM : public BaseReaction {
  
 public:
  
  SymmetricTransportFastMM(std::vector<double> &paraValue, 
			   std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!Moves molecules activly with a Michaelis Menten type of equation
class PolarizationTransportInfluxFastMM : public BaseReaction {
  
 public:
  
  PolarizationTransportInfluxFastMM(std::vector<double> &paraValue, 
				    std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!Moves molecules activly with a Michaelis Menten type of equation
class PolarizationHillTransportInfluxFastMM : public BaseReaction {
  
 public:
  
  PolarizationHillTransportInfluxFastMM(std::vector<double> &paraValue, 
					std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!Moves molecules activly symmetric with a Michaelis Menten equation
class SymmetricTransportInfluxFastMM : public BaseReaction {
  
 public:
  
  SymmetricTransportInfluxFastMM(std::vector<double> &paraValue, 
				    std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!Moves molecules activly with a Michaelis Menten type of equation
class PolarizationTransportFastMMSpatial : public BaseReaction {
  
 public:
  
  PolarizationTransportFastMMSpatial(std::vector<double> &paraValue, 
				     std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!The class polarizationHillTransportFast moves molecules activly
class PolarizationHillTransportFast : public BaseReaction {
  
 public:
  
  PolarizationHillTransportFast(std::vector<double> &paraValue, 
				std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!The class polarizationHillTransportFastMM moves molecules activly with MM
class PolarizationHillTransportFastMM : public BaseReaction {
  
 public:
  
  PolarizationHillTransportFastMM(std::vector<double> &paraValue, 
				std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!Moves molecules activly with MM, and exponential polarization feedback
class PolarizationExpTransportFastMM : public BaseReaction {
  
 public:
  
  PolarizationExpTransportFastMM(std::vector<double> &paraValue, 
				 std::vector< std::vector<size_t> > 
				 &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief This class moves molecules activly with a const+lin polarization feedback
///
/// The Polarized transporter molecule P is divided into the cell cytosol
/// (@f$P_i@f$) and its membranes (@f$P_ij@f$) following the dynamics
///
/// @f[ \frac{dP_i}{dt} = -a P_i \sum_k (c+A_k) + b \sum_k P_{ik} @f]
/// @f[ \frac{dP_{ij}}{dt} = a P_i (c+A_j) - b P_{ij} @f]
///
/// where k is summed over neighboring cells, A is the polarizing signal and
/// a,b,c are constants. The polarization is assumed to be fast and the fixed
/// point values are assumed
///
/// @f[ P_i = (b/a)*P / (\sum_k A_k + (b/a)) @f]
/// @f[ P_{ij} = \frac{P (c+A_j)}{(\sum_k (c+A_k) + (b/a))} @f]
///
/// where P is the total amount of polarized molecule in a cell.
///
/// The polarized transport of a molecule X between two neighboring cells is
/// then given by
///
/// @f[ \frac{dX_i}{dt} = T ( P_{ij}*X_i - P_{ji}*X_j ). @f]
///
/// The parameters given to this function are T and K'=b/a, and c.  a_ij is
/// the crossing area. The volumes for the cells are also included for correct
/// concentration update.
///
class PolarizationConstLinTransportFast : public BaseReaction {
  
 public:
  
  PolarizationConstLinTransportFast(std::vector<double> &paraValue, 
				    std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!The class polarizationTransportFastB moves molecules activly
class PolarizationTransportFastB : public BaseReaction {
  
 public:
  
  PolarizationTransportFastB(std::vector<double> &paraValue, 
			     std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!The class polarizationTransportFastBMM moves molecules activly
class PolarizationTransportFastBMM : public BaseReaction {
  
 public:
  
  PolarizationTransportFastBMM(std::vector<double> &paraValue, 
			       std::vector< std::vector<size_t> > &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

//!Moves molecules activly with a built in switch behavior
class PolarizationTransportFastSwitchedMM : public BaseReaction {
  
 public:
  
  PolarizationTransportFastSwitchedMM(std::vector<double> &paraValue, 
				      std::vector< std::vector<size_t> > 
				      &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief Moves molecules activly with a MM type of equation and polarized
///
/// The Polarized transporter molecule P is divided into the cell cytosol
/// (@f$P_i@f$) and its membranes (@f$P_ij@f$) following the dynamics
///
/// @f[ \frac{dP_i}{dt} = -a P_i \sum_k (A_k) + b \sum_k P_{ik} @f]
/// @f[ \frac{dP_{ij}}{dt} = a P_i (A_j) - b P_{ij} @f]
///
/// where k is summed over neighboring cells (which are assumed to be in
/// neighbor level 1), A is the polarizing signal and a,b are constants. The
/// polarization is assumed to be fast and the fixed point values are assumed
///
/// @f[ P_i = \frac{(b/a)P}{(\sum_k a_k*A_k + V_i*(b/a))} @f]
/// @f[ P_{ij} = \frac{P (A_j)}{(\sum_k a_k*A_k + V_i*(b/a))} @f]
///
/// where P is the total amount of polarized molecule in a cell (P_i*V_i), and
/// a_k is the membrane area (if those are given, otherwise a_k=1).
///
/// The polarized transport of a molecule X between a cell (i) and neighboring
/// walls (ij) is then given by
///
/// @f[ dX_i/dt = 1/V_i sum_j a_ij ( T_inf P_ij*X_ij/(K+X_ij) 
/// - T_eff P_ij*X_i/(K+X_i) ). @f]
/// @f[ dX_ij/dt = 1/V_ij sum_j a_ij ( -T_inf P_ij*X_ij/(K+X_ij) 
/// + T_eff P_ij*X_i/(K+X_i) ). @f]
///
/// The parameters given to this function are T_eff, T_inf and K'=b/a. a_ij is
/// the crossing area. The volumes for the cells are also included for correct
/// concentration update. The derivative is updated from cellular
/// compartments.
///
class PolarizationLinFastTransportRestrictedMM : public BaseReaction {
  
 public:
  
  PolarizationLinFastTransportRestrictedMM(std::vector<double> 
					   &paraValue, 
					   std::vector< std::vector<size_t> > 
					   &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief Moves molecules activly with a MM type of equation and polarized
///
/// The Polarized transporter molecule P is divided into the cell cytosol
/// (P_i) and its membranes (P_ij) following the dynamics
///  
/// @f[ dP_i/dt = -a P_i \sum_k f(A_k) + b \sum_k P_{ik} @f]
/// @f[ dP_{ij}/dt = a P_i f(A_j) - b P_{ij} @f] 
///
/// where k is summed over neighboring cells (which are assumed to be in
/// neighboe level 1), f(A_j)=((1-c)+cA_j^n/(Kh^n+a_j^n)) is the polarizing
/// signal and a,b are constants. The polarization is assumed to be fast and
/// the fixed point values are assumed
///  
/// @f[ P_i = (b/a)*P / (\sum_k a_k*f(A_k) + V_i*(b/a)) @f]
/// @f[ P_{ij} = P * f(A_j) / (\sum_k a_k*f(A_k) + V_i*(b/a)) @f]
///
/// where P is the total amount of polarized molecule in a cell (P_i*V_i), and
/// a_k is the membrane area (if those are given, otherwise a_k=1).
///  
/// The polarized transport of a molecule X between a cell (i) and neighboring
/// walls (ij) is then given by
///  
/// @f[ dX_i/dt = 1/V_i sum_j a_ij ( T_inf P_ij*X_ij/(K+X_ij) 
/// - T_eff P_ij*X_i/(K+X_i) ). @f]
/// @f[ dX_ij/dt = 1/V_ij sum_j a_ij ( -T_inf P_ij*X_ij/(K+X_ij) 
/// + T_eff P_ij*X_i/(K+X_i) ). @f]
///
/// The parameters given to this function are T_eff, T_inf and K'=b/a, c, Kh,
/// n, and K.  a_ij is the crossing area. The volumes for the cells are also
/// included for correct concentration update. The derivative is updated from
/// cellular compartments.
///
class PolarizationConstHillFastTransportRestrictedMM : 
public BaseReaction {
  
 public:
  
  PolarizationConstHillFastTransportRestrictedMM(std::vector<double> 
						 &paraValue, 
						 std::vector< 
						 std::vector<size_t> > 
						 &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief Moves molecules activly with a MM type of equation and unpolarized
///
/// The transporter molecule P is divided into the cell cytosol (P_i) and its
/// membranes (P_ij) following the dynamics
///
/// @f[ \frac{dP_i}{dt} = -a P_i \sum_k 1 + b \sum_k P_{ik} @f]
/// @f[ \frac{dP_{ij}}{dt} = a P_i - b P_{ij} @f]
///
/// where k is summed over neighboring cells (which are assumed to be in
/// neighbor level 1), a,b are constants. The membranization is assumed to be
/// fast and the fixed point values are assumed
///
/// @f[ P_i = \frac{(b/a)*P}{(\sum_k a_k*1 + V_i*(b/a))} @f]
/// @f[ P_ij = \frac{P * 1}{(\sum_k a_k*1 + V_i*(b/a))} @f]
///
/// where P is the total amount of transport molecule in a cell (P_i*V_i), and
/// a_k is the membrane area (if those are given, otherwise a_k=1).
///
/// The polarized transport of a molecule X between a cell (i) and neighboring
/// walls (ij) is then given by
///
/// @f[ dX_i/dt = 1/V_i sum_j a_{ij} ( T_inf P_{ij}*X_{ij}/(K+X_{ij}) 
/// - T_{eff} P_{ij}*X_i/(K+X_i) ) . @f]
/// @f[dX_{ij}/dt = 1/V_{ij} sum_j a_{ij} ( -T_inf P_{ij}*X_ij/(K+X_{ij}) 
/// + T_{eff} P_{ij}*X_i/(K+X_i) ). @f]
///
/// The parameters given to this function are T_eff, T_inf and K'=b/a, and K.
/// a_ij is the crossing area. The volumes for the cells are also included for
/// correct concentration update. The derivative is updated from cellular
/// compartments.
///
class PolarizationConstFastTransportRestrictedMM : public BaseReaction {
  
 public:
  
  PolarizationConstFastTransportRestrictedMM(std::vector<double> 
																						 &paraValue, 
																						 std::vector< 
																						 std::vector<size_t> > 
																						 &indValue );
  
  void derivs(Compartment &compartment,size_t species,
							std::vector< std::vector<double> > &y,
							std::vector< std::vector<double> > &dydt );
};

///
/// @brief Moves molecules activly with a MM type of equation and unpolarized
///
/// The transporter molecule P is divided into the cell cytosol (P_i) and its
/// membranes (P_ij) following the dynamics
///
/// @f[ \frac{dP_i}{dt} = -a P_i \sum_k 1 + b \sum_k P_{ik} @f]
/// @f[ \frac{dP_{ij}}{dt} = a P_i - b P_{ij} @f] 
///
/// where k is summed over neighboring cells (which are assumed to be in
/// neighboe level 1), a,b are constants. The membranization is assumed to be
/// fast and the fixed point values are assumed
///
/// @f[ P_i = \frac{(b/a)*P}{(\sum_k a_k*1 + V_i*(b/a))} @f]
/// @f[ P_{ij} = \frac{P * 1}{(\sum_k a_k*1 + V_i*(b/a))} @f]
///
/// where P is the total amount of transport molecule in a cell (P_i*V_i), and
/// a_k is the membrane area (if those are given, otherwise a_k=1).
///
/// The polarized transport of a molecule X between a cell (i) and neighboring
/// walls (ij) is then given by
///
/// @f[ dX_i/dt = 1/V_i sum_j a_ij ( T_inf P_ij*f_ij*X_ij/(K+f_ij*X_ij) 
/// - T_eff P_ij*f_i*X_i/(K+f_i*X_i) ). @f]
/// @f[ dX_ij/dt = 1/V_ij sum_j a_ij ( -T_inf P_ij*f_ij*X_ij/(K+f_ij*X_ij) 
/// + T_eff P_ij*f_i*X_i/(K+f_i*X_i) ). @f]
///
/// The parameters given to this function are T_eff, T_inf and K'=b/a, , K,
/// f_i, and f_ij.  a_ij is the crossing area. The volumes for the cells are
/// also included for correct concentration update. The derivative is updated
/// from cellular compartments.
///
class PolarizationConstFastTransportFractionRestrictedMM : 
public BaseReaction {
  
 public:
  
  PolarizationConstFastTransportFractionRestrictedMM(std::vector<double> 
						     &paraValue, 
						     std::vector< 
						     std::vector<size_t> > 
						     &indValue );
  
  void derivs(Compartment &compartment,size_t species,
	      std::vector< std::vector<double> > &y,
	      std::vector< std::vector<double> > &dydt );
};

///
/// @brief Moves molecules activly with a MM type of equation and polarized
///
/// The transporter molecule P is divided into the cell cytosol (P_i) and its
/// membranes (P_ij) following the dynamics
///
/// @f[ dP_i/dt = -a P_i \sum_k f(A_k) + b \sum_k P_{ik} @f]
/// @f[ dP_{ij}/dt = a P_i f(A_j) - b P_{ij} @f] 
///
/// where k is summed over neighboring cells (which are assumed to be in
/// neighboe level 1), f(A_j)=((1-c)+cA_j^n/(Kh^n+a_j^n)) is the polarizing
/// signal and a,b are constants. The polarization is assumed to be fast and
/// the fixed point values are assumed
///  
/// @f[ P_i = (b/a)*P / (\sum_k a_k*f(A_k) + V_i*(b/a)) @f]
/// @f[ P_{ij} = P * f(A_j) / (\sum_k a_k*f(A_k) + V_i*(b/a)) @f]
///
/// where P is the total amount of polarized molecule in a cell (P_i*V_i), and
/// a_k is the membrane area (if those are given, otherwise a_k=1).
///  
/// The polarized transport of a molecule X between a cell (i) and neighboring
/// walls (ij) is then given by
///
/// @f[ dX_i/dt = 1/V_i sum_j a_ij ( T_inf P_ij*f_ij*X_ij/(K+f_ij*X_ij) 
/// - T_eff P_ij*f_i*X_i/(K+f_i*X_i) ). @f]
/// @f[ dX_ij/dt = 1/V_ij sum_j a_ij ( -T_inf P_ij*f_ij*X_ij/(K+f_ij*X_ij) 
/// + T_eff P_ij*f_i*X_i/(K+f_i*X_i) ). @f]
///
/// The parameters given to this function are T_eff, T_inf and K'=b/a, , K,
/// f_i, and f_ij.  a_ij is the crossing area. The volumes for the cells are
/// also included for correct concentration update. The derivative is updated
/// from cellular compartments.
///
class PolarizationConstHillFastTransportFractionRestrictedMM : 
	public BaseReaction 
{
public:
  
  PolarizationConstHillFastTransportFractionRestrictedMM(std::vector<double> 
																												 &paraValue, 
																												 std::vector< 
																												 std::vector<size_t> > 
																												 &indValue );
  
  void derivs(Compartment &compartment,size_t species,
							std::vector< std::vector<double> > &y,
							std::vector< std::vector<double> > &dydt );
};

/// @brief The class polarizationTransportFastGeneralFGRestricted
///
/// The Polarized transporter molecule \f$P\f$ is divided into the cell
/// cytosol (\f$P_i\f$) and its membranes (\f$P_{ij}\f$) following the
/// dynamics
///
///  \f[ \frac{dP_i}{dt} = - P_i sum_k f(A_{i,k}) + sum_k g(A_{i,k})P_{i,k}
///  \f]
///
/// \f[ \frac{dP_{i,j}}{dt} = P_i f(A_{i,j}) - P_{i,j} g(A_{i,j}) \f]
///
///  where \f$k\f$ is summed over neighboring cells, \f$A_i\f$ is the
///  polarizing signal in cell \f$i\f$ and \f$A_{i,j}\f$ is the polarizing
///  signal in the wall between cell \f$i\f$ and \f$j\f$. The polarization is
///  assumed to be fast and the fixed point values are assumed
///
///  \f[ P^\ast_{i,j} = \frac
///  {\frac{f(A_{i,j})}{g(A_{i,j})}P_{\textnormal{tot}}}
///  {1+sum_k\frac{f(A_{i,k})}{g(A_{i,k})}} \f]
///
///  where \f$P_{\textnormal{tot}}\f$ is the total amount of polarized
///  molecule in a cell.
///
///  The polarized transport of a molecule \f$A\f$ between two neighboring
///  compartments (of different types) are then given by
///  
///  \f[ \frac{dA_{i}}{dt} = - T A_i sum_k P^\ast_{i,k} \f] \f[
///  \frac{dA_{i,j}}{dt} = T(P^\ast_{i,j} A_i + P^\ast_{j,i} A_j) \f]
///
class PolarizationTransportFastGeneralFGRestricted : public BaseReaction
{
 public:
  PolarizationTransportFastGeneralFGRestricted(
    std::vector<double> &paraValue, 
    std::vector< std::vector<size_t> > &indValue);
  
  void  derivs(Compartment &compartment, size_t species,
	       std::vector< std::vector<double> > &y,
	       std::vector< std::vector<double> > &dydt);

///
/// @brief Function to be used in PolarizationTransportFastGeneralFGRestricted
///
  double f(double arg);

///
/// @brief Function to be used in PolarizationTransportFastGeneralFGRestricted
///
  double g(double arg);			       
};

#endif
