#ifndef EULER_H
#define EULER_H
//
// Filename     : euler.h
// Description  : Simple Euler numerical solvers
// Author(s)    : Henrik Jonsson (henrik@thep.lu.se)
// Created      : March 2009
// Revision     : $Id:$
//

#include "baseSolver.h"

///
/// @brief A simple Euler step solver class
///
class Euler : public BaseSolver {

 private:
  
  double h_;
  
 public:
  ///
  /// @brief Main constructor
  ///
  Euler(Organism *O,std::ifstream &IN);
  
  ///
  /// @brief Reads the parameters used by the Euler solver
  ///
  /// This function is responsible for reading parameters used by the
  /// Euler algorithm. The parameter file sent to the simulator
  /// binary looks like:
  ///
  /// <pre> 
  /// Euler
  /// T_start T_end 
  /// printFlag printNum 
  /// h 
  /// </pre> 
  ///
  /// where Euler is the identity string used by BaseSolver::getSolver
  /// to identify that the Euler algorithm should be used. T_start
  /// (T_end) is the start (end) time for the simulation, printFlag is an
  /// integer which sets the output format (read by BaseSolver::print()),
  /// printNum is the number of equally spread time points to be printed. h
  /// is the step size for each Euler step.
  ///
  /// Comments can be included in the parameter file by starting the line with
  /// an #. Caveat: No check on the validity of the read data is applied.
  ///
  /// @see BaseSolver::getSolver()
  /// @see BaseSolver::print()
  ///
  void readParameterFile(std::ifstream &IN);
	
  ///
  /// @brief Runs a simulation of an organism model
  ///
  void simulate(void);
	
  ///
  /// @brief An Euler step
  ///
  void euler();
};

///
/// @brief A simple Euler step solver class, where negative values are truncated to zero
///
/// WARNING! This solver will give numerical errors, and the results
/// cannot be trusted. It is implemented for use as a quick test of a
/// simulation.
///
/// The algorithm takes Euler steps and truncates values at zero,
/// i.e. negative values will be set to zero.
///
class EulerTruncated : public BaseSolver {
  
 private:
  
  double h_;
  
 public:
  ///
  /// @brief Main constructor
  ///
  EulerTruncated(Organism *O,std::ifstream &IN);
  
  ///
  /// @brief Reads the parameters used by the Euler solver
  ///
  /// This function is responsible for reading parameters used by the
  /// Euler algorithm. The parameter file sent to the simulator
  /// binary looks like:
  ///
  /// <pre> 
  /// EulerTruncated
  /// T_start T_end 
  /// printFlag printNum 
  /// h 
  /// </pre> 
  ///
  /// where EulerTruncated is the identity string used by BaseSolver::getSolver
  /// to identify that the Euler algorithm should be used. T_start
  /// (T_end) is the start (end) time for the simulation, printFlag is an
  /// integer which sets the output format (read by BaseSolver::print()),
  /// printNum is the number of equally spread time points to be printed. h
  /// is the step size for each Euler step.
  ///
  /// Comments can be included in the parameter file by starting the line with
  /// an #. Caveat: No check on the validity of the read data is applied.
  ///
  /// @see BaseSolver::getSolver()
  /// @see BaseSolver::print()
  ///
  void readParameterFile(std::ifstream &IN);
	
  ///
  /// @brief Runs a simulation of an organism model
  ///
  void simulate(void);
	
  ///
  /// @brief An Euler step with truncation of negative numbers to zero
  ///
  void eulerTrunc();
};

#endif /* EULER_H */

