//
// Filename     : rungeKutta.cc
// Description  : Different Runge Kutta solvers
// Author(s)    : Patrik Sahlin (sahlin@thep.lu.se)
// Created      : August 2006
// Revision     : $Id: rungeKutta.cc 478 2011-02-21 08:34:45Z henrik $
//
#include <cmath>
#include "rungeKutta.h"

RK5Adaptive::RK5Adaptive(Organism *O,std::ifstream &IN)
	:BaseSolver(O,IN)
{
	readParameterFile(IN);
}

void RK5Adaptive::readParameterFile(std::ifstream &IN)
{
  IN >> startTime_;
  t_= startTime_;
  IN >> endTime_;
  
  IN >> printFlag_;
  IN >> numPrint_;
  
  IN >> h1_;
  IN >> eps_;
}

void RK5Adaptive::simulate(void)
{ 
  // double tiny = 1e-30; // Using NR definition
  double tiny = 1e-9*eps_; // Caveat! Using new definition
  double  h, hNext, hDid;
  
  //Check that h1 and endTime - startTime are > 0
  //////////////////////////////////////////////////////////////////////
  if (h1_ > 0.0 && (endTime_ - startTime_) > 0.0)
    h = h1_;
  else {//either h or (endTime-startTime) <=0
    std::cerr << "solverRk5Adaptive::simulate() - "
	      << "Wrong time borders or time step for simulation. "
	      << "No simulation performed.\n";
    exit(-1);
  }
  //Introduce the neighborhood
  //////////////////////////////////////////////////////////////////////
  if (O_->numNeighborhood())
    O_->neighborhoodCreate(y_, t_);
  if (y_.size() && y_.size() != dydt_.size()) {
    dydt_.resize(y_.size(), y_[0]);
  }
  assert( y_.size() == O_->numCompartment() && y_.size()==dydt_.size());
  //Create all vectors that will be needed here and by rkqs and rkck!
  //////////////////////////////////////////////////////////////////////
  //Used here
  std::vector< std::vector<double> > yScal( N() );
  //Used by rkqs
  std::vector< std::vector<double> > yTemp( N() ),yErr( N() );
  //used by rkck
  std::vector< std::vector<double> > ak2( N() ),ak3( N() ),ak4( N() ),
    ak5( N() ),ak6( N() ),yTempRkck( N() );
  //Resize each vector
  for (size_t i = 0; i < N(); i++) {
    yScal[i].resize(M());
    yErr[i].resize(M());
    yTemp[i].resize(M());
    ak2[i].resize(M());
    ak3[i].resize(M());
    ak4[i].resize(M());
    ak5[i].resize(M());
    ak6[i].resize(M());
    yTempRkck[i].resize(M());
  }
  assert( y_.size()==yScal.size() && y_.size()==yErr.size() );
  assert( y_.size()==yTemp.size() && y_.size()==ak2.size() );
  assert( y_.size()==ak3.size() && y_.size()==ak4.size() );
  assert( y_.size()==ak5.size() && y_.size()==ak6.size() );
  assert( y_.size()==yTempRkck.size() );
  
  // Initiate print times
  //////////////////////////////////////////////////////////////////////
  double printTime = endTime_ + tiny;
  double printDeltaTime = endTime_ + 2.0 * tiny;
  if (numPrint_ <= 0) //No printing
    printFlag_ = 0;
  else if (numPrint_ == 1) { // Print last point (default)
  }
  else if (numPrint_ == 2) { //Print first/last point
    printTime = startTime_ - tiny;
  } 
  else { //Print first/last points and spread the rest uniformly
    printTime = startTime_ - tiny;
    printDeltaTime = (endTime_ - startTime_) / ((double) (numPrint_ - 1));
  }

  // Go
  //////////////////////////////////////////////////////////////////////
  t_ = startTime_;
  numOk_ = numBad_ = 0;
  for (unsigned int nstp = 0;; nstp++) {
    if (debugFlag()) {
			yCopy_[debugCount()] = y_;
		} 
    // Update the derivatives
    O_->derivs(y_, dydt_);
    // Calculate 'scaling' for error measure
    for (size_t i = 0; i < N(); i++)
      for (size_t j = 0; j < M(); j++) {
        yScal[i][j] = fabs(y_[i][j]) + fabs(dydt_[i][j] * h) + tiny;
      }

    // Print if applicable 
    if (printFlag_ && t_ >= printTime) {
      printTime += printDeltaTime;
      print();
    }

    // Check if step is larger than max allowed
    // max step end is min of endTime_ and printTime
    double tMin = endTime_< printTime ? endTime_ : printTime;
    if (t_+h > tMin) h = tMin - t_;

    // Update
    rkqs(h, hDid, hNext, yScal, yTemp, yErr, ak2, ak3, ak4, ak5, ak6, 
				 yTempRkck);
    if (hDid == h) ++numOk_; else ++numBad_;
				
		// Make updates of rections that requires it
		O_->reactionUpdate(hDid, t_, y_);

    // Check for cell divisions, compartmental removal etc
    O_->compartmentChange(y_, dydt_, t_);
		
    // Rescale all temporary vectors as well
    if (y_.size() != yScal.size()) {
      yScal.resize(N(), yScal[0]);
      yTemp.resize(N(), yTemp[0]);
      yErr.resize(N(), yErr[0]);
      ak2.resize(N(), ak2[0]);
      ak3.resize(N(), ak3[0]);
      ak4.resize(N(), ak4[0]);
      ak5.resize(N(), ak5[0]);
      ak6.resize(N(), ak6[0]);
      yTempRkck.resize(N(), yTempRkck[0]);
    }
		
    // Update neighborhood
    if (O_->numNeighborhood())
      O_->neighborhoodUpdate(y_, t_);
		
    // If the end t is passed return (print if applicable)
    if (t_ >= endTime_) {
      if (printFlag_) {
				// Update the derivatives
				O_->derivs(y_, dydt_);
				print();
      }
      std::cerr << "Simulation done.\n"; 
      return;
    }
    //Warn for small step sizes...
    //  if (fabs(hNext) <= hMin) {
    //        std::cerr << "Warning: Step size small (" << hNext
    //  		<< ") in rk5Adaptive::simulate at time " 
    //  		<< t << "\n"; /*exit(-1);*/ }
    h = hNext;
    //Do not take larger steps than h1
    if (h > h1_)
      h = h1_;
  }
}

void RK5Adaptive::
simulateWithConvergingMechanics(double cEps,double cTmax,
				std::vector<size_t> &mechEq,
				std::vector<size_t> &mechPos )
{  
  //double tiny=1e-30;//Using NR definition
  double tiny=1e-9*eps_;//Caveat! Using new definition
  double  h,hNext,hDid;
  
  //Check that h1 and endTime-startTime are > 0
  //////////////////////////////////////////////////////////////////////
  if( h1_>0. && (endTime_-startTime_)>0. )
    h=h1_;
  else {//either h or (endTime-startTime) <=0
    std::cerr << "Rk5Adaptive::simulate() Wrong time borders or time step for "
	      << "simulation. No simulation performed.\n";
    return;
  }
  
  //Create all vectors that will be needed here and by rkqs and rkck!
  //////////////////////////////////////////////////////////////////////
  //Used here
  std::vector< std::vector<double> > yScal( N() );
  //Used by rkqs
  std::vector< std::vector<double> > yTemp( N() ),yErr( N() );
  //used by rkck
  std::vector< std::vector<double> > ak2( N() ),ak3( N() ),ak4( N() ),
    ak5( N() ),ak6( N() ),yTempRkck( N() );
  //Resize each vector
  for( size_t i=0 ; i<N() ; i++ ) {
    yScal[i].resize(M());
    yErr[i].resize( M() );
    yTemp[i].resize( M() );
    ak2[i].resize(M());
    ak3[i].resize(M());
    ak4[i].resize(M());
    ak5[i].resize(M());
    ak6[i].resize(M());
    yTempRkck[i].resize(M());
  }
  
  // Initiate print times
  //////////////////////////////////////////////////////////////////////
  double printTime=endTime_+tiny;
  double printDeltaTime=endTime_+2.*tiny;
  if( numPrint_<=0 )//No printing
    printFlag_=0;
  else if( numPrint_==1 ) {//Print last point (default)
  }
  else if( numPrint_==2 ) {//Print first/last point
    printTime=startTime_-tiny;
  } 
  else {//Print first/last points and spread the rest uniformly
    printTime=startTime_-tiny;
    printDeltaTime=(endTime_-startTime_)/double(numPrint_-1);
  }
  
  //Introduce the neighborhood
  //////////////////////////////////////////////////////////////////////
  if( O_->numNeighborhood() )
    O_->neighborhoodCreate(y_,t_);
  
  // Go
  //////////////////////////////////////////////////////////////////////
  t_=startTime_;
  numOk_ = numBad_ = 0;
  for( unsigned int nstp=0 ; ; nstp++ ) {
    if (debugFlag()) {
			yCopy_[debugCount()] = y_;
		} 
    //Update the derivatives
    O_->derivs(y_,dydt_);
    
    //Calculate 'scaling' for error measure
    for( size_t i=0 ; i<N() ; i++ )
      for( size_t j=0 ; j<M() ; j++ ) {
        yScal[i][j]=fabs(y_[i][j])+fabs(dydt_[i][j]*h)+tiny;
      }

    //Print if applicable 
    if( printFlag_ && t_ >= printTime ) {
      printTime += printDeltaTime;
      print();
    }
    
    //Check if step is larger than max allowed
    //max step end is min of endTime_ and printTime
    double tMin= endTime_<printTime ? endTime_ : printTime;
    if( t_+h>tMin ) h=tMin-t_;
    
    //Update
    rkqs(h,hDid,hNext,yScal,
	 yTemp,yErr,ak2,ak3,ak4,ak5,ak6,yTempRkck);
    if (hDid == h) ++numOk_; else ++numBad_;
    
		//Converge mechanics by simulating mech eq 
    updateMechanicsUntilConvergence(cEps,cTmax,yScal,
				    yTemp,yErr,ak2,ak3,ak4,ak5,ak6,
				    yTempRkck,mechEq,mechPos);
    
		// Make updates of rections that requires it
		O_->reactionUpdate(hDid, t_, y_);

    // Check for cell divisions, compartmental removal etc
    O_->compartmentChange(y_,dydt_,t_);
    
    //Rescale all temporary vectors as well
    if(y_.size() != yScal.size() ) {
      yScal.resize( N(), yScal[0] );
      yTemp.resize( N(), yTemp[0] );
      yErr.resize( N(), yErr[0] );
      ak2.resize( N(), ak2[0] );
      ak3.resize( N(), ak3[0] );
      ak4.resize( N(), ak4[0] );
      ak5.resize( N(), ak5[0] );
      ak6.resize( N(), ak6[0] );
      yTempRkck.resize( N(), yTempRkck[0] );
    }
    
    //Update neighborhood
    if( O_->numNeighborhood() )
      O_->neighborhoodUpdate(y_,t_);

    //If the end t is passed return (print if applicable)
    if ( t_>=endTime_ ) {
      if( printFlag_ ) {
	//Update the derivatives
				O_->derivs(y_,dydt_);
				print();
      }
      std::cerr << "Simulation done.\n"; 
      return;
    }
    //Warn for small step sizes...
    //  if (fabs(hNext) <= hMin) {
    //        std::cerr << "Warning: Step size small (" << hNext
    //  		<< ") in rk5Adaptive::simulate at time " 
    //  		<< t << "\n"; /*exit(-1);*/ }
    h=hNext;
    //Do not take larger steps than h1
    if( h>h1_ )
      h=h1_;
  }  
}

void RK5Adaptive::
updateMechanicsUntilConvergence(double cEps,double cTmax,
				std::vector< std::vector<double> > &yScal,
				std::vector< std::vector<double> > &yTemp,
				std::vector< std::vector<double> > &yErr,
				std::vector< std::vector<double> > &ak2,
				std::vector< std::vector<double> > &ak3,
				std::vector< std::vector<double> > &ak4,
				std::vector< std::vector<double> > &ak5,
				std::vector< std::vector<double> > &ak6,
				std::vector< std::vector<double> > &yTempRkck,
				std::vector<size_t> &mechEq,
				std::vector<size_t> &mechPos ) 
{  
  //double tiny=1e-30;//Using NR definition
  double tiny=1e-9*eps_;//Caveat! Using new definition
  double h=h1_,hNext,hDid;

  // Go
  //////////////////////////////////////////////////////////////////////
  double t=0.0;
  //numOk_ = numBad_ = 0;
  for( unsigned int nstp=0 ; ; nstp++ ) {
		if (debugFlag()) {
			yCopy_[debugCount()] = y_;
		} 
    //Update the derivatives
    O_->derivsMechanical(y_,dydt_,mechEq,mechPos);
    
    //Calculate 'scaling' for error measure
    for( size_t i=0 ; i<N() ; i++ )
      for( size_t k=0 ; k<mechPos.size() ; k++ ) {
				size_t j=mechPos[k];
        yScal[i][j]=fabs(y_[i][j])+fabs(dydt_[i][j]*h)+tiny;
      }
    
    //Update
    rkqsMechanical(t,h,hDid,hNext,yScal,
		   yTemp,yErr,ak2,ak3,ak4,ak5,ak6,yTempRkck,
		   mechEq,mechPos);
    //if (hDid == h) ++numOk_; else ++numBad_;
    
		// Make updates of rections that requires it
		O_->reactionUpdate(hDid, t_, y_);
    
    //Update neighborhood
    if( O_->numNeighborhood() )
      O_->neighborhoodUpdate(y_,t_);

    //If the positional variables have converged 
    //or maximal T is reached, return
    double maxDydt=0.0;
    for( size_t i=0 ; i<N() ; i++ )
      for( size_t k=0 ; k<mechPos.size() ; k++ ) {
	size_t j=mechPos[k];
	if( std::fabs(dydt_[i][j])>maxDydt )
	  maxDydt = std::fabs(dydt_[i][j]); 
      }
    if( maxDydt<cEps ) {
      //std::cerr << "Rk5Adaptive::updateMechanicsUntilConvergance "
      //	<< "t=" << t << " maxDydt=" << maxDydt << "\n";
      return;
    }
    else if ( t>=cTmax ) {
      std::cerr << "Rk5Adaptive::updateMechanicsUntilConvergance\n"
		<< "Warning, final time " << cTmax << " reached with"
		<< " Max dy/dt " << maxDydt << ".\n"; 
      return;
    }
    //Warn for small step sizes...
    //  if (fabs(hNext) <= hMin) {
    //        std::cerr << "Warning: Step size small (" << hNext
    //  		<< ") in rk5Adaptive::simulate at time " 
    //  		<< t << "\n"; /*exit(-1);*/ }
    h=hNext;
    //Do not take larger steps than h1
    if( h>h1_ )
      h=h1_;
  }  
}

#define SAFETY 0.9
#define PGROW -0.2
#define PSHRNK -0.25
#define ERRCON 1.89e-4
void RK5Adaptive::rkqs(double hTry, double &hDid, double &hNext,
											 std::vector< std::vector<double> > &yScal,
											 std::vector< std::vector<double> > &yTemp,
											 std::vector< std::vector<double> > &yErr,
											 std::vector< std::vector<double> > &ak2,
											 std::vector< std::vector<double> > &ak3,
											 std::vector< std::vector<double> > &ak4,
											 std::vector< std::vector<double> > &ak5,
											 std::vector< std::vector<double> > &ak6,
											 std::vector< std::vector<double> > &yTempRkck)
{
  double errMax, h, hTemp, tNew, aux;
  h = hTry;
  for (;;) {
    rkck(h, yTemp, yErr, ak2, ak3, ak4, ak5, ak6, yTempRkck);
    errMax = 0.0;
    for (size_t i = 0; i < N(); i++)
      for (size_t j = 0; j < M(); j++) {
        aux = fabs(yErr[i][j] / yScal[i][j]);
        if (aux > errMax)
          errMax = aux;
      }
    errMax /= eps_;
    if (errMax <= 1.0) break;
    hTemp = SAFETY * h * pow(errMax, PSHRNK);
    if (h >= 0.0)
      h = hTemp > 0.1 * h ? hTemp : 0.1 * h;
    else
      h = hTemp > 0.1 * h ? 0.1 * h : hTemp;
    tNew = t_ + h;
    if (tNew == t_) { 
      std::cerr << "Warning stepsize underflow in solverRk5Adaptive::rkqs\n"; 
      exit(-1);
    }
  }
  if (errMax > ERRCON) hNext = SAFETY * h * pow(errMax, PGROW);
  else hNext = 5.0 * h;
  t_ += (hDid = h);
  
  for (size_t i = 0; i < N(); i++)
    for (size_t j = 0; j < M(); j++)
      y_[i][j] = yTemp[i][j];
}
#undef SAFETY
#undef PGROW
#undef PSHRNK
#undef ERRCON

#define SAFETY 0.9
#define PGROW -0.2
#define PSHRNK -0.25
#define ERRCON 1.89e-4
void RK5Adaptive::
rkqsMechanical(double &t,double hTry,double &hDid,double &hNext,
	       std::vector< std::vector<double> > &yScal,
	       std::vector< std::vector<double> > &yTemp,
	       std::vector< std::vector<double> > &yErr,
	       std::vector< std::vector<double> > &ak2,
	       std::vector< std::vector<double> > &ak3,
	       std::vector< std::vector<double> > &ak4,
	       std::vector< std::vector<double> > &ak5,
	       std::vector< std::vector<double> > &ak6,
	       std::vector< std::vector<double> > &yTempRkck,
	       std::vector<size_t> &mechEq,std::vector<size_t> &posVar) 
{  
  double errMax,h,hTemp,tNew,aux;
  h=hTry;
  for (;;) {
    rkckMechanical(h,yTemp,yErr,ak2,ak3,ak4,ak5,ak6,yTempRkck,
		   mechEq,posVar);
    errMax=0.;
    for(size_t i=0 ; i<N() ; i++ )
      for( size_t k=0 ; k<posVar.size() ; k++ ) {
	size_t j=posVar[k];
        aux = fabs(yErr[i][j]/yScal[i][j]);
        if( aux>errMax )
          errMax = aux;
      }
    errMax /= eps_;
    if (errMax <= 1.0 ) break;
    hTemp=SAFETY*h*pow(errMax,PSHRNK);
    if( h >= 0. )
      h = hTemp > 0.1*h ? hTemp : 0.1*h;
    else
      h = hTemp > 0.1*h ? 0.1*h : hTemp;
    tNew=t+h;
    if( tNew==t ) { 
      std::cerr << "Warning stepsize underflow in Rk5Adaptive::rkqs\n"; 
      exit(-1); }
  }
  if (errMax > ERRCON) hNext=SAFETY*h*pow(errMax,PGROW);
  else hNext=5.0*h;
  t += (hDid=h);
  
  for(size_t i=0 ; i<N() ; i++ )
    for( size_t k=0 ; k<posVar.size() ; k++ ) {
      size_t j=posVar[k];
      y_[i][j]=yTemp[i][j];
    }
}
#undef SAFETY
#undef PGROW
#undef PSHRNK
#undef ERRCON

void RK5Adaptive::rkck(double h,
		       std::vector< std::vector<double> > &yOut,
		       std::vector< std::vector<double> > &yErr,
		       std::vector< std::vector<double> > &ak2,
		       std::vector< std::vector<double> > &ak3,
		       std::vector< std::vector<double> > &ak4,
		       std::vector< std::vector<double> > &ak5,
		       std::vector< std::vector<double> > &ak6,
		       std::vector< std::vector<double> > &yTempRkck ) {
  
  //static double a2=0.2,a3=0.3,a4=0.6,a5=1.0,a6=0.875;
  static double b21=0.2,
    b31=3.0/40.0,b32=9.0/40.0,b41=0.3,b42 = -0.9,b43=1.2,
    b51 = -11.0/54.0, b52=2.5,b53 = -70.0/27.0,b54=35.0/27.0,
    b61=1631.0/55296.0,b62=175.0/512.0,b63=575.0/13824.0,
    b64=44275.0/110592.0,b65=253.0/4096.0,c1=37.0/378.0,
    c3=250.0/621.0,c4=125.0/594.0,c6=512.0/1771.0,
    dc5 = -277.00/14336.0;
  double dc1=c1-2825.0/27648.0,dc3=c3-18575.0/48384.0,
    dc4=c4-13525.0/55296.0,dc6=c6-0.25;
  double n = N();
	double m = M();
	
  for (size_t i = 0; i < n; ++i)
    for (size_t j = 0; j < m; ++j)
      yTempRkck[i][j]=y_[i][j]+b21*h*dydt_[i][j];
  
  O_->derivs(yTempRkck, ak2); // t + a2h
	for (size_t i = 0; i < n; ++i)
		for (size_t j = 0; j < m; ++j)
			yTempRkck[i][j]=y_[i][j]+h*(b31*dydt_[i][j]+b32*ak2[i][j]);
  
  O_->derivs(yTempRkck, ak3); // t + a3h
	for (size_t i = 0; i < n; ++i)
		for (size_t j = 0; j < m; ++j)
			yTempRkck[i][j]=y_[i][j]+h*(b41*dydt_[i][j]+b42*ak2[i][j]+b43*ak3[i][j]);
  
  O_->derivs(yTempRkck, ak4); // t + a4 * h
	for (size_t i = 0; i < n; ++i)
		for (size_t j = 0; j < m; ++j)
			yTempRkck[i][j]=y_[i][j]+h*
				(b51*dydt_[i][j]+b52*ak2[i][j]+b53*ak3[i][j]+b54*ak4[i][j]);
  
  O_->derivs(yTempRkck, ak5); // t + a5 * h
	for (size_t i = 0; i < n; ++i)
		for (size_t j = 0; j < m; ++j)
			yTempRkck[i][j]=y_[i][j]+h*
				(b61*dydt_[i][j]+b62*ak2[i][j]+b63*ak3[i][j]+b64*ak4[i][j]+
				 b65*ak5[i][j]);
  
  O_->derivs(yTempRkck, ak6); // t + a6 * h
	for (size_t i = 0; i < n; ++i)
		for (size_t j = 0; j < m; ++j)
			yOut[i][j]=y_[i][j]+h*
				(c1*dydt_[i][j]+c3*ak3[i][j]+c4*ak4[i][j]+c6*ak6[i][j]);
  
	for (size_t i = 0; i < n; ++i)
    for (size_t j = 0; j < m; ++j)
      yErr[i][j]=h*
        (dc1*dydt_[i][j]+dc3*ak3[i][j]+dc4*ak4[i][j]+
         dc5*ak5[i][j]+dc6*ak6[i][j]);
}

void RK5Adaptive::
rkckMechanical(double h,
	       std::vector< std::vector<double> > &yOut,
	       std::vector< std::vector<double> > &yErr,
	       std::vector< std::vector<double> > &ak2,
	       std::vector< std::vector<double> > &ak3,
	       std::vector< std::vector<double> > &ak4,
	       std::vector< std::vector<double> > &ak5,
	       std::vector< std::vector<double> > &ak6,
	       std::vector< std::vector<double> > &yTempRkck,
	       std::vector<size_t> &mechEq,std::vector<size_t> &posVar ) 
{  
  //static double a2=0.2,a3=0.3,a4=0.6,a5=1.0,a6=0.875;
  static double b21=0.2,
    b31=3.0/40.0,b32=9.0/40.0,b41=0.3,b42 = -0.9,b43=1.2,
    b51 = -11.0/54.0, b52=2.5,b53 = -70.0/27.0,b54=35.0/27.0,
    b61=1631.0/55296.0,b62=175.0/512.0,b63=575.0/13824.0,
    b64=44275.0/110592.0,b65=253.0/4096.0,c1=37.0/378.0,
    c3=250.0/621.0,c4=125.0/594.0,c6=512.0/1771.0,
    dc5 = -277.00/14336.0;
  double dc1=c1-2825.0/27648.0,dc3=c3-18575.0/48384.0,
    dc4=c4-13525.0/55296.0,dc6=c6-0.25;
  
  for(size_t i=0 ; i<N() ; i++ )
    for(size_t k=0 ; k<posVar.size() ; k++ ) {
      size_t j=posVar[k];
      yTempRkck[i][j]=y_[i][j]+b21*h*dydt_[i][j];
    }
  O_->derivsMechanical(yTempRkck,ak2,mechEq,posVar);//t+a2h
  for(size_t i=0 ; i<N() ; i++ )
    for(size_t k=0 ; k<posVar.size() ; k++ ) {
      size_t j=posVar[k];
      yTempRkck[i][j]=y_[i][j]+h*(b31*dydt_[i][j]+b32*ak2[i][j]);
    }
  O_->derivsMechanical(yTempRkck,ak3,mechEq,posVar);//t+a3h
  for(size_t i=0 ; i<N() ; i++ )
    for(size_t k=0 ; k<posVar.size() ; k++ ) {
      size_t j=posVar[k];
      yTempRkck[i][j]=y_[i][j]+h*(b41*dydt_[i][j]+b42*ak2[i][j]+b43*ak3[i][j]);
    }
  O_->derivsMechanical(yTempRkck,ak4,mechEq,posVar);//t+a4*h
  for(size_t i=0 ; i<N() ; i++ )
    for(size_t k=0 ; k<posVar.size() ; k++ ) {
      size_t j=posVar[k];
      yTempRkck[i][j]=y_[i][j]+h*
        (b51*dydt_[i][j]+b52*ak2[i][j]+b53*ak3[i][j]+b54*ak4[i][j]);
    }
  O_->derivsMechanical(yTempRkck,ak5,mechEq,posVar);//t+a5*h
  for(size_t i=0 ; i<N() ; i++ )
    for(size_t k=0 ; k<posVar.size() ; k++ ) {
      size_t j=posVar[k];
      yTempRkck[i][j]=y_[i][j]+h*
        (b61*dydt_[i][j]+b62*ak2[i][j]+b63*ak3[i][j]+b64*ak4[i][j]+
         b65*ak5[i][j]);
    }  
  O_->derivsMechanical(yTempRkck,ak6,mechEq,posVar);//t+a6*h
  for(size_t i=0 ; i<N() ; i++ )
    for(size_t k=0 ; k<posVar.size() ; k++ ) {
      size_t j=posVar[k];
      yOut[i][j]=y_[i][j]+h*
        (c1*dydt_[i][j]+c3*ak3[i][j]+c4*ak4[i][j]+c6*ak6[i][j]);
    }
  for(size_t i=0 ; i<N() ; i++ )
    for(size_t k=0 ; k<posVar.size() ; k++ ) {
      size_t j=posVar[k];
      yErr[i][j]=h*
        (dc1*dydt_[i][j]+dc3*ak3[i][j]+dc4*ak4[i][j]+
         dc5*ak5[i][j]+dc6*ak6[i][j]);
    }
}

///////////////////////////////////////////////////////////////////////
// CLASS SOLVERRK5ADAPTIVETEMPLATE
///////////////////////////////////////////////////////////////////////
RK5AdaptiveTemplate::RK5AdaptiveTemplate(Organism *O,std::ifstream &IN)
	:BaseSolver(O,IN)
{
	readParameterFile(IN);
}

RK5AdaptiveTemplate::~RK5AdaptiveTemplate()
{
	//std::cerr << "delete costFunction...";
	delete C_;
	//std::cerr << "Done!\n";
}

void RK5AdaptiveTemplate::readParameterFile(std::ifstream &IN, int verbose)
{
  verbose=1;
  if (verbose) {
    std::cerr << "RK5AdaptiveTemplate: Reading parameter file:" << std::endl;
  }
  std::string costType;
  IN >> costType;
  C_ = BaseCostFunction::createCostFunction(costType);
  
  IN >> startTime_;
  t_= startTime_;
  IN >> endTime_;
  
  IN >> printFlag_;
  IN >> numPrint_;
  
  IN >> h1_;
  IN >> eps_;
  
  if (verbose) {
    std::cerr << "Cost function: " << costType << std::endl
	      << "T_start/end: " << startTime_ << " " << endTime_ << std::endl
	      << "Print_flag: " << printFlag_ << " PrintNum: " << numPrint_ << std::endl
	      << "Initial/max step: " << h1_ << " eps: " << eps_ << std::endl;
  }
  
  //simulation template parameters
  size_t Mtmp;
  IN >> Mtmp;
  
  simulationIndex_.resize( Mtmp );
  
  if (Mtmp) {
    if (verbose)
      std::cerr << Mtmp << " variables read from template." << std::endl;
    for (size_t i=0 ; i<Mtmp ; ++i) {
      IN >> simulationIndex_[i];
      if (verbose)
	std::cerr << simulationIndex_[i] << " ";
    }
    IN >> inputTemplateFile_;
    if (verbose)
      std::cerr << std::endl << "Template file " << inputTemplateFile_ << std::endl; 
  }
  else
    if (verbose)
      std::cerr << "No simulation template file used." << std::endl; 
  
  //cost template parameters
  IN >> Mtmp;
  std::vector<size_t> costList;
  
  if( Mtmp ) {
    if (verbose)
      std::cerr << Mtmp << " variables used in cost calculation\n";
    costList.resize(Mtmp);
    for (size_t i=0 ; i<Mtmp ; i++ ) {
      IN >> costList[i];
      if (verbose)
	std::cerr << costList[i] << " ";
    }
    if (verbose)
      std::cerr << "\n";
    setCostList(costList);
    std::string costTemplateFile;	
    IN >> costTemplateFile;
    setCostTemplateFile(costTemplateFile);
    if (verbose)
      std::cerr << "Cost template file " << costTemplateFile << "\n"; 
    C_->initiateCostCalculation(startTime_, endTime_);
  }
  else
    if (verbose)
      std::cerr << "No cost template file used\n";
  //IN.close();
  if (verbose) {
    std::cerr << "RK5AdaptiveTemplate: Done reading parameter file." << std::endl << std::endl;
  }
}

void RK5AdaptiveTemplate::simulate(void)
{ 
	static bool flag = true;
	if(flag)
		setSimulationFlag();
	flag = false;
	
  // double tiny = 1e-30; // Using NR definition
  double tiny = 1e-9*eps_; // Caveat! Using new definition
	double  h, hNext, hDid;
  //Vectors to store division data from template file
  std::vector<int> divIndex;
  std::vector<double> divTime;
  size_t division=0;

  //Check if template reading is needed...
  size_t simCount=0;
  size_t templateFlag=1;
  for (size_t i=0 ; i<M() ; i++ ) {
    simCount += simulationFlag_[i];
  }
  if( simCount==M() ) 
    templateFlag = 0;
  //Check if the costList is well defined
  for (size_t i=0; i<C_->sizeOfCostList(); ++i) {
    if (C_->costList(i)>=M()) {
      std::cerr << "RK5AdaptiveTemplate::simulate(): "
		<< C_->costList(i) << " is out of scope\n";
      exit(-1);
    }
  }
  //Check that h1 and endTime - startTime are > 0
  //////////////////////////////////////////////////////////////////////
  if (h1_ > 0.0 && (endTime_ - startTime_) > 0.0)
    h = h1_;
  else {//either h or (endTime-startTime) <=0
    std::cerr << "solverRk5Adaptive::simulate() - "
	      << "Wrong time borders or time step for simulation. "
	      << "No simulation performed.\n";
    exit(-1);
  }
  //Introduce the neighborhood
  //////////////////////////////////////////////////////////////////////
  if (O_->numNeighborhood())
    O_->neighborhoodCreate(y_, t_);
  if (y_.size() && y_.size() != dydt_.size()) {
    dydt_.resize(y_.size(), y_[0]);
  }
  assert( y_.size() == O_->numCompartment() && y_.size()==dydt_.size());
  //Create all vectors that will be needed here and by rkqs and rkck!
  //////////////////////////////////////////////////////////////////////
  //Used here
  std::vector< std::vector<double> > yScal( N() );
  //Used by rkqs
  std::vector< std::vector<double> > yTemp( N() ),yErr( N() );
  //used by rkck
  std::vector< std::vector<double> > ak2( N() ),ak3( N() ),ak4( N() ),
    ak5( N() ),ak6( N() ),yTempRkck( N() );
  //Resize each vector
  for (size_t i = 0; i < N(); i++) {
    yScal[i].resize(M());
    yErr[i].resize(M());
    yTemp[i].resize(M());
    ak2[i].resize(M());
    ak3[i].resize(M());
    ak4[i].resize(M());
    ak5[i].resize(M());
    ak6[i].resize(M());
    yTempRkck[i].resize(M());
  }
  assert( y_.size()==yScal.size() && y_.size()==yErr.size() );
  assert( y_.size()==yTemp.size() && y_.size()==ak2.size() );
  assert( y_.size()==ak3.size() && y_.size()==ak4.size() );
  assert( y_.size()==ak5.size() && y_.size()==ak6.size() );
  assert( y_.size()==yTempRkck.size() );
  
  
  // Initiate cost Calculation if applicable
  //////////////////////////////////////////////////////////////////////
  double endTimeLocal = endTime_;
  if( C_->sizeOfCostList() ) {
    //C_->initiateCostCalculation(startTime_, endTime_);
    C_->resetCostCalculation();
    if (C_->sizeOfCostTemplateTime() ) {
      endTimeLocal = C_->endTime() +tiny;  
    }
  }
  // Initiate print times
  //////////////////////////////////////////////////////////////////////
  double printTime = endTimeLocal + tiny;
  double printDeltaTime = endTimeLocal + 2.0 * tiny;
  if (numPrint_ <= 0) //No printing
    printFlag_ = 0;
  else if (numPrint_ == 1) { // Print last point (default)
  }
  else if (numPrint_ == 2) { //Print first/last point
    printTime = startTime_ - tiny;
  } 
  else { //Print first/last points and spread the rest uniformly
    printTime = startTime_ - tiny;
    printDeltaTime = (endTimeLocal - startTime_) / ((double) (numPrint_ - 1));
  }
  
  // Go
  //////////////////////////////////////////////////////////////////////
  t_ = startTime_;
  // Initiate template from file if applicable
  //////////////////////////////////////////////////////////////////////
  std::ifstream IN;
  if( templateFlag ) {
    const char *tmp = inputTemplateFile_.c_str();
    IN.open( tmp );
    if( !IN ) {
      std::cerr << "Rk5::simulateTemplate() "
		<< "Cannot open templatefile " << inputTemplateFile_
		<< "\n\n\7";exit(-1);}
    //Read previous and future time points
    initiateValuesTemplate(IN,divIndex,divTime);
  }
  
  numOk_ = numBad_ = 0;
  for (unsigned int nstp = 0;; nstp++) {
    if (debugFlag()) {
      yCopy_[debugCount()] = y_;
    } 
    // Update the derivatives
    O_->derivsTemplate(y_,dydt_,simulationFlag_);
    
    // Calculate 'scaling' for error measure
    for (size_t i = 0; i < N(); i++)
      for (size_t j = 0; j < M(); j++) {
	yScal[i][j] = fabs(y_[i][j]) + fabs(dydt_[i][j] * h) + tiny;
	// Try this modification to see whether it fasten up things
	// if( yScal[i][j]<1.0 )
	// yScal[i][j] = 1.0;
      }
    
    // Print if applicable 
    if (printFlag_ && t_ >= printTime) {
      printTime += printDeltaTime;
      print();
    }
    
    // Check if step is larger than max allowed
    // max step end is min of endTime_ and printTime
    double tMin = endTimeLocal < printTime ? endTimeLocal : printTime;
    
    int maxStepFlag = 0;//1 if stepsize shortened
    double hTemp=h; //stepsize sugested before shortened
    
    if( templateFlag && futureTime()<tMin ) tMin = futureTime(); 
    if( templateFlag && division<divIndex.size() && divTime[division]<tMin ) 
      tMin=divTime[division];
    if( C_->sizeOfCostList() && C_->costTemplateIndex()<C_->sizeOfCostTemplateTime() 
	&& C_->costTemplateTime( C_->costTemplateIndex() )<tMin )
      tMin=C_->costTemplateTime( C_->costTemplateIndex() );
    if( t_+h>tMin ){
      h=tMin-t_;
      maxStepFlag = 1;
    }
    
    // Update
    rkqsTemplate(h, hDid, hNext, yScal, yTemp, yErr, ak2, ak3, ak4, ak5, ak6, 
		 yTempRkck);
    if (hDid == h) ++numOk_; else ++numBad_;
    //Update other cols of y from template
    if( templateFlag )
      allValuesTemplate();//time is already updated
    
    // Make updates of rections that requires it
    O_->reactionUpdate(hDid, t_, y_);
    
    // Check for cell divisions, compartmental removal etc
    //O_->compartmentChange(y_, dydt_, t_);
    if( templateFlag ) {
      int tmpDivCount=0;
      while( division<divIndex.size() && t_>=divTime[division] ) {
	divideTemplate(divIndex[division]);
	division++;
	tmpDivCount++;
      }
      if( tmpDivCount ) {
	// Rescale all temporary vectors as well
	if (y_.size() != yScal.size()) {
	  yScal.resize(N(), yScal[0]);
	  yTemp.resize(N(), yTemp[0]);
	  yErr.resize(N(), yErr[0]);
	  ak2.resize(N(), ak2[0]);
	  ak3.resize(N(), ak3[0]);
	  ak4.resize(N(), ak4[0]);
	  ak5.resize(N(), ak5[0]);
	  ak6.resize(N(), ak6[0]);
	  yTempRkck.resize(N(), yTempRkck[0]);
	}
      }
    }
    //Add cost from template comparison if applicable
    //////////////////////////////////////////////////////////////////////
    //size_t tmpCount=0;
    //if (!tmpCount) {
    //std::cerr << "Cost calculation variables:" << std::endl;
    //std::cerr << C_->sizeOfCostList() << " "
    //		<< C_->costTemplateIndex() << " "
    //		<< C_->sizeOfCostTemplateTime() << " "
    //		<< C_->costTemplateTime(C_->costTemplateIndex()) << std::endl;
    //++tmpCount;
    //}
    if( C_->sizeOfCostList() && C_->costTemplateIndex()< C_->sizeOfCostTemplateTime()
	&& t_>= C_->costTemplateTime(C_->costTemplateIndex() ) ) {
      //std::cerr << "Cost added at time " << t_ << std::endl;
      C_->addCost(y_,t_); 
    }
    
    // Check if new time point data needs to be read from file
    //////////////////////////////////////////////////////////////////////
    if( templateFlag && t_>=futureTime_ && t_<endTimeLocal ) {
      updateValuesTemplate(IN,endTimeLocal,divIndex,divTime);
      division=0;    
    }
    // Update neighborhood
    if (O_->numNeighborhood())
      O_->neighborhoodUpdate(y_, t_);
    
    // If the end t is passed return (print if applicable)
    if (t_ >= endTimeLocal) {
      if (printFlag_) {
	// Update the derivatives
	O_->derivs(y_, dydt_);
	print();
      }
      //std::cerr << "Simulation done.\n"; 
      return;
    }
    //Warn for small step sizes...
    //  if (fabs(hNext) <= hMin) {
    //        std::cerr << "Warning: Step size small (" << hNext
    //  		<< ") in rk5Adaptive::simulate at time " 
    //  		<< t << "\n"; /*exit(-1);*/ }
    if(!maxStepFlag)
      h=hNext;
    else
      h=hTemp;
    //Do not take larger steps than h1
    if (h > h1_)
      h = h1_;
  }
}

#define SAFETY 0.9
#define PGROW -0.2
#define PSHRNK -0.25
#define ERRCON 1.89e-4
void RK5AdaptiveTemplate::rkqsTemplate(double hTry,double &hDid,double &hNext,
		       std::vector< std::vector<double> > &yScal,
		       std::vector< std::vector<double> > &yTemp,
		       std::vector< std::vector<double> > &yErr,
		       std::vector< std::vector<double> > &ak2,
		       std::vector< std::vector<double> > &ak3,
		       std::vector< std::vector<double> > &ak4,
		       std::vector< std::vector<double> > &ak5,
		       std::vector< std::vector<double> > &ak6,
		       std::vector< std::vector<double> > &yTempRkck ) 
{  
  double errMax,h,hTemp,tNew,aux;
  h=hTry;
  //rkckTemplate(h,yTemp,yErr,
  // ak2,ak3,ak4,ak5,ak6,yTempRkck);
  for (;;) {
    rkckTemplate(h,yTemp,yErr,
	 ak2,ak3,ak4,ak5,ak6,yTempRkck);
    errMax=0.;
    for (size_t j=0 ; j<M() ; j++ )
      if( simulationFlag_[j] )
	for (size_t i=0 ; i<N() ; i++ ) {
	  aux = fabs(yErr[i][j]/yScal[i][j]);
	  if( aux>errMax )
	    errMax = aux;
	}
    errMax /= eps_;
    if (errMax <= 1.0) break;
    //if (errMax <= 1.0 || h <= 0.01) break;
    hTemp=SAFETY*h*pow(errMax,PSHRNK);
    if( h >= 0. )
      h = hTemp > 0.1*h ? hTemp : 0.1*h;
    else
      h = hTemp > 0.1*h ? 0.1*h : hTemp;
    tNew=t_+h;
    if( tNew==t_ ) { std::cerr << "stepsize underflow in rkqs\n"; exit(-1); }
  }
  if (errMax  <= 1.0) {
    if (errMax > ERRCON) hNext=SAFETY*h*pow(errMax,PGROW);
    else hNext=5.0*h;
  }
  else
    hNext = h;
  //std::cerr << h << " " << hTry << " " << hNext << " " << hDid << " " << "\n";
  t_ += (hDid=h);
  for (size_t j=0 ; j<M() ; j++ )
    if( simulationFlag_[j] )
      for (size_t i=0 ; i<N() ; i++ )
	y_[i][j]=yTemp[i][j];

}
#undef SAFETY
#undef PGROW
#undef PSHRNK
#undef ERRCON

void RK5AdaptiveTemplate::rkckTemplate(double h,
		       std::vector< std::vector<double> > &yOut,
		       std::vector< std::vector<double> > &yErr,
		       std::vector< std::vector<double> > &ak2,
		       std::vector< std::vector<double> > &ak3,
		       std::vector< std::vector<double> > &ak4,
		       std::vector< std::vector<double> > &ak5,
		       std::vector< std::vector<double> > &ak6,
		       std::vector< std::vector<double> > &yTempRkck ) 
{  
    //static double a2=0.2,a3=0.3,a4=0.6,a5=1.0,a6=0.875;
    static double b21=0.2,
    b31=3.0/40.0,b32=9.0/40.0,b41=0.3,b42 = -0.9,b43=1.2,
    b51 = -11.0/54.0, b52=2.5,b53 = -70.0/27.0,b54=35.0/27.0,
    b61=1631.0/55296.0,b62=175.0/512.0,b63=575.0/13824.0,
    b64=44275.0/110592.0,b65=253.0/4096.0,c1=37.0/378.0,
    c3=250.0/621.0,c4=125.0/594.0,c6=512.0/1771.0,
    dc5 = -277.00/14336.0;
  double dc1=c1-2825.0/27648.0,dc3=c3-18575.0/48384.0,
    dc4=c4-13525.0/55296.0,dc6=c6-0.25;
  
  for (size_t i=0 ; i<N() ; i++ )
    for (size_t j=0 ; j<M() ; j++ )
      yTempRkck[i][j]=y_[i][j]+b21*h*dydt_[i][j];
  
  O_->derivsTemplate(yTempRkck,ak2,simulationFlag_);//t+a2h
  for (size_t i=0 ; i<N() ; i++ )
    for (size_t j=0 ; j<M() ; j++ )
      yTempRkck[i][j]=y_[i][j]+h*(b31*dydt_[i][j]+b32*ak2[i][j]);
  
  O_->derivsTemplate(yTempRkck,ak3,simulationFlag_);//t+a3h
  for (size_t i=0 ; i<N() ; i++ )
    for (size_t j=0 ; j<M() ; j++ )
      yTempRkck[i][j]=y_[i][j]+h*(b41*dydt_[i][j]+b42*ak2[i][j]+b43*ak3[i][j]);
  
  O_->derivsTemplate(yTempRkck,ak4,simulationFlag_);//t+a4*h
  for (size_t i=0 ; i<N() ; i++ )
    for (size_t j=0 ; j<M() ; j++ )
      yTempRkck[i][j]=y_[i][j]+h*
        (b51*dydt_[i][j]+b52*ak2[i][j]+b53*ak3[i][j]+b54*ak4[i][j]);
  
  O_->derivsTemplate(yTempRkck,ak5,simulationFlag_);//t+a5*h
  for (size_t i=0 ; i<N() ; i++ )
    for (size_t j=0 ; j<M() ; j++ )
      yTempRkck[i][j]=y_[i][j]+h*
        (b61*dydt_[i][j]+b62*ak2[i][j]+b63*ak3[i][j]+b64*ak4[i][j]+
         b65*ak5[i][j]);
  
  O_->derivsTemplate(yTempRkck,ak6,simulationFlag_);//t+a6*h
  for (size_t j=0 ; j<M() ; j++ )
    if( simulationFlag_[j] )
      for (size_t i=0 ; i<N() ; i++ )
	yOut[i][j]=y_[i][j]+h*
	  (c1*dydt_[i][j]+c3*ak3[i][j]+c4*ak4[i][j]+c6*ak6[i][j]);
  
  for (size_t j=0 ; j<M() ; j++ )
    if( simulationFlag_[j] )
      for (size_t i=0 ; i<N() ; i++ )
	yErr[i][j]=h*
	  (dc1*dydt_[i][j]+dc3*ak3[i][j]+dc4*ak4[i][j]+
	   dc5*ak5[i][j]+dc6*ak6[i][j]);
}

///////////////////////////////////////////////////////////////////////
// CLASS SOLVERRK4
///////////////////////////////////////////////////////////////////////
RK4::RK4(Organism *O,std::ifstream &IN)
	:BaseSolver(O,IN)
{
	readParameterFile(IN);
}

void RK4::readParameterFile(std::ifstream &IN)
{
  //Read in the needed parameters
  IN >> startTime_;
  t_=startTime_;
  IN >> endTime_;
  
  IN >> printFlag_;
  IN >> numPrint_;
  
  IN >> h_;
}

void RK4::simulate() 
{
	//Check that h1 and endTime-startTime are > 0
  //////////////////////////////////////////////////////////////////////
  if( !(h_>0. && (endTime_-startTime_)>0.) ) {
    std::cerr << "Rk4::simulate() Wrong time borders or time step for "
	      << "simulation. No simulation performed.\n";
    return;
  }
	std::cerr << "Simulating using fourth-order Runge-Kutta\n";
  //Introduce the neighborhood
  //////////////////////////////////////////////////////////////////////
  if( O_->numNeighborhood() )
    O_->neighborhoodCreate(y_,t_);
  if( y_.size() && y_.size() != dydt_.size() ) {
    dydt_.resize( y_.size(),y_[0]);
  }
  assert( y_.size() == O_->numCompartment() && y_.size()==dydt_.size());
	
  //Create all vectors that will be needed by rk4()!
  //////////////////////////////////////////////////////////////////////
  std::vector< std::vector<double> > yt( N() ),dyt( N() ),dym( N() );
  //Resize each vector
  for( size_t i=0 ; i<N() ; i++ ) {
    yt[i].resize(M());
    dyt[i].resize( M() );
    dym[i].resize( M() );
  }
  assert( y_.size()==yt.size() && y_.size()==dyt.size() &&
	  y_.size()==dym.size() );
  
  // Initiate print times
  //////////////////////////////////////////////////////////////////////
  double tiny = 1e-10;
  double printTime=endTime_+tiny;
  double printDeltaTime=endTime_+2.*tiny;
  if( numPrint_<=0 )//No printing
    printFlag_=0;
  else if( numPrint_==1 ) {//Print last point (default)
  }
  else if( numPrint_==2 ) {//Print first/last point
    printTime=startTime_-tiny;
  } 
  else {//Print first/last points and spread the rest uniformly
    printTime=startTime_-tiny;
    printDeltaTime=(endTime_-startTime_)/double(numPrint_-1);
  }
  
  // Go
  //////////////////////////////////////////////////////////////////////
  t_=startTime_;
  numOk_ = numBad_ = 0;
  while( t_<endTime_ ) {
		if (debugFlag()) {
			yCopy_[debugCount()] = y_;
		} 
   //Update the derivatives
    O_->derivs(y_,dydt_);
    
    //Print if applicable 
    if( printFlag_ && t_ >= printTime ) {
      printTime += printDeltaTime;
      print();
    }

    //Check if step is larger than max allowed
    //max step end is min of endTime_ and printTime
    //double tMin= endTime_<printTime ? endTime_ : printTime;
    //if( t_+h>tMin ) h=tMin-t_;

    //Update
    rk4(yt,dyt,dym);
    numOk_++;
    
		// Make updates of rections that requires it
		O_->reactionUpdate(h_, t_, y_);

    // Check for cell divisions, compartmental removal etc
    O_->compartmentChange(y_,dydt_,t_);
    
    //Rescale all temporary vectors as well
    if(y_.size() != yt.size() ) {
      yt.resize( N(), yt[0] );
      dyt.resize( N(), dyt[0] );
      dym.resize( N(), dym[0] );
    }
    
    //Update neighborhood
    if( O_->numNeighborhood() )
      O_->neighborhoodUpdate(y_,t_);
    
    //update time variable
    if( (t_+h_)==t_ ) {
      std::cerr << "Rk4::simulate() Step size too small.";
      exit(-1);
    }
    t_ += h_;
  }
  if( printFlag_ ) {
    //Update the derivatives
    O_->derivs(y_,dydt_);
    print();
  }
  std::cerr << "Simulation done.\n"; 
  return;
}

void RK4::simulateWithConvergingMechanics(double cEps,double cTmax,
					  std::vector<size_t> &mechEq,
					  std::vector<size_t> &mechPos) 
{
  //Check that h1 and endTime-startTime are > 0
  //////////////////////////////////////////////////////////////////////
  if( !(h_>0. && (endTime_-startTime_)>0.) ) {
    std::cerr << "Rk4::simulate() Wrong time borders or time step for "
	      << "simulation. No simulation performed.\n";
    return;
  }
  
  //Introduce the neighborhood
  //////////////////////////////////////////////////////////////////////
  if( O_->numNeighborhood() )
    O_->neighborhoodCreate(y_,t_);
  if( y_.size() && y_.size() != dydt_.size() ) {
    dydt_.resize( y_.size(),y_[0]);
  }
  assert( y_.size() == O_->numCompartment() && y_.size()==dydt_.size());
  
  //Create all vectors that will be needed by rk4()!
  //////////////////////////////////////////////////////////////////////
  std::vector< std::vector<double> > yt( N() ),dyt( N() ),dym( N() );
  //Resize each vector
  for( size_t i=0 ; i<N() ; i++ ) {
    yt[i].resize(M());
    dyt[i].resize( M() );
    dym[i].resize( M() );
  }
  assert( y_.size()==yt.size() && y_.size()==dyt.size() &&
	  y_.size()==dym.size() );
  
  // Initiate print times
  //////////////////////////////////////////////////////////////////////
  double tiny = 1e-10;
  double printTime=endTime_+tiny;
  double printDeltaTime=endTime_+2.*tiny;
  if( numPrint_<=0 )//No printing
    printFlag_=0;
  else if( numPrint_==1 ) {//Print last point (default)
  }
  else if( numPrint_==2 ) {//Print first/last point
    printTime=startTime_-tiny;
  } 
  else {//Print first/last points and spread the rest uniformly
    printTime=startTime_-tiny;
    printDeltaTime=(endTime_-startTime_)/double(numPrint_-1);
  }
  
  // Go
  //////////////////////////////////////////////////////////////////////
  t_=startTime_;
  numOk_ = numBad_= 0;
  while( t_<endTime_ ) {
    if (debugFlag()) {
			yCopy_[debugCount()] = y_;
		} 
    //Update the derivatives
    O_->derivs(y_,dydt_);
    
    //Print if applicable 
    if( printFlag_ && t_ >= printTime ) {
      printTime += printDeltaTime;
      print();
    }

    //Check if step is larger than max allowed
    //max step end is min of endTime_ and printTime
    //double tMin= endTime_<printTime ? endTime_ : printTime;
    //if( t_+h>tMin ) h=tMin-t_;

    //Update
    rk4(yt,dyt,dym);
    numOk_++;

    //Converge the mechanics
    updateMechanicsUntilConvergence(cEps,cTmax,yt,dyt,dym,mechEq,mechPos);

		// Make updates of rections that requires it
		O_->reactionUpdate(h_, t_, y_);

    // Check for cell divisions, compartmental removal etc
    O_->compartmentChange(y_,dydt_,t_);
    
    //Rescale all temporary vectors as well
    if(y_.size() != yt.size() ) {
      yt.resize( N(), yt[0] );
      dyt.resize( N(), dyt[0] );
      dym.resize( N(), dym[0] );
    }
    
    //Update neighborhood
    if( O_->numNeighborhood() )
      O_->neighborhoodUpdate(y_,t_);
    
    //update time variable
    if( (t_+h_)==t_ ) {
      std::cerr << "Rk4::simulate() Step size too small.";
      exit(-1);
    }
    t_ += h_;
  }
  if( printFlag_ ) {
    //Update the derivatives
    O_->derivs(y_,dydt_);
    print();
  }
  std::cerr << "Simulation done.\n"; 
  return;
}

void RK4::updateMechanicsUntilConvergence(double cEps,double cTmax,
					  std::vector< std::vector<double> > 
					  &yt,
					  std::vector< std::vector<double> > 
					  &dyt,
					  std::vector< std::vector<double> > 
					  &dym,
					  std::vector<size_t> &mechEq,
					  std::vector<size_t> &mechPos ) 
{ 
  // Go
  //////////////////////////////////////////////////////////////////////
  double t = 0.0, maxDydt=0.0;
  while( t<cTmax ) {
    if (debugFlag()) {
			yCopy_[debugCount()] = y_;
		} 
    //Update the derivatives
    O_->derivsMechanical(y_,dydt_,mechEq,mechPos);
    
    //Update
    rk4Mechanical(yt,dyt,dym,mechEq,mechPos);
    
		// Make updates of rections that requires it
		O_->reactionUpdate(h_, t_, y_);

    // Check for cell divisions, compartmental removal etc
    O_->compartmentChange(y_,dydt_,t_);
    
    //Rescale all temporary vectors as well
    if(y_.size() != yt.size() ) {
      yt.resize( N(), yt[0] );
      dyt.resize( N(), dyt[0] );
      dym.resize( N(), dym[0] );
    }
    
    //Update neighborhood
    if( O_->numNeighborhood() )
      O_->neighborhoodUpdate(y_,t_);
    
    //Check if derivatives have converged
    maxDydt=0.0;
    for( size_t i=0 ; i<N() ; i++ )
      for( size_t k=0 ; k<mechPos.size() ; k++ ) {
	size_t j=mechPos[k];
	if( std::fabs(dydt_[i][j])>maxDydt )
	  maxDydt = std::fabs(dydt_[i][j]); 
      }
    if( maxDydt<cEps ) {
      //std::cerr << "Rk4::updateMechanicsUntilConvergance "
      //	<< "t=" << t << " maxDydt=" << maxDydt << "\n";
      return;
    }
    
    
    t += h_;
  }
  std::cerr << "Rk4::updateMechanicsUntilConvergance "
	    << "Warning max time " << cTmax << " reached with"
	    << " Max dy/dt " << maxDydt << "." << std::endl; 
  return;
}

void RK4::rk4(std::vector< std::vector<double> > &yt,
	      std::vector< std::vector<double> > &dyt,
	      std::vector< std::vector<double> > &dym ) 
{  
  double hh=0.5*h_;
  double h6=h_/6.0;
	size_t n = N();
	size_t m = M();
  for(size_t i=0 ; i<n ; ++i )
    for( size_t j=0 ; j<m ; ++j )
      yt[i][j]=y_[i][j]+hh*dydt_[i][j];
  
  O_->derivs(yt,dyt);//second step
  for(size_t i=0 ; i<n ; ++i )
    for( size_t j=0 ; j<m ; ++j )
      yt[i][j]=y_[i][j]+hh*dyt[i][j];
  
  O_->derivs(yt,dym);//third step
  for(size_t i=0 ; i<n ; ++i )
    for( size_t j=0 ; j<m ; ++j ) {
      yt[i][j]=y_[i][j]+h_*dym[i][j];
      dym[i][j] += dyt[i][j];
    }
  
  O_->derivs(yt,dyt);//fourth step
  for(size_t i=0 ; i<n ; ++i )
    for( size_t j=0 ; j<m ; ++j )
      y_[i][j]=y_[i][j]+h6*(dydt_[i][j]+dyt[i][j]+2.0*dym[i][j]);
}

void RK4::rk4Mechanical(std::vector< std::vector<double> > &yt,
			std::vector< std::vector<double> > &dyt,
			std::vector< std::vector<double> > &dym,
			std::vector<size_t> &mechEq,
			std::vector<size_t> &mechPos ) 
{  
  double hh=0.5*h_;
  double h6=h_/6.0;

  for(size_t i=0 ; i<N() ; i++ )
    for( size_t k=0 ; k<mechPos.size() ; k++ ) {
      size_t j=mechPos[k];
      yt[i][j]=y_[i][j]+hh*dydt_[i][j];
    }
  
  O_->derivsMechanical(yt,dyt,mechEq,mechPos);//second step
  for(size_t i=0 ; i<N() ; i++ )
    for( size_t k=0 ; k<mechPos.size() ; k++ ) {
      size_t j=mechPos[k];
      yt[i][j]=y_[i][j]+hh*dyt[i][j];
    }
  
  O_->derivsMechanical(yt,dym,mechEq,mechPos);//third step
  for(size_t i=0 ; i<N() ; i++ )
    for( size_t k=0 ; k<mechPos.size() ; k++ ) {
      size_t j=mechPos[k];
      yt[i][j]=y_[i][j]+h_*dym[i][j];
      dym[i][j] += dyt[i][j];
    }
  
  O_->derivsMechanical(yt,dyt,mechEq,mechPos);//fourth step
  for(size_t i=0 ; i<N() ; i++ )
    for( size_t k=0 ; k<mechPos.size() ; k++ ) {
      size_t j=mechPos[k];
      y_[i][j]=y_[i][j]+h6*(dydt_[i][j]+dyt[i][j]+2.0*dym[i][j]);
    }
}

//!Finds the maximal |dydt|/|y| for the system
double RK4::maxDerivative() {  
  double max=0.0,val;
  for( size_t i=0 ; i<N() ; i++ ) {
    for( size_t j=0 ; j<M() ; j++ ) {
      if( y_[i][j]!=0 )
	val = fabs( dydt_[i][j]/y_[i][j] );
      else
	val = fabs( dydt_[i][j] );
      
      if( val>max ) 
	max = val;
    }
  }
  return max;
}

