#ifndef RUNGEKUTTA_H
#define RUNGEKUTTA_H

#include "../solvers/baseSolver.h"

///
/// @brief A fifth order Runge-Kutta solver for ODEs
///
class RK5Adaptive : public BaseSolver {

private:

	double eps_;
	double h1_;

public:
	///
	/// @brief Main constructor
	///
	RK5Adaptive(Organism *O,std::ifstream &IN);
	
	///
	/// @brief Reads the parameters used by the RK5Adaptive algorithm
	///
	/// This function is responsible for reading parameters used by the fifth
	/// order Runge-Kutta algorithm. The parameter file sent to the simulator
	/// binary looks like:
	///
	/// <pre> 
	/// RK5Adaptive
	/// T_start T_end 
	/// printFlag printNum 
	/// h1 eps 
	/// </pre> 
	///
	/// where RK5Adaptive is the identity string used by BaseSolver::getSolver
	/// to identify that the RK5Adaptive algorithm should be used. T_start
	/// (T_end) is the start (end) time for the simulation, printFlag is an
	/// integer which sets the output format (read by BaseSolver::print()),
	/// printNum is the number of equally spread time points to be printed. h1
	/// is the maximal (and initial) step size for each RK5 step, and eps sets
	/// the error threshold (should be <<1.0).
	///
	/// Comments can be included in the parameter file by starting the line with
	/// an #. Caveat: No check on the validity of the read data is applied.
	///
	/// @see BaseSolver::getSolver()
	/// @see BaseSolver::print()
	///
	void readParameterFile(std::ifstream &IN);
	
	void simulate(void);
	
	///
	/// @brief Runs a simulation of an organism model where mechanics is assumed
	/// fast
	///
	/// Runs a simulation where the mechanics equations are treated specifically
	/// to always be in equilibrium. At each step the mechanical equations are
	/// run until convergence.
	///
	void simulateWithConvergingMechanics(double cEps, double cTdiff, std::vector<size_t> &mechEq, std::vector<size_t> &mechPos);
	
	///
	/// @brief Updates mechanical equations until convergance 
	///
	/// Runs a simulation on mechanical equations only until it converges. It
	/// takes a system state as input and updates only positional variables
	/// using all mechanical equations until this system converges. During the
	/// process also the neighborhood is allowed to be updated, but no
	/// compartmentChanges are allowed for.
	///
	void updateMechanicsUntilConvergence(double cEps,double cTdiff, std::vector< std::vector<double> >  &yScal, std::vector< std::vector<double> >  &yTemp,
	 std::vector< std::vector<double> >  &yErr, std::vector< std::vector<double> >  &ak2, std::vector< std::vector<double> >  &ak3,
	 std::vector< std::vector<double> >  &ak4, std::vector< std::vector<double> >  &ak5, std::vector< std::vector<double> >  &ak6,
	 std::vector< std::vector<double> >  &yTempRkck, std::vector<size_t> &mechEq, std::vector<size_t> &mechPos);
	
	///
	/// @brief Fifth order Runge-Kutta adaptive stepper.
	///
	void rkqs(double hTry, double &hDid, double &hNext,
	std::vector< std::vector<double> > &yScal,
	std::vector< std::vector<double> > &yTemp,
	std::vector< std::vector<double> > &yErr,
	std::vector< std::vector<double> > &ak2,
	std::vector< std::vector<double> > &ak3,
	std::vector< std::vector<double> > &ak4,
	std::vector< std::vector<double> > &ak5,
	std::vector< std::vector<double> > &ak6,
	std::vector< std::vector<double> > &yTempRkck);
	
	///
	/// @brief Fifth order Runge-Kutta adaptive stepper for mechanical update.
	///
	void rkqsMechanical(double &t, double hTry, double &hDid, double &hNext,
	std::vector< std::vector<double> > &yScal,
	std::vector< std::vector<double> > &yTemp,
	std::vector< std::vector<double> > &yErr,
	std::vector< std::vector<double> > &ak2,
	std::vector< std::vector<double> > &ak3,
	std::vector< std::vector<double> > &ak4,
	std::vector< std::vector<double> > &ak5,
	std::vector< std::vector<double> > &ak6,
	std::vector< std::vector<double> > &yTempRkck,
	std::vector<size_t> &mechEq,
	std::vector<size_t> &posVar);
	
	///
	/// @brief One fifth order Runge-Kutta step
	///
	void rkck(double h,
	std::vector< std::vector<double> > &yOut,
	std::vector< std::vector<double> > &yErr,
	std::vector< std::vector<double> > &ak2,
	std::vector< std::vector<double> > &ak3,
	std::vector< std::vector<double> > &ak4,
	std::vector< std::vector<double> > &ak5,
	std::vector< std::vector<double> > &ak6,
	std::vector< std::vector<double> > &yTempRkck);
	
	///
	/// @brief One fifth order Runge-Kutta step for positional variables
	///
	/// A fifth order Runge-Kutta step. Equivalent to the normal rkck, but only
	/// updates according to the equations provided in mechEq and updates only
	/// variables provided in posVar. Caveat: The check that mechEq only update
	/// positional variables are left to the user!
	///
	void rkckMechanical(double h,
	std::vector< std::vector<double> > &yOut,
	std::vector< std::vector<double> > &yErr,
	std::vector< std::vector<double> > &ak2,
	std::vector< std::vector<double> > &ak3,
	std::vector< std::vector<double> > &ak4,
	std::vector< std::vector<double> > &ak5,
	std::vector< std::vector<double> > &ak6,
	std::vector< std::vector<double> > &yTempRkck,
	std::vector<size_t> &mechEq,
	std::vector<size_t> &posVar );
	double maxDerivative();
};

///
/// @brief A ODE solver using a fifth order Runge-Kutta method with
/// possibility to compare the result to a cos template and read some
/// variables from a file.
///
/// A fifth order Runge-Kutta algorithm that allows for cost calculations and
/// reading some of the varables from a file.
///
class RK5AdaptiveTemplate : public BaseSolver {

private:

	double eps_;
	double h1_;

public:

	RK5AdaptiveTemplate(Organism *O,std::ifstream &IN);
	~RK5AdaptiveTemplate();
	
	///
	/// @brief Reads the parameters used by the RK5AdaptiveTemplate algorithm
	///
	/// This function is responsible for reading parameters used by the fifth
	/// order Runge-Kutta algorithm which allows for cost calculations and
	/// reading some of the varables from a file. The parameter file sent to the
	/// simulator binary looks like:
	///
	/// <pre> 
	/// RK5AdaptiveTemplate
	/// costType
	/// T_start T_end 
	/// printFlag printNum 
	/// h1 eps 
	/// </pre> 
	///
	/// where RK5AdaptiveTemplate is the identity string used by
	/// BaseSolver::getSolver to identify that the RK5AdaptiveTemplate algorithm
	/// should be used. costType is the functional form (class) used for
	/// calculating the 'cost' (objective function value, energy...) of a
	/// simulation and possible functions for the moment are meanSquare and
	/// meanSquareRelative. T_start (T_end) is the start (end) time for the
	/// simulation, printFlag is an integer which sets the output format (read
	/// by BaseSolver::print()), printNum is the number of equally spread time
	/// points to be printed. h1 is the maximal (and initial) step size for each
	/// RK5 step, and eps sets the error limit (should be <<1.0).
	///
	/// Comments can be included in the parameter file by starting the line with
	/// an #. Caveat: No check on the validity of the read data is applied.
	///
	/// @see BaseSolver::getSolver()
	/// @see BaseSolver::print()
	///	@see BaseCostFunction::createCostFunction()
	///
	void readParameterFile(std::ifstream &IN, int verbose=0);
	
	void simulate(void);
	
	///
	/// @brief Fifth order Runge-Kutta adaptiveTemplate stepper.
	///
	void rkqsTemplate(double hTry, double &hDid, double &hNext,
	std::vector< std::vector<double> > &yScal,
	std::vector< std::vector<double> > &yTemp,
	std::vector< std::vector<double> > &yErr,
	std::vector< std::vector<double> > &ak2,
	std::vector< std::vector<double> > &ak3,
	std::vector< std::vector<double> > &ak4,
	std::vector< std::vector<double> > &ak5,
	std::vector< std::vector<double> > &ak6,
	std::vector< std::vector<double> > &yTempRkck);
	
	///
	/// @brief One fifth order Runge-Kutta step
	///
	/// A fifth order Runge-Kutta step. It uses y,dydt as initial conditions at
	/// time t and takes a step of size h, returning the new function values in
	/// yOut. yErr is the estimated error in each variable (calculated as the
	/// difference of a fourth and fifth order step). The function and all of
	/// the parameters are taken from Numerical Recipes.
	///
	void rkckTemplate(double h,
	std::vector< std::vector<double> > &yOut,
	std::vector< std::vector<double> > &yErr,
	std::vector< std::vector<double> > &ak2,
	std::vector< std::vector<double> > &ak3,
	std::vector< std::vector<double> > &ak4,
	std::vector< std::vector<double> > &ak5,
	std::vector< std::vector<double> > &ak6,
	std::vector< std::vector<double> > &yTempRkck);
};

///
/// @brief This class solves the ODE using a fourth order Runge-Kutta solver
///
class RK4 : public BaseSolver {

private:

	double h_;
	unsigned int numStep_;

public:
	
	///
	/// @brief Main constructor.
	///
	RK4(Organism *O,std::ifstream &IN);
	
	///
	/// @brief Reads the parameters used by the RK4 algorithm
	///
	/// This function is responsible for reading parameters used by the fourth
	/// order Runge-Kutta algorithm. The parameter file sent to the simulator
	/// binary looks like:
	///
	/// <pre> 
	/// RK4 
	/// T_start T_end 
	/// printFlag printNum 
	/// h 
	/// </pre> 
	///
	/// where RK4 is the identity string used by BaseSolver::getSolver to
	/// identify that the RK4 algorithm should be used. T_start (T_end) is the
	/// start (end) time for the simulation, printFlag is an integer which sets
	/// the output format (read by BaseSolver::print()), printNum is the number
	/// of equally spread time points to be printed, and h is the step size for
	/// each RK4 step.
	///
	/// Comments can be included in the parameter file by starting the line with
	/// an #. Caveat: No check on the validity of the read data is applied.
	///
	/// @see BaseSolver::getSolver()
	/// @see BaseSolver::print()
	///
	void readParameterFile(std::ifstream &IN);
	
	///
	/// @brief Runs a simulation of an organism model
	///
	void simulate(void);
	
	///
	/// @brief Runs a simulation of an organism model where the mechanics is
	/// updated until convergance between each step
	///
	void simulateWithConvergingMechanics(double cEps,double cTmax,
	 std::vector<size_t> &mechEq,
	 std::vector<size_t> &mechPos);
	
	///
	/// @brief The mechanics update function for the
	/// simulateWithConvergingMechanics solver
	///
	void updateMechanicsUntilConvergence(double cEps,double cTmax,
	 std::vector< std::vector<double> > 
	 &yt,
	 std::vector< std::vector<double> > 
	 &dyt,
	 std::vector< std::vector<double> > 
	 &dym,
	 std::vector<size_t> &mechEq,
	 std::vector<size_t> &mechPos );
	///
	/// @brief Fourth order Runge-Kutta stepper
	///
	void rk4(std::vector< std::vector<double> > &yt,
	 std::vector< std::vector<double> > &dyt,
	 std::vector< std::vector<double> > &dym );
	
	///
	/// @brief Fourth order Runge-Kutta stepper for the positional variables
	///
	void rk4Mechanical(std::vector< std::vector<double> > &yt,
	 std::vector< std::vector<double> > &dyt,
	 std::vector< std::vector<double> > &dym,
	 std::vector<size_t> &mechEq,
	 std::vector<size_t> &mechPos);
	double maxDerivative();
};

#endif /* RUNGEKUTTA_H */

